SystemOrganization addCategory: #'XML-Parser'!
SystemOrganization addCategory: 'XML-Parser-DOM'!
SystemOrganization addCategory: 'XML-Parser-DTD'!
SystemOrganization addCategory: 'XML-Parser-Drivers'!
SystemOrganization addCategory: 'XML-Parser-Exceptions'!
SystemOrganization addCategory: 'XML-Parser-Files'!
SystemOrganization addCategory: 'XML-Parser-HTTP'!
SystemOrganization addCategory: 'XML-Parser-Namespaces'!
SystemOrganization addCategory: 'XML-Parser-Streams'!
SystemOrganization addCategory: 'XML-Parser-URI'!
SystemOrganization addCategory: 'XML-Parser-Utils'!
SystemOrganization addCategory: 'XML-Parser-Validation'!
SystemOrganization addCategory: 'XML-Parser-Validation-DFA'!

!Collection methodsFor: '*XML-Parser' stamp: 'monty 8/26/2016 02:16'!
asXMLAttributeSpec
	^ XMLBasicAttributeSpec attributes: self! !

!Collection methodsFor: '*XML-Parser' stamp: 'monty 4/15/2016 02:07'!
includesXMLName: aName
	"optimized with do: instead of anySatisfy:"
	self do: [:each |
		(aName isXMLQualifiedOrLocalName: each)
			ifTrue: [^ true]].
	^ false.! !

Collection subclass: #XMLNFAStateSet
	instanceVariableNames: 'set isAccepting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAStateSet commentStamp: 'monty 1/24/2014 22:44' prior: 0!
This class is a set of NFA states. It wraps an IdentitySet and keeps track of if any of its states is accepting.!

!XMLNFAStateSet class methodsFor: 'instance creation' stamp: 'monty 12/11/2014 05:39'!
new
	^ self new: 5! !

!XMLNFAStateSet class methodsFor: 'instance creation' stamp: 'monty 12/11/2014 05:39'!
new: aCapacity
	^ self basicNew initialize: aCapacity! !

!XMLNFAStateSet class methodsFor: 'instance creation' stamp: 'monty 3/29/2016 07:11'!
newFrom: aCollection
	"for GS portability"
	^ self withAll: aCollection! !

!XMLNFAStateSet methodsFor: 'copying' stamp: 'monty 3/30/2016 05:25'!
, aCollection
	"GS does not define , for non-sequenceable collections"
	^ self copy
		addAll: aCollection;
		yourself! !

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 3/29/2016 06:53'!
= anObject
	self == anObject
		ifTrue: [^ true].

	(self species == anObject species
		and: [self size = anObject size])
		ifFalse: [^ false].

	"avoid allSatisfy: for speed"
	self do: [:each |
		(anObject includes: each)
			ifFalse: [^ false]].
	^ true.! !

!XMLNFAStateSet methodsFor: 'adding' stamp: 'monty 12/28/2013 18:52'!
add: aState
	isAccepting
		ifFalse: [isAccepting := aState isAccepting].

	^ set add: aState.! !

!XMLNFAStateSet methodsFor: 'converting' stamp: 'monty 4/25/2015 15:15'!
asDFAState
	^ self asDFAStateWithStateCache: nil! !

!XMLNFAStateSet methodsFor: 'converting' stamp: 'monty 4/16/2015 01:12'!
asDFAStateWithStateCache: aDFAStateCache
	^ XMLDFAState
		nfaStates: self
		dfaStateCache: aDFAStateCache! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 3/29/2016 08:17'!
capacity
	^ set capacity! !

!XMLNFAStateSet methodsFor: 'enumerating' stamp: 'monty 12/28/2013 18:47'!
do: aBlock
	set do: aBlock! !

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 3/29/2016 07:00'!
hash
	| hash |

	hash := self species hash.
	self size > 5
		ifFalse: [
			self do: [:each | hash := hash bitXor: each hash]].
	^ hash bitXor: self size hash.! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:48'!
includes: aState
	^ set includes: aState! !

!XMLNFAStateSet methodsFor: 'initialization' stamp: 'monty 12/11/2014 05:39'!
initialize: aCapacity
	set := IdentitySet new: aCapacity.
	isAccepting := false.! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:53'!
isAccepting
	^ isAccepting! !

!XMLNFAStateSet methodsFor: 'copying' stamp: 'monty 3/29/2016 08:09'!
postCopy
	set := set copy! !

!XMLNFAStateSet methodsFor: 'removing' stamp: 'monty 3/29/2016 08:28'!
remove: aState ifAbsent: aBlock
	set
		remove: aState
		ifAbsent: [^ aBlock value].

	isAccepting
		ifTrue: [isAccepting := set anySatisfy: [:each | each isAccepting]].

	^ aState.! !

!XMLNFAStateSet methodsFor: 'removing' stamp: 'monty 4/2/2016 01:52'!
removeAll
	"Redefined because Collection>>removeAll assumes the collection can
	be safely modified while iterating it with do:, and 'set removeAll'
	is not used because GS's Set/IdentitySet don't implement it"

	self initialize: self capacity! !

!XMLNFAStateSet methodsFor: 'private' stamp: 'monty 12/28/2013 18:50'!
set
	^ set! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 19:01'!
size
	^ set size! !

Collection subclass: #XMLOrderedList
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLOrderedList commentStamp: 'monty 6/15/2015 22:15' prior: 0!
This class functions as an OrderedCollection that will not raise exceptions when attempts are made to access absent objects. This means messages like #at:, #after:, and #first will return nil when the requested object cannot be retrieved. Messages that modify the collection will still raise exceptions as usual.

!

XMLOrderedList subclass: #XMLObservableList
	instanceVariableNames: 'observer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLObservableList commentStamp: 'monty 3/29/2016 06:02' prior: 0!
This class is an ordered list that sends notifications to an observer when objects are added to it or removed from it. Subclasses should override the #beforeAdding:, #afterAdding:at:, #afterAddingLast: and #afterRemoving: messages to send custom notifications.!

XMLObservableList subclass: #XMLAttributeList
	instanceVariableNames: 'attributeNodesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeList commentStamp: 'monty 6/16/2015 01:29' prior: 0!
This class represents an ordered list of attribute nodes that also keeps a dictionary mapping attribute names to nodes for faster lookup.!

!XMLAttributeList class methodsFor: 'instance creation' stamp: 'monty 4/16/2015 01:05'!
new
	^ self new: 5! !

!XMLAttributeList methodsFor: 'adding' stamp: 'monty 3/2/2016 15:53'!
addAttributeAssociation: anAssociation
	^ self
		addAttributeName: anAssociation key
		value: anAssociation value! !

!XMLAttributeList methodsFor: 'adding' stamp: 'monty 3/2/2016 15:51'!
addAttributeName: aName value: aValue
	^ self addLast:
		(self attributeClass
			name: aName
			value: aValue)! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/18/2016 08:59'!
afterAdding: aNode at: anIndex
	self attributeNodesByName
		at: aNode name ifPresent: [:oldNode |
			self
				replaced: oldNode
				with: aNode 
				at: anIndex];
		at: aNode name put: aNode.
	observer ifNotNil: [observer addedAttribute: aNode].! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/18/2016 08:59'!
afterAddingLast: aNode
	self attributeNodesByName
		at: aNode name ifPresent: [:oldNode |
			self
				replaced: oldNode
				with: aNode 
				at: self size];
		at: aNode name put: aNode.
	observer ifNotNil: [observer addedAttribute: aNode].! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/18/2016 08:57'!
afterRemoving: aNode
	self attributeNodesByName removeKey: aNode name.
	observer ifNotNil: [observer removedAttribute: aNode].! !

!XMLAttributeList methodsFor: 'converting' stamp: 'monty 4/6/2016 22:33'!
asDictionary
	^ self attributes! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 11/6/2014 15:43'!
attributeAssociations
	^ self collect: [:each | each name -> each value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 4/6/2016 22:47'!
attributeAt: aName
	^ self attributeValueAt: aName! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 4/6/2016 22:47'!
attributeAt: aName ifAbsent: aBlock
	^ self
		attributeValueAt: aName
		ifAbsent: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 4/6/2016 22:47'!
attributeAt: aName ifAbsentPut: aBlock
	^ self
		attributeValueAt: aName
		ifAbsentPut: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 4/6/2016 22:47'!
attributeAt: aName put: aValue
	^ self
		attributeValueAt: aName
		put: aValue! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 14:15'!
attributeClass
	^ XMLAttribute
! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 14:14'!
attributeNames
	^ self collect: [:each | each name]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'monty 3/2/2016 18:21'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self do: [:each |
		aTwoArgumentBlock
			value: each name
			value: each value]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 14:15'!
attributeNamesDo: aBlock
	self do: [:each | aBlock value: each name]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 3/2/2016 18:21'!
attributeNodeAt: aName
	^ self
		attributeNodeAt: aName
		ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 4/18/2016 09:02'!
attributeNodeAt: aName ifAbsent: aBlock
	self do: [:each |
		(each isNamed: aName)
			ifTrue: [^ each]].
	^ aBlock value.! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 4/6/2016 21:53'!
attributeNodeAt: aName namespaceURI: aURI
	^ self
		attributeNodeAt: aName
		namespaceURI: aURI
		ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 4/18/2016 09:04'!
attributeNodeAt: aName namespaceURI: aURI ifAbsent: aBlock
	self do: [:each |
		((each isNamed: aName)
			and: [each namespaceURI = aURI])
			ifTrue: [^ each]].
	^ aBlock value.! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/19/2016 08:54'!
attributeNodesByName
	^ attributeNodesByName
		ifNil: [attributeNodesByName := Dictionary new: self capacity]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/6/2016 22:45'!
attributeValueAt: aName
	^ self
		attributeValueAt: aName
		ifAbsent: ['']! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/18/2016 03:53'!
attributeValueAt: aName ifAbsent: aBlock
	^ (self attributeNodesByName
		at: aName
		ifAbsent: [^ aBlock value]) value! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/6/2016 22:45'!
attributeValueAt: aName ifAbsentPut: aBlock
	^ self
		attributeValueAt: aName
		ifAbsent: [
			self
				attributeValueAt: aName
				put: aBlock value]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/6/2016 22:46'!
attributeValueAt: aName put: aValue
	(self includesAttribute: aName)
		ifTrue: [(self attributeNodeAt: aName) value: aValue]
		ifFalse: [self addAttributeName: aName value: aValue].
	^ aValue.! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 4/6/2016 22:33'!
attributes
	| dictionary |

	dictionary := OrderPreservingStringDictionary new: self size.
	self do: [:each |
		dictionary
			at: each name
			put: each value].
	^ dictionary.! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/18/2016 08:57'!
beforeAdding: aNode
	observer ifNotNil: [observer addingAttribute: aNode]! !

!XMLAttributeList methodsFor: 'testing' stamp: 'monty 4/18/2016 03:53'!
includes: aNode
	^ aNode isAttribute
		and: [
			(self attributeNodesByName
				at: aNode name
				ifAbsent: [^ false]) = aNode]! !

!XMLAttributeList methodsFor: 'testing' stamp: 'monty 4/18/2016 03:53'!
includesAttribute: aName
	^ self attributeNodesByName includesKey: aName! !

!XMLAttributeList methodsFor: 'testing' stamp: 'monty 4/19/2016 10:33'!
includesAttributeNode: aName
	self
		attributeNodeAt: aName
		ifAbsent: [^ false].
	^ true.! !

!XMLAttributeList methodsFor: 'testing' stamp: 'monty 4/19/2016 10:34'!
includesAttributeNode: aName namespaceURI: aURI
	self
		attributeNodeAt: aName
		namespaceURI: aURI
		ifAbsent: [^ false].
	^ true.! !

!XMLAttributeList methodsFor: 'removing' stamp: 'monty 4/18/2016 09:13'!
removeAttribute: aName
	^ (self remove: (self attributeNodesByName at: aName)) value! !

!XMLAttributeList methodsFor: 'removing' stamp: 'monty 4/18/2016 09:13'!
removeAttribute: aName ifAbsent: aBlock
	^ (self remove:
		(self attributeNodesByName
			at: aName
			ifAbsent: [^ aBlock value])) value! !

!XMLAttributeList methodsFor: 'notifying' stamp: 'monty 4/18/2016 07:37'!
renamedAttribute: anAttributeNode from: anOldName to: aNewName
	self attributeNodesByName
		removeKey: anOldName;
		at: aNewName ifPresent: [:attribute | self remove: attribute];
		at: aNewName put: anAttributeNode! !

!XMLAttributeList methodsFor: 'initialization' stamp: 'monty 4/6/2016 22:35'!
setAttributes: anAssociationCollection
	self removeAll.
	anAssociationCollection associationsDo: [:each |
		self addAttributeAssociation: each].! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/18/2016 03:53'!
setWithOrderPreservingDictionary: anOrderPreservingDictionary
	"hack to reuse a dictionary for speed"
	attributeNodesByName := anOrderPreservingDictionary dictionary.

	"add the XMLAttributes in order directly to the collection
	to avoid sending notifications and also replace the value in
	the attributesByName association with the XMLAttribute"
	anOrderPreservingDictionary keysDo: [:each | | association |
		(association := attributeNodesByName associationAt: each ifAbsent: nil)
			value:
				(collection addLast: 
					(self attributeClass new
						setName: each
						value: association value
						element: observer))].! !

!XMLAttributeList methodsFor: 'sorting' stamp: 'monty 12/2/2014 23:05'!
sort
	self sort: [:a :b | a sortKey <= b sortKey]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/18/2016 03:47'!
writeAttributesOn: aWriter
	"use to:do: for speed"
	1 to: collection size do: [:i | | attribute |
		attribute := collection at: i.
		aWriter
			writeAttributeName: attribute name
			value: attribute value]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 4/18/2016 03:48'!
writeSortedAttributesOn: aWriter
	| sortedAttributes |

	sortedAttributes := self sorted.
	"use to:do: for speed"
	1 to: sortedAttributes size do: [:i | | attribute |
		attribute := sortedAttributes at: i.
		aWriter
			writeAttributeName: attribute name
			value: attribute value]! !

XMLObservableList subclass: #XMLInternalSubsetList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLInternalSubsetList commentStamp: 'JAAyer 7/27/2010 12:45' prior: 0!
This class represents the list of nodes that comprise a document's internal DTD subset.!

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 1/21/2015 11:55'!
afterAdding: aNode at: anIndex
	observer ifNotNil: [observer addedNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/18/2014 00:54'!
afterRemoving: aNode
	observer ifNotNil: [observer removedNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/29/2014 09:00'!
beforeAdding: aNode
	observer ifNotNil: [observer addingNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'sorting' stamp: 'monty 1/18/2015 09:31'!
sort
	self sort: [:a :b | a sortKey <= b sortKey]! !

XMLObservableList subclass: #XMLNodeList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeList commentStamp: 'monty 6/16/2015 02:17' prior: 0!
This class represents an ordered list of child nodes for an XML document or element. Non-modifying accessing messages like #first and #last return nil instead of raising exceptions.!

XMLNodeList subclass: #XMLCachingNodeList
	instanceVariableNames: 'cachedElements cachedElementsByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCachingNodeList commentStamp: 'monty 4/3/2016 02:02' prior: 0!
This is a node list that caches child elements by name, keeping both an ordered list of elements and a dictionary that allows for faster retrieval of elements by qualified or local name.

You can inject it into the DOM parser like this:
	(parser := XMLDOMParser on: xmlSource)
		nodeFactory:
			(XMLPluggableNodeFactory new
				nodeListClass: XMLCachingNodeList);
		parseDocument!

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 02:54'!
afterAdding: aNode at: anIndex
	aNode isElement
		ifTrue: [
			anIndex = self size
				ifTrue: [self cacheElementAddedLast: aNode]
				ifFalse: [
					self
						cacheElement: aNode
						addedAt: anIndex]].
	super
		afterAdding: aNode
		at: anIndex.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 02:06'!
afterAddingLast: aNode
	aNode isElement
		ifTrue: [self cacheElementAddedLast: aNode].
	super afterAddingLast: aNode.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 02:07'!
afterRemoving: aNode
	aNode isElement
		ifTrue: [self uncacheElement: aNode].
	super afterRemoving: aNode.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:00'!
cacheElement: anElement addedAt: anIndex
	(self findElementAfterIndex: anIndex)
		ifNil: [self cachedElements addLast: anElement]
		ifNotNil: [:elementAfter |
			self cachedElements
				add: anElement
				before: elementAfter].
	self
		cachedElementsByNameAt: anElement name
		add: anElement
		beforeNextAfterIndex: anIndex.
	anElement hasPrefix
		ifTrue: [
			self
				cachedElementsByNameAt: anElement localName
				add: anElement
				beforeNextAfterIndex: anIndex].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:00'!
cacheElementAddedLast: anElement
	self cachedElements addLast: anElement.
	self
		cachedElementsByNameAt: anElement name
		addLast: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				cachedElementsByNameAt: anElement localName
				addLast: anElement].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 1/16/2015 13:46'!
cachedElements
	^ cachedElements ifNil: [cachedElements := OrderedCollection new]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 1/16/2015 13:50'!
cachedElementsByName
	^ cachedElementsByName ifNil: [cachedElementsByName := Dictionary new]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:06'!
cachedElementsByNameAt: aName add: anElement beforeNextAfterIndex: anIndex
	| elementAfter elementsWithName |

	(elementAfter :=
		self
			findElementAfterIndex: anIndex
			named: aName)
		ifNil: [
			^ self
				cachedElementsByNameAt: aName
				addLast: anElement].

	(elementsWithName := self cachedElementsByName at: aName) isCollection
		ifTrue: [elementsWithName add: anElement before: elementAfter]
		ifFalse: [
			self cachedElementsByName
				at: aName
				put: (OrderedCollection with: anElement with: elementAfter)].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:08'!
cachedElementsByNameAt: aName addLast: anElement
	| elementsWithName |

	elementsWithName :=
		self cachedElementsByName
			at: aName
			ifAbsent: [
				self cachedElementsByName at: aName put: anElement.
				^ self].

	 elementsWithName isCollection
		ifTrue: [elementsWithName addLast: anElement]
		ifFalse: [
			self cachedElementsByName
				at: aName
				put: (OrderedCollection with: elementsWithName with: anElement)].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:01'!
cachedElementsByNameAt: aName remove: anElement
	| elementsWithName |

	(elementsWithName := self cachedElementsByName at: aName) isCollection
		ifTrue: [
			elementsWithName remove: anElement.
			elementsWithName isEmpty
				ifTrue: [self cachedElementsByName removeKey: aName]]
		ifFalse: [self cachedElementsByName removeKey: aName]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 1/17/2015 08:29'!
copyClass
	"use superclass for copies for performance"
	^ XMLNodeList! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/17/2015 08:18'!
elementAt: aName ifAbsent: aBlock
	| elementsWithName |

	(elementsWithName :=
		self cachedElementsByName
			at: aName
			ifAbsent: [^ aBlock value]) isCollection
		ifTrue: [^ elementsWithName first]
		ifFalse: [^ elementsWithName]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/16/2015 13:43'!
elements
	^ self copyEmpty
		addAll: self cachedElements;
		yourself! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/16/2015 13:53'!
elementsAt: aName
	| elementsWithName copiedElements |

	copiedElements := self copyEmpty.
	(elementsWithName :=
		self cachedElementsByName
			at: aName
			ifAbsent: [^ copiedElements]) isCollection
		ifTrue: [copiedElements addAll: elementsWithName]
		ifFalse: [copiedElements addLast: elementsWithName].
	^ copiedElements.! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 4/18/2016 02:51'!
elementsAt: aName do: aBlock
	(self cachedElementsByName
		at: aName
		ifAbsent: [^ self]) do: aBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 08:16'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self cachedElements
		detect: aBlock
		ifNone: anExceptionBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 1/16/2015 13:44'!
elementsDo: aBlock
	self cachedElements do: aBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 1/17/2015 08:24'!
elementsSelect: aBlock
	| selectedElements |

	selectedElements := self copyEmpty.
	self cachedElements do: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 15:34'!
findElementAfterIndex: anIndex
	anIndex + 1 to: self size do: [:i | | node |
		node := self at: i.
		node isElement
			ifTrue: [^ node]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 00:30'!
findElementAfterIndex: anIndex named: aName
	anIndex + 1 to: self size do: [:i | | node |
		node := self at: i.
		(node isElement
			and: [node isNamed: aName])
			ifTrue: [^ node]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/16/2015 13:59'!
firstElement
	^ self cachedElements
		at: 1
		ifAbsent: [self defaultValue]! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 1/16/2015 13:47'!
hasElements
	^ cachedElements notNil and: [cachedElements notEmpty]! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 1/16/2015 13:48'!
includesElement: aName
	^ self cachedElementsByName includesKey: aName! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 1/16/2015 14:00'!
lastElement
	^ self cachedElements
		at: self cachedElements size
		ifAbsent: [self defaultValue]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/2/2015 02:58'!
rebuildCachedElements
	self cachedElements removeAll.
	self cachedElementsByName removeAll.
	self do: [:each |
		each isElement
			ifTrue: [self cacheElementAddedLast: each]].! !

!XMLCachingNodeList methodsFor: 'notifying' stamp: 'monty 4/1/2016 00:57'!
renamedElement: anElement from: anOldName to: aNewName
	| oldPrefix index |

	self
		cachedElementsByNameAt: anOldName
		remove: anElement.
	(oldPrefix := anOldName xmlPrefixBeforeLocalName)
		ifNotEmpty: [
			self
				cachedElementsByNameAt: (anOldName xmlLocalNameAfterPrefix: oldPrefix)
				remove: anElement].
	index := self indexOf: anElement.
	self
		cachedElementsByNameAt: anElement name
		add: anElement
		beforeNextAfterIndex: index.
	anElement hasPrefix
		ifTrue: [
			self
				cachedElementsByNameAt: anElement localName
				add: anElement
				beforeNextAfterIndex: index].! !

!XMLCachingNodeList methodsFor: 'sorting' stamp: 'monty 2/2/2015 01:32'!
sort: aSortBlock
	super sort: aSortBlock.

	self cachedElements sort: aSortBlock.
	self cachedElementsByName valuesDo: [:each |
		each isCollection
			ifTrue: [each sort: aSortBlock]].! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 2/2/2015 02:58'!
swap: aFirstIndex with: aSecondIndex
	super swap: aFirstIndex with: aSecondIndex.

	((self at: aFirstIndex) isElement
		or: [(self at: aSecondIndex) isElement])
		ifTrue: [self rebuildCachedElements].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 2/9/2015 06:00'!
uncacheElement: anElement
	self cachedElements remove: anElement.
	self
		cachedElementsByNameAt: anElement name
		remove: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				cachedElementsByNameAt: anElement localName
				remove: anElement].! !

!XMLNodeList class methodsFor: 'instance creation' stamp: 'monty 4/16/2015 01:05'!
new
	^ self new: 5! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 1/21/2015 11:54'!
afterAdding: aNode at: anIndex
	observer ifNotNil: [
		aNode isElement
			ifTrue: [observer addedElement: aNode]
			ifFalse: [observer addedNode: aNode]]! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 1/21/2015 12:27'!
afterAddingLast: aNode
	observer ifNotNil: [
		aNode isElement
			ifTrue: [observer addedElement: aNode]
			ifFalse: [observer addedNode: aNode]]! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/28/2014 19:05'!
afterRemoving: aNode
	observer ifNotNil: [
		aNode isElement
			ifTrue: [observer removedElement: aNode]
			ifFalse: [observer removedNode: aNode]]! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/28/2014 13:48'!
beforeAdding: aNode
	observer ifNotNil: [observer addingNode: aNode]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:08'!
contentNodes
	^ self select: [:each | each isContentNode]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:08'!
contentNodesDo: aBlock
	self do: [:each | each isContentNode ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 4/15/2016 06:50'!
contentStringAt: aName
	^ (self
		elementAt: aName
		ifAbsent: [^ '']) contentString! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:27'!
elementAfter: anElement
	| last |

	self elementsDo: [:each |
		last = anElement
			ifTrue: [^ each].
		last := each].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName
	^ self elementAt: aName ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 4/18/2016 09:07'!
elementAt: aName ifAbsent: aBlock
	self do: [:each |
		(each isElementNamed: aName)
			ifTrue: [^ each]].
	^ aBlock value.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementAt: aName namespaceURI: aURI
	^ self
		elementAt: aName
		namespaceURI: aURI
		ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 4/18/2016 09:08'!
elementAt: aName namespaceURI: aURI ifAbsent: aBlock
	self
		elementsAt: aName
		do: [:each |
			each namespaceURI = aURI
				ifTrue: [^ each]].
	^ aBlock value.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:27'!
elementBefore: anElement
	| last |

	last := self defaultValue.
	self elementsDo: [:each |
		each = anElement
			ifTrue: [^ last].
		last := each].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/26/2014 08:19'!
elementNames
	^ self elementsCollect: [:each | each name]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:07'!
elements
	^ self select: [:each | each isElement]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:12'!
elementsAt: aName
	^ self elementsSelect: [:each | each isNamed: aName]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 4/18/2016 02:54'!
elementsAt: aName do: aBlock
	self do: [:each |
		(each isElement
			and: [each isNamed: aName])
			ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 4/18/2016 09:09'!
elementsAt: aName namespaceURI: aURI
	^ self elementsSelect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:34'!
elementsCollect: aBlock
	^ self elements collect: aBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self elementsDetect: aBlock ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:10'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self
		detect: [:each | each isElement and: [aBlock value: each]]
		ifNone: anExceptionBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:11'!
elementsDo: aBlock
	self do: [:each | each isElement ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 4/20/2016 09:53'!
elementsSelect: aBlock
	^ self select: [:each |
		each isElement
			and: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:23'!
firstElement
	^ self
		detect: [:each | each isElement]
		ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasContentNodes
	^ self anySatisfy: [:each | each isContentNode]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:03'!
hasElements
	^ self anySatisfy: [:each | each isElement]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasStringNodes
	^ self anySatisfy: [:each | each isStringNode]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:04'!
includesElement: aName
	self elementAt: aName ifAbsent: [^ false].
	^ true.! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 4/19/2016 10:32'!
includesElement: aName namespaceURI: aURI
	self
		elementAt: aName
		namespaceURI: aURI
		ifAbsent: [^ false].
	^ true.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:23'!
lastElement
	self reverseDo: [:each |
		each isElement
			ifTrue: [^ each]].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 4/15/2016 06:50'!
rawContentStringAt: aName
	^ (self
		elementAt: aName
		ifAbsent: [^ '']) rawContentString! !

!XMLNodeList methodsFor: 'notifying' stamp: 'monty 10/18/2014 01:06'!
renamedElement: anElement from: anOldName to: aNewName! !

!XMLNodeList methodsFor: 'sorting' stamp: 'monty 12/3/2014 00:45'!
sort
	self sort: [:a :b | a sortKey <= b sortKey]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/25/2014 20:59'!
stringNodes
	^ self select: [:each | each isStringNode]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 20:57'!
stringNodesDo: aBlock
	self do: [:each | each isStringNode ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:00'!
strings
	^ self
		select: [:each | each isStringNode]
		thenCollect: [:each | each string]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 20:58'!
stringsDo: aBlock
	self stringNodesDo: [:each | aBlock value: each string]! !

!XMLObservableList class methodsFor: 'instance creation' stamp: 'monty 7/15/2016 07:35'!
observer: anObserver
	^ self new observer: anObserver! !

!XMLObservableList class methodsFor: 'instance creation' stamp: 'monty 1/17/2015 11:52'!
ofSize: aSize
	self shouldNotImplement! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 14:53'!
add: anObject after: aPreviousObject
	^ self
		add: anObject
		afterIndex:
			(collection
				indexOf: aPreviousObject
				ifAbsent: [-1])! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 11:51'!
add: anObject afterIndex: anIndex
	self beforeAdding: anObject.
	collection
		add: anObject
		afterIndex: anIndex.
	self
		afterAdding: anObject
		at: anIndex + 1.
	^ anObject.! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 14:53'!
add: anObject before: aNextObject
	^ self
		add: anObject
		beforeIndex:
			(collection
				indexOf: aNextObject
				ifAbsent: [-1])! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 2/2/2015 01:29'!
add: anObject beforeIndex: anIndex
	self beforeAdding: anObject.
	collection
		add: anObject
		beforeIndex: anIndex.
	self
		afterAdding: anObject
		at: anIndex.
	^ anObject.! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 11:44'!
addFirst: anObject
	self beforeAdding: anObject.
	collection addFirst: anObject.
	self afterAdding: anObject at: 1.
	^ anObject.! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 1/21/2015 12:26'!
addLast: anObject
	self beforeAdding: anObject.
	collection addLast: anObject.
	self afterAddingLast: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 1/21/2015 11:46'!
afterAdding: anObject at: anIndex
	observer ifNotNil: [observer added: anObject to: self]! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 1/21/2015 12:26'!
afterAddingLast: anObject
	self
		afterAdding: anObject
		at: self size! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
afterRemoving: anObject
	observer ifNotNil: [observer removed: anObject from: self]! !

!XMLObservableList methodsFor: 'accessing' stamp: 'monty 1/21/2015 12:35'!
at: anIndex put: anObject
	| oldObject |

	"gemstone behavior"
	anIndex = (self size + 1)
		ifTrue: [^ self addLast: anObject].

	self beforeAdding: anObject.

	oldObject := collection at: anIndex.
	collection at: anIndex put: nil.
	oldObject
		ifNotNil: [self afterRemoving: oldObject].
	
	collection at: anIndex put: anObject.
	self
		afterAdding: anObject
		at: anIndex.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
beforeAdding: anObject
	observer ifNotNil: [observer adding: anObject to: self]! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 4/20/2016 09:20'!
indexOf: anObject notAt: anIndex
	"used by subclasses to remove duplicates"
	1 to: anIndex - 1 do: [:i |
		(collection at: i) = anObject
			ifTrue: [^ i]].
	anIndex + 1 to: collection size do: [:i |
		(collection at: i) = anObject
			ifTrue: [^ i]].
	^ 0.! !

!XMLObservableList methodsFor: 'observing' stamp: 'monty 10/18/2014 00:50'!
observer
	^ observer! !

!XMLObservableList methodsFor: 'observing' stamp: 'monty 10/18/2014 00:50'!
observer: anObserver
	observer := anObserver! !

!XMLObservableList methodsFor: 'removing' stamp: 'monty 1/21/2015 10:00'!
remove: anObject ifAbsent: aBlock
	collection remove: anObject ifAbsent: [^ aBlock value].
	self afterRemoving: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'removing' stamp: 'monty 5/25/2015 20:50'!
removeAt: anIndex
	| removed |

	removed := super removeAt: anIndex.
	self afterRemoving: removed.
	^ removed.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 4/20/2016 09:20'!
replaced: anOldObject with: aNewObject at: aNewIndex
	| i |

	(i :=
		self
			indexOf: anOldObject
			notAt: aNewIndex) > 0
		ifTrue: [
			anOldObject == aNewObject
				ifTrue: [
					"exact same object being moved to another index; can be
					removed directly without signaling observers"
					collection removeAt: i]
				ifFalse: [self removeAt: i]]! !

!XMLObservableList methodsFor: 'removing' stamp: 'monty 1/21/2015 09:55'!
reset
	| removed |

	removed := collection copy.
	collection reset.
	removed do: [:each | self afterRemoving: each].! !

!XMLOrderedList class methodsFor: 'private' stamp: 'monty 1/17/2015 08:28'!
collection: aCollection
	^ self basicNew setCollection: aCollection! !

!XMLOrderedList class methodsFor: 'private' stamp: 'monty 1/29/2014 01:12'!
collectionClass
	^ OrderedCollection! !

!XMLOrderedList class methodsFor: 'inspecting' stamp: 'monty 3/10/2016 14:28'!
inspectorClass
	"for pre-GTInspector Pharo"
	^ self collectionClass inspectorClass! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 4/16/2015 01:05'!
new
	^ self new: 10! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/16/2015 07:12'!
new: aCapacity
	^ self basicNew setCollection: (self collectionClass new: aCapacity)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/16/2015 07:12'!
new: aSize withAll: anObject
	| newList |

	"reimplement for portability"
	newList := self new: aSize.
	1 to: aSize do: [:i | newList addLast: anObject].
	^ newList.! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/16/2015 07:11'!
newFrom: aCollection
	^ (self new: aCollection size)
		addAll: aCollection;
		yourself! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:43'!
ofSize: aSize
	^ self new: aSize withAll: nil! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/23/2014 10:54'!
, aCollection 
	^ self
		copyReplaceFrom: self size + 1
		to: self size
		with: aCollection! !

!XMLOrderedList methodsFor: 'comparing' stamp: 'monty 4/11/2015 08:00'!
= aCollection
	self == aCollection
		ifTrue: [^ true].

	(self species == aCollection species
		and: [self size = aCollection size])
		ifFalse: [^ false].

	"to:do: used for speed"
	1 to: self size do: [:i |
		(self at: i) = (aCollection at: i)
			ifFalse: [^ false]].
	^ true.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:01'!
add: anObject
	^ self addLast: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'monty 4/3/2016 01:52'!
add: anObject after: aPreviousObject
	^ collection add: anObject after: aPreviousObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject afterIndex: anIndex
	^ collection add: anObject afterIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'monty 4/3/2016 01:52'!
add: anObject before: aNextObject
	^ collection add: anObject before: aNextObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:10'!
add: anObject beforeIndex: anIndex
	^ collection add: anObject beforeIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:02'!
addAll: aCollection
	^ self addAllLast: aCollection! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:10'!
addAllFirst: aCollection
	aCollection reverseDo: [:each | self addFirst: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:11'!
addAllLast: aCollection
	aCollection do: [:each | self addLast: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:11'!
addFirst: anObject
	^ collection addFirst: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:12'!
addLast: anObject
	^ collection addLast: anObject! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:10'!
after: anObject
	^ self after: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
after: anObject ifAbsent: aBlock
	^ collection after: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:56'!
allButFirst
	^ self allButFirst: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 18:05'!
allButFirst: anInteger
	^ self copyFrom: anInteger + 1 to: self size! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:32'!
allButFirstDo: aBlock
	"reimplemented for portability"
	2 to: collection size do: [:i |
		aBlock value: (collection at: i)]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:56'!
allButLast
	^ self allButLast: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 18:02'!
allButLast: anInteger
	^ self copyFrom: 1 to: self size - anInteger! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:32'!
allButLastDo: aBlock
	"reimplemented for portability"
	1 to: collection size - 1 do: [:i |
		aBlock value: (collection at: i)]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
anyOne
	^ self first! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 5/7/2015 21:46'!
asArray
	"use withAll: not newFrom: for portability"
	^ Array withAll: collection! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 5/7/2015 21:46'!
asOrderedCollection
	"use withAll: not newFrom: for portability"
	^ OrderedCollection withAll: collection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 1/16/2015 13:06'!
at: anIndex
	"optimized"
	(anIndex > 0 and: [anIndex <= collection size])
		ifTrue: [^ collection at: anIndex]
		ifFalse: [^ self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
at: anIndex ifAbsent: aBlock
	^ collection at: anIndex ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 1/18/2015 18:06'!
at: anIndex put: aValue
	"treats index = size + 1 as addLast: gemstone compatibility"
	anIndex = (collection size + 1)
		ifTrue: [^ collection addLast: aValue]
		ifFalse: [^ collection at: anIndex put: aValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 00:55'!
atLast: anIndexFromEnd
	^ self atLast: anIndexFromEnd ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:18'!
atLast: anIndexFromEnd ifAbsent: aBlock
	^ collection atLast: anIndexFromEnd ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:11'!
before: anObject
	^ self before: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
before: anObject ifAbsent: aBlock
	 ^ collection before: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
beginsWith: aCollection
	^ collection beginsWith: aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 2/23/2016 13:36'!
capacity
	^ collection capacity! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:27'!
collect: aBlock
	| collected |

	collected := self copyEmptyForCollect.
	"use to:do: for speed"
	1 to: collection size do: [:i |
		collected addLast: (aBlock value: (collection at: i))].
	^ collected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:34'!
collect: aCollectBlock thenSelect: aSelectBlock
	| selected |

	selected := self copyEmpty.
	"use to:do: for speed"
	1 to: collection size do: [:i | | item |
		item := aCollectBlock value: (collection at: i).
		(aSelectBlock value: item)
			ifTrue: [selected addLast: item]].
	^ selected.! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAA 3/25/2011 15:35'!
collection
	^ collection! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 12:35'!
copy
	^ self copyEmpty
		addAll: self;
		yourself! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 06:16'!
copyAfter: anObject
	| copy |

	copy := self copyEmpty.
	(self
		indexOf: anObject
		ifAbsent: [^ copy]) + 1
			to: self size
			do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 06:16'!
copyAfterLast: anObject
	| copy |

	copy := self copyEmpty.
	(self
		lastIndexOf: anObject
		ifAbsent: [^ copy]) + 1
			to: self size
			do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 4/7/2016 05:38'!
copyClass
	"the class to use for copies"
	^ self class! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 08:28'!
copyEmpty
	^ self copyClass collection: collection copyEmpty! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 4/20/2016 11:01'!
copyEmptyForCollect
	"collect: methods return instances of the generic ordered list species class
	because a collect block can return any value, which may not be appropiate to
	store in copies of particular subclasses (like XMLNodeList)"

	^ self species new: self size! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:04'!
copyFrom: aStart to: anEnd
	| copy |

	copy := self copyEmpty.
	(aStart max: 1)
		to: (anEnd min: self size)
		do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 15:28'!
copyReplaceAll: aSearchCollection with: aReplacementCollection
	| copy index size searchCollectionSize |

	"this implementation always returns a copy, even if it doesn't replace"
	(searchCollectionSize := aSearchCollection size) > 0
		ifFalse: [^ self copy].

	copy := self copyEmpty.
	index := 1.
	size := self size.
	[index <= size]
		whileTrue: [
			(self at: index) = (aSearchCollection at: 1)
				ifTrue: [| searchCollectionIndex |
					searchCollectionIndex := 2.
					[searchCollectionIndex <= searchCollectionSize
						and: [
							(self at: index + searchCollectionIndex - 1) =
								(aSearchCollection at: searchCollectionIndex)]]
						whileTrue: [searchCollectionIndex := searchCollectionIndex + 1].
					searchCollectionIndex > searchCollectionSize
						ifTrue: [
							copy addAll: aReplacementCollection.
							index := index + searchCollectionSize]
						ifFalse: [
							copy addLast: (self at: index).
							index := index + 1]]
				ifFalse: [ 
					copy addLast: (self at: index).
					index := index + 1]].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 4/20/2016 05:27'!
copyReplaceFrom: aStart to: aStop with: aReplacementCollection
	| copy start stop |

	(self size < 1
		or: [self size < aStart])
		ifTrue: [
			^ self copy
				addAll: aReplacementCollection;
				yourself].

	copy := self copyEmpty.
	start := aStart max: 1.
	stop := aStop min: self size.
	1 to: self size do: [:i |
		(i = start)
			ifTrue: [
				copy addAll: aReplacementCollection.
				start > stop
					ifTrue: [copy addLast: (self at: i)]]
			ifFalse: [
				(i > start and: [i <= stop])
					ifFalse: [copy addLast: (self at: i)]]].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 06:21'!
copyUpTo: anObject
	| copy |

	copy := self copyEmpty.
	1
		to:
			(self
				indexOf: anObject
				ifAbsent: [self size + 1]) - 1
		do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 06:21'!
copyUpToLast: anObject
	| copy |

	copy := self copyEmpty.
	1
		to:
			(self
				lastIndexOf: anObject
				ifAbsent: [collection size + 1]) - 1
		do: [:i | copy addLast: (self at: i)].
	^ copy.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:29'!
copyWith: anObject
	^ self copy
		addLast: anObject;
		yourself! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 12:35'!
copyWithFirst: anObject
	^ self copyEmpty
		addLast: anObject;
		addAll: self;
		yourself! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:30'!
copyWithout: anObject
	^ self copy
		remove: anObject ifAbsent: [nil];
		yourself! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/16/2015 13:30'!
copyWithoutAll: aCollection
	| copy |

	copy := self copy.
	aCollection do: [:each | copy remove: each ifAbsent: [nil]].
	^ copy.! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 3/12/2010 06:48'!
defaultValue
	^ nil! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:21'!
detect: aBlock
	^ self detect: aBlock ifNone: [self defaultValue]! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:21'!
detect: aBlock ifNone: anExceptionBlock
	^ collection detect: aBlock ifNone: anExceptionBlock! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 3/12/2015 20:20'!
do: aBlock
	collection do: aBlock! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:21'!
doWithIndex: aBlock
	^ self withIndexDo: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
eighth
	^ self at: 8! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
endsWith: aCollection
	^ collection endsWith: aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
fifth
	^ self at: 5! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 7/15/2016 02:08'!
first
	"optimized implementation"
	^ collection size > 0
		ifTrue: [collection at: 1]
		ifFalse: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:54'!
first: anInteger
	^ self copyFrom: 1 to: anInteger! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
fourth
	^ self at: 4! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 1/17/2015 08:28'!
grownBy: aSize
	^ self copyClass collection: (collection grownBy: aSize)! !

!XMLOrderedList methodsFor: 'comparing' stamp: 'monty 3/29/2016 07:00'!
hash
	| hash|

	hash := self species hash.
	self size > 10
		ifFalse: [
			self do: [:each | hash := hash bitXor: each hash]].
	^ hash bitXor: self size hash.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:59'!
identityIndexOf: anObject
	^ self identityIndexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:09'!
identityIndexOf: anObject ifAbsent: aBlock
	^ collection identityIndexOf: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:59'!
indexOf: anObject
	^ self indexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 4/20/2016 05:30'!
indexOf: anObject ifAbsent: aBlock
	^ collection
		indexOf: anObject
		ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 4/20/2016 05:29'!
indexOf: anObject startingAt: anIndex 
	^ self
		indexOf: anObject
		startingAt: anIndex
		ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 05:45'!
indexOf: anObject startingAt: anIndex ifAbsent: aBlock
	^ collection
		indexOf: anObject
		startingAt: (anIndex max: 1)
		ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 4/20/2016 05:31'!
indexOfSubCollection: aSubCollection startingAt: anIndex
	^ self
		indexOfSubCollection: aSubCollection
		startingAt: anIndex
		ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 4/20/2016 05:31'!
indexOfSubCollection: aSubCollection startingAt: anIndex ifAbsent: aBlock
	^ collection
		indexOfSubCollection: aSubCollection
		startingAt: anIndex
		ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'inspecting' stamp: 'monty 3/10/2016 14:27'!
inspectorClass
	"for Squeak"
	^ collection inspectorClass! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
isSequenceable
	^ true! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 7/15/2016 02:00'!
last
	| lastIndex |

	"optimized implementation"
	^ (lastIndex := collection size) > 0
		ifTrue: [collection at: lastIndex]
		ifFalse: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:55'!
last: anInteger
	^ self copyFrom: self size - anInteger + 1 to: self size! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:06'!
lastIndexOf: anObject
	^ self lastIndexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 07:32'!
lastIndexOf: anObject ifAbsent: aBlock
	^ collection lastIndexOf: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
ninth
	^ self at: 9! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:23'!
pairsDo: aBlock
	collection pairsDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/22/2014 16:10'!
readStream
	^ ReadStream on: self! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:25'!
reject: aBlock
	| selected |

	selected := self copyEmpty.
	"use to:do: for speed"
	1 to: collection size do: [:i | | item |
		item := collection at: i.
		(aBlock value: item)
			ifFalse: [selected addLast: item]].
	^ selected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:31'!
reject: aRejectBlock thenCollect: aCollectBlock
	| collected |

	collected := self copyEmptyForCollect.
	"use to:do: for speed"
	1 to: collection size do: [:i | | item |
		item := collection at: i.
		(aRejectBlock value: item)
			ifFalse: [collected addLast: (aCollectBlock value: item)]].
	^ collected.! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:33'!
remove: anObject ifAbsent: aBlock
	^ collection remove: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 4/2/2016 01:52'!
removeAll
	"Redefined because Collection>>removeAll assumes the collection can
	be safely modified while iterating it with do:"

	[self isEmpty] whileFalse: [self removeLast]! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 15:46'!
removeAt: anIndex
	^ collection removeAt: anIndex! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:25'!
removeFirst
	^ self removeAt: 1! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:32'!
removeFirst: aNumber
	| removed |

	removed := self copyFrom: 1 to: aNumber.
	aNumber timesRepeat: [self removeFirst].
	^ removed.! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:24'!
removeLast
	^ self removeAt: self size! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:33'!
removeLast: aNumber
	| removed |

	removed := self copyFrom: self size - (aNumber - 1) to: self size.
	aNumber timesRepeat: [self removeLast].
	^ removed.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:02'!
replaceFrom: aStart to: aStop with: aReplacementCollection
	^ self
		replaceFrom: aStart
		to: aStop
		with: aReplacementCollection
		startingAt: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:04'!
replaceFrom: aStart to: aStop with: aReplacementCollection startingAt: aReplacementStart
	| offset |

	offset := 0.
	[aStart + offset <= aStop]
		whileTrue: [
			self
				at: aStart + offset
				put: (aReplacementCollection at: aReplacementStart + offset).
			offset := offset + 1].! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 10/19/2014 06:31'!
reset
	collection reset! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 4/20/2016 11:00'!
reverse
	"despite its name, this returns a reversed copy rather than doing in-place
	reversal. Same as SequenceableCollection's reverse"

	^ self reversed! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:35'!
reverseDo: aBlock
	collection reverseDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 1/17/2015 12:33'!
reversed
	| reversed |

	reversed := self copyEmpty.
	self reverseDo: [:each | reversed addLast: each].
	^ reversed.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
second
	^ self at: 2! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:24'!
select: aBlock
	| selected |

	selected := self copyEmpty.
	"use to:do: for speed"
	1 to: collection size do: [:i | | item |
		item := collection at: i.
		(aBlock value: item)
			ifTrue: [selected addLast: item]].
	^ selected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:30'!
select: aSelectBlock thenCollect: aCollectBlock
	| collected |

	collected := self copyEmptyForCollect.
	"use to:do: for speed"
	1 to: collection size do: [:i | | item |
		item := collection at: i.
		(aSelectBlock value: item)
			ifTrue: [collected addLast: (aCollectBlock value: item)]].
	^ collected.! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 8/4/2010 13:38'!
setCollection: aCollection
	collection := aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
seventh
	^ self at: 7! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
sixth
	^ self at: 6! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
size
	^ collection size! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/22/2014 16:18'!
sort
	self sort: [:a :b | a <= b]! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 2/2/2015 01:33'!
sort: aSortBlock
	collection sort: aSortBlock! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 1/18/2015 09:31'!
sorted
	^ self copy sort! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:07'!
sorted: aSortBlock
	^ self copy sort: aSortBlock! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 4/7/2016 05:39'!
species
	"this makes instances of this class and all subclasses with the same
	items in the same order be equal"
	^ XMLOrderedList! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 2/1/2015 23:47'!
swap: aFirstIndex with: aSecondIndex
	| temp |

	temp := collection at: aFirstIndex.
	collection
		at: aFirstIndex
		put: (collection at: aSecondIndex).
	collection
		at: aSecondIndex
		put: temp.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
third
	^ self at: 3! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:28'!
with: aCollection collect: aBlock
	| collected |

	collected := self copyEmptyForCollect.
	1 to: (collection size min: aCollection size) do: [:i |
		collected addLast:
			(aBlock
				value: (collection at: i)
				value: (aCollection at: i))].
	^ collected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:42'!
with: aCollection do: aBlock
	1 to: (collection size min: aCollection size) do: [:i |
		aBlock
			value: (collection at: i)
			value: (aCollection at: i)]! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 4/17/2016 08:28'!
withIndexCollect: aTwoArgumentBlock
	| collected |

	collected := self copyEmptyForCollect.
	1 to: collection size do: [:i |
		collected addLast:
			(aTwoArgumentBlock
				value: (collection at: i)
				value: i)].
	^ collected.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:47'!
withIndexDo: aBlock
	collection withIndexDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/22/2014 16:10'!
writeStream
	^ WriteStream on: self! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:56'!
xmlLastOrNil
	^ self last! !

!String methodsFor: '*XML-Parser' stamp: 'monty 3/13/2016 20:11'!
asXMLEscapedString
	"returns a copy of self with &, <, and > escaped with &amp;, &lt;, and &gt;"
	| stream |

	stream := (self class new: self size) writeStream.
	"to:do: for performance"
	1 to: self size do: [:i | | nextChar |
		(nextChar := self at: i) == $<
			ifTrue: [stream nextPutAll: '&lt;']
			ifFalse: [
				nextChar == $> "needed to escape ]]> in PCDATA"
					ifTrue: [stream nextPutAll: '&gt;']
					ifFalse: [
						nextChar == $&
							ifTrue: [stream nextPutAll: '&amp;']
							ifFalse: [stream nextPut: nextChar]]]].
	^ stream contents.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 2/2/2016 17:37'!
asXMLFileHandle
	^ XMLFileHandle path: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 2/13/2016 00:35'!
asXMLHTTPRequest
	^ XMLHTTPRequest url: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 3/5/2016 06:36'!
asXMLStreamConverter
	^ XMLStreamConverter newForEncoding: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 8/16/2016 21:57'!
asXMLString
	"will be deprecated; use #asXMLStringNode instead"

	^ self asXMLStringNode! !

!String methodsFor: '*XML-Parser' stamp: 'monty 8/5/2016 16:39'!
asXMLStringNode
	^ XMLString string: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 1/10/2014 00:56'!
asXMLURI
	^ XMLURI fromString: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/1/2016 00:56'!
expandWithXMLNamespaceURI: aURI
	aURI size > 0
		ifTrue: [^ 'Q{', aURI, '}', self]
		ifFalse: [^ self]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 3/24/2016 20:54'!
isXMLNCName
	"optimized"
	self size > 0
		ifFalse: [^ false].
	(self at: 1) isXMLNCNameStartChar
		ifFalse: [^ false].
	2 to: self size do: [:i |
		(self at: i) isXMLNCNameChar
			ifFalse: [^ false]].
	^ true.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/12/2016 10:41'!
isXMLNSAttributeName
	"optimized"
	self size >= 5
		ifFalse: [^ false].
	(self at: 1) == $x
		ifFalse: [^ false].
	(self at: 2) == $m
		ifFalse: [^ false].
	(self at: 3) == $l
		ifFalse: [^ false].
	(self at: 4) == $n
		ifFalse: [^ false].
	(self at: 5) == $s
		ifFalse: [^ false].
	self size = 5
		ifTrue: [^ true].
	^ (self at: 6) == $:.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 3/24/2016 20:54'!
isXMLName
	"optimized"
	self size > 0
		ifFalse: [^ false].
	(self at: 1) isXMLNameStartChar
		ifFalse: [^ false].
	2 to: self size do: [:i |
		(self at: i) isXMLNameChar
			ifFalse: [^ false]].
	^ true.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 3/25/2016 05:09'!
isXMLNmtoken
	"optimized"
	self size > 0
		ifFalse: [^ false].
	1 to: self size do: [:i |
		(self at: i) isXMLNameChar
			ifFalse: [^ false]].
	^ true.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/15/2016 02:09'!
isXMLQualifiedOrLocalName: aQualifiedOrLocalName
	"returns true if self and aQualifiedOrLocalName are the
	same QName or if self is a QName and aQualifiedOrLocalName
	is the local part, or if they are the same NCName."
	| i |

	i := aQualifiedOrLocalName size.
	self size to: 1 by: -1 do: [:j |
		i > 0
			ifFalse: [^ (self at: j) == $:].
		(aQualifiedOrLocalName at: i) = (self at: j)
			ifFalse: [^ false].
		i := i - 1].
	^ i = 0.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/12/2016 10:00'!
isXMLReservedAttributeName
	"optimized"
	self size > 3
		ifFalse: [^ false].
	(self at: 1) == $x
		ifFalse: [^ false].
	(self at: 2) == $m
		ifFalse: [^ false].
	^ (self at: 3) == $l.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/21/2016 07:35'!
isXMLWhitespace
	"optimized"
	self size > 0
		ifFalse: [^ false].
	1 to: self size do: [:i |
		(self at: i) isXMLWhitespace
			ifFalse: [^ false]].
	^ true.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 10/18/2014 05:47'!
parseXML
	^ XMLDOMParser parse: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/1/2016 00:57'!
qualifyWithXMLPrefix: aPrefix
	"optimized"
	aPrefix size > 0
		ifTrue: [^ (aPrefix copyWith: $:), self]
		ifFalse: [^ self]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/1/2016 00:57'!
xmlLocalNameAfterPrefix
	^ self xmlLocalNameAfterPrefix: self xmlPrefixBeforeLocalName! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/1/2016 00:56'!
xmlLocalNameAfterPrefix: aPrefix
	"optimized"
	| prefixSize |

	(prefixSize := aPrefix size) > 0
		ifTrue: [^ self copyFrom: prefixSize + 2 to: self size]
		ifFalse: [^ self]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/1/2016 00:57'!
xmlPrefixBeforeLocalName
	"Split the name into namespace prefix
	(Optimized code)"
	| i |

	i := self size.
	[i > 1]
		whileTrue: [
			((self at: i) == $:)
				ifTrue: [^ (self copyFrom: 1 to: i - 1)].
			i := i - 1].
	^ ''.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 4/1/2016 00:57'!
xmlPrefixBeforeLocalName: aLocalName
	"optimized"
	| prefixSize |

	(prefixSize := self size - aLocalName size - 1) > 0
		ifTrue: [^ self copyFrom: 1 to: prefixSize]
		ifFalse: [^ '']! !

!SequenceableCollection methodsFor: '*XML-Parser' stamp: 'monty 4/14/2016 13:04'!
includesXMLName: aName
	"optimized with to:do:"
	1 to: self size do: [:i |
		(aName isXMLQualifiedOrLocalName: (self at: i))
			ifTrue: [^ true]].
	^ false.! !

!SequenceableCollection methodsFor: '*XML-Parser' stamp: 'monty 4/1/2016 00:56'!
xmlLastOrNil
	"for speed to avoid at:ifAbsent: closure"
	| lastOrNilOffset |

	(lastOrNilOffset := self size) > 0
		ifTrue: [^ self at: lastOrNilOffset]
		ifFalse: [^ nil]! !

!StandardOrderedDictionary methodsFor: '*XML-Parser' stamp: 'monty 3/15/2016 00:09'!
xmlAt: aKey newPut: aValue
	"should be identical to at:put: except it returns whether the key is new"
	| oldSize |

	oldSize := dictionary size.
	dictionary at: aKey put: aValue.
	dictionary size > oldSize
		ifTrue: [
			orderedKeys size > oldSize
				ifFalse: [self growOrderedKeys].
			orderedKeys at: oldSize + 1 put: aKey.
			^ true].
	^ false.! !

Error subclass: #XMLException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLException commentStamp: 'monty 3/11/2016 23:36' prior: 0!
A generic exception class that supports message formatting. Used as a base class for exceptions in XMLParser and related packages.!

!XMLException class methodsFor: 'signalling' stamp: 'monty 7/3/2016 18:38'!
formatSignal: aString with: aFirstValue
	^ self
		formatSignal: aString
		withArguments: (Array with: aFirstValue)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 7/3/2016 18:38'!
formatSignal: aString with: aFirstValue with: aSecondValue
	^ self
		formatSignal: aString
		withArguments: (Array with: aFirstValue with: aSecondValue)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 7/3/2016 18:38'!
formatSignal: aString with: aFirstValue with: aSecondValue with: aThirdValue
	^ self
		formatSignal: aString
		withArguments: (Array with: aFirstValue with: aSecondValue with: aThirdValue)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 7/3/2016 18:38'!
formatSignal: aString with: aFirstValue with: aSecondValue with: aThirdValue with: aFourthValue
	^ self
		formatSignal: aString
		withArguments:
			(Array
				with: aFirstValue
				with: aSecondValue
				with: aThirdValue
				with: aFourthValue)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 7/3/2016 18:38'!
formatSignal: aString withArguments: anArray
	^ self signal: (aString format: anArray)! !

!XMLException class methodsFor: 'signalling' stamp: 'monty 12/3/2015 05:31'!
signalMessageFrom: anException
	^ self
		formatSignal: '({1}) {2}'
		with: anException class name
		with: anException messageText! !

!XMLException methodsFor: 'private' stamp: 'monty 3/30/2016 05:30'!
append: anObject withDescription: aDescription
	| oldMessageText |

	"older Pharo versions of #messageText can return nil, so ifNil: is used"
	oldMessageText := self messageText ifNil: [''].
	self messageText:
		('{1} ({2} {3})' format:
			(Array
				with: oldMessageText
				with: aDescription
				with: anObject))! !

XMLException subclass: #XMLParserException
	instanceVariableNames: 'lineNumber columnNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLParserException commentStamp: 'monty 3/11/2016 23:36' prior: 0!
A base class for XMLParser exceptions that supports line number error info and can be used with on:do: to catch any XMLParser-related errors.!

XMLParserException subclass: #XMLDOMException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLDOMException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked when a DOM tree is manipulated wrong.!

XMLParserException subclass: #XMLEncodingException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLEncodingException commentStamp: 'monty 11/11/2014 04:38' prior: 0!
Invoked for encoding errors.!

XMLParserException subclass: #XMLFileException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLFileException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked for file IO errors.!

XMLParserException subclass: #XMLHTTPException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLHTTPException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked for HTTP errors.!

XMLParserException subclass: #XMLLimitException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLLimitException commentStamp: 'monty 1/25/2014 00:19' prior: 0!
Invoked when a security limit on input is exceeded. Use the messages in XMLParserLimits and DTDExternalEntityLimits to configure these limits or the removeLimits message to remove them completely.!

XMLParserException subclass: #XMLNamespaceException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLNamespaceException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when namespaces are used improperly and namespace checking is on.!

!XMLParserException methodsFor: 'accessing' stamp: 'monty 3/18/2016 01:44'!
columnNumber
	^ columnNumber! !

!XMLParserException methodsFor: 'accessing' stamp: 'monty 3/18/2016 01:44'!
columnNumber: aColumnNumber
	columnNumber := aColumnNumber.
	self
		append: aColumnNumber
		withDescription: 'column'.! !

!XMLParserException methodsFor: 'testing' stamp: 'monty 3/19/2016 16:24'!
hasColumnNumber
	^ self columnNumber notNil! !

!XMLParserException methodsFor: 'testing' stamp: 'monty 3/19/2016 16:24'!
hasLineNumber
	^ self lineNumber notNil! !

!XMLParserException methodsFor: 'accessing' stamp: 'monty 4/12/2015 07:29'!
lineNumber
	^ lineNumber! !

!XMLParserException methodsFor: 'accessing' stamp: 'monty 4/12/2015 07:30'!
lineNumber: aLineNumber
	lineNumber := aLineNumber.
	self
		append: aLineNumber
		withDescription: 'line'.! !

!XMLParserException methodsFor: 'testing' stamp: 'monty 11/26/2014 06:03'!
shouldPassOnWhenCaught
	^ true! !

XMLParserException subclass: #XMLParsingInterruptException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLParsingInterruptException commentStamp: 'monty 11/26/2014 05:54' prior: 0!
Used to interrupt parsing from a SAXHandler. Will be silently caught.!

!XMLParsingInterruptException methodsFor: 'testing' stamp: 'monty 11/26/2014 06:03'!
shouldPassOnWhenCaught
	^ false! !

XMLParserException subclass: #XMLValidationException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLValidationException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when invalid XML is detected and validation is on.!

XMLParserException subclass: #XMLWellFormednessException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLWellFormednessException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when XML markup is not well-formed.!

Object subclass: #DTDAttributeDefinitionParser
	instanceVariableNames: 'element attribute type defaultValidator'
	classVariableNames: 'AttributeDefaultValidatorClasses NonEnumerationAttributeValidatorClasses'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDAttributeDefinitionParser commentStamp: '<historical>' prior: 0!
This class parses an attribute definition in an ATTLIST declaration and creates a validator for it.!

!DTDAttributeDefinitionParser class methodsFor: 'instance creation' stamp: 'monty 2/19/2016 14:49'!
element: anElement attribute: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		setElement: anElement
		attribute: anAttribute
		type: aType
		defaultValidator:
			((AttributeDefaultValidatorClasses at: aDefaultPragma)
				attribute: anAttribute
				defaultValue: aDefaultValue)! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 2/20/2016 04:38'!
initialize
	"self initialize"

	self
		initializeAttributeDefaultValidatorClasses;
		initializeNonEnumerationAttributeValidatorClasses! !

DTDAttributeDefinitionParser initialize!

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 2/19/2016 14:45'!
initializeAttributeDefaultValidatorClasses
	(AttributeDefaultValidatorClasses := Dictionary new)
		at: '' put: XMLAttributeDefaultValueValidator;
		at: 'FIXED' put: XMLAttributeDefaultFixedValueValidator;
		at: 'IMPLIED' put: XMLAttributeDefaultImpliedValidator;
		at: 'REQUIRED' put: XMLAttributeDefaultRequiredValidator! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 2/19/2016 12:05'!
initializeNonEnumerationAttributeValidatorClasses
	(NonEnumerationAttributeValidatorClasses := Dictionary new)
		at: 'CDATA' put: XMLCDataAttributeValidator;
		at: 'ID' put: XMLIDAttributeValidator;
		at: 'IDREF' put: XMLIDRefAttributeValidator;
		at: 'IDREFS' put: XMLIDRefsAttributeValidator;
		at: 'ENTITY' put: XMLEntityAttributeValidator;
		at: 'ENTITIES' put: XMLEntitiesAttributeValidator;
		at: 'NMTOKEN' put: XMLNmtokenAttributeValidator;
		at: 'NMTOKENS' put: XMLNmtokensAttributeValidator! !

!DTDAttributeDefinitionParser methodsFor: 'parsing' stamp: 'monty 2/20/2016 09:51'!
parse
	^ (NonEnumerationAttributeValidatorClasses
		at: type
		ifAbsent: [^ self parseEnumerationAttributeDefinition])
		element: element
		attribute: attribute
		defaultValidator: defaultValidator! !

!DTDAttributeDefinitionParser methodsFor: 'private' stamp: 'monty 2/20/2016 10:06'!
parseEnumerationAttributeDefinition
	| attributeValidatorClass enumerationListParserClass |

	"either a '(...)' or 'NOTATION (...)' enumeration type"
	(type at: 1) == $(
		ifTrue: [
			attributeValidatorClass := XMLEnumerationAttributeValidator.
			enumerationListParserClass := DTDEnumerationListParser]
		ifFalse: [
			attributeValidatorClass := XMLNotationEnumerationAttributeValidator.
			enumerationListParserClass := DTDNotationEnumerationListParser].

	^ (attributeValidatorClass
		element: element
		attribute: attribute
		defaultValidator: defaultValidator)
			allowedValues: (enumerationListParserClass on: type) parse.! !

!DTDAttributeDefinitionParser methodsFor: 'initialization' stamp: 'monty 2/19/2016 14:49'!
setElement: anElement attribute: anAttribute type: aType defaultValidator: aDefaultValidator
	element := anElement.
	attribute := anAttribute.
	type := aType.
	defaultValidator := aDefaultValidator.! !

Object subclass: #DTDContentModelParser
	instanceVariableNames: 'element contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDContentModelParser commentStamp: '<historical>' prior: 0!
This is a base class for DTD content model parsers that turn element declarations into validators.!

!DTDContentModelParser class methodsFor: 'instance creation' stamp: 'monty 6/7/2015 19:43'!
element: anElement contentModel: aContentModel
	^ self new
		setElement: anElement
		contentModel: aContentModel! !

!DTDContentModelParser class methodsFor: 'parsing' stamp: 'monty 4/13/2016 06:26'!
parserForContentModel: aContentModel
	"the tokenizer removes all spaces from the content model"
	(aContentModel size > 0
		and: [(aContentModel at: 1) == $(])
		ifTrue: [
			(aContentModel size > 1
				and: [(aContentModel at: 2) == $#]) "#PCDATA mixed content"
				ifTrue: [^ DTDMixedContentModelParser]
				ifFalse: [^ DTDElementContentModelParser]]
		ifFalse: [^ DTDLiteralContentModelParser]! !

!DTDContentModelParser methodsFor: 'parsing' stamp: 'monty 12/21/2013 19:57'!
parse
	self subclassResponsibility! !

!DTDContentModelParser methodsFor: 'initialization' stamp: 'monty 6/7/2015 19:43'!
setElement: anElement contentModel: aContentModel
	element := anElement.
	contentModel := aContentModel.! !

DTDContentModelParser subclass: #DTDElementContentModelParser
	instanceVariableNames: 'nfaBuilder binaryOperatorStack'
	classVariableNames: 'OperatorApplicators SpecialCharacterParsers'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDElementContentModelParser commentStamp: 'monty 8/4/2016 09:11' prior: 0!
This class parses a content model using postfix conversion and an XMLNFABuilder to build an NFA before converting it to a lazy DFA (states are built when needed and cached) and returned as an element validator.

It assumes all spaces have been removed from the input and that it's been checked for well-formedness (meaning no missing names or misplaced operators/parentheses).!

!DTDElementContentModelParser class methodsFor: 'class initialization' stamp: 'monty 4/1/2016 22:59'!
initialize
	"self initialize"

	self
		initializeSpecialCharacterParsers;
		initializeOperatorApplicators! !

DTDElementContentModelParser initialize!

!DTDElementContentModelParser class methodsFor: 'class initialization' stamp: 'monty 2/20/2016 02:42'!
initializeOperatorApplicators
	"use Array for faster lookup"
	(OperatorApplicators := Array new: 255)
		at: $, asciiValue put: #applyConcatenation;
		at: $| asciiValue put: #applyBranch;
		at: $? asciiValue put: #applyOptional;
		at: $* asciiValue put: #applyStar;
		at: $+ asciiValue put: #applyPlus! !

!DTDElementContentModelParser class methodsFor: 'class initialization' stamp: 'monty 4/1/2016 23:02'!
initializeSpecialCharacterParsers
	"use Array for faster lookup"
	(SpecialCharacterParsers := Array new: 255)
		at: $( asciiValue put: #parseOpeningParenthesis:;
		at: $) asciiValue put: #parseClosingParenthesis:;
		at: $, asciiValue put: #parseBinaryOperator:;
		at: $| asciiValue put: #parseBinaryOperator:;
		at: $? asciiValue put: #parseUnaryOperator:;
		at: $* asciiValue put: #parseUnaryOperator:;
		at: $+ asciiValue put: #parseUnaryOperator:! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 4/1/2016 21:40'!
applyBinaryOperatorOnStack
	self applyOperator: binaryOperatorStack removeLast! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 4/1/2016 23:03'!
applyOperator: aCharacter
	nfaBuilder perform: (OperatorApplicators at: aCharacter asciiValue)! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 4/1/2016 21:44'!
hasBinaryOperatorOnStack
	^ (binaryOperatorStack isEmpty
		or: [binaryOperatorStack last == $(]) not! !

!DTDElementContentModelParser methodsFor: 'parsing' stamp: 'monty 4/21/2016 08:15'!
parse
	"the tokenizer removes all spaces from the content model and checks
	it for well-formedness (meaning no missing names or misplaced
	operators/parentheses), which simplifies parsing"
	| nameStart |

	binaryOperatorStack := OrderedCollection new.
	nfaBuilder := XMLNFABuilder new.
	1 to: contentModel size do: [:i |
		(self parserForSpecialCharacter: (contentModel at: i))
			ifNil: [
				nameStart
					ifNil: [nameStart := i]]
			ifNotNil: [:selector |
				nameStart
					ifNotNil: [
						nfaBuilder pushMatcher:
							(contentModel copyFrom: nameStart to: i - 1).
						nameStart := nil].
				self
					perform: selector
					with: (contentModel at: i)]].
	self popRemainingBinaryOperators.

	^ XMLDFAElementContentElementValidator
		element: element
		dfa: nfaBuilder buildAcceptingNFAFromFragments asDFAState.! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 4/1/2016 21:39'!
parseBinaryOperator: aCharacter
	binaryOperatorStack addLast: aCharacter! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 4/1/2016 23:02'!
parseClosingParenthesis: aCharacter
	self popRemainingBinaryOperators.
	(binaryOperatorStack notEmpty
		and: [binaryOperatorStack last == $(])
		ifTrue: [binaryOperatorStack removeLast].! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 4/1/2016 23:02'!
parseOpeningParenthesis: aCharacter
	binaryOperatorStack addLast: $(! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 4/1/2016 21:39'!
parseUnaryOperator: aCharacter
	self applyOperator: aCharacter! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 1/8/2016 07:08'!
parserForSpecialCharacter: aCharacter
	| asciiValue |

	"avoid at:ifAbsent: for speed"
	((asciiValue := aCharacter asciiValue) > 0
		and: [asciiValue <= SpecialCharacterParsers size])
			ifTrue: [^ SpecialCharacterParsers at: asciiValue]
			ifFalse: [^ nil]! !

!DTDElementContentModelParser methodsFor: 'private' stamp: 'monty 4/1/2016 21:44'!
popRemainingBinaryOperators
	[self hasBinaryOperatorOnStack]
		whileTrue: [self applyBinaryOperatorOnStack]! !

DTDContentModelParser subclass: #DTDLiteralContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDLiteralContentModelParser commentStamp: 'monty 8/16/2016 22:16' prior: 0!
This class parses literal "ANY" and "EMPTY" element content models and returns element validators accepting any content or no content (not even whitespace, comments, or PIs).!

!DTDLiteralContentModelParser methodsFor: 'parsing' stamp: 'monty 2/20/2016 10:14'!
parse
	"either ANY or EMPTY"
	(contentModel at: 1) == $A
		ifTrue: [^ XMLAnyElementValidator element: element]
		ifFalse: [^ XMLEmptyElementValidator element: element]! !

DTDContentModelParser subclass: #DTDMixedContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDMixedContentModelParser commentStamp: 'monty 8/4/2016 09:14' prior: 0!
This class parses mixed content declarations like "(#PCDATA|one|two)*" using a DTDEnumerationListParser and returns an element validator accepting any sequence of PCDATA and the named elements.!

!DTDMixedContentModelParser methodsFor: 'parsing' stamp: 'monty 8/4/2016 09:16'!
parse
	^ XMLMixedContentElementValidator
		element: element
		allowedElements:
			((DTDEnumerationListParser on: contentModel) parse
				remove: '#PCDATA';
				yourself)! !

Object subclass: #DTDDoctypeDefinition
	instanceVariableNames: 'root elementValidators attributeValidators generalEntities parameterEntities unparsedEntities notations elementIDValidator unparsedEntityValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDoctypeDefinition commentStamp: 'monty 4/6/2015 08:06' prior: 0!
This class represents the parsed internal and external subset of a DTD taken together.!

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 4/8/2016 06:44'!
addExternalGeneralEntity: aName resolver: aResolver publicID: aPublicID systemID: aSystemID baseURI: aBaseURIString
	self generalEntities
		at: aName
		ifAbsentPut: [
			DTDExternalGeneralEntity
				name: aName
				resolver: aResolver
				publicID: aPublicID
				systemID: aSystemID
				baseURI: aBaseURIString]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 4/8/2016 06:44'!
addExternalParameterEntity: aName resolver: aResolver publicID: aPublicID systemID: aSystemID baseURI: aBaseURIString
	self parameterEntities
		at: aName
		ifAbsentPut: [
			DTDExternalParameterEntity
				name: aName
				resolver: aResolver
				publicID: aPublicID
				systemID: aSystemID
				baseURI: aBaseURIString]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 5/9/2015 16:50'!
addGeneralEntitiesFrom: aDoctypeDefinition
	aDoctypeDefinition generalEntities keysAndValuesDo: [:name :entity |
		self generalEntities
			at: name
			ifAbsentPut: [entity]]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 4/8/2016 06:16'!
addInternalGeneralEntity: aName replacement: aReplacement baseURI: aBaseURIString
	self generalEntities
		at: aName
		ifAbsentPut: [
			DTDInternalGeneralEntity 
				name: aName
				replacement: aReplacement
				uri: aBaseURIString]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 4/8/2016 06:17'!
addInternalParameterEntity: aName replacement: aReplacement baseURI: aBaseURIString
	self parameterEntities
		at: aName
		ifAbsentPut: [
			DTDInternalParameterEntity
				name: aName
				replacement: aReplacement
				uri: aBaseURIString]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 11/14/2013 23:59'!
addNotation: aName
	self notations add: aName! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 7/13/2016 00:04'!
addUnparsedEntity: aName ndata: aNotation
	self generalEntities
		at: aName
		ifAbsentPut: [
			self unparsedEntities
				at: aName
				put:
					(DTDUnparsedEntity
						name: aName
						ndata: aNotation)]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 7/18/2016 00:32'!
addUnresolvableExternalGeneralEntity: aName
	self generalEntities
		at: aName
		ifAbsentPut: [DTDUnresolvableExternalGeneralEntity named: aName]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 7/18/2016 00:32'!
addUnresolvableExternalParameterEntity: aName
	self parameterEntities
		at: aName
		ifAbsentPut: [DTDUnresolvableExternalParameterEntity named: aName]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 4/8/2016 17:30'!
addValidatorForAttributeIn: anElement named: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	| attributeValidatorsForElement |

	attributeValidatorsForElement :=
		self attributeValidators
			at: anElement
			ifAbsentPut: [Dictionary new].

	"additional declarations for the same attribute are legal, unlike with
	element declarations, but are ignored"
	(attributeValidatorsForElement includesKey: anAttribute)
		ifFalse: [
			attributeValidatorsForElement
				at: anAttribute
				put: 
					((DTDAttributeDefinitionParser
						element:  anElement
						attribute: anAttribute
						type: aType
						defaultPragma: aDefaultPragma
						defaultValue: aDefaultValue) parse
							initializeWithValidatorsFrom: self;
							validateAttributeDeclaration)].! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 3/21/2016 03:24'!
addValidatorForElement: anElement contentModel: aContentModel
	| validator oldSize |

	validator :=
		((DTDContentModelParser parserForContentModel: aContentModel)
			element: anElement
			contentModel: aContentModel) parse.

	"testing the size after is faster than an includes: test before"
	oldSize := self elementValidators size.
	(self elementValidators
		at: anElement put: validator;
		size) > oldSize
		ifFalse: [self errorMultipleDeclarationsForElement: anElement]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:01'!
attributeValidators
	^ attributeValidators ifNil: [attributeValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 7/13/2016 00:21'!
attributeValidatorsAt: aName
	^ self
		attributeValidatorsAt: aName
		ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 7/13/2016 00:21'!
attributeValidatorsAt: aName ifAbsent: aBlock
	^ self attributeValidators
		at: aName
		ifAbsent: aBlock! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 3/21/2015 23:03'!
elementIDValidator
	^ elementIDValidator ifNil: [elementIDValidator := XMLElementIDValidator new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/18/2013 19:26'!
elementValidators
	^ elementValidators ifNil: [elementValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 7/13/2016 00:21'!
elementValidatorsAt: aName
	^ self
		elementValidatorsAt: aName
		ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 7/13/2016 00:22'!
elementValidatorsAt: aName ifAbsent: aBlock
	^ self elementValidators
		at: aName
		ifAbsent: aBlock! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 3/21/2016 03:25'!
errorMultipleDeclarationsForElement: anElement
	XMLValidationException
		formatSignal: 'Multiple declarations for <{1}> element'
		with: anElement! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 20:51'!
generalEntities
	^ generalEntities ifNil: [generalEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/3/2015 03:11'!
generalEntityAt: aName
	^ self
		generalEntityAt: aName
		ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/3/2015 03:11'!
generalEntityAt: aName ifAbsent: aBlock
	^ self generalEntities
		at: aName
		ifAbsent: aBlock! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/23/2016 22:01'!
hasAttributeValidators
	"optimized to avoid lazy initialization"
	attributeValidators == nil
		ifTrue: [^ false].
	^ attributeValidators size > 0.! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/21/2015 23:03'!
hasElementIDValidator
	^ elementIDValidator notNil! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/23/2016 22:01'!
hasElementValidators
	"optimized to avoid lazy initialization"
	elementValidators == nil
		ifTrue: [^ false].
	^ elementValidators size > 0.! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:22'!
hasEntities
	^ self hasGeneralEntities
		or: [self hasParameterEntities
			or: [self hasUnparsedEntities]]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:12'!
hasGeneralEntities
	^ generalEntities notNil and: [generalEntities notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 7/13/2016 01:56'!
hasNotationAttributeValidators
	^ self hasAttributeValidators
		and: [self unparsedEntityValidator hasNotationAttributeValidators]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:12'!
hasNotations
	^ notations notNil and: [notations notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:13'!
hasParameterEntities
	^ parameterEntities notNil and: [parameterEntities notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 4/24/2015 18:28'!
hasRoot
	^ root notNil and: [root notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/8/2015 14:22'!
hasUnparsedEntities
	^ unparsedEntities notNil and: [unparsedEntities notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 3/23/2016 21:53'!
hasValidators
	^ self hasElementValidators
		or: [self hasAttributeValidators]! !

!DTDDoctypeDefinition methodsFor: 'instance creation' stamp: 'monty 3/18/2016 01:10'!
newValidator
	^ DTDDocumentValidator doctypeDefinition: self! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:59'!
notations
	^ notations ifNil: [notations := Set new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 20:51'!
parameterEntities
	^ parameterEntities ifNil: [parameterEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/3/2015 03:11'!
parameterEntityAt: aName
	^ self
		parameterEntityAt: aName
		ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/3/2015 03:11'!
parameterEntityAt: aName ifAbsent: aBlock
	^ self parameterEntities
		at: aName
		ifAbsent: aBlock! !

!DTDDoctypeDefinition methodsFor: 'removing' stamp: 'monty 11/14/2015 20:03'!
removeIDsAndIDReferences
	self hasElementIDValidator
		ifTrue: [self elementIDValidator removeIDsAndIDReferences]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:56'!
root
	^ root ifNil: [root := '']! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 3/8/2015 09:42'!
root: aRootElement
	root := aRootElement! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:50'!
unparsedEntities
	^ unparsedEntities ifNil: [unparsedEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:48'!
unparsedEntityValidator
	^ unparsedEntityValidator
		ifNil: [
			unparsedEntityValidator :=
				XMLUnparsedEntityValidator
					unparsedEntities: self unparsedEntities
					notations: self notations]! !

!DTDDoctypeDefinition methodsFor: 'validation' stamp: 'monty 7/13/2016 02:54'!
validateCompletedDefinition
	self hasUnparsedEntities
		ifTrue: [self unparsedEntityValidator validateUnparsedEntities].
	self hasNotationAttributeValidators
		ifTrue: [self unparsedEntityValidator validateNotationAttributes].! !

Object subclass: #DTDEntity
	instanceVariableNames: 'name replacement uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDEntity commentStamp: 'monty 4/8/2016 01:26' prior: 0!
This is an abstract class for internal and external parsed entities with replacements that can be inserted into an XML document.!

!DTDEntity class methodsFor: 'instance creation' stamp: 'monty 3/20/2016 23:12'!
name: aName replacement: aReplacementString
	^ self
		name: aName
		replacement: aReplacementString
		uri: ''! !

!DTDEntity class methodsFor: 'instance creation' stamp: 'monty 5/23/2015 03:04'!
name: aName replacement: aReplacementString uri: aURIString
	^ self new
		setName: aName
		replacement: aReplacementString
		uri: aURIString! !

!DTDEntity class methodsFor: 'instance creation' stamp: 'monty 5/23/2015 03:05'!
named: aName
	^ self
		name: aName
		replacement: ''! !

!DTDEntity methodsFor: 'converting' stamp: 'monty 4/24/2015 14:25'!
asReference
	^ String streamContents: [:stream | self printReferenceOn: stream]! !

!DTDEntity methodsFor: 'testing' stamp: 'monty 4/5/2015 07:55'!
hasReplacement
	^ self replacement notEmpty! !

!DTDEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:56'!
isExternal
	^ false! !

!DTDEntity methodsFor: 'testing' stamp: 'monty 4/8/2016 01:22'!
isInternalTo: aURI
	^ false! !

!DTDEntity methodsFor: 'testing' stamp: 'monty 4/3/2016 04:05'!
isParameter
	^ false! !

!DTDEntity methodsFor: 'testing' stamp: 'monty 7/12/2016 23:53'!
isUnparsed
	^ false! !

!DTDEntity methodsFor: 'accessing' stamp: 'monty 11/13/2013 23:45'!
name
	^ name ifNil: [name := '']! !

!DTDEntity methodsFor: 'accessing' stamp: 'monty 4/8/2016 02:54'!
name: aName
	name := aName! !

!DTDEntity methodsFor: 'printing' stamp: 'monty 4/8/2016 06:23'!
printNameOn: aStream
	aStream nextPutAll: self name! !

!DTDEntity methodsFor: 'printing' stamp: 'monty 4/8/2016 06:23'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	self printNameOn: aStream.
	aStream nextPut: $).! !

!DTDEntity methodsFor: 'printing' stamp: 'monty 4/8/2016 01:25'!
printReferenceOn: aStream
	aStream
		nextPut: self referencePrefix;
		nextPutAll: self name;
		nextPut: $;! !

!DTDEntity methodsFor: 'private ' stamp: 'monty 4/8/2016 01:21'!
referencePrefix
	^ self isParameter
		ifTrue: [$%]
		ifFalse: [$&]! !

!DTDEntity methodsFor: 'accessing' stamp: 'monty 11/13/2013 19:15'!
replacement
	^ replacement ifNil: [replacement := '']! !

!DTDEntity methodsFor: 'accessing' stamp: 'monty 4/9/2016 04:06'!
replacement: aReplacementString
	replacement := aReplacementString! !

!DTDEntity methodsFor: 'accessing' stamp: 'monty 4/8/2016 01:23'!
replacementStream
	^ self replacement readStream! !

!DTDEntity methodsFor: 'initialization' stamp: 'monty 5/23/2015 03:04'!
setName: aName replacement: aReplacementString uri: aURIString
	name := aName.
	replacement := aReplacementString.
	uri := aURIString.! !

!DTDEntity methodsFor: 'accessing' stamp: 'monty 11/16/2013 02:55'!
uri
	^ uri ifNil: [uri := '']! !

!DTDEntity methodsFor: 'accessing' stamp: 'monty 4/8/2016 02:55'!
uri: aURIString
	uri := aURIString! !

DTDEntity subclass: #DTDExternalEntity
	instanceVariableNames: 'unresolvedReplacement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntity commentStamp: 'monty 4/8/2016 17:52' prior: 0!
This is an abstract class for external parsed entities with external replacements that can be inserted into an XML document.

The external replacements are resolved lazily the first time #replacement or #replacementStream is sent.!

!DTDExternalEntity class methodsFor: 'instance creation' stamp: 'monty 4/8/2016 17:37'!
name: aName resolver: aResolver publicID: aPublicID systemID: aSystemID baseURI: aBaseURIString
	^ self new
		setName: aName
		unresolvedReplacement:
			(DTDUnresolvedExternalEntityReplacement
				resolver: aResolver
				publicID: aPublicID
				systemID: aSystemID
				baseURI: aBaseURIString)! !

!DTDExternalEntity methodsFor: 'testing' stamp: 'monty 4/8/2016 17:37'!
hasUnresolvedReplacement
	^ self unresolvedReplacement notNil! !

!DTDExternalEntity methodsFor: 'testing' stamp: 'monty 4/8/2016 01:21'!
isExternal
	^ true! !

!DTDExternalEntity methodsFor: 'printing' stamp: 'monty 4/8/2016 07:30'!
printNameOn: aStream
	aStream
		nextPut: ${;
		nextPutAll: self uri;
		nextPut: $}.
	super printNameOn: aStream.! !

!DTDExternalEntity methodsFor: 'accessing' stamp: 'monty 4/8/2016 06:52'!
replacement
	self resolveReplacement.
	^ super replacement.! !

!DTDExternalEntity methodsFor: 'accessing' stamp: 'monty 4/8/2016 01:23'!
replacementStream
	^ XMLDecodingReadStreamAdapter on: self replacement readStream! !

!DTDExternalEntity methodsFor: 'resolving' stamp: 'monty 4/8/2016 17:38'!
resolveReplacement
	self hasUnresolvedReplacement
		ifTrue: [self unresolvedReplacement resolveForExternalEntity: self]! !

!DTDExternalEntity methodsFor: 'initialization' stamp: 'monty 4/9/2016 04:08'!
setName: aName unresolvedReplacement: anUnresolvedReplacement
	name := aName.
	unresolvedReplacement := anUnresolvedReplacement.! !

!DTDExternalEntity methodsFor: 'accessing' stamp: 'monty 4/8/2016 17:37'!
unresolvedReplacement
	^ unresolvedReplacement! !

!DTDExternalEntity methodsFor: 'accessing' stamp: 'monty 4/9/2016 04:07'!
unresolvedReplacement: anUnresolvedReplacement
	unresolvedReplacement := anUnresolvedReplacement! !

DTDExternalEntity subclass: #DTDExternalGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalGeneralEntity commentStamp: 'monty 4/8/2016 17:52' prior: 0!
This is a class for external general parsed entities with external replacements that can be inserted into an XML document.!

DTDExternalEntity subclass: #DTDExternalParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalParameterEntity commentStamp: 'monty 4/8/2016 17:52' prior: 0!
This is a class for external parameter parsed entities with external replacements that can be inserted into an XML document.!

!DTDExternalParameterEntity methodsFor: 'testing' stamp: 'monty 4/8/2016 01:21'!
isParameter
	^ true! !

DTDExternalEntity subclass: #DTDUnresolvableExternalEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDUnresolvableExternalEntity commentStamp: 'monty 7/18/2016 08:18' prior: 0!
This is an abstract class for external parsed entities that are unresolvable because external entity resolution was not enabled. Resolving them will raise an exception.
!

!DTDUnresolvableExternalEntity methodsFor: 'testing' stamp: 'monty 7/18/2016 00:25'!
hasUnresolvedReplacement
	^ true! !

!DTDUnresolvableExternalEntity methodsFor: 'resolving' stamp: 'monty 7/18/2016 03:56'!
resolveReplacement
	XMLParserException
		formatSignal:
			'Unresolvable external entity {1}. ',
			'Enable resolution with #resolvesExternalEntityReferences:'
		with: self asReference! !

DTDUnresolvableExternalEntity subclass: #DTDUnresolvableExternalGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDUnresolvableExternalGeneralEntity commentStamp: 'monty 7/18/2016 08:18' prior: 0!
This is a class for unresolvable external general parsed entities. Resolving them will raise an exception.!

DTDUnresolvableExternalEntity subclass: #DTDUnresolvableExternalParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDUnresolvableExternalParameterEntity commentStamp: 'monty 7/18/2016 08:18' prior: 0!
This is a class for unresolvable external parameter parsed entities. Resolving them will raise an exception.!

!DTDUnresolvableExternalParameterEntity methodsFor: 'testing' stamp: 'monty 7/18/2016 00:30'!
isParameter
	^ true! !

DTDEntity subclass: #DTDInternalEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDInternalEntity commentStamp: 'monty 4/8/2016 01:26' prior: 0!
This is an abstract class for internal parsed entities with replacements that can be inserted into an XML document.!

!DTDInternalEntity methodsFor: 'testing' stamp: 'monty 4/8/2016 01:22'!
isInternalTo: aURI
	^ self uri = aURI! !

DTDInternalEntity subclass: #DTDInternalGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDInternalGeneralEntity commentStamp: 'monty 4/8/2016 01:27' prior: 0!
This is a class for internal general parsed entities with replacements that can be inserted into documents.!

DTDInternalEntity subclass: #DTDInternalParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDInternalParameterEntity commentStamp: 'monty 4/8/2016 01:27' prior: 0!
This is a class for internal parameter parsed entities with replacements that can be inserted into documents.!

!DTDInternalParameterEntity methodsFor: 'testing' stamp: 'monty 4/3/2016 04:05'!
isParameter
	^ true! !

DTDEntity subclass: #DTDUnparsedEntity
	instanceVariableNames: 'ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDUnparsedEntity commentStamp: 'monty 7/18/2016 03:19' prior: 0!
A class for unparsed entities. These should never be referenced with the &name; syntax but can be named in values of ENTITY and ENTITIES attributes.!

!DTDUnparsedEntity class methodsFor: 'instance creation' stamp: 'monty 7/13/2016 00:04'!
name: aName ndata: aNotationName
	^ self new
		setName: aName
		ndata: aNotationName! !

!DTDUnparsedEntity methodsFor: 'testing' stamp: 'monty 7/12/2016 23:54'!
isUnparsed
	^ true! !

!DTDUnparsedEntity methodsFor: 'accessing' stamp: 'monty 7/13/2016 02:15'!
ndata
	^ ndata ifNil: [ndata := '']! !

!DTDUnparsedEntity methodsFor: 'accessing' stamp: 'monty 7/12/2016 23:52'!
ndata: aNotationName
	ndata := aNotationName! !

!DTDUnparsedEntity methodsFor: 'initialization' stamp: 'monty 7/12/2016 23:53'!
setName: aName ndata: aNotationName
	name := aName.
	ndata := aNotationName.! !

Object subclass: #DTDEntityConfiguration
	instanceVariableNames: 'replacesContentEntityReferences resolvesExternalEntities preservesUndeclaredEntityReferences externalEntityResolver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDEntityConfiguration commentStamp: 'monty 1/3/2015 04:39' prior: 0!
This class contains configurations for entity replacement/retrieval. It is separate from XMLConfiguration so it is only lazy initialized when needed and saves memory.!

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 4/8/2016 03:05'!
externalEntityResolver
	"uses the caching resolver by default"
	^ externalEntityResolver
		ifNil: [externalEntityResolver := DTDExternalEntityResolver default]! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:22'!
externalEntityResolver: anExternalEntityResolver
	externalEntityResolver := anExternalEntityResolver! !

!DTDEntityConfiguration methodsFor: 'copying' stamp: 'monty 6/19/2015 20:34'!
postCopy
	super postCopy.

	externalEntityResolver
		ifNotNil: [externalEntityResolver := externalEntityResolver copy].! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 04:17'!
preservesUndeclaredEntityReferences
	^ preservesUndeclaredEntityReferences ifNil: [preservesUndeclaredEntityReferences := false]! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 04:13'!
preservesUndeclaredEntityReferences: aBoolean
	preservesUndeclaredEntityReferences := aBoolean! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 09:37'!
replacesContentEntityReferences
	^ replacesContentEntityReferences ifNil: [replacesContentEntityReferences := true]! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 09:37'!
replacesContentEntityReferences: aBoolean
	replacesContentEntityReferences := aBoolean! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:22'!
resolvesExternalEntities
	^ resolvesExternalEntities ifNil: [resolvesExternalEntities := false]! !

!DTDEntityConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:22'!
resolvesExternalEntities: aBoolean
	resolvesExternalEntities := aBoolean! !

Object subclass: #DTDEnumerationListParser
	instanceVariableNames: 'listString listValues startIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDEnumerationListParser commentStamp: 'monty 12/8/2014 15:21' prior: 0!
This is a parser of lists in the form of "(one|two|three)"
(Assumes spaces have been removed)!

!DTDEnumerationListParser class methodsFor: 'instance creation' stamp: 'monty 10/30/2014 12:58'!
on: aString
	^ self new on: aString! !

!DTDEnumerationListParser methodsFor: 'private' stamp: 'monty 12/11/2014 05:50'!
addListValue: aListValue
	| oldSize |

	"checking the size avoids includes: hash lookup"
	oldSize := listValues size.
	listValues add: aListValue.
	(listValues size > oldSize)
		ifFalse: [self errorDuplicateListValue: aListValue].! !

!DTDEnumerationListParser methodsFor: 'private' stamp: 'monty 10/31/2014 21:20'!
errorDuplicateListValue: aValue
	XMLValidationException
		formatSignal: 'Duplicate value "{1}" in list'
		with: aValue! !

!DTDEnumerationListParser methodsFor: 'initialization' stamp: 'monty 12/11/2014 04:51'!
on: aString
	listString := aString.
	listValues := Set new.
	startIndex := 2.! !

!DTDEnumerationListParser methodsFor: 'parsing' stamp: 'monty 12/11/2014 05:50'!
parse
	"the tokenizer removes all spaces from lists"
	| valueStartIndex nextChar |

	valueStartIndex := startIndex.
	startIndex to: listString size do: [:i | 
		((nextChar := listString at: i) == $)
			or: [nextChar == $|])
			ifTrue: [
				self addListValue:
					(listString
						copyFrom: valueStartIndex
						to: i - 1).
				nextChar == $)
					ifTrue: [^ listValues].
				valueStartIndex := i + 1]].
	^ listValues.! !

DTDEnumerationListParser subclass: #DTDNotationEnumerationListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDNotationEnumerationListParser commentStamp: 'monty 1/25/2014 03:36' prior: 0!
This class parses notation attribute value lists.!

!DTDNotationEnumerationListParser methodsFor: 'initialization' stamp: 'monty 12/11/2014 04:55'!
on: aString
	super on: aString.
	startIndex := 11. "'NOTATION (' size + 1"! !

Object subclass: #DTDExternalEntityLimits
	instanceVariableNames: 'externalSubsetReadLimit maxExternalEntities maxExternalEntitySize resolvesLocalFromNonLocal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityLimits commentStamp: 'monty 1/25/2014 04:01' prior: 0!
This class places security limits on the number and size of external parsed entities. You can set these using the configuration messages in DTDExternalEntityResolver or remove them using removeLimits.!

!DTDExternalEntityLimits class methodsFor: 'accessing' stamp: 'monty 1/27/2015 22:48'!
defaultExternalSubsetReadLimit
	^ XMLParserLimits defaultReadLimit
		ifNotNil: [:limit | (limit / 4) asInteger]! !

!DTDExternalEntityLimits class methodsFor: 'accessing' stamp: 'monty 6/21/2015 02:02'!
defaultMaxExternalEntities
	^ 20! !

!DTDExternalEntityLimits class methodsFor: 'accessing' stamp: 'monty 12/9/2014 07:57'!
defaultMaxExternalEntitySize
	^ 2500000! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/27/2015 22:46'!
externalSubsetReadLimit
	^ externalSubsetReadLimit! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 4/14/2016 03:31'!
externalSubsetReadLimit: anInteger
	externalSubsetReadLimit := anInteger! !

!DTDExternalEntityLimits methodsFor: 'initialization' stamp: 'monty 6/21/2015 02:02'!
initialize
	super initialize.

	externalSubsetReadLimit := self class defaultExternalSubsetReadLimit.
	maxExternalEntities := self class defaultMaxExternalEntities.
	maxExternalEntitySize := self class defaultMaxExternalEntitySize.
	resolvesLocalFromNonLocal := false.! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntities
	^ maxExternalEntities! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 4/14/2016 03:31'!
maxExternalEntities: anInteger
	maxExternalEntities := anInteger! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntitySize
	^ maxExternalEntitySize! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 4/14/2016 03:31'!
maxExternalEntitySize: anInteger
	maxExternalEntitySize := anInteger! !

!DTDExternalEntityLimits methodsFor: 'removing' stamp: 'monty 5/25/2015 01:35'!
removeAll
	externalSubsetReadLimit := nil.
	maxExternalEntities := nil.
	maxExternalEntitySize := nil.
	resolvesLocalFromNonLocal := true.! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/24/2014 06:47'!
resolvesLocalFromNonLocal
	^ resolvesLocalFromNonLocal! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/24/2014 06:47'!
resolvesLocalFromNonLocal: aBoolean
	resolvesLocalFromNonLocal := aBoolean! !

Object subclass: #DTDExternalEntityResolver
	instanceVariableNames: 'externalEntityLimits totalResolvedExternalEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityResolver commentStamp: 'monty 4/3/2016 01:11' prior: 0!
This class resolves external entity references using XMLURI, XMLHTTPRequest, and XMLFileHandle.

It supports retrieving external general entities as DTDExternalGeneralEntity objects with #resolveExternalGeneralEntity:publicID:systemID:baseURI: and external parameter entities as DTDExternalParameterEntity objects with #resolveExternalParameterEntity:publicID:systemID:baseURI:.

#resolveExternalEntityURI: and  #streamResolveExternalEntityURI: messages can resolve generic external entity URIs as strings or read streams.!

DTDExternalEntityResolver subclass: #DTDCachingExternalEntityResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

DTDCachingExternalEntityResolver class
	instanceVariableNames: 'entityCache'!

!DTDCachingExternalEntityResolver commentStamp: 'monty 3/22/2016 01:08' prior: 0!
This class is a resolver that caches non-local entities so they don't have to be retrieved again (like over HTTP). (The default resolver)!

DTDCachingExternalEntityResolver class
	instanceVariableNames: 'entityCache'!

!DTDCachingExternalEntityResolver class methodsFor: 'clearing' stamp: 'monty 11/15/2014 20:59'!
clearEntityCache
	"self clearEntityCache"
	self entityCache clear! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:56'!
entityCache
	^ entityCache ifNil: [entityCache := XMLKeyValueCache maxSize: 64]! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 3/22/2016 19:43'!
entityCacheAtURI: anXMLURIOrURIString
	^ self entityCache at: anXMLURIOrURIString asXMLURI! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 3/22/2016 19:43'!
entityCacheAtURI: anXMLURIOrURIString ifAbsent: aBlock
	^ self entityCache
		at: anXMLURIOrURIString asXMLURI
		ifAbsent: aBlock! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 3/22/2016 19:43'!
entityCacheAtURI: anXMLURIOrURIString ifAbsentPut: aBlock
	^ self entityCache
		at: anXMLURIOrURIString asXMLURI
		ifAbsentPut: aBlock! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 3/23/2016 01:06'!
entityCacheAtURI: anXMLURIOrURIString put: aReplacement
	^ self entityCache
		at: anXMLURIOrURIString asXMLURI
		put: aReplacement! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:57'!
maxCachedEntities
	^ self entityCache maxSize! !

!DTDCachingExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:56'!
maxCachedEntities: anInteger
	self entityCache maxSize: anInteger! !

!DTDCachingExternalEntityResolver methodsFor: 'private' stamp: 'monty 4/3/2016 00:45'!
cachingResolveEntityURI: anXMLURIOrURIString
	| uri |

	uri := anXMLURIOrURIString asXMLURI.
	^ self class
		entityCacheAtURI: uri
		ifAbsentPut: [
			self resolveExternalEntityWith: [:maxSize |
				uri
					getUpToLimit: maxSize
					decoding: false]]! !

!DTDCachingExternalEntityResolver methodsFor: 'private' stamp: 'monty 3/22/2016 19:39'!
cachingStreamResolveEntityURI: anXMLURIOrURIString
	^ (self cachingResolveEntityURI: anXMLURIOrURIString) readStream! !

!DTDCachingExternalEntityResolver methodsFor: 'clearing' stamp: 'monty 1/13/2014 03:45'!
clearEntityCache
	self class clearEntityCache! !

!DTDCachingExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 1/13/2014 17:52'!
maxCachedEntities
	^ self class maxCachedEntities! !

!DTDCachingExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 1/13/2014 17:53'!
maxCachedEntities: anInteger
	self class maxCachedEntities: anInteger! !

!DTDCachingExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 4/3/2016 00:44'!
resolveExternalEntityURI: anXMLURIOrURIString
	| uri |

	(uri := anXMLURIOrURIString asXMLURI) isLocalURI
		ifTrue: [^ super resolveExternalEntityURI: uri]
		ifFalse: [
			"non-local entities are cached"
			^ self cachingResolveEntityURI: uri]! !

!DTDCachingExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 4/3/2016 00:44'!
streamResolveExternalEntityURI: anXMLURIOrURIString
	| uri |

	(uri := anXMLURIOrURIString asXMLURI) isLocalURI
		ifTrue: [^ super streamResolveExternalEntityURI: uri]
		ifFalse: [
			"non-local entities are cached"
			^ self cachingStreamResolveEntityURI: uri]! !

!DTDExternalEntityResolver class methodsFor: 'instance creation' stamp: 'monty 4/8/2016 03:03'!
default
	^ self defaultClass new! !

!DTDExternalEntityResolver class methodsFor: 'accessing' stamp: 'monty 4/8/2016 03:04'!
defaultClass
	^ DTDCachingExternalEntityResolver! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 4/11/2016 03:22'!
checkExternalEntityLimit
	(self maxExternalEntities notNil
		and: [self totalResolvedExternalEntities >= self maxExternalEntities])
		ifTrue: [self erorrExternalEntityLimitExceeded]! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 11/6/2014 12:21'!
erorrExternalEntityLimitExceeded
	XMLLimitException signal: 'External entity limit exceeded; use #maxExternalEntities: to change'! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 10/31/2014 21:33'!
errorLocalFromNonLocal
	XMLLimitException signal:
		'Local URI referenced from non-local URI; use #resolvesLocalFromNonLocal: to change'! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:15'!
externalEntityLimits
	^ externalEntityLimits ifNil: [externalEntityLimits := DTDExternalEntityLimits new]! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 3/22/2016 04:41'!
externalEntityLimits: anExternalEntityLimits
	externalEntityLimits := anExternalEntityLimits! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 4/3/2016 00:44'!
externalEntityURIFromPublicID: aPublicID systemID: aSystemID
	^ aSystemID asXMLURI! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 4/3/2016 01:12'!
externalEntityURIFromPublicID: aPublicID systemID: aSystemID baseURI: aBaseXMLURIOrURIString
	| baseURI externalEntityURI |

	baseURI := aBaseXMLURIOrURIString asXMLURI.
	externalEntityURI :=
		baseURI combinedWith:
			(self
				externalEntityURIFromPublicID: aPublicID
				systemID: aSystemID).
	(externalEntityURI isLocalURI
		and: [baseURI isLocalURI not
			and: [self resolvesLocalFromNonLocal not]])
		ifTrue: [self errorLocalFromNonLocal].

	^ externalEntityURI.! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/27/2015 22:53'!
externalSubsetReadLimit
	^ self externalEntityLimits externalSubsetReadLimit! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 4/14/2016 03:31'!
externalSubsetReadLimit: anInteger
	"change the external subset read limit or nil to disable"

	self externalEntityLimits externalSubsetReadLimit: anInteger! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 4/3/2016 00:46'!
incrementTotalResolvedExternalEntities
	totalResolvedExternalEntities :=
		totalResolvedExternalEntities
			ifNil: [1]
			ifNotNil: [totalResolvedExternalEntities + 1]! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:17'!
maxExternalEntities
	^ self externalEntityLimits maxExternalEntities! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:54'!
maxExternalEntities: aLimit
	self externalEntityLimits maxExternalEntities: aLimit! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:20'!
maxExternalEntitySize
	^ self externalEntityLimits maxExternalEntitySize! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:54'!
maxExternalEntitySize: aLimit
	self externalEntityLimits maxExternalEntitySize: aLimit! !

!DTDExternalEntityResolver methodsFor: 'copying' stamp: 'monty 4/3/2016 00:46'!
postCopy
	super postCopy.

	totalResolvedExternalEntities := 0.
	externalEntityLimits
		ifNotNil: [externalEntityLimits := externalEntityLimits copy].! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/16/2014 06:16'!
removeLimits
	self externalEntityLimits removeAll! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 4/3/2016 00:45'!
resolveExternalEntityURI: anXMLURIOrURIString
	^ self resolveExternalEntityWith: [:maxSize |
		anXMLURIOrURIString asXMLURI
			getUpToLimit: maxSize
			decoding: false]! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 4/3/2016 00:45'!
resolveExternalEntityWith: aBlock
	| result |

	self checkExternalEntityLimit.
	result := aBlock value: self maxExternalEntitySize.
	self incrementTotalResolvedExternalEntities.

	^ result.! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 1/24/2014 06:50'!
resolvesLocalFromNonLocal
	^ self externalEntityLimits resolvesLocalFromNonLocal! !

!DTDExternalEntityResolver methodsFor: 'configuring' stamp: 'monty 5/25/2015 03:54'!
resolvesLocalFromNonLocal: aBoolean
	"allow non-local (http) entities to reference local (file) entities?
	(default disabled)"

	self externalEntityLimits resolvesLocalFromNonLocal: aBoolean! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 4/3/2016 00:45'!
streamResolveExternalEntityURI: anXMLURIOrURIString
	^ self resolveExternalEntityWith: [:maxSize |
		anXMLURIOrURIString asXMLURI
			streamGetUpToLimit: maxSize
			decoding: false]! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 4/3/2016 00:46'!
totalResolvedExternalEntities
	^ totalResolvedExternalEntities ifNil: [totalResolvedExternalEntities := 0]! !

DTDExternalEntityResolver subclass: #DTDStaticExternalEntityResolver
	instanceVariableNames: 'externalEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDStaticExternalEntityResolver commentStamp: 'monty 6/27/2016 04:13' prior: 0!
This class is a resolver that maps entity URIs to string replacements so resolution can be done without HTTP or file access. Used during testing.!

!DTDStaticExternalEntityResolver class methodsFor: 'instance creation' stamp: 'monty 4/3/2016 00:52'!
externalEntities: anAssociationCollection
	^ self new externalEntities: anAssociationCollection! !

!DTDStaticExternalEntityResolver methodsFor: 'private' stamp: 'monty 4/3/2016 00:54'!
errorMaxExternalEntitySize: aMaxSize exceededByURI: anXMLURIOrURIString
	XMLLimitException
		formatSignal: 'Entity exceeds max size of {1} bytes: {2}'
		with: aMaxSize
		with: anXMLURIOrURIString asString! !

!DTDStaticExternalEntityResolver methodsFor: 'private' stamp: 'monty 4/3/2016 00:54'!
errorUnresolvableExternalEntityURI: anXMLURIOrURIString
	XMLParserException
		formatSignal: 'Unresolvable external entity URI: {1}'
		with: anXMLURIOrURIString asString! !

!DTDStaticExternalEntityResolver methodsFor: 'private' stamp: 'monty 4/3/2016 00:54'!
externalEntities
	^ externalEntities ifNil: [externalEntities := Dictionary new]! !

!DTDStaticExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 4/3/2016 00:56'!
externalEntities: anAssociationCollection
	self hasExternalEntities
		ifTrue: [self removeExternalEntities].
	anAssociationCollection associationsDo: [:each |
		self
			externalEntityAtURI: each key
			put: each value].! !

!DTDStaticExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 4/3/2016 00:53'!
externalEntityAtURI: anXMLURIOrURIString
	^ self
		externalEntityAtURI: anXMLURIOrURIString asXMLURI
		ifAbsent: ['']! !

!DTDStaticExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 4/3/2016 00:53'!
externalEntityAtURI: anXMLURIOrURIString ifAbsent: aBlock
	^ self externalEntities
		at: anXMLURIOrURIString asXMLURI
		ifAbsent: aBlock! !

!DTDStaticExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 4/3/2016 00:53'!
externalEntityAtURI: anXMLURIOrURIString ifAbsentPut: aBlock
	^ self externalEntities
		at: anXMLURIOrURIString asXMLURI
		ifAbsentPut: aBlock! !

!DTDStaticExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 4/3/2016 00:53'!
externalEntityAtURI: anXMLURIOrURIString put: aReplacement
	^ self externalEntities
		at: anXMLURIOrURIString asXMLURI
		put: aReplacement! !

!DTDStaticExternalEntityResolver methodsFor: 'testing' stamp: 'monty 4/3/2016 00:56'!
hasExternalEntities
	^ self externalEntities notEmpty! !

!DTDStaticExternalEntityResolver methodsFor: 'testing' stamp: 'monty 4/3/2016 00:56'!
includesExternalEntityURI: anXMLURIOrURIString
	^ self externalEntities includesKey: anXMLURIOrURIString asXMLURI! !

!DTDStaticExternalEntityResolver methodsFor: 'removing' stamp: 'monty 4/3/2016 00:54'!
removeExternalEntities
	self externalEntities removeAll! !

!DTDStaticExternalEntityResolver methodsFor: 'removing' stamp: 'monty 4/3/2016 00:55'!
removeExternalEntityURI: anXMLURIOrURIString
	^ self externalEntities removeKey: anXMLURIOrURIString asXMLURI! !

!DTDStaticExternalEntityResolver methodsFor: 'removing' stamp: 'monty 4/3/2016 00:55'!
removeExternalEntityURI: anXMLURIOrURIString ifAbsent: aBlock
	^ self externalEntities
		removeKey: anXMLURIOrURIString asXMLURI
		ifAbsent: aBlock! !

!DTDStaticExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 4/3/2016 00:54'!
resolveExternalEntityURI: anXMLURIOrURIString
	^ self resolveExternalEntityWith: [:maxSize | | replacement |
		replacement :=
			self
				externalEntityAtURI: anXMLURIOrURIString
				ifAbsent: [self errorUnresolvableExternalEntityURI: anXMLURIOrURIString].
		(maxSize notNil
			and: [replacement size > maxSize])
			ifTrue: [
				self
					errorMaxExternalEntitySize: maxSize
					exceededByURI: anXMLURIOrURIString].
		replacement]! !

!DTDStaticExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 4/3/2016 00:44'!
streamResolveExternalEntityURI: anXMLURIOrURIString
	^ (self resolveExternalEntityURI: anXMLURIOrURIString) readStream! !

Object subclass: #DTDUnresolvedExternalEntityReplacement
	instanceVariableNames: 'resolver publicID systemID baseURI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDUnresolvedExternalEntityReplacement commentStamp: 'monty 4/8/2016 07:31' prior: 0!
This class stores an entity resolver and the public ID, system ID, and base URI of an external entity so its replacement can be lazily resolved only if the entity is referenced.!

!DTDUnresolvedExternalEntityReplacement class methodsFor: 'instance creation' stamp: 'monty 4/8/2016 02:49'!
resolver: aResolver publicID: aPublicID systemID: aSystemID baseURI: aBaseURIString
	^ self new
		setResolver: aResolver
		publicID: aPublicID
		systemID: aSystemID
		baseURI: aBaseURIString! !

!DTDUnresolvedExternalEntityReplacement methodsFor: 'accessing' stamp: 'monty 4/8/2016 02:59'!
baseURI
	^ baseURI ifNil: [baseURI := '']! !

!DTDUnresolvedExternalEntityReplacement methodsFor: 'printing' stamp: 'monty 4/8/2016 06:29'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	self publicID
		ifEmpty: [aStream nextPutAll: 'SYSTEM']
		ifNotEmpty: [
			aStream
				nextPutAll: 'PUBLIC "';
				nextPutAll: self publicID;
				nextPut: $"].
	aStream
		nextPutAll: ' "';
		nextPutAll: self systemID;
		nextPutAll: '")'.! !

!DTDUnresolvedExternalEntityReplacement methodsFor: 'accessing' stamp: 'monty 4/8/2016 02:58'!
publicID
	^ publicID ifNil: [publicID := '']! !

!DTDUnresolvedExternalEntityReplacement methodsFor: 'resolving' stamp: 'monty 4/8/2016 17:38'!
resolveForExternalEntity: anExternalEntity
	| externalEntityURI |

	externalEntityURI :=
		self resolver
			externalEntityURIFromPublicID: self publicID
			systemID: self systemID
			baseURI: self baseURI.

	anExternalEntity
		replacement: (self resolver resolveExternalEntityURI: externalEntityURI);
		uri: externalEntityURI asString;
		unresolvedReplacement: nil.! !

!DTDUnresolvedExternalEntityReplacement methodsFor: 'accessing' stamp: 'monty 4/8/2016 03:05'!
resolver
	^ resolver ifNil: [resolver := DTDExternalEntityResolver default]! !

!DTDUnresolvedExternalEntityReplacement methodsFor: 'initialization' stamp: 'monty 4/8/2016 02:50'!
setResolver: aResolver publicID: aPublicID systemID: aSystemID baseURI: aBaseURIString
	resolver := aResolver.
	publicID := aPublicID.
	systemID := aSystemID.
	baseURI := aBaseURIString.! !

!DTDUnresolvedExternalEntityReplacement methodsFor: 'accessing' stamp: 'monty 4/8/2016 02:58'!
systemID
	^ systemID ifNil: [systemID := '']! !

!Object methodsFor: '*XML-Parser' stamp: 'monty 4/1/2016 00:55'!
isXMLConvertingStreamAdapter
	^ false! !

Object subclass: #SAX2ContentHandler
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2ContentHandler commentStamp: 'monty 8/20/2016 07:02' prior: 0!
This is a base class for SAX2 content handler classes that can be injected into SAX2Parser instances with #contentHandler: before parsing:
	(SAX2Parser on: xmlStringOrStream)
		contentHandler: MyContentHandlerClass new;
		parseDocument.

See SAX2Parser and SAXHandler for more info.!

!SAX2ContentHandler class methodsFor: 'instance creation' stamp: 'monty 8/14/2016 18:00'!
parser: aParser
	^ self new parser: aParser! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 8/21/2016 00:25'!
characters: aString
	"Handles character data in an element, such as the 'test' in
	'<characters>test</characters>' or in the CDATA section '<!![CDATA[test]]>'.
	
	It may be sent multiple times for different sections of character data
	in a single element if they're separated by other nodes or entity
	references like in '<characters>one<!!--comment-->two</characters>'.
	
	See ignorableWhitespace:"! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
endDocument
	"Handles the end of an XML document. No more handlers are invoked after this."! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
endElement: aQualifiedName
	"Handles the end of an element if a subclass doesn't implement
	endElement:prefix:uri:localName: or endElement:prefix:uri:."! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
endElement: aQualifiedName prefix: aPrefix uri: aURI
	"Handles the end of an element if a subclass doesn't implement
	endElement:prefix:uri:localName:. By default it forwards to a simpler
	handler, endElement:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string"

	self endElement: aQualifiedName! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	"Handles the end of an element, either an end tag like '</end>' or a
	self-closed tag like '<empty/>'. By default it forwards to a simpler handler,
	endElement:prefix:uri:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aLocalName - the local part of the qualified name"

	self
		endElement: aQualifiedName
		prefix: aPrefix
		uri:  aURI! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
endPrefixMapping: aPrefix
	"Handles the end of a namespace prefix/URI mapping when namespaces are
	enabled. After, aPrefix (empty for the default namespace) will be either
	unmapped or mapped to a different URI. The 'xml' prefix is predefined
	and never unmapped."! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
ignorableWhitespace: aString
	"When validation is enabled and a DTD with element declarations is
	present, this can be used to handle ignorable whitespace in elements
	with 'element content' (meaning they can only contain other elements
	and no PCDATA), such as between '<element-content>' and '<mixed-content>'
	or between '</mixed-content>' and '</element-content>' in this sample:
		'<!!DOCTYPE element-content [
			<!!--this has element content, so its whitespace is ignorable-->
			<!!ELEMENT element-content (mixed-content)>
			<!!--this has mixed content, so its whitespace is not ignorable-->
			<!!ELEMENT mixed-content (#PCDATA|any-content)*>
			<!!--this has any content, so its whitespace is not ignorable-->
			<!!ELEMENT any-content ANY>
		]>
		<element-content>
			<mixed-content>
				<any-content> any </any-content>
			</mixed-content>
		</element-content>'

	Not sent if validation is disabled or if there's no DTD with element
	declarations."! !

!SAX2ContentHandler methodsFor: 'accessing' stamp: 'monty 8/20/2016 06:44'!
parser
	"Returns the SAX2Parser this handler was injected into"

	^ parser! !

!SAX2ContentHandler methodsFor: 'accessing' stamp: 'monty 8/14/2016 17:56'!
parser: aParser
	parser := aParser! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
processingInstruction: aTargetString data: aDataString
	"Handles processing instructions like '<?name value?>', where 'name' is
	the target and 'value' is the data.
	
	XML declarations may look like PIs but are different and are handled
	with xmlVersion:encoding:standalone: instead."! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
skippedEntity: anEntityName
	"Sent when a general parsed entity reference like '&name;' is
	unreplaced in content. Only sent when validation is turned off
	or entity replacement is disabled."! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
startDocument
	"Handles the start of an XML document, the first handler invoked."! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
startElement: aQualifiedName attributes: aDictionary
	"Handles the start of an element if a subclass doesn't implement
	startElement:prefix:uri:localName:attributes: or
	startElement:prefix:uri:attributes:.

	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aDictionary - an OrderPreservingStringDictionary of attributes"! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
startElement: aQualifiedName prefix: aPrefix uri: aURI attributes: aDictionary
	"Handles the start of an element if a subclass doesn't implement
	startElement:prefix:uri:localName:attributes:. By default it forwards
	to a simpler handler, startElement:attributes:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aDictionary - an OrderPreservingStringDictionary of attributes"

	self
		startElement: aQualifiedName
		attributes: aDictionary! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	"Handles the start of an element, either a start tag like '<start>' or a
	self-closed tag like '<empty/>'. By default it forwards to a simpler handler,
	startElement:prefix:uri:attributes:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aLocalName - the local part of the qualified name
	aDictionary - an OrderPreservingStringDictionary of attributes"

	self
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aURI
		attributes: aDictionary! !

!SAX2ContentHandler methodsFor: 'handling - content' stamp: 'monty 6/23/2016 01:29'!
startPrefixMapping: aPrefix uri: aURI
	"Handles the start of a namespace prefix/URI mapping when namespaces are
	enabled. The 'xml' prefix is predefined and never remapped.

	aPrefix - the prefix of the namespace (empty for the default namesapce)
	aURI - the namespace URI"! !

Object subclass: #SAX2DeclarationHandler
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2DeclarationHandler commentStamp: 'monty 8/20/2016 07:02' prior: 0!
This is a base class for SAX2 declaration handler classes that can be injected into SAX2Parser instances with #declarationHandler: before parsing:
	(SAX2Parser on: xmlStringOrStream)
		declarationHandler: MyDeclarationHandlerClass new;
		parseDocument.

See SAX2Parser and SAXHandler for more info.!

!SAX2DeclarationHandler class methodsFor: 'instance creation' stamp: 'monty 8/14/2016 18:00'!
parser: aParser
	^ self new parser: aParser! !

!SAX2DeclarationHandler methodsFor: 'handling - declaration' stamp: 'monty 6/23/2016 01:29'!
attributeDeclaration: anElementName name: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!SAX2DeclarationHandler methodsFor: 'handling - declaration' stamp: 'monty 6/23/2016 01:29'!
elementDeclaration: anElementName contentModel: aContentModel
! !

!SAX2DeclarationHandler methodsFor: 'handling - declaration' stamp: 'monty 6/23/2016 01:29'!
generalEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotationName! !

!SAX2DeclarationHandler methodsFor: 'handling - declaration' stamp: 'monty 6/23/2016 01:29'!
generalEntityDeclaration: aName replacement: aReplacement! !

!SAX2DeclarationHandler methodsFor: 'handling - declaration' stamp: 'monty 6/23/2016 01:29'!
notationDeclaration: aName publicID: aPublicID systemID: aSystemID
! !

!SAX2DeclarationHandler methodsFor: 'handling - declaration' stamp: 'monty 6/23/2016 01:29'!
parameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID! !

!SAX2DeclarationHandler methodsFor: 'handling - declaration' stamp: 'monty 6/23/2016 01:29'!
parameterEntityDeclaration: aName replacement: aReplacement! !

!SAX2DeclarationHandler methodsFor: 'accessing' stamp: 'monty 8/20/2016 06:44'!
parser
	"Returns the SAX2Parser this handler was injected into"

	^ parser! !

!SAX2DeclarationHandler methodsFor: 'accessing' stamp: 'monty 8/14/2016 17:57'!
parser: aParser
	parser := aParser! !

Object subclass: #SAX2ElementHandler
	instanceVariableNames: 'parser parent parentFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2ElementHandler commentStamp: 'monty 8/21/2016 00:23' prior: 0!
This is base class for SAX2 element handlers to be used with SAX2ElementParsers.

SAX2ElementParsers use separate SAX2ElementHandlers to handle the  #startElement*, #endElement*, and #characters: events of each element in the document instead of using a single SAX2ContentHandler for everything.

The parser can be accessed with #parser, and the parent element handler (if any) can be accessed with #parent.!

SAX2ElementHandler subclass: #SAX2ElementCharacterHandler
	instanceVariableNames: 'characters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2ElementCharacterHandler commentStamp: 'monty 8/21/2016 00:11' prior: 0!
This class is a SAX2ElementHandler that handles #characters: events. Sending #characters returns the complete character data of the element as a string.!

!SAX2ElementCharacterHandler methodsFor: 'accessing' stamp: 'monty 8/16/2016 22:20'!
characters
	"This returns the combined #characters: events handled by the receiver
	as a string."

	^ characters ifNil: [characters := '']! !

!SAX2ElementCharacterHandler methodsFor: 'handling - content' stamp: 'monty 8/20/2016 02:11'!
characters: aString
	characters :=
		(characters == nil "optimization"
			or: [characters size = 0]) "optimization"
			ifTrue: [aString]
			ifFalse: [characters, aString]! !

!SAX2ElementHandler class methodsFor: 'parsing' stamp: 'monty 8/17/2016 23:41'!
parser: aParser
	^ self
		parser: aParser
		parent: aParser elementHandler
		parentFactory: aParser elementHandlerFactory! !

!SAX2ElementHandler class methodsFor: 'parsing' stamp: 'monty 8/18/2016 23:29'!
parser: aParser parent: anElementHandler parentFactory: anElementHandlerFactory
	^ self new
		setParser: aParser
		parent: anElementHandler
		parentFactory: anElementHandlerFactory! !

!SAX2ElementHandler methodsFor: 'accessing' stamp: 'monty 8/19/2016 23:12'!
ancestors	
	"Returns a list of all ancestor element handlers of the receiver."

	| ancestors |

	ancestors := XMLOrderedList new.
	self ancestorsDo: [:each | ancestors addLast: each].
	^ ancestors.! !

!SAX2ElementHandler methodsFor: 'enumerating' stamp: 'monty 8/19/2016 23:12'!
ancestorsDo: aBlock
	| ancestor |

	ancestor := self parent.
	[ancestor notNil]
		whileTrue: [
			aBlock value: ancestor.
			ancestor := ancestor parent].! !

!SAX2ElementHandler methodsFor: 'accessing' stamp: 'monty 8/16/2016 22:19'!
characters
	"This handler ignores #characters: events, so this always returns
	an empty string."

	^ ''! !

!SAX2ElementHandler methodsFor: 'handling - content' stamp: 'monty 8/21/2016 00:24'!
characters: aString
	"Handles character data in an element, such as the 'test' in
	'<characters>test</characters>' or in the CDATA section '<!![CDATA[test]]>'.
	
	It may be sent multiple times for different sections of character data
	in a single element if they're separated by other nodes or entity
	references like in '<characters>one<!!--comment-->two</characters>'."! !

!SAX2ElementHandler methodsFor: 'accessing' stamp: 'monty 8/20/2016 02:31'!
elementHandlerFactory
	"Can return a default factory to use by a factory mapper if no other
	one is mapped to the receiver."

	^ nil! !

!SAX2ElementHandler methodsFor: 'handling - content' stamp: 'monty 7/22/2016 12:59'!
endElement: aQualifiedName
	"Handles the end of an element if a subclass doesn't implement
	endElement:prefix:uri:localName: or endElement:prefix:uri:."! !

!SAX2ElementHandler methodsFor: 'handling - content' stamp: 'monty 7/22/2016 12:59'!
endElement: aQualifiedName prefix: aPrefix uri: aURI
	"Handles the end of an element if a subclass doesn't implement
	endElement:prefix:uri:localName:. By default it forwards to a simpler
	handler, endElement:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string"

	self endElement: aQualifiedName! !

!SAX2ElementHandler methodsFor: 'handling - content' stamp: 'monty 7/22/2016 12:59'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	"Handles the end of an element, either an end tag like '</end>' or a
	self-closed tag like '<empty/>'. By default it forwards to a simpler handler,
	endElement:prefix:uri:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aLocalName - the local part of the qualified name"

	self
		endElement: aQualifiedName
		prefix: aPrefix
		uri:  aURI! !

!SAX2ElementHandler methodsFor: 'testing' stamp: 'monty 8/18/2016 06:07'!
hasParent
	^ parent notNil! !

!SAX2ElementHandler methodsFor: 'accessing' stamp: 'monty 8/20/2016 18:59'!
parent
	"Returns the parent element handler or nil if there isn't one (if
	the receiver is handling the root element)."

	^ parent! !

!SAX2ElementHandler methodsFor: 'accessing' stamp: 'monty 8/20/2016 18:59'!
parentFactory
	"Returns the factory that created the receiver."

	^ parentFactory! !

!SAX2ElementHandler methodsFor: 'accessing' stamp: 'monty 8/16/2016 04:16'!
parser
	"Returns the SAX2ElementParser"

	^ parser! !

!SAX2ElementHandler methodsFor: 'initialization' stamp: 'monty 8/18/2016 23:29'!
setParser: aParser parent: anElementHandler parentFactory: anElementHandlerFactory
	parser := aParser.
	parent := anElementHandler.
	parentFactory := anElementHandlerFactory.! !

!SAX2ElementHandler methodsFor: 'handling - content' stamp: 'monty 7/22/2016 13:00'!
startElement: aQualifiedName attributes: aDictionary
	"Handles the start of an element if a subclass doesn't implement
	startElement:prefix:uri:localName:attributes: or
	startElement:prefix:uri:attributes:.

	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aDictionary - an OrderPreservingStringDictionary of attributes"! !

!SAX2ElementHandler methodsFor: 'handling - content' stamp: 'monty 7/22/2016 12:59'!
startElement: aQualifiedName prefix: aPrefix uri: aURI attributes: aDictionary
	"Handles the start of an element if a subclass doesn't implement
	startElement:prefix:uri:localName:attributes:. By default it forwards
	to a simpler handler, startElement:attributes:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aDictionary - an OrderPreservingStringDictionary of attributes"

	self
		startElement: aQualifiedName
		attributes: aDictionary! !

!SAX2ElementHandler methodsFor: 'handling - content' stamp: 'monty 7/22/2016 12:59'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	"Handles the start of an element, either a start tag like '<start>' or a
	self-closed tag like '<empty/>'. By default it forwards to a simpler handler,
	startElement:prefix:uri:attributes:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aLocalName - the local part of the qualified name
	aDictionary - an OrderPreservingStringDictionary of attributes"

	self
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aURI
		attributes: aDictionary! !

Object subclass: #SAX2ElementHandlerFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2ElementHandlerFactory commentStamp: 'monty 8/16/2016 22:00' prior: 0!
This is a base class for element handler factories used by SAX2ElementParsers to create element handlers. Instances can be injected into a SAX2ElementParser with #elementHandlerFactory: before parsing.

This  factory just users the default handler class SAX2ElementHandler for all elements.!

!SAX2ElementHandlerFactory methodsFor: 'accessing' stamp: 'monty 8/12/2016 23:21'!
classForElement: aQualifiedName namespaceURI: aURI localName: aLocalName attributes: anAssociationCollection
	^ self elementHandlerClass! !

!SAX2ElementHandlerFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:01'!
elementHandlerClass
	^ SAX2ElementHandler! !

SAX2ElementHandlerFactory subclass: #SAX2PluggableElementHandlerFactory
	instanceVariableNames: 'elementClassMapper elementHandlerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2PluggableElementHandlerFactory commentStamp: 'monty 8/26/2016 03:16' prior: 0!
This class is a pluggable element handler factory with an API like XMLPluggableElementFactory. Instances can be injected into a SAX2ElementParser with #elementHandlerFactory: before parsing.

You can map combinations of element name, namespace URI, and attributes to specific element handler classes using the "handling" messages, and change the default element handler class for when there's no mapping with #elementHandlerClass:.

Instances can be saved and resused for performance and safely modified after copying.!

!SAX2PluggableElementHandlerFactory methodsFor: 'accessing' stamp: 'monty 8/12/2016 23:22'!
classForElement: aQualifiedName namespaceURI: aURI localName: aLocalName attributes: anAssociationCollection
	^ elementClassMapper
		classForElement: aQualifiedName
		namespaceURI: aURI
		attributes: anAssociationCollection
		ifNone: [self elementHandlerClass]! !

!SAX2PluggableElementHandlerFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:00'!
elementClassMapper
	^ elementClassMapper! !

!SAX2PluggableElementHandlerFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:00'!
elementClassMapper: anElementClassMapper
	elementClassMapper := anElementClassMapper! !

!SAX2PluggableElementHandlerFactory methodsFor: 'private ' stamp: 'monty 8/26/2016 02:16'!
elementClassMapperClass
	^ XMLBasicElementClassMapper! !

!SAX2PluggableElementHandlerFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:02'!
elementHandlerClass
	^ elementHandlerClass! !

!SAX2PluggableElementHandlerFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:02'!
elementHandlerClass: aClass
	elementHandlerClass := aClass! !

!SAX2PluggableElementHandlerFactory methodsFor: 'handling' stamp: 'monty 8/18/2016 23:10'!
handleElement: anElementName attributes: aCollectionOrAttributeSpec withClass: aClass
	"aCollectionOrAttributeSpec can be some kind of XMLAttributeSpec or a
	collection of associations of required attribute names and values, and the
	required values can be strings, blocks, or nil meaning the attribute is
	required but can take any value"

	elementClassMapper
		mapElement: anElementName
		attributeSpec: aCollectionOrAttributeSpec asXMLAttributeSpec
		toClass: aClass! !

!SAX2PluggableElementHandlerFactory methodsFor: 'handling' stamp: 'monty 8/18/2016 23:10'!
handleElement: anElementName namespaceURI: aURI attributes: aCollectionOrAttributeSpec withClass: aClass
	"aCollectionOrAttributeSpec can be some kind of XMLAttributeSpec or a
	collection of associations of required attribute names and values, and the
	required values can be strings, blocks, or nil meaning the attribute is
	required but can take any value"

	elementClassMapper
		mapElement: anElementName
		namespaceURI: aURI
		attributeSpec: aCollectionOrAttributeSpec asXMLAttributeSpec
		toClass: aClass! !

!SAX2PluggableElementHandlerFactory methodsFor: 'handling' stamp: 'monty 8/5/2016 11:03'!
handleElement: anElementName namespaceURI: aURI withClass: aClass
	elementClassMapper
		mapElement: anElementName
		namespaceURI: aURI
		toClass: aClass! !

!SAX2PluggableElementHandlerFactory methodsFor: 'handling' stamp: 'monty 8/5/2016 11:03'!
handleElement: anElementName withClass: aClass
	elementClassMapper
		mapElement: anElementName
		toClass: aClass! !

!SAX2PluggableElementHandlerFactory methodsFor: 'handling' stamp: 'monty 8/5/2016 11:03'!
handleElements: anElementNameCollection namespaceURI: aNamespaceURI withClass: aClass
	elementClassMapper
		mapElements: anElementNameCollection
		namespaceURI: aNamespaceURI
		toClass: aClass! !

!SAX2PluggableElementHandlerFactory methodsFor: 'handling' stamp: 'monty 8/5/2016 11:03'!
handleElements: anElementNameCollection withClass: aClass
	elementClassMapper
		mapElements: anElementNameCollection
		toClass: aClass! !

!SAX2PluggableElementHandlerFactory methodsFor: 'handling' stamp: 'monty 8/26/2016 03:19'!
handleElementsMatchingClass: aClass
	"This will handle all elements matching the name of a class with that
	class, but igoring the class's prefix and any 'Element' or 'ElementHandler'
	suffix.

	For example, it will match MyUserDefinedElement to:
		<UserDefined>
		<userDefined>
		<user-defined>
		<user_defined>"

	elementClassMapper
		mapAllMatchingElementsToClass: aClass
		ignoringSuffixes: self ignorableClassNameSuffixes! !

!SAX2PluggableElementHandlerFactory methodsFor: 'handling' stamp: 'monty 8/26/2016 01:41'!
handleElementsMatchingClasses: aClassCollection
	"See the comment in #handleElementsMatchingClass:"

	elementClassMapper
		mapAllMatchingElementsToClasses: aClassCollection
		ignoringSuffixes: self ignorableClassNameSuffixes! !

!SAX2PluggableElementHandlerFactory methodsFor: 'private ' stamp: 'monty 8/26/2016 00:48'!
ignorableClassNameSuffixes
	^ #('Element' 'ElementHandler')! !

!SAX2PluggableElementHandlerFactory methodsFor: 'initialization' stamp: 'monty 8/5/2016 11:02'!
initialize
	super initialize.

	elementClassMapper := self elementClassMapperClass new.
	elementHandlerClass := SAX2ElementHandler.! !

!SAX2PluggableElementHandlerFactory methodsFor: 'copying' stamp: 'monty 8/5/2016 11:05'!
postCopy
	super postCopy.

	elementClassMapper := elementClassMapper copy.! !

Object subclass: #SAX2ElementHandlerFactoryMapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2ElementHandlerFactoryMapper commentStamp: 'monty 8/23/2016 18:38' prior: 0!
This is a base class for mappers to map element handler objects to element handler factories.

Requesting the factory for an element handler checks the handler's #elementHandlerFactory message first and then the mapper's own #elementHandlerFactory message which can be (and is) overriden in subclasses to return a default factory.

When a SAX2ElementParser requests the factory for a new element handler, if none is found, it will reuse (inherit) the current factory.!

!SAX2ElementHandlerFactoryMapper methodsFor: 'accessing' stamp: 'monty 8/26/2016 02:42'!
elementHandlerFactory
	"Can be overriden to return a default element handler factory to use
	if no other mapping is defined for it."

	^ nil! !

!SAX2ElementHandlerFactoryMapper methodsFor: 'accessing' stamp: 'monty 8/15/2016 01:45'!
factoryForElementHandler: anElementHandler
	^ self
		factoryForElementHandler: anElementHandler
		ifNone: [nil]! !

!SAX2ElementHandlerFactoryMapper methodsFor: 'accessing' stamp: 'monty 8/17/2016 23:53'!
factoryForElementHandler: anElementHandler ifNone: aBlock
	^ anElementHandler elementHandlerFactory
		ifNil: [
			self elementHandlerFactory
				ifNil: [aBlock value]]! !

SAX2ElementHandlerFactoryMapper subclass: #SAX2PluggableElementHandlerFactoryMapper
	instanceVariableNames: 'elementHandlerFactoryMap elementHandlerFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2PluggableElementHandlerFactoryMapper commentStamp: 'monty 8/26/2016 03:16' prior: 0!
This class is a pluggable mapper to map element handler objects to element handler factories.

Requesting the factory for an element handler checks for a mapping set with  #mapElementHandlerClass: toFactory:, then the handler's  #elementHandlerFactory message, then the mapper's own #elementHandlerFactory message, which returns a default factory (unlike in the base class) which can be changed with #elementHandlerFactory:.

When a SAX2ElementParser requests the factory for a new element handler, if none is found, it will reuse (inherit) the current factory, but this won't happen unless the default factory is cleared by setting #elementHandlerFactory: to nil.

Instances can be saved and resused for performance and safely modified after copying.!

!SAX2PluggableElementHandlerFactoryMapper methodsFor: 'private' stamp: 'monty 8/15/2016 23:57'!
defaultElementHandlerFactoryClass
	^ SAX2ElementHandlerFactory! !

!SAX2PluggableElementHandlerFactoryMapper methodsFor: 'accessing' stamp: 'monty 8/15/2016 23:58'!
elementHandlerFactory
	^ elementHandlerFactory! !

!SAX2PluggableElementHandlerFactoryMapper methodsFor: 'accessing' stamp: 'monty 8/15/2016 23:58'!
elementHandlerFactory: anElementHandlerFactory
	elementHandlerFactory := anElementHandlerFactory! !

!SAX2PluggableElementHandlerFactoryMapper methodsFor: 'private' stamp: 'monty 8/16/2016 00:01'!
elementHandlerFactoryMap
	^ elementHandlerFactoryMap! !

!SAX2PluggableElementHandlerFactoryMapper methodsFor: 'accessing' stamp: 'monty 8/16/2016 23:36'!
factoryForElementHandler: anElementHandler ifNone: aBlock
	"optimization to skip any unneeded hash lookups"
	elementHandlerFactoryMap size > 0
		ifTrue: [
			^ elementHandlerFactoryMap
				at: anElementHandler class
				ifAbsent: [
					super
						factoryForElementHandler: anElementHandler
						ifNone: aBlock]]
		ifFalse: [
			^ super
				factoryForElementHandler: anElementHandler
				ifNone: aBlock]! !

!SAX2PluggableElementHandlerFactoryMapper methodsFor: 'initialization' stamp: 'monty 8/15/2016 23:57'!
initialize
	super initialize.

	elementHandlerFactoryMap := IdentityDictionary new.
	elementHandlerFactory := self defaultElementHandlerFactoryClass new.! !

!SAX2PluggableElementHandlerFactoryMapper methodsFor: 'mapping' stamp: 'monty 8/15/2016 04:26'!
mapElementHandlerClass: aClass toFactory: aFactory
	elementHandlerFactoryMap
		at: aClass
		put: aFactory! !

!SAX2PluggableElementHandlerFactoryMapper methodsFor: 'copying' stamp: 'monty 8/16/2016 00:02'!
postCopy
	super postCopy.

	elementHandlerFactoryMap := elementHandlerFactoryMap copy.! !

Object subclass: #SAX2LexicalHandler
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2LexicalHandler commentStamp: 'monty 8/20/2016 07:02' prior: 0!
This is a base class for SAX2 lexical handler classes that can be injected into SAX2Parser instances with #lexicalHandler: before parsing:
	(SAX2Parser on: xmlStringOrStream)
		lexicalHandler: MyLexicalHandlerClass new;
		parseDocument.

See SAX2Parser and SAXHandler for more info.!

!SAX2LexicalHandler class methodsFor: 'instance creation' stamp: 'monty 8/14/2016 18:00'!
parser: aParser
	^ self new parser: aParser! !

!SAX2LexicalHandler methodsFor: 'handling - lexical' stamp: 'monty 6/23/2016 01:29'!
comment: aString
	"Handles '<!!--comment-->' sections in documents."! !

!SAX2LexicalHandler methodsFor: 'handling - lexical' stamp: 'monty 6/23/2016 01:29'!
endCData
	"Sent after the character data of a '<!![CDATA[...]]>' section."! !

!SAX2LexicalHandler methodsFor: 'handling - lexical' stamp: 'monty 6/23/2016 01:29'!
endDTD
! !

!SAX2LexicalHandler methodsFor: 'handling - lexical' stamp: 'monty 6/23/2016 01:29'!
endEntity: anEntityName
	"Handles the end of some general entity's replacement in content.
	Not sent if the entity is undeclared or if content entity replacement
	is disabled."! !

!SAX2LexicalHandler methodsFor: 'accessing' stamp: 'monty 8/20/2016 06:44'!
parser
	"Returns the SAX2Parser this handler was injected into"

	^ parser! !

!SAX2LexicalHandler methodsFor: 'accessing' stamp: 'monty 8/14/2016 17:57'!
parser: aParser
	parser := aParser! !

!SAX2LexicalHandler methodsFor: 'handling - lexical' stamp: 'monty 6/23/2016 01:29'!
startCData
	"Sent before the character data of a '<!![CDATA[...]]>' section."! !

!SAX2LexicalHandler methodsFor: 'handling - lexical' stamp: 'monty 6/23/2016 01:29'!
startDTD: aRootElementName publicID: aPublicID systemID: aSystemID
	"Handles the start of a '<!!DOCTYPE ...>' declaration. Any events
	after this and until endDTD is sent are part of the internal subset
	of the DTD.
	
	aRootElementName - the expected name of the document's root element
	aPublicID - the public ID of the external DTD subset or an empty string
	aSystemID - the URI where the external subset can be found or an empty string"! !

!SAX2LexicalHandler methodsFor: 'handling - lexical' stamp: 'monty 6/23/2016 01:29'!
startEntity: anEntityName
	"Handles the start of some general entity's replacement in content.
	Not sent if the entity is undeclared or if content entity replacement
	is disabled."! !

!SAX2LexicalHandler methodsFor: 'handling - lexical' stamp: 'monty 6/23/2016 01:29'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	"Handles the '<?xml ...?>' declaration of a document.
	
	aVersionFloat - the version of XML the document uses, usually 1.0
	anEncoding - the name of the encoding or an empty string
	aBoolean - true if standalone='yes'"! !

Object subclass: #SAXHandler
	instanceVariableNames: 'input documentURI configuration isInContent isInCDataSection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXHandler commentStamp: 'monty 3/14/2016 00:11' prior: 0!
This class is an event-handling XML parser. To use it, subclass it and override event handlers in the "handling" categories, such as startDocument, startElement:attributes:, endElement:, and endDocument.

By default XML namespace support and validation are enabled but not external entity resolution.

Once the subclass is ready, use the class "parsing" messages to parse XML:
	result := SAXHandlerSubclass parse: xmlStringOrStream.
	resultFromURL := SAXHandlerSubclass parseURL: xmlURLString.
	resultFromFile := SAXHandlerSubclass parseFileNamed: xmlFileName.

The class "instance creation" messages create and return new parsers on the input so they can be configured with messages in the  "configuring" category before parsing:
	result :=
		(SAXHandlerSubclass on: xmlStringOrStream)
			isValidating: true;
			resolvesExternalEntities: true;
			parseDocument.

#interruptParsing can be sent from within a handler to stop parsing, and there is also #parseDocumentWhile: and #parseDocumentUntil: 
	result :=
		(SAXHandlerSubclass on: xmlStringOrStream)
			parseDocumentWhile: [self shouldKeepParsing].

To parse incrementally, send #parser to an instance to get the underlying XMLParser object and send it #parseToken repeatedly:
	(handler := SAXHandlerSubclass on: xmlStringOrStream)
		isValidating: false; 
		preservesUndeclaredEntityReferences: true.
	parser := handler parser.

	"Only parse the first 10 tokens:"
	10 timesRepeat: [parser parseToken].

There are security limits on input you can remove with #removeLimits or change with messages like #documentReadLimit:
	result :=
		(SAXHandlerSubclass on: xmlStringOrStream)
			removeLimits;
			documentReadLimit: newReadLimit;
			maxEntityReplacementDepth: newMaxEntityDepth;
			parseDocument.
		
#optimizeForLargeDocuments can be used when parsing large documents if you don't care for validating or namespaces.!

SAXHandler subclass: #SAX2Parser
	instanceVariableNames: 'contentHandler lexicalHandler declarationHandler parsingResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2Parser commentStamp: 'monty 8/20/2016 07:04' prior: 0!
This class is a parser with an API like SAX 2.0, with separate SAX2ContentHandler, SAX2LexicalHandler, and SAX2DeclarationHandler classes that are subclassed unlike SAX 1.0 where a single,  unified handler class is subclassed directly. Parsers can be created with #on: or other instance creation messages, and instances of the content, lexical, and declaration handlers can be injected before parsing with #contentHandler:, #lexicalHandler:, and #declarationHandler:, like this:
	(SAX2Parser on: xmlStringOrStream)
		contentHandler: MyContentHandlerClass new;
		declarationHandler: MyDeclarationHandlerClass new;
		lexicalHandler: MyLexicalHandlerClass new;
		parseDocument.

Each handler can access the SAX2Parser object with #parser and set the parsing result returned by #parseDocument with #parsingResult:.

SAXHandler is not deprecated and can still be used if you prefer the SAX 1.0 style API.!

SAX2Parser subclass: #SAX2ElementParser
	instanceVariableNames: 'elementHandler elementHandlerFactory elementHandlerFactoryMapper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAX2ElementParser commentStamp: 'monty 8/26/2016 03:05' prior: 0!
This class is a SAX2Parser that uses separate SAX2ElementHandlers to handle the #startElement*, #endElement*, and #characters: events of each element in the document instead of using a single SAX2ContentHandler for everything.

Custom subclasses of SAX2ElementHandler can be mapped to element names, namespace URIs, and attributes using a SAX2PluggableElementHandlerFactory. Factories can be injected with #elementHandlerFactory: before parsing. See #testParsingSampleBooks for an example.

You can have multiple factories by mapping specific SAX2ElementHandlers to specific factories using a SAX2PluggableElementHandlerFactoryMapper, which can be injected with #elementHandlerFactoryMapper: before parsing. This can be used to create a finite-state machine, with the element handler instances being the states and the mapped factories being their transitions. See #testParsingSamplePerson for an example.!

!SAX2ElementParser methodsFor: 'handling - content' stamp: 'monty 7/22/2016 13:11'!
characters: aString
	self elementHandler characters: aString! !

!SAX2ElementParser methodsFor: 'private' stamp: 'monty 8/15/2016 01:01'!
defaultElementHandlerFactoryClass
	^ SAX2ElementHandlerFactory! !

!SAX2ElementParser methodsFor: 'private' stamp: 'monty 8/15/2016 01:31'!
defaultElementHandlerFactoryMapperClass
	^ SAX2ElementHandlerFactoryMapper! !

!SAX2ElementParser methodsFor: 'accessing' stamp: 'monty 7/23/2016 04:14'!
elementHandler
	^ elementHandler! !

!SAX2ElementParser methodsFor: 'accessing' stamp: 'monty 8/15/2016 01:54'!
elementHandler: anElementHandler
	elementHandler := anElementHandler! !

!SAX2ElementParser methodsFor: 'accessing' stamp: 'monty 7/22/2016 13:10'!
elementHandlerFactory
	^ elementHandlerFactory! !

!SAX2ElementParser methodsFor: 'accessing' stamp: 'monty 7/22/2016 13:10'!
elementHandlerFactory: anElementHandlerFactory
	elementHandlerFactory := anElementHandlerFactory! !

!SAX2ElementParser methodsFor: 'accessing' stamp: 'monty 8/15/2016 01:31'!
elementHandlerFactoryMapper
	^ elementHandlerFactoryMapper! !

!SAX2ElementParser methodsFor: 'accessing' stamp: 'monty 8/15/2016 01:31'!
elementHandlerFactoryMapper: anElementHandlerFactoryMapper
	elementHandlerFactoryMapper := anElementHandlerFactoryMapper! !

!SAX2ElementParser methodsFor: 'handling - content' stamp: 'monty 8/20/2016 02:39'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	| oldElementHandler |

	(oldElementHandler := self elementHandler)
		endElement: aQualifiedName
		prefix: aPrefix
		uri: aURI
		localName: aLocalName.
	self
		elementHandler: oldElementHandler parent;
		elementHandlerFactory: oldElementHandler parentFactory.! !

!SAX2ElementParser methodsFor: 'initialization' stamp: 'monty 8/15/2016 01:31'!
on: aStringOrStream
	super on: aStringOrStream.

	elementHandlerFactory := self defaultElementHandlerFactoryClass new.
	elementHandlerFactoryMapper := self defaultElementHandlerFactoryMapperClass new.! !

!SAX2ElementParser methodsFor: 'handling - content' stamp: 'monty 8/20/2016 02:39'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	| newElementHandler |

	newElementHandler :=
		(self elementHandlerFactory
			classForElement: aQualifiedName
			namespaceURI: aURI
			localName: aLocalName
			attributes: aDictionary)
				parser: self
				parent: self elementHandler
				parentFactory: self elementHandlerFactory.
	self
		elementHandler: newElementHandler;
		elementHandlerFactory:
			(self elementHandlerFactoryMapper
				factoryForElementHandler: newElementHandler
				ifNone: [self elementHandlerFactory]).

	newElementHandler
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aURI
		localName: aLocalName
		attributes: aDictionary.! !

!SAX2Parser class methodsFor: 'private ' stamp: 'monty 7/4/2016 06:45'!
generateHandlerForwardingMethods
	"
	self generateHandlerForwardingMethods
	"

	(Array
		with: 'contentHandler' -> #'handling - content'
		with: 'lexicalHandler' -> #'handling - lexical'
		with: 'declarationHandler' -> #'handling - declaration')
		do: [:handlerAccessorAndCategory | | handlerAccessor handlerCategory |
			handlerAccessor := handlerAccessorAndCategory key.
			handlerCategory := handlerAccessorAndCategory value.

			(SAXHandler selectorsInCategory: handlerCategory) do: [:each | | parsedMethod sourceStream |
				parsedMethod := RBParser parseMethod: (SAXHandler sourceCodeAt: each).
				(sourceStream := String new writeStream)
					nextPutAll: parsedMethod selectorAndArgumentNames;
					cr;
					nextPutAll: '	self ';
					nextPutAll: handlerAccessor.
				1 to: parsedMethod keywords size do: [:i |
					parsedMethod keywords size > 1
						ifTrue: [
							sourceStream
								cr;
								tab;
								tab]
						ifFalse: [sourceStream space].
					sourceStream nextPutAll: (parsedMethod keywords at: i).
					parsedMethod argumentNames
						ifNotEmpty: [
							sourceStream
								space;
								nextPutAll: (parsedMethod argumentNames at: i)]].

				self
					compile: sourceStream contents
					classified: handlerCategory]]! !

!SAX2Parser class methodsFor: 'private ' stamp: 'monty 7/10/2016 02:42'!
generateMethodsInHandlerClasses
	"
	self generateMethodsInHandlerClasses
	"

	(Array
		with: SAX2ContentHandler -> #'handling - content'
		with: SAX2LexicalHandler -> #'handling - lexical'
		with: SAX2DeclarationHandler -> #'handling - declaration')
		do: [:handlerClassAndCategory | | handlerClass handlerCategory |
			handlerClass := handlerClassAndCategory key.
			handlerCategory := handlerClassAndCategory value.

			(SAXHandler selectorsInCategory: handlerCategory) do: [:each |
				handlerClass
					compile: (SAXHandler sourceCodeAt: each)
					classified: handlerCategory]]! !

!SAX2Parser methodsFor: 'handling - declaration' stamp: 'monty 6/24/2016 23:32'!
attributeDeclaration: anElementName name: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self declarationHandler
		attributeDeclaration: anElementName
		name: anAttributeName
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
characters: aString
	self contentHandler characters: aString! !

!SAX2Parser methodsFor: 'handling - lexical' stamp: 'monty 6/24/2016 23:32'!
comment: aString
	self lexicalHandler comment: aString! !

!SAX2Parser methodsFor: 'accessing' stamp: 'monty 6/22/2016 16:27'!
contentHandler
	^ contentHandler! !

!SAX2Parser methodsFor: 'accessing' stamp: 'monty 8/14/2016 17:58'!
contentHandler: aContentHandler
	(contentHandler :=
		aContentHandler ifNil: [self defaultContentHandlerClass new])
			parser: self! !

!SAX2Parser methodsFor: 'accessing' stamp: 'monty 6/22/2016 16:27'!
declarationHandler
	^ declarationHandler! !

!SAX2Parser methodsFor: 'accessing' stamp: 'monty 8/14/2016 17:58'!
declarationHandler: aDeclarationHandler
	(declarationHandler :=
		aDeclarationHandler ifNil: [self defaultDeclarationHandlerClass new])
			parser: self! !

!SAX2Parser methodsFor: 'private' stamp: 'monty 7/10/2016 02:41'!
defaultContentHandlerClass
	^ SAX2ContentHandler! !

!SAX2Parser methodsFor: 'private' stamp: 'monty 7/10/2016 02:42'!
defaultDeclarationHandlerClass
	^ SAX2DeclarationHandler! !

!SAX2Parser methodsFor: 'private' stamp: 'monty 7/10/2016 02:42'!
defaultLexicalHandlerClass
	^ SAX2LexicalHandler! !

!SAX2Parser methodsFor: 'handling - declaration' stamp: 'monty 6/24/2016 23:32'!
elementDeclaration: anElementName contentModel: aContentModel
	self declarationHandler
		elementDeclaration: anElementName
		contentModel: aContentModel! !

!SAX2Parser methodsFor: 'handling - lexical' stamp: 'monty 6/24/2016 23:32'!
endCData
	self lexicalHandler endCData! !

!SAX2Parser methodsFor: 'handling - lexical' stamp: 'monty 6/24/2016 23:32'!
endDTD
	self lexicalHandler endDTD! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
endDocument
	self contentHandler endDocument! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	self contentHandler
		endElement: aQualifiedName
		prefix: aPrefix
		uri: aURI
		localName: aLocalName! !

!SAX2Parser methodsFor: 'handling - lexical' stamp: 'monty 6/24/2016 23:32'!
endEntity: anEntityName
	self lexicalHandler endEntity: anEntityName! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
endPrefixMapping: aPrefix
	self contentHandler endPrefixMapping: aPrefix! !

!SAX2Parser methodsFor: 'handling - declaration' stamp: 'monty 6/24/2016 23:32'!
generalEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotationName
	self declarationHandler
		generalEntityDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID
		ndata: aNotationName! !

!SAX2Parser methodsFor: 'handling - declaration' stamp: 'monty 6/24/2016 23:32'!
generalEntityDeclaration: aName replacement: aReplacement
	self declarationHandler
		generalEntityDeclaration: aName
		replacement: aReplacement! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
ignorableWhitespace: aString
	self contentHandler ignorableWhitespace: aString! !

!SAX2Parser methodsFor: 'accessing' stamp: 'monty 6/22/2016 16:27'!
lexicalHandler
	^ lexicalHandler! !

!SAX2Parser methodsFor: 'accessing' stamp: 'monty 8/14/2016 17:58'!
lexicalHandler: aLexicalHandler
	(lexicalHandler :=
		aLexicalHandler ifNil: [self defaultLexicalHandlerClass new])
			parser: self! !

!SAX2Parser methodsFor: 'handling - declaration' stamp: 'monty 6/24/2016 23:32'!
notationDeclaration: aName publicID: aPublicID systemID: aSystemID
	self declarationHandler
		notationDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID! !

!SAX2Parser methodsFor: 'initialization' stamp: 'monty 8/16/2016 23:35'!
on: aStringOrStream
	super on: aStringOrStream.

	contentHandler := self defaultContentHandlerClass parser: self.
	lexicalHandler := self defaultLexicalHandlerClass parser: self.
	declarationHandler := self defaultDeclarationHandlerClass parser: self.
	
	"self is the returns parsing result SAXHandler returns"
	parsingResult := self.! !

!SAX2Parser methodsFor: 'handling - declaration' stamp: 'monty 6/24/2016 23:32'!
parameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID
	self declarationHandler
		parameterEntityDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID! !

!SAX2Parser methodsFor: 'handling - declaration' stamp: 'monty 6/24/2016 23:32'!
parameterEntityDeclaration: aName replacement: aReplacement
	self declarationHandler
		parameterEntityDeclaration: aName
		replacement: aReplacement! !

!SAX2Parser methodsFor: 'parsing' stamp: 'monty 8/14/2016 17:53'!
parsingResult
	^ parsingResult! !

!SAX2Parser methodsFor: 'parsing' stamp: 'monty 8/14/2016 17:53'!
parsingResult: anObject
	parsingResult := anObject! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
processingInstruction: aTargetString data: aDataString
	self contentHandler
		processingInstruction: aTargetString
		data: aDataString! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
skippedEntity: anEntityName
	self contentHandler skippedEntity: anEntityName! !

!SAX2Parser methodsFor: 'handling - lexical' stamp: 'monty 6/24/2016 23:32'!
startCData
	self lexicalHandler startCData! !

!SAX2Parser methodsFor: 'handling - lexical' stamp: 'monty 6/24/2016 23:32'!
startDTD: aRootElementName publicID: aPublicID systemID: aSystemID
	self lexicalHandler
		startDTD: aRootElementName
		publicID: aPublicID
		systemID: aSystemID! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
startDocument
	self contentHandler startDocument! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	self contentHandler
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aURI
		localName: aLocalName
		attributes: aDictionary! !

!SAX2Parser methodsFor: 'handling - lexical' stamp: 'monty 6/24/2016 23:32'!
startEntity: anEntityName
	self lexicalHandler startEntity: anEntityName! !

!SAX2Parser methodsFor: 'handling - content' stamp: 'monty 6/24/2016 23:32'!
startPrefixMapping: aPrefix uri: aURI
	self contentHandler
		startPrefixMapping: aPrefix
		uri: aURI! !

!SAX2Parser methodsFor: 'handling - lexical' stamp: 'monty 6/24/2016 23:32'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self lexicalHandler
		xmlVersion: aVersionFloat
		encoding: anEncoding
		standalone: aBoolean! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 3/12/2016 14:48'!
on: aStringOrStream
	"Returns a new parser on the string or stream argument but doesn't parse it.

	Can be used to configure a parser with 'configuring' messages before
	parsing with parseDocument."

	^ self new on: aStringOrStream! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 3/12/2016 14:48'!
on: aStringOrStream documentURI: aURIString
	"Returns a new parser on the string or stream argument treating the
	specified URI as the base URI of the document but doesn't parse it.

	Can be used to configure a parser with 'configuring' messages before
	parsing with parseDocument."

	^ (self on: aStringOrStream) documentURI: aURIString! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 3/13/2016 10:59'!
onFileNamed: aFileName
	"Returns a new parser on the specified file but doesn't parse it.
	If aFileName is a relative path, it will be resolved in the context of
	the current working directory.

	Can be used to configure file parsers with 'configuring' messages before
	parsing with parseDocument."

	^ self
		on:
			(aFileName asXMLFileHandle
				streamGetUpToLimit: nil
				decoding: false)
		documentURI: aFileName! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 3/12/2016 14:52'!
onURL: aURLString
	"Fetches the URL and returns new parser on the response content but doesn't
	parse it. If the URL has no scheme, it will be treated as a file path.

	Can be used to configure URL parsers with 'configuring' messages before
	parsing with parseDocument."

	^ self
		onURL: aURLString
		upToLimit: XMLParserLimits defaultDocumentReadLimit! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 3/14/2016 00:54'!
onURL: aURLString upToLimit: aMaxSize
	"Fetches a URL and returns new parser on the response content up to the
	input limit (use nil to disable) but doesn't parse it. If the URL has
	no scheme, it will be treated as a file path.

	Can be used to configure limited URL parsers with 'configuring' messages
	before parsing with parseDocument."

	^ (self
		on:
			(XMLDecodingReadStreamAdapter on:
				(aURLString asXMLURI
					streamGetUpToLimit: aMaxSize
					"disable decoding so we can do our own looking at byte order
					marks and <?xml ...?> encoding attributes"
					decoding: false))
		documentURI: aURLString) documentReadLimit: aMaxSize! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 3/11/2016 14:52'!
parse: aStringOrStream
	"Parses the document in the string or stream argument"

	^ (self on: aStringOrStream) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 3/11/2016 14:54'!
parse: aStringOrStream documentURI: aURIString
	"Parses the document in the string or stream argument treating the specified
	URI as the base URI of the document."

	^ (self
		on: aStringOrStream
		documentURI: aURIString) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 3/12/2016 15:07'!
parse: aStringOrStream usingNamespaces: aBoolean
	"Parses the document in the string or stream argument possibly with namespace
	checking enabled (default)"

	^ (self on: aStringOrStream)
		usesNamespaces: aBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 3/12/2016 15:08'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean
	"Parses the document in the string or stream argument possibly with namespace
	checking (default) and validation (default) enabled."

	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 3/12/2016 15:10'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean externalEntities: anEntityResolutionBoolean
	"Parses the document in the string or stream argument possibly with namespace
	checking (default), validation (default), or external entity resolution (not
	default) enabled."

	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		resolvesExternalEntities: anEntityResolutionBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'deprecated' stamp: 'monty 4/18/2015 03:20'!
parseDocumentFrom: aStringOrStream
	self deprecated: 'Use #parse: instead'.

	^ self parse: aStringOrStream.! !

!SAXHandler class methodsFor: 'deprecated' stamp: 'monty 3/12/2016 14:51'!
parseDocumentFromFileNamed: aFileName
	self deprecated: 'Use #parseFileNamed: instead'.

	^ self parseFileNamed: aFileName.! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 3/12/2016 14:57'!
parseFileNamed: aFileNameString
	"Parses the specified file. If aFileName is a relative path, it will be
	resolved in the context of the current working directory."

	^ (self onFileNamed: aFileNameString) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 3/12/2016 14:53'!
parseURL: aURLString
	"Fetches the URL and parses the response content. If the URL has no scheme,
	it will be treated as a file path."

	^ (self onURL: aURLString) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 3/12/2016 22:25'!
parseURL: aURLString upToLimit: aMaxSize
	"Fetches the URL upto the input limit (nil to disable) and parses the
	response content. If the URL has no scheme, it will be treated as a file
	path."

	^ (self
		onURL: aURLString
		upToLimit: aMaxSize) parseDocument! !

!SAXHandler methodsFor: 'handling - declaration' stamp: 'monty 6/7/2015 19:38'!
attributeDeclaration: anElementName name: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/14/2016 00:47'!
characters: aString
	"Handles character data in an element, such as the 'test' in
	'<characters>test</characters>' or in the CDATA section '<!![CDATA[test]]>'.
	
	It may be sent multiple times for different sections of character data
	in a single element if they're separated by other nodes or entity
	references like in '<characters>one<!!--comment-->two</characters>'.
	
	See isInCDataSection and ignorableWhitespace:"! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 3/11/2016 14:43'!
comment: aString
	"Handles '<!!--comment-->' sections in documents."! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:14'!
configuration
	^ configuration! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 12/3/2014 11:59'!
configuration: aConfiguration
	(configuration := aConfiguration) parserHandlerClass: self class! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/23/2010 15:35'!
configurationClass
	^ XMLConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'monty 12/27/2013 03:36'!
currentScope: aNamespaceScope! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 5/8/2015 19:04'!
decodesCharacters
	^ self configuration decodesCharacters! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 8/16/2016 23:38'!
decodesCharacters: aBoolean
	"When enabled, character decoding will be tried if the input is binary, if
	it starts with a byte order mark (BOM) sequence, if the input looks like
	undecoded UTF-16 or UTF-32 (because of null chars before or after the first
	non-null ASCII char), or if the input is an XMLDecodingReadStreamAdapter,
	and when decoding is being done and an <?xml ...?> declarations is parsed,
	its encoding attribute will be obeyed. (default)"

	self configuration decodesCharacters: aBoolean! !

!SAXHandler methodsFor: 'private' stamp: 'monty 3/20/2016 20:38'!
defaultDocumentURI
	^ ''! !

!SAXHandler methodsFor: 'private' stamp: 'monty 3/8/2015 09:29'!
doctypeDefinition: aDoctypeDefinition! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:32'!
documentReadLimit
	^ self configuration documentReadLimit! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 4/14/2016 03:29'!
documentReadLimit: anInteger
	"change the document read limit (maximum number of chars that can be read)
	or nil to disable"

	self configuration documentReadLimit: anInteger! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 5/26/2015 01:42'!
documentURI
	"Returns current document base URI string"
	^ documentURI ifNil: [documentURI := self defaultDocumentURI]! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 5/23/2015 03:03'!
documentURI: aURIString
	documentURI := aURIString! !

!SAXHandler methodsFor: 'private' stamp: 'monty 2/9/2014 02:43'!
driverClass
	^ SAXParserDriver! !

!SAXHandler methodsFor: 'handling - declaration' stamp: 'monty 6/7/2015 19:39'!
elementDeclaration: anElementName contentModel: aContentModel
! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 3/11/2016 14:44'!
endCData
	"Sent after the character data of a '<!![CDATA[...]]>' section."! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 11/12/2013 18:01'!
endDTD
! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/11/2016 23:35'!
endDocument
	"Handles the end of an XML document. No more handlers are invoked after this."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/11/2016 15:01'!
endElement: aQualifiedName
	"Handles the end of an element if a subclass doesn't implement
	endElement:prefix:uri:localName: or endElement:prefix:uri:."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/11/2016 15:00'!
endElement: aQualifiedName prefix: aPrefix uri: aURI
	"Handles the end of an element if a subclass doesn't implement
	endElement:prefix:uri:localName:. By default it forwards to a simpler
	handler, endElement:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string"

	self endElement: aQualifiedName! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/11/2016 14:37'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	"Handles the end of an element, either an end tag like '</end>' or a
	self-closed tag like '<empty/>'. By default it forwards to a simpler handler,
	endElement:prefix:uri:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aLocalName - the local part of the qualified name"

	self
		endElement: aQualifiedName
		prefix: aPrefix
		uri:  aURI! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 3/27/2016 02:30'!
endEntity: anEntityName
	"Handles the end of some general entity's replacement in content.
	Not sent if the entity is undeclared or if content entity replacement
	is disabled."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/14/2016 00:35'!
endPrefixMapping: aPrefix
	"Handles the end of a namespace prefix/URI mapping when namespaces are
	enabled. After, aPrefix (empty for the default namespace) will be either
	unmapped or mapped to a different URI. The 'xml' prefix is predefined
	and never unmapped."! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/8/2013 02:33'!
externalEntityResolver
	^ self configuration externalEntityResolver! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/29/2013 01:55'!
externalEntityResolver: anExternalEntityResolver
	"This can be used to set a different DTDExternalEntityResolver"

	self configuration externalEntityResolver: anExternalEntityResolver! !

!SAXHandler methodsFor: 'handling - declaration' stamp: 'monty 6/7/2015 19:32'!
generalEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotationName! !

!SAXHandler methodsFor: 'handling - declaration' stamp: 'monty 11/14/2013 04:02'!
generalEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 4/19/2016 08:59'!
ignorableWhitespace: aString
	"When validation is enabled and a DTD with element declarations is
	present, this can be used to handle ignorable whitespace in elements
	with 'element content' (meaning they can only contain other elements
	and no PCDATA), such as between '<element-content>' and '<mixed-content>'
	or between '</mixed-content>' and '</element-content>' in this sample:
		'<!!DOCTYPE element-content [
			<!!--this has element content, so its whitespace is ignorable-->
			<!!ELEMENT element-content (mixed-content)>
			<!!--this has mixed content, so its whitespace is not ignorable-->
			<!!ELEMENT mixed-content (#PCDATA|any-content)*>
			<!!--this has any content, so its whitespace is not ignorable-->
			<!!ELEMENT any-content ANY>
		]>
		<element-content>
			<mixed-content>
				<any-content> any </any-content>
			</mixed-content>
		</element-content>'

	Not sent if validation is disabled or if there's no DTD with element
	declarations."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/22/2014 03:19'!
input
	^ input! !

!SAXHandler methodsFor: 'private' stamp: 'monty 8/21/2016 00:20'!
input: aStringOrStream
	"here for backwards compatibility; use #on: instead"

	input := aStringOrStream! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:17'!
interruptParsing
	"Can be sent within a handler to stop parsing;
	will be silently caught within parseDocument or other parsing methods"

	XMLParsingInterruptException signal! !

!SAXHandler methodsFor: 'testing' stamp: 'monty 3/14/2016 00:36'!
isInCDataSection
	"Can be used by characters: handlers to distinguish ordinary character
	data from characters within '<!![CDATA[...]]>' sections"

	^ isInCDataSection! !

!SAXHandler methodsFor: 'private' stamp: 'monty 3/11/2016 19:59'!
isInCDataSection: aBoolean
	isInCDataSection := aBoolean! !

!SAXHandler methodsFor: 'testing' stamp: 'monty 3/11/2016 20:08'!
isInContent
	"True if parsing has reached the first element of the document."

	^ isInContent! !

!SAXHandler methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:20'!
isValidating
	^ self configuration isValidating! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 7/18/2016 08:28'!
isValidating: aBoolean
	"If true and the document has a schema (like a DTD with ELEMENT and ATTLIST
	declarations) the document will be validated against it. Otherwise only
	entity references and xml:id attributes will be validated.
	
	(This is unlike the standard which requires validating parsers to reject
	all documents without DTDs or other schemas. To get that behavior,
	use requiresDocumentConstraints:)

	Enabling disables preservesUndeclaredEntityReferences.
	Disabling disables requiresDocumentConstraints."

	self configuration isValidating: aBoolean! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:33'!
maxEntityReplacementDepth
	^ self configuration maxEntityReplacementDepth! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 4/14/2016 03:30'!
maxEntityReplacementDepth: anInteger
	"change the max replacement depth or nil to disable"

	self configuration maxEntityReplacementDepth: anInteger! !

!SAXHandler methodsFor: 'handling - declaration' stamp: 'monty 3/21/2015 22:36'!
notationDeclaration: aName publicID: aPublicID systemID: aSystemID
! !

!SAXHandler methodsFor: 'initialization' stamp: 'monty 6/19/2015 20:26'!
on: aStringOrStream
	input := aStringOrStream.

	"a single instance can parse multiple documents with on:,
	so all these values must be reset"
	documentURI := nil.
	isInContent := false.
	isInCDataSection := false.
	configuration
		ifNil: [self configuration: self configurationClass new]
		ifNotNil: [self configuration: configuration copy].! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/11/2016 13:59'!
optimizeForLargeDocuments
	"use this for faster parsing/less memory use with large documents"

	self
		isValidating: false;
		usesNamespaces: false;
		documentReadLimit: nil! !

!SAXHandler methodsFor: 'handling - declaration' stamp: 'monty 3/21/2015 22:36'!
parameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID! !

!SAXHandler methodsFor: 'handling - declaration' stamp: 'monty 11/14/2013 04:05'!
parameterEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:00'!
parseDocument
	"parser the entire input document"

	self parser parseDocument.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:27'!
parseDocumentUntil: aBlock
	"parse the input document until aBlock is true"

	self parser parseDocumentUntil: aBlock.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:01'!
parseDocumentWhile: aBlock
	"parse the input document while aBlock is true"

	self parser parseDocumentWhile: aBlock.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/3/2014 12:19'!
parser
	"returns a new XMLParser object for this handler"

	^ self parserClass
		driver: (self driverClass saxHandler: self)
		on: self input! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/3/2014 00:18'!
parserClass
	^ XMLParser! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/4/2014 14:07'!
parsingResult
	"value returned by parseDocument and other parsing methods"

	^ self! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/6/2015 04:12'!
preservesUndeclaredEntityReferences
	^ self configuration preservesUndeclaredEntityReferences! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/13/2016 08:47'!
preservesUndeclaredEntityReferences: aBoolean
	"If true, references to undeclared entities in content are
	preserved (not default).
	Disables validation.
	
	(See also replacesContentEntityReferences:, which is usually more appropriate.)"

	self configuration preservesUndeclaredEntityReferences: aBoolean! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/14/2016 00:37'!
processingInstruction: aTargetString data: aDataString
	"Handles processing instructions like '<?name value?>', where 'name' is
	the target and 'value' is the data.
	
	XML declarations may look like PIs but are different and are handled
	with xmlVersion:encoding:standalone: instead."! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 3/17/2015 17:38'!
removeLimits
	"remove all input security limits (unsafe)"

	self configuration removeLimits! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/3/2015 23:55'!
replacesContentEntityReferences
	^ self configuration replacesContentEntityReferences! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/6/2015 22:31'!
replacesContentEntityReferences: aBoolean
	"Replace general parsed entity and character references
	in content (default)"

	self configuration replacesContentEntityReferences: aBoolean! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/21/2013 20:07'!
requiresDocumentConstraints
	^ self configuration requiresDocumentConstraints! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/29/2013 03:23'!
requiresDocumentConstraints: aBoolean
	"If true, constraints on elements and attributes (in a DTD)
	will be required (not default).
	Enables validation."

	self configuration requiresDocumentConstraints: aBoolean! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 11/16/2013 07:33'!
resolvesExternalEntities
	^ self configuration resolvesExternalEntities! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 1/24/2014 23:52'!
resolvesExternalEntities: aBoolean
	"When true, external DTDs and parsed entities will be
	fetched (not default)."

	self configuration resolvesExternalEntities: aBoolean! !

!SAXHandler methodsFor: 'private' stamp: 'monty 3/18/2016 01:11'!
schema
	^ nil! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/14/2016 00:39'!
skippedEntity: anEntityName
	"Sent when a general parsed entity reference like '&name;' is
	unreplaced in content. Only sent when validation is turned off
	or entity replacement is disabled."! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 3/11/2016 14:44'!
startCData
	"Sent before the character data of a '<!![CDATA[...]]>' section."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 3/24/2015 20:25'!
startContent
	isInContent := true! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 3/14/2016 00:40'!
startDTD: aRootElementName publicID: aPublicID systemID: aSystemID
	"Handles the start of a '<!!DOCTYPE ...>' declaration. Any events
	after this and until endDTD is sent are part of the internal subset
	of the DTD.
	
	aRootElementName - the expected name of the document's root element
	aPublicID - the public ID of the external DTD subset or an empty string
	aSystemID - the URI where the external subset can be found or an empty string"! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/11/2016 23:33'!
startDocument
	"Handles the start of an XML document, the first handler invoked."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/11/2016 15:00'!
startElement: aQualifiedName attributes: aDictionary
	"Handles the start of an element if a subclass doesn't implement
	startElement:prefix:uri:localName:attributes: or
	startElement:prefix:uri:attributes:.

	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aDictionary - an OrderPreservingStringDictionary of attributes"! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/11/2016 15:00'!
startElement: aQualifiedName prefix: aPrefix uri: aURI attributes: aDictionary
	"Handles the start of an element if a subclass doesn't implement
	startElement:prefix:uri:localName:attributes:. By default it forwards
	to a simpler handler, startElement:attributes:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aDictionary - an OrderPreservingStringDictionary of attributes"

	self
		startElement: aQualifiedName
		attributes: aDictionary! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/11/2016 14:30'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	"Handles the start of an element, either a start tag like '<start>' or a
	self-closed tag like '<empty/>'. By default it forwards to a simpler handler,
	startElement:prefix:uri:attributes:.
	
	aQualifiedName - the full name, including any prefix, such as 'prefix:name'
	aPrefix - the prefix of the qualified name or an empty string
	aURI - the namespace URI of the element or an empty string
	aLocalName - the local part of the qualified name
	aDictionary - an OrderPreservingStringDictionary of attributes"

	self
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aURI
		attributes: aDictionary! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 3/27/2016 02:30'!
startEntity: anEntityName
	"Handles the start of some general entity's replacement in content.
	Not sent if the entity is undeclared or if content entity replacement
	is disabled."! !

!SAXHandler methodsFor: 'handling - content' stamp: 'monty 3/14/2016 00:31'!
startPrefixMapping: aPrefix uri: aURI
	"Handles the start of a namespace prefix/URI mapping when namespaces are
	enabled. The 'xml' prefix is predefined and never remapped.

	aPrefix - the prefix of the namespace (empty for the default namesapce)
	aURI - the namespace URI"! !

!SAXHandler methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:20'!
usesNamespaces
	^ self configuration usesNamespaces! !

!SAXHandler methodsFor: 'configuring' stamp: 'monty 12/29/2013 03:03'!
usesNamespaces: aBoolean
	"Namespace checking when true (default)"

	self configuration usesNamespaces: aBoolean! !

!SAXHandler methodsFor: 'handling - lexical' stamp: 'monty 3/14/2016 00:52'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	"Handles the '<?xml ...?>' declaration of a document.
	
	aVersionFloat - the version of XML the document uses, usually 1.0
	anEncoding - the name of the encoding or an empty string
	aBoolean - true if standalone='yes'"! !

SAXHandler subclass: #XMLDOMParser
	instanceVariableNames: 'document nodeStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMParser commentStamp: 'monty 3/14/2016 00:58' prior: 0!
This class is an XML parser that parses XML into a tree of nodes with an XMLDocument node as the root:
	document := XMLDOMParser parse: xmlStringOrStream.

See the XML-Parser-DOM category for info on the node classes.

By default, XML comments are ignored during parsing and "<!![CDATA[...]]>" sections are merged with adjacent character data as XMLString nodes. Use #preservesCommentNodes: or #preservesCDataNodes: before parsing to change this. You can also use #preservesIgnorableWhitespace: to preserve ignorable whitespace as XMLString nodes.

To control what node classes the parser uses to construct the tree, inject a custom node factory with #nodeFactory: prior to parsing. See XMLNodeFactory.

See the superclass for more info.!

!XMLDOMParser methodsFor: 'handling - declaration' stamp: 'monty 6/7/2015 19:38'!
attributeDeclaration: anElementName name: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self topNode addNode:
		(self nodeFactory attributeDeclarationClass
			element: anElementName
			attribute: anAttributeName
			type: aType
			defaultPragma: aDefaultPragma
			defaultValue: aDefaultValue)! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 8/14/2016 21:26'!
characters: aString
	(self isInCDataSection
		and: [self preservesCDataNodes])
		ifTrue: [self topNode addNode: (self nodeFactory newCData: aString)]
		ifFalse: [self topNode addString: aString]! !

!XMLDOMParser methodsFor: 'handling - lexical' stamp: 'monty 3/11/2016 14:17'!
comment: aString
	self preservesCommentNodes
		ifTrue: [self topNode addNode: (self nodeFactory newComment: aString)]! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/23/2010 14:37'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 12/27/2013 03:43'!
currentScope: aNamespaceScope
	self topNode setNamespaceScope: aNamespaceScope! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 3/8/2015 10:12'!
doctypeDefinition: aDoctypeDefinition
	self document doctypeDefinition: aDoctypeDefinition! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 3/9/2010 01:38'!
document
	^ document! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'monty 11/6/2014 14:07'!
document: aDocument
	document := aDocument! !

!XMLDOMParser methodsFor: 'handling - declaration' stamp: 'monty 6/7/2015 19:39'!
elementDeclaration: anElementName contentModel: aContentModel
	self topNode addNode:
		(self nodeFactory elementDeclarationClass
			element: anElementName
			contentModel: aContentModel)! !

!XMLDOMParser methodsFor: 'handling - lexical' stamp: 'monty 11/12/2013 23:00'!
endDTD
	self popNode! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:36'!
endElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName
	self popNode! !

!XMLDOMParser methodsFor: 'handling - declaration' stamp: 'monty 6/7/2015 19:32'!
generalEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotationName
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			publicID: aPublicID
			systemID: aSystemID
			ndata: aNotationName)! !

!XMLDOMParser methodsFor: 'handling - declaration' stamp: 'monty 12/21/2013 03:02'!
generalEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 1/3/2015 00:05'!
ignorableWhitespace: aString
	self preservesIgnorableWhitespace
		ifTrue: [self characters: aString]! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLDOMParser methodsFor: 'deprecated' stamp: 'monty 4/17/2015 10:55'!
nodeListClass
	"will be deprecated"
	^ self configuration nodeListClass! !

!XMLDOMParser methodsFor: 'deprecated' stamp: 'monty 4/17/2015 10:55'!
nodeListClass: aClass
	"will be deprecated"
	self configuration nodeListClass: aClass! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 10/27/2014 09:12'!
nodeStack
	^ nodeStack! !

!XMLDOMParser methodsFor: 'handling - declaration' stamp: 'monty 3/21/2015 22:36'!
notationDeclaration: aName publicID: aPublicID systemID: aSystemID
	self topNode addNode:
		(self nodeFactory notationDeclarationClass
			name: aName
			publicID: aPublicID
			systemID: aSystemID)! !

!XMLDOMParser methodsFor: 'initialization' stamp: 'monty 4/7/2015 05:08'!
on: aStringOrStream
	super on: aStringOrStream.

	nodeStack := OrderedCollection new: 10.! !

!XMLDOMParser methodsFor: 'handling - declaration' stamp: 'monty 3/21/2015 22:36'!
parameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			publicID: aPublicID
			systemID: aSystemID)! !

!XMLDOMParser methodsFor: 'handling - declaration' stamp: 'monty 12/21/2013 03:02'!
parameterEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 5/10/2015 03:29'!
parser
	| parser |

	parser := super parser.
	self document
		ifNotNil: [self setupInnerXMLDocumentForParser: parser].
	self nodeStack isEmpty
		ifFalse: [self setupInnerXMLNodesForParser: parser].
	^ parser.! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:38'!
parsingResult
	^ self document! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:31'!
popNode
	^ self nodeStack removeLast! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCDataNodes
	^ self configuration preservesCDataNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 4/11/2015 03:13'!
preservesCDataNodes: aBoolean
	"if enabled, <!![CDATA[...]]> sections will be preserved as CDATA nodes
	and not ordinary string nodes and won't be merged with adjacent string
	nodes (not default)"
	self configuration preservesCDataNodes: aBoolean! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes
	^ self configuration preservesCommentNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 4/10/2015 21:54'!
preservesCommentNodes: aBoolean
	"if enabled, comments will be preserved as comment nodes (not default)"
	self configuration preservesCommentNodes: aBoolean! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 1/3/2015 00:05'!
preservesIgnorableWhitespace
	^ self configuration preservesIgnorableWhitespace! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 4/20/2016 13:48'!
preservesIgnorableWhitespace: aBoolean
	"If enabled, ignorable whitespace (identified by DTD element declarations
	with element content) will be preserved as string nodes. Enables validation.
	(Not default)"

	self configuration preservesIgnorableWhitespace: aBoolean! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 3/21/2015 22:51'!
processingInstruction: aTargetString data: aDataString
	self topNode addNode:
		(self nodeFactory newPI
			target: aTargetString;
			data: aDataString)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 2/4/2014 01:33'!
pushElement: anElement
	self topNode addNode: anElement.
	^ self pushNode: anElement.! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 7/11/2016 23:20'!
pushInnerXMLRootNode: aRootNode
	self
		configuration: aRootNode configuration;
		document: aRootNode document;
		pushNode: aRootNode! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:25'!
pushNode: aNode
	^ self nodeStack addLast: aNode
! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 5/10/2015 03:29'!
setupInnerXMLDocumentForParser: aParser
	self document hasDoctypeDefinition
		ifTrue: [
			aParser driver doctypeDefinition addGeneralEntitiesFrom:
				self document doctypeDefinition]! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 5/10/2015 03:29'!
setupInnerXMLNodesForParser: aParser
	self nodeStack do: [:each |
		each isElement
			ifTrue: [
				self
					startInnerXMLElement: each
					forParser: aParser]].
	aParser context: self topNode innerXMLContext.! !

!XMLDOMParser methodsFor: 'handling - lexical' stamp: 'monty 3/21/2015 22:36'!
startDTD: aRootElementName publicID: aPublicID systemID: aSystemID
	| doctypeDeclaration |

	doctypeDeclaration := 
		self nodeFactory doctypeDeclarationClass
			root: aRootElementName
			publicID: aPublicID
			systemID: aSystemID.
	self pushNode: doctypeDeclaration.
	self document doctypeDeclaration: doctypeDeclaration.! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 11/6/2014 14:07'!
startDocument
	self topNode ifNotNil: [^ self].

	self
		document: self nodeFactory newDocument;
		pushNode: self document.

	self document configuration: self configuration.! !

!XMLDOMParser methodsFor: 'handling - content' stamp: 'monty 8/8/2016 23:59'!
startElement: aQualifiedName prefix: aPrefix uri: aURI localName: aLocalName attributes: aDictionary
	self pushElement:
		((self nodeFactory
			classForElement: aQualifiedName
			namespaceURI: aURI
			localName: aLocalName
			attributes: aDictionary) new
				setQualifiedName: aQualifiedName
				localName: aLocalName
				attributes: aDictionary)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 11/15/2014 04:07'!
startInnerXMLElement: anElement forParser: aParser
	aParser driver
		setRootElement: anElement name
		currentScope: anElement namespaceScope! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 4/1/2016 00:56'!
topNode
	^ self nodeStack xmlLastOrNil! !

!XMLDOMParser methodsFor: 'handling - lexical' stamp: 'monty 11/19/2013 05:37'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self document ifNotNil: [
		self document
			version: aVersionFloat;
			encoding: anEncoding;
			isStandalone: aBoolean]! !

Object subclass: #XMLAbstractFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Utils'!

XMLAbstractFactory class
	instanceVariableNames: 'implementation'!

!XMLAbstractFactory commentStamp: 'monty 3/5/2016 11:50' prior: 0!
An abstract class for dynamically choosing a supported concrete implementation from a class hierarchy for this platform.

Each root subclass needs to override preferredImplementation to return the preferred subclass and errorNoSupportedImplementation to signal an error if none are supported, and then each subclass of the root needs to define isSupportedImplementation to return true if it supports this platform.!

XMLAbstractFactory class
	instanceVariableNames: 'implementation'!

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 3/4/2016 16:37'!
clearAllImplementations
	"self clearImplementations"

	self implementation: nil.
	self allSubclassesDo: [:each | each implementation: nil]! !

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 3/4/2016 16:27'!
errorNoSupportedImplementation
	self subclassResponsibility! !

!XMLAbstractFactory class methodsFor: 'private' stamp: 'monty 4/8/2016 22:31'!
findSupportedImplementation
	self isSupportedImplementation
		ifTrue: [^ self].

	self preferredImplementation isSupportedImplementation
		ifTrue: [^ self preferredImplementation].
	self allSubclassesDo: [:each |
		each isSupportedImplementation
			ifTrue: [^ each]].

	^ nil.! !

!XMLAbstractFactory class methodsFor: 'testing' stamp: 'monty 3/4/2016 16:30'!
hasSupportedImplementation
	^ self findSupportedImplementation notNil! !

!XMLAbstractFactory class methodsFor: 'accessing' stamp: 'monty 3/4/2016 16:30'!
implementation
	^ implementation
		ifNil: [
			(implementation := self findSupportedImplementation)
				ifNil: [self errorNoSupportedImplementation].
			implementation]! !

!XMLAbstractFactory class methodsFor: 'accessing' stamp: 'monty 12/18/2014 19:07'!
implementation: aClass
	implementation := aClass! !

!XMLAbstractFactory class methodsFor: 'class initialization' stamp: 'monty 3/4/2016 16:37'!
initialize
	"self initialize"

	self clearAllImplementations! !

XMLAbstractFactory initialize!

!XMLAbstractFactory class methodsFor: 'testing' stamp: 'monty 3/4/2016 16:23'!
isSupportedImplementation
	^ false! !

!XMLAbstractFactory class methodsFor: 'instance creation' stamp: 'monty 11/12/2014 18:24'!
new
	^ self implementation basicNew initialize! !

!XMLAbstractFactory class methodsFor: 'accessing' stamp: 'monty 11/12/2014 18:22'!
preferredImplementation
	self subclassResponsibility! !

XMLAbstractFactory subclass: #XMLFileHandle
	instanceVariableNames: 'path'
	classVariableNames: 'XMLDocumentFileExtensions'
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLFileHandle commentStamp: 'monty 3/13/2016 23:21' prior: 0!
This is an abstract class for file handles to read/write/delete files. Subclasses need to implement the abstract methods, and then instances of the correct concrete class for this platform can be created by sending #asXMLFileHandle to a string.

The #readStream/#writeStream messages return file streams that do decoding/encoding, while the #rawReadStream/#rawWriteStream streams don't.

The #get/#getUpToLimit:decoding: and #streamGet/#streamGetUpToLimit:decoding: interface of XMLURI is supported.!

XMLFileHandle subclass: #XMLFileDirectoryFileHandle
	instanceVariableNames: 'fileDirectory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLFileDirectoryFileHandle commentStamp: 'monty 3/13/2016 23:22' prior: 0!
This class uses FileDirectory/DirectoryEntry to implement file IO. This is used on Squeak and Gemstone, which don't support the Pharo FileSystem API.!

!XMLFileDirectoryFileHandle class methodsFor: 'private' stamp: 'monty 7/6/2016 02:35'!
basicCurrentWorkingDirectory
	^ (XMLClassFinder classNamed: #FileDirectory) default pathName! !

!XMLFileDirectoryFileHandle class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:40'!
isSupportedImplementation
	^ XMLClassFinder hasClassNamed: #FileDirectory! !

!XMLFileDirectoryFileHandle methodsFor: 'private' stamp: 'monty 3/13/2016 21:33'!
basicDelete
	"FileDirectory's deleteFileNamed: is used because DirectoryEntry
	doesn't understand #delete on GS"
	self fileDirectory deleteFileNamed: self path! !

!XMLFileDirectoryFileHandle methodsFor: 'accessing' stamp: 'monty 3/13/2016 23:11'!
directoryEntry
	"a bug on Squeak (but not GS) with the handling of dot relative paths
	like './file.xml' requires fully qualifying the path before creating
	the DirectoryEntry"
	^ self fileDirectory directoryEntryFor:
		(self fileDirectory fullPathFor: self path)! !

!XMLFileDirectoryFileHandle methodsFor: 'testing' stamp: 'monty 3/13/2016 00:31'!
exists
	^ self directoryEntry notNil! !

!XMLFileDirectoryFileHandle methodsFor: 'accessing' stamp: 'monty 7/6/2016 02:46'!
fileDirectory
	^ fileDirectory
		ifNil: [fileDirectory := (XMLClassFinder classNamed: #FileDirectory) default]! !

!XMLFileDirectoryFileHandle methodsFor: 'accessing' stamp: 'monty 3/13/2016 21:35'!
fileSize
	^ self exists
		ifTrue: [self directoryEntry fileSize]
		ifFalse: [0]! !

!XMLFileDirectoryFileHandle methodsFor: 'testing' stamp: 'monty 3/13/2016 00:32'!
isFile
	^ self exists
		and: [self directoryEntry isDirectory not]! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 3/13/2016 21:17'!
basicCurrentWorkingDirectory
	self subclassResponsibility! !

!XMLFileHandle class methodsFor: 'accessing' stamp: 'monty 3/13/2016 23:21'!
currentWorkingDirectory
	"returns a path string for the current working directory"

	^ self implementation basicCurrentWorkingDirectory! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 3/4/2016 16:27'!
errorNoSupportedImplementation
	XMLFileException signal: 'File IO unsupported'! !

!XMLFileHandle class methodsFor: 'class initialization' stamp: 'monty 6/22/2016 00:06'!
initialize
	"self initialize"

	"a class variable so it's shared by all subclasses"
	XMLDocumentFileExtensions :=
		Set withAll: "avoid newFrom: for GS portability"
			#('atom' "Atom Syndication Format"
				'dbk' "DocBook XML"
				'docm' "Office Open XML Document"
				'docx' "Office Open XML Document"
				'pptm' "Office Open XML Presentation"
				'pptx' "Office Open XML Presentation"
				'rdf' "Resource Description Framework"
				'rss' "Rich Site Summary"
				'svg' "Scalable Vector Graphics"
				'xbel' "XML Bookmark Exchange Language"
				'xfd' "Extensible Forms Description Language"
				'xfdl' "Extensible Forms Description Language"
				'xhtml'
				'xlsm' "Office Open XML Workbook"
				'xlsx' "Office Open XML Workbook"
				'xml'
				'xsd' "XML Schema Definition"
				'xspf' "XML Shareable Playlist Format"
				'xul') "XML User Interface Language"! !

XMLFileHandle initialize!

!XMLFileHandle class methodsFor: 'instance creation' stamp: 'monty 3/14/2016 00:07'!
path: aPathString
	^ self new path: aPathString! !

!XMLFileHandle class methodsFor: 'accessing' stamp: 'monty 10/25/2014 18:56'!
preferredImplementation
	^ XMLFileSystemFileHandle! !

!XMLFileHandle methodsFor: 'converting' stamp: 'monty 2/2/2016 17:36'!
asXMLFileHandle
	^ self! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 3/13/2016 21:33'!
basicDelete
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 3/13/2016 23:17'!
contents
	"the read stream adapters don't support #contents so #upToEnd is used instead"
	^ self readStreamDo: [:readStream | readStream upToEnd]! !

!XMLFileHandle methodsFor: 'deleting' stamp: 'monty 3/13/2016 22:39'!
delete
	self exists
		ifFalse: [self errorFileDoesNotExist].

	[self basicDelete]
		on: Error
		do: [:error | XMLFileException signalMessageFrom: error].! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 3/13/2016 22:39'!
errorFileDoesNotExist
	XMLFileException
		formatSignal: 'File does does not exist: {1}'
		with: self path! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 3/19/2016 19:11'!
errorFileExceedsMaxSize: aMaxSize
	XMLLimitException
		formatSignal: 'File exceeds max size of {1} bytes: {2}'
		with: aMaxSize
		with: self path! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 3/12/2016 19:28'!
errorNotAFile
	XMLFileException
		formatSignal: 'Not a file: {1}'
		with: self path! !

!XMLFileHandle methodsFor: 'testing' stamp: 'monty 3/13/2016 00:31'!
exists
	"returns true if the file exists"

	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 7/6/2016 06:31'!
extension
	"Returns the file extension, everything after the first '.' of the
	last path segment:
		'one.two/three.txt' asXMLFileHandle extension = 'txt'.
		'one.two/three.tar.gz' asXMLFileHandle extension = 'tar.gz'.
		'one.two/three.four/' asXMLFileHandle extension = ''.
	"
	| i |

	^ ((i := self path lastIndexOf: $/) > 0
		ifTrue: [self path copyFrom: i + 1 to: self path size]
		ifFalse: [self path]) copyAfter: $.! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/22/2014 19:07'!
fileSize
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'retrieving' stamp: 'monty 11/29/2015 22:40'!
get
	^ self
		getUpToLimit: nil
		decoding: true! !

!XMLFileHandle methodsFor: 'retrieving' stamp: 'monty 3/13/2016 23:17'!
getUpToLimit: aMaxSize decoding: aBoolean
	| stream contents |

	"can be safely implemented using streamGetUpToLimit:decoding: rather
	than the other way around like XMLHTTPRequest requires, because files
	are local and can be safely read incrementally, unlike an HTTP response
	which must be read immediately and stored entirely in memory"
	stream :=
		self
			streamGetUpToLimit: aMaxSize
			decoding: aBoolean.
	"the read stream adapters don't support #contents so #upToEnd is used instead"
	contents := stream upToEnd.
	stream close.

	^ contents.! !

!XMLFileHandle methodsFor: 'testing' stamp: 'monty 3/13/2016 00:31'!
isFile
	"returns true if the file exists and is a file (not a directory)"

	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'testing' stamp: 'monty 3/19/2016 16:40'!
isXMLDocumentFile
	"Returns true if the file has a known extension of an XML document format
	such as .xml, .xhtml, .xul, but not .dtd, .ent, or other XML-related
	formats that aren't proper XML documents."

	^ XMLDocumentFileExtensions includes: self extension asLowercase! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 3/13/2016 21:08'!
newFileReadStreamOnPath
	^ XMLFileReadStreamFactory onPath: self path! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 3/13/2016 21:08'!
newFileWriteStreamOnPath
	^ XMLFileWriteStreamFactory onPath: self path! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/9/2014 04:34'!
path
	^ path! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 3/14/2016 00:07'!
path: aPathString
	path := aPathString! !

!XMLFileHandle methodsFor: 'printing' stamp: 'monty 3/12/2016 19:28'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self path;
		nextPut: $).! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 3/13/2016 22:39'!
rawReadStream
	"returns a raw file read stream with no decoding"

	self exists
		ifFalse: [self errorFileDoesNotExist].
	self isFile
		ifFalse: [self errorNotAFile].
	
	^ [self newFileReadStreamOnPath]
		on: Error
		do: [:error | XMLFileException signalMessageFrom: error].! !

!XMLFileHandle methodsFor: 'writing' stamp: 'monty 3/13/2016 21:08'!
rawWriteStream
	"returns a raw file write stream with no encoding; truncates existing files"

	^ [self newFileWriteStreamOnPath]
		on: Error
		do: [:error | XMLFileException signalMessageFrom: error]! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 3/13/2016 11:01'!
readStream
	"returns a file read stream with decoding"

	| decodingReadStreamAdapter |
	
	decodingReadStreamAdapter :=
		XMLDecodingReadStreamAdapter on: self rawReadStream.
	(XMLFileStreamEncodingDetector on: decodingReadStreamAdapter)
			detectCurrentEncoding.
	^ decodingReadStreamAdapter.! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 2/12/2016 23:50'!
readStreamDo: aBlock
	| readStream |

	readStream := self readStream.
	^ [aBlock value: readStream]
		ensure: [readStream close].! !

!XMLFileHandle methodsFor: 'retrieving' stamp: 'monty 11/29/2015 22:40'!
streamGet
	^ self
		streamGetUpToLimit: nil
		decoding: true! !

!XMLFileHandle methodsFor: 'retrieving' stamp: 'monty 3/19/2016 15:51'!
streamGetUpToLimit: aMaxSize decoding: aBoolean
	(aMaxSize notNil
		and: [self fileSize > aMaxSize])
		ifTrue: [self errorFileExceedsMaxSize: aMaxSize].

	^ aBoolean
		ifTrue: [self readStream]
		ifFalse: [self rawReadStream].! !

!XMLFileHandle methodsFor: 'writing' stamp: 'monty 3/13/2016 20:55'!
writeStream
	^ XMLEncodingWriteStreamAdapter on: self rawWriteStream! !

!XMLFileHandle methodsFor: 'writing' stamp: 'monty 2/12/2016 23:50'!
writeStreamDo: aBlock
	| writeStream |

	writeStream := self writeStream.
	^ [aBlock value: writeStream]
		ensure: [writeStream close].! !

XMLFileHandle subclass: #XMLFileSystemFileHandle
	instanceVariableNames: 'fileReference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLFileSystemFileHandle commentStamp: 'monty 2/2/2016 00:15' prior: 0!
This class uses the Pharo FileSystem API to implement file IO. The preferred implementation.!

!XMLFileSystemFileHandle class methodsFor: 'private' stamp: 'monty 7/6/2016 02:35'!
basicCurrentWorkingDirectory
	^ String streamContents: [:stream |
		"the class-side workingDirectory isn't used for backwards compatibility"
		(XMLClassFinder classNamed: #FileSystem) disk workingDirectory
			path segments do: [:each |
				stream
					nextPut: $/;
					nextPutAll: each]].! !

!XMLFileSystemFileHandle class methodsFor: 'instance creation' stamp: 'monty 3/13/2016 13:11'!
fileReference: aFileReference
	^ self new fileReference: aFileReference! !

!XMLFileSystemFileHandle class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:40'!
isSupportedImplementation
	^ XMLClassFinder hasClassNamed: #FileSystem! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 3/13/2016 21:33'!
basicDelete
	self fileReference delete! !

!XMLFileSystemFileHandle methodsFor: 'testing' stamp: 'monty 3/13/2016 00:31'!
exists
	^ self fileReference exists! !

!XMLFileSystemFileHandle methodsFor: 'accessing' stamp: 'monty 3/12/2016 19:29'!
fileReference
	^ fileReference ifNil: [fileReference := self path asFileReference]! !

!XMLFileSystemFileHandle methodsFor: 'accessing' stamp: 'monty 3/12/2016 19:40'!
fileReference: aFileReference
	fileReference := aFileReference.
	self path: aFileReference fullName.! !

!XMLFileSystemFileHandle methodsFor: 'accessing' stamp: 'monty 1/22/2014 19:07'!
fileSize
	^ self fileReference size! !

!XMLFileSystemFileHandle methodsFor: 'testing' stamp: 'monty 1/13/2014 04:37'!
isFile
	^ self fileReference isFile! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 3/13/2016 23:10'!
newFileReadStreamOnPath
	"memory file references need special handling"
	self fileReference fileSystem isMemoryFileSystem
		ifTrue: [^ self fileReference readStream]
		ifFalse: [^ super newFileReadStreamOnPath]! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 3/13/2016 23:10'!
newFileWriteStreamOnPath
	"memory file references need special handling"
	self fileReference fileSystem isMemoryFileSystem
		ifTrue: [^ self fileReference writeStream]
		ifFalse: [^ super newFileWriteStreamOnPath]! !

XMLAbstractFactory subclass: #XMLFileReadStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLFileReadStreamFactory commentStamp: 'monty 3/14/2016 19:40' prior: 0!
An abstract factory for file read streams used by XMLFileHandles.

Subclasses should create read streams that return raw byte characters and not characters decoded from some encoding (like UTF-8), so that XMLParser can do its own decoding later looking at byte-order marks and the <?xml ...?> encoding attribute.!

!XMLFileReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 3/14/2016 00:07'!
basicOnPath: aPathString
	self subclassResponsibility! !

!XMLFileReadStreamFactory class methodsFor: 'private' stamp: 'monty 3/4/2016 16:27'!
errorNoSupportedImplementation
	XMLFileException signal: 'File reading unsupported'! !

!XMLFileReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 3/14/2016 00:22'!
onPath: aPathString
	^ self implementation basicOnPath: aPathString! !

!XMLFileReadStreamFactory class methodsFor: 'accessing' stamp: 'monty 11/20/2014 02:32'!
preferredImplementation
	^ XMLStandardFileStreamReadStreamFactory! !

XMLFileReadStreamFactory subclass: #XMLGsFileReadStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLGsFileReadStreamFactory commentStamp: 'monty 3/13/2016 23:23' prior: 0!
A GsFile read stream factory. This stream is preferred for Gemstone because it does no decoding and returns byte characters so XMLParser can do its own decoding.!

!XMLGsFileReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 7/6/2016 02:33'!
basicOnPath: aPathString
	^ (XMLClassFinder classNamed: #GsFile)
		open: aPathString
		mode: 'rb' "binary to avoid CRLF<->LF conversions on Windows"
		onClient: false "use server files by default"! !

!XMLGsFileReadStreamFactory class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:40'!
isSupportedImplementation
	^ XMLClassFinder hasClassNamed: #GsFile! !

XMLFileReadStreamFactory subclass: #XMLStandardFileStreamReadStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLStandardFileStreamReadStreamFactory commentStamp: 'monty 3/13/2016 23:23' prior: 0!
A StandardFileStream read stream factory. This stream is preferred for Squeak/Pharo because it does no decoding and returns byte characters so XMLParser can do its own decoding.!

!XMLStandardFileStreamReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 7/6/2016 02:33'!
basicOnPath: aPathString
	^ (XMLClassFinder classNamed: #StandardFileStream) readOnlyFileNamed: aPathString! !

!XMLStandardFileStreamReadStreamFactory class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:40'!
isSupportedImplementation
	^ XMLClassFinder hasClassNamed: #StandardFileStream! !

XMLAbstractFactory subclass: #XMLFileWriteStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLFileWriteStreamFactory commentStamp: 'monty 3/14/2016 19:40' prior: 0!
An abstract factory for file write streams used by XMLFileHandles.

Subclasses should create write streams that do no automatic encoding and accept raw byte characters, so XMLParser can do its own encoding.!

!XMLFileWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 3/14/2016 00:08'!
basicOnPath: aPathString
	self subclassResponsibility! !

!XMLFileWriteStreamFactory class methodsFor: 'private' stamp: 'monty 3/4/2016 16:27'!
errorNoSupportedImplementation
	XMLFileException signal: 'File writing unsupported'! !

!XMLFileWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 3/14/2016 00:22'!
onPath: aPathString
	^ self implementation basicOnPath: aPathString! !

!XMLFileWriteStreamFactory class methodsFor: 'accessing' stamp: 'monty 2/2/2016 00:08'!
preferredImplementation
	^ XMLStandardFileStreamWriteStreamFactory! !

XMLFileWriteStreamFactory subclass: #XMLGsFileWriteStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLGsFileWriteStreamFactory commentStamp: 'monty 3/13/2016 23:24' prior: 0!
A GsFile write stream factory. This stream is preferred for Gemstone because it does no encoding and accepts byte characters so XMLParser can do its own encoding.!

!XMLGsFileWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 7/6/2016 02:34'!
basicOnPath: aPathString
	^ (XMLClassFinder classNamed: #GsFile)
		open: aPathString
		mode: 'wb' "binary to avoid CRLF<->LF conversions on Windows"
		onClient: false "use server files by default"! !

!XMLGsFileWriteStreamFactory class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:40'!
isSupportedImplementation
	^ XMLClassFinder hasClassNamed: #GsFile! !

XMLFileWriteStreamFactory subclass: #XMLStandardFileStreamWriteStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Files'!

!XMLStandardFileStreamWriteStreamFactory commentStamp: 'monty 3/13/2016 23:24' prior: 0!
A StandardFileStream write stream factory. This stream is preferred for Pharo/Squeak because it does no encoding and accepts byte characters so XMLParser can do its own encoding.!

!XMLStandardFileStreamWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 7/6/2016 02:34'!
basicOnPath: aPathString
	"use forceNewFileNamed: to ensure truncation of existing files before writing"
	^ (XMLClassFinder classNamed: #StandardFileStream) forceNewFileNamed: aPathString! !

!XMLStandardFileStreamWriteStreamFactory class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:40'!
isSupportedImplementation
	^ XMLClassFinder hasClassNamed: #StandardFileStream! !

XMLAbstractFactory subclass: #XMLHTTPRequest
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLHTTPRequest commentStamp: 'monty 8/2/2016 08:10' prior: 0!
A base class for HTTP requests. Subclasses need to implement #basicGetUpToLimit:decoding: to return the HTTP response as a string or raise an exception if an HTTP/network error happens or the response is too large.

These classes implement the #get/#getUpToLimit:decoding: and #streamGet/#streamGetUpToLimit:decoding: interface of XMLURI.!

!XMLHTTPRequest class methodsFor: 'private' stamp: 'monty 3/14/2016 15:42'!
errorNoSupportedImplementation
	XMLHTTPException signal: 'HTTP unsupported; install Zinc or WebClient'! !

!XMLHTTPRequest class methodsFor: 'accessing' stamp: 'monty 10/25/2014 18:56'!
preferredImplementation
	^ XMLZincHTTPRequest! !

!XMLHTTPRequest class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
url: aURL
	^ self new url: aURL! !

!XMLHTTPRequest methodsFor: 'converting' stamp: 'monty 2/13/2016 00:35'!
asXMLHTTPRequest
	^ self! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 3/13/2016 23:40'!
basicGetUpToLimit: aMaxSize decoding: aBoolean
	self subclassResponsibility! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 10/26/2014 09:14'!
defaultTimeout
	^ 30! !

!XMLHTTPRequest methodsFor: 'retrieving' stamp: 'monty 11/29/2015 22:40'!
get
	^ self
		getUpToLimit: nil
		decoding: true! !

!XMLHTTPRequest methodsFor: 'retrieving' stamp: 'monty 3/13/2016 23:39'!
getUpToLimit: aMaxSize decoding: aBoolean
	^ [self
			basicGetUpToLimit: aMaxSize
			decoding: aBoolean]
			on: Error
			do: [:error |
				(self isMaxSizeException: error)
					ifTrue: [XMLLimitException signal: 'HTTP reponse limit exceeded']
					ifFalse: [XMLHTTPException signalMessageFrom: error]]! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 4/10/2015 10:41'!
isMaxSizeException: anException
	^ anException isKindOf: XMLLimitException! !

!XMLHTTPRequest methodsFor: 'printing' stamp: 'monty 3/12/2016 19:30'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self url;
		nextPut: $).! !

!XMLHTTPRequest methodsFor: 'retrieving' stamp: 'monty 11/29/2015 22:40'!
streamGet
	^ self
		streamGetUpToLimit: nil
		decoding: true! !

!XMLHTTPRequest methodsFor: 'retrieving' stamp: 'monty 2/19/2016 11:09'!
streamGetUpToLimit: aMaxSize decoding: aBoolean
	"must be implemented using getUpToLimit:decoding: rather than the other
	way around like XMLFileHandle does, because HTTP responses must be read
	immediately and stored entirely in memory, unlike a file stream which
	can safely be left open and read from incrementally"
	^ (self
		getUpToLimit: aMaxSize
		decoding: aBoolean) readStream! !

!XMLHTTPRequest methodsFor: 'accessing' stamp: 'monty 1/10/2014 00:10'!
url
	^ url! !

!XMLHTTPRequest methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
url: aURL
	url := aURL! !

XMLHTTPRequest subclass: #XMLWebClientHTTPRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLWebClientHTTPRequest commentStamp: 'monty 4/11/2015 00:15' prior: 0!
This class uses the Squeak WebClient library, but does not refer to WebClient classes directly to avoid compile errors on platforms without it.!

!XMLWebClientHTTPRequest class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:40'!
isSupportedImplementation
	^ XMLClassFinder hasClassNamed: #WebClient! !

!XMLWebClientHTTPRequest methodsFor: 'private' stamp: 'monty 7/6/2016 02:37'!
basicGetUpToLimit: aMaxSize decoding: aBoolean
	| response |

	response :=
		(XMLClassFinder classNamed: #WebClient) new
			timeout: self defaultTimeout;
			httpGet: self url.
	response isSuccess
		ifFalse: [XMLHTTPException signal: response status].
	aMaxSize
		ifNotNil: [
			response contentLength
				ifNil: [
					response content size > aMaxSize
						ifTrue: [XMLLimitException signal]]
				ifNotNil: [
					response contentLength > aMaxSize
						ifTrue: [XMLLimitException signal]]].
	^ response content.! !

XMLHTTPRequest subclass: #XMLZincHTTPRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLZincHTTPRequest commentStamp: 'monty 4/10/2015 10:54' prior: 0!
This client uses Zinc-HTTP, but does not refer to Zinc classes directly to avoid compile errors on platforms without it.!

!XMLZincHTTPRequest class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:40'!
isSupportedImplementation
	^ XMLClassFinder hasClassNamed: #ZnClient! !

!XMLZincHTTPRequest methodsFor: 'private' stamp: 'monty 7/6/2016 02:38'!
basicGetUpToLimit: aMaxSize decoding: aBoolean
	"value:during: on GS doesn't return the result of evaluating its second
	argument, so instead the value:during: block is returned from"
	(XMLClassFinder classNamed: #ZnMaximumEntitySize)
		value: aMaxSize
		during: [
			"Use streaming entities to get the undecoded response when
			decoding is disabled. This is used to not conflict with our
			own decoding that relies on byte-order marks and <?xml?>
			encoding attributes."
			^ ((XMLClassFinder classNamed: #ZnClient) new
				streaming: aBoolean not; "use streaming if not decoding"
				timeout: self defaultTimeout;
				url: self url;
				get;
				entity) contents]! !

!XMLZincHTTPRequest methodsFor: 'private' stamp: 'monty 7/6/2016 02:37'!
isMaxSizeException: anException
	^ anException isKindOf: (XMLClassFinder classNamed: #ZnEntityTooLarge)! !

Object subclass: #XMLAttributeDefaultValidator
	instanceVariableNames: 'attribute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultValidator commentStamp: 'monty 2/20/2016 11:10' prior: 0!
This is a base class for attribute default validators that implement constraints from the default declaration part of an ATTLIST attribute definition.!

XMLAttributeDefaultValidator subclass: #XMLAttributeDefaultImpliedValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultImpliedValidator commentStamp: 'monty 2/20/2016 05:34' prior: 0!
This class allows an attribute to be absent (the #IMPLIED constraint).!

!XMLAttributeDefaultImpliedValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 20:04'!
validateAttributeDefaultIn: aDictionary! !

XMLAttributeDefaultValidator subclass: #XMLAttributeDefaultRequiredValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultRequiredValidator commentStamp: 'monty 2/20/2016 11:10' prior: 0!
This class makes sure an attribute is present (the #REQUIRED constrant).!

!XMLAttributeDefaultRequiredValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:11'!
errorRequiredAttributeMissing
	XMLValidationException
		formatSignal: 'Required attribute "{1}" is missing'
		with: self attribute! !

!XMLAttributeDefaultRequiredValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 20:04'!
validateAttributeDefaultIn: aDictionary
	(aDictionary includesKey: self attribute)
		ifFalse: [self errorRequiredAttributeMissing]! !

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 2/19/2016 11:41'!
attribute: anAttribute
	^ self
		attribute: anAttribute
		defaultValue: ''! !

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 2/19/2016 11:41'!
attribute: anAttribute defaultValue: aDefaultValue
	^ self new
		setAttribute: anAttribute
		defaultValue: aDefaultValue! !

!XMLAttributeDefaultValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:48'!
attribute
	^ attribute! !

!XMLAttributeDefaultValidator methodsFor: 'accessing' stamp: 'monty 2/20/2016 09:04'!
defaultValue
	^ ''! !

!XMLAttributeDefaultValidator methodsFor: 'testing' stamp: 'monty 4/7/2016 03:08'!
hasDefaultValue
	^ false! !

!XMLAttributeDefaultValidator methodsFor: 'printing' stamp: 'monty 4/24/2015 14:16'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self attribute;
		nextPutAll: '="';
		nextPutAll: self defaultValue;
		nextPutAll: '")'.! !

!XMLAttributeDefaultValidator methodsFor: 'initialization' stamp: 'monty 2/20/2016 09:04'!
setAttribute: anAttribute defaultValue: aDefaultValue
	attribute := anAttribute! !

!XMLAttributeDefaultValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 20:04'!
validateAttributeDefaultIn: aDictionary
	self subclassResponsibility! !

XMLAttributeDefaultValidator subclass: #XMLAttributeDefaultValueValidator
	instanceVariableNames: 'defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultValueValidator commentStamp: 'monty 2/20/2016 11:15' prior: 0!
This class will put a default value for an attribute if the attribute is absent.!

XMLAttributeDefaultValueValidator subclass: #XMLAttributeDefaultFixedValueValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultFixedValueValidator commentStamp: 'monty 2/20/2016 11:16' prior: 0!
This class will put a default value for an attribute if the attribute is absent and also makes sure that any value matches the default (the #FIXED constraint).!

!XMLAttributeDefaultFixedValueValidator methodsFor: 'private' stamp: 'monty 2/20/2016 11:07'!
errorNonDefaultValue
	XMLValidationException
		formatSignal: 'Fixed attribute "{1}" can only have "{2}" as its value'
		with: self attribute
		with: self defaultValue! !

!XMLAttributeDefaultFixedValueValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 20:05'!
validateAttributeDefaultIn: aDictionary
	"unless the attribute is set (even just to an empty value), then it's set
	to the fixed default value, which it must match"
	(aDictionary
		at: self attribute
		ifAbsentPut: [self defaultValue]) = self defaultValue
		ifFalse: [self errorNonDefaultValue]! !

!XMLAttributeDefaultValueValidator methodsFor: 'accessing' stamp: 'monty 2/20/2016 09:04'!
defaultValue
	^ defaultValue! !

!XMLAttributeDefaultValueValidator methodsFor: 'accessing' stamp: 'monty 4/7/2016 20:18'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!XMLAttributeDefaultValueValidator methodsFor: 'testing' stamp: 'monty 4/7/2016 03:08'!
hasDefaultValue
	^ true! !

!XMLAttributeDefaultValueValidator methodsFor: 'initialization' stamp: 'monty 2/20/2016 09:04'!
setAttribute: anAttribute defaultValue: aDefaultValue
	attribute := anAttribute.
	defaultValue := aDefaultValue.! !

!XMLAttributeDefaultValueValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 20:05'!
validateAttributeDefaultIn: aDictionary
	"unless the attribute is set (even just to an empty value), then it's set
	to the default value"
	aDictionary
		at: self attribute
		ifAbsentPut: [self defaultValue]! !

Object subclass: #XMLAttributeSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Utils'!

!XMLAttributeSpec commentStamp: 'monty 7/24/2016 06:57' prior: 0!
This is a base class for attribute specifications. Subclasses need to implement matchesAttributes: to return true if the argument attribute dictionary matches the spec and false otherwise.!

!XMLAttributeSpec methodsFor: 'converting' stamp: 'monty 7/24/2016 07:01'!
asXMLAttributeSpec
	^ self! !

!XMLAttributeSpec methodsFor: 'matching' stamp: 'monty 8/10/2016 23:07'!
matchesAttributeAssociations: anAssociationCollection
	anAssociationCollection isDictionary
		ifTrue: [
			^ self matchesAttributes: anAssociationCollection]
		ifFalse: [| dictionary |
			"can't use Dictionary newFrom: because of GS portability, and
			withAll: won't work either because it uses addAll: which sends
			keysAndValuesDo: to its arg instead of associationsDo: with add:,
			so when anAssociationCollection is an array, the keys end up being
			array indexes mapped to associations"
			dictionary := Dictionary new: anAssociationCollection size.
			anAssociationCollection associationsDo: [:each |
				dictionary add: each].
			^ self matchesAttributes: dictionary]! !

!XMLAttributeSpec methodsFor: 'matching' stamp: 'monty 7/24/2016 06:48'!
matchesAttributes: aDictionary
	self subclassResponsibility! !

XMLAttributeSpec subclass: #XMLBasicAttributeSpec
	instanceVariableNames: 'attributeSpec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Utils'!

!XMLBasicAttributeSpec commentStamp: 'monty 7/24/2016 07:05' prior: 0!
A simple attribute spec that checks for attributes being present and optionally having a specific value.!

!XMLBasicAttributeSpec class methodsFor: 'instance creation' stamp: 'monty 7/24/2016 09:09'!
attributes: aDictionaryOrCollectionOfAssociationsAndStrings
	^ self new setAttributes: aDictionaryOrCollectionOfAssociationsAndStrings! !

!XMLBasicAttributeSpec methodsFor: 'adding' stamp: 'monty 8/17/2016 19:56'!
addAttribute: anAttribute
	^ attributeSpec
		at: anAttribute
		ifAbsentPut: [nil]! !

!XMLBasicAttributeSpec methodsFor: 'adding' stamp: 'monty 7/24/2016 08:59'!
addAttributeAssociation: anAssociation
	^ self
		addAttributeName: anAssociation key
		value: anAssociation value! !

!XMLBasicAttributeSpec methodsFor: 'adding' stamp: 'monty 8/17/2016 19:56'!
addAttributeName: aName value: aValue
	^ attributeSpec
		at: aName
		put: aValue! !

!XMLBasicAttributeSpec methodsFor: 'initialization' stamp: 'monty 8/17/2016 19:56'!
initialize
	attributeSpec := Dictionary new! !

!XMLBasicAttributeSpec methodsFor: 'matching' stamp: 'monty 8/26/2016 02:35'!
matchesAttributes: aDictionary
	attributeSpec keysAndValuesDo: [:specName :specValue |
		specValue
			ifNil: [
				"nil values mean the attribute has to be present and can have
				any value"
				(aDictionary includesKey: specName)
					ifFalse: [^ false]]
			ifNotNil: [
				(specValue isBlock
					ifTrue: [
						specValue value: (aDictionary at: specName ifAbsent: [^ false])]
					ifFalse: [
						specValue = (aDictionary at: specName ifAbsent: [^ false])])
							ifFalse: [^ false]]].
	^ true.! !

!XMLBasicAttributeSpec methodsFor: 'initialization' stamp: 'monty 8/17/2016 19:56'!
setAttributes: aDictionaryOrCollectionOfAssociationsAndStrings
	attributeSpec removeAll.

	"associationsDo: works for dictionaries and non-dictionaries, including
	arrays, to support specs like {'one'. 'two'->'three'}"
	aDictionaryOrCollectionOfAssociationsAndStrings associationsDo: [:each |
		each isString
			ifTrue: [self addAttribute: each]
			ifFalse: [self addAttributeAssociation: each]].! !

Object subclass: #XMLAttributeValidator
	instanceVariableNames: 'element attribute defaultValidator'
	classVariableNames: 'NormalizedSpace'
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute validators.!

!XMLAttributeValidator class methodsFor: 'instance creation' stamp: 'monty 2/19/2016 14:47'!
element: anElement attribute: anAttribute defaultValidator: aDefaultValidator
	^ self new
		setElement: anElement
		attribute: anAttribute
		defaultValidator: aDefaultValidator! !

!XMLAttributeValidator class methodsFor: 'class initialization' stamp: 'monty 12/10/2014 11:36'!
initialize
	"self initialize"

	NormalizedSpace := Character space! !

XMLAttributeValidator initialize!

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
attribute
	^ attribute! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 2/19/2016 14:46'!
defaultValidator
	^ defaultValidator! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
element
	^ element! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 4/8/2016 23:03'!
errorInvalidDefaultValue: aDefaultValue
	XMLValidationException
		formatSignal: 'Invalid default value "{1}" in attribute "{2}" of element <3>'
		with: aDefaultValue
		with: self attribute
		with: self element! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 4/9/2016 04:13'!
errorInvalidXMLIDAttributeValidator
	XMLValidationException signal:
		'All "xml:id" attributes must be declared to be of ID type'! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 4/8/2016 23:10'!
errorInvalidXMLSpaceAttributeValidator
	XMLValidationException
		formatSignal:
			'Attribute "xml:space" of element <{1}> must be declared as ',
			'an enumeration type with "default" and/or "preserve" values'
		with: self element! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 4/8/2016 16:09'!
furtherNormalizeAttributeValue: aSemiNormalizedAttributeValue
	"The tokenizer normalizes all (non-char escaped) whitespace in attribute
	values to spaces (0x20), and this further normalizes them by removing
	leading and trailing spaces and turing multiple spaces to single spaces."
	| writeStream isInWhitespace |

	"must use #writeStream instead of WriteStream on: to get a 0-based stream
	on Gemstone"
	writeStream := (String new: aSemiNormalizedAttributeValue size) writeStream.
	isInWhitespace := false.
	"to:do: for speed"
	1 to: aSemiNormalizedAttributeValue size do: [:i | | nextChar |
		(nextChar := aSemiNormalizedAttributeValue at: i) == NormalizedSpace
			ifTrue: [
				isInWhitespace
					ifFalse: [isInWhitespace := true]]
			ifFalse: [
				isInWhitespace
					ifTrue: [
						writeStream position > 0
							ifTrue: [writeStream nextPut: NormalizedSpace].
						isInWhitespace := false].
				writeStream nextPut: nextChar]].
	^ writeStream contents.! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 8/6/2016 20:06'!
furtherNormalizeAttributeValueIn: aDictionary
	| association |

	association :=
		aDictionary
			associationAt: self attribute
			ifAbsent: [^ self].

	association value:
		(self furtherNormalizeAttributeValue: association value).! !

!XMLAttributeValidator methodsFor: 'initialization' stamp: 'monty 11/14/2015 20:32'!
initializeWithValidatorsFrom: aValidatorContainer! !

!XMLAttributeValidator methodsFor: 'printing' stamp: 'monty 4/24/2015 08:54'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPutAll: '(<';
		nextPutAll: self element;
		space;
		nextPutAll: self attribute;
		nextPutAll: '="">)'.! !

!XMLAttributeValidator methodsFor: 'initialization' stamp: 'monty 2/19/2016 14:48'!
setElement: anElement attribute: anAttribute defaultValidator: aDefaultValidator
	element := anElement.
	attribute := anAttribute.
	defaultValidator := aDefaultValidator.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ true! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 4/7/2016 03:31'!
spaceSeparatedListValuesIn: aSpaceSeparatedList do: aBlock
	NormalizedSpace
		xmlSplit: aSpaceSeparatedList
		do: aBlock! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 4/8/2016 17:30'!
validateAttributeDeclaration
	self defaultValidator hasDefaultValue
		ifTrue: [| defaultValue |
			self shouldFurtherNormalizeAttributeValue
				ifTrue: [
					defaultValue :=
						self furtherNormalizeAttributeValue:
							self defaultValidator defaultValue.
					self defaultValidator defaultValue: defaultValue]
				ifFalse: [defaultValue := self defaultValidator defaultValue].
			self validateAttributeDefaultValue: defaultValue].

	(self attribute at: 1) == $x "optimization"
		ifTrue: [
			(self attribute = 'xml:id')
				ifTrue: [^ self validateXMLIDAttributeValidator].
			(self attribute = 'xml:space')
				ifTrue: [^ self validateXMLSpaceAttributeValidator]].! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 4/9/2016 02:34'!
validateAttributeDefaultValue: aDefaultValue
	"This is overriden in subclasses to check default values, but CDATA types
	should not be validated, and ID types cannot have default values at all"! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 12/28/2013 20:03'!
validateAttributeValue: aValue
	self subclassResponsibility! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 20:04'!
validateAttributes: aDictionary
	self shouldFurtherNormalizeAttributeValue
		ifTrue: [self furtherNormalizeAttributeValueIn: aDictionary].

	self defaultValidator validateAttributeDefaultIn: aDictionary.

	self validateAttributeValue:
		(aDictionary
			at: self attribute
			ifAbsent: [^ self]).! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:58'!
validateXMLIDAttributeValidator
	self errorInvalidXMLIDAttributeValidator! !

!XMLAttributeValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:55'!
validateXMLSpaceAttributeValidator
	self errorInvalidXMLSpaceAttributeValidator! !

XMLAttributeValidator subclass: #XMLCDataAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLCDataAttributeValidator commentStamp: 'monty 2/20/2016 05:32' prior: 0!
This class validates CDATA attributes, which are not whitespace-normalized further.!

!XMLCDataAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ false! !

!XMLCDataAttributeValidator methodsFor: 'validating' stamp: 'monty 12/28/2013 20:04'!
validateAttributeValue: aValue
	! !

XMLAttributeValidator subclass: #XMLEntityAttributeValidator
	instanceVariableNames: 'unparsedEntityValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntityAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes that contain unparsed entity names (that must be declared).!

XMLEntityAttributeValidator subclass: #XMLEntitiesAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntitiesAttributeValidator commentStamp: 'monty 2/20/2016 05:27' prior: 0!
This class validates attributes with lists of unparsed entity name values (that must be declared) separated by spaces.!

!XMLEntitiesAttributeValidator methodsFor: 'validating' stamp: 'monty 4/8/2016 17:31'!
validateAttributeDefaultValue: aDefaultValue
	self
		spaceSeparatedListValuesIn: aDefaultValue
		do: [:each |
			each isXMLName
				ifFalse: [self errorInvalidDefaultValue: aDefaultValue]]! !

!XMLEntitiesAttributeValidator methodsFor: 'validating' stamp: 'monty 4/7/2016 03:31'!
validateAttributeValue: aValue
	self
		spaceSeparatedListValuesIn: aValue
		do: [:each |
			self unparsedEntityValidator
				validateEntityReference: each
				inAttribute: self attribute]! !

!XMLEntityAttributeValidator methodsFor: 'initialization' stamp: 'monty 11/14/2015 20:29'!
initializeWithValidatorsFrom: aValidatorContainer
	unparsedEntityValidator := aValidatorContainer unparsedEntityValidator! !

!XMLEntityAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:35'!
unparsedEntityValidator
	^ unparsedEntityValidator! !

!XMLEntityAttributeValidator methodsFor: 'validating' stamp: 'monty 4/8/2016 17:31'!
validateAttributeDefaultValue: aDefaultValue
	aDefaultValue isXMLName
		ifFalse: [self errorInvalidDefaultValue: aDefaultValue]! !

!XMLEntityAttributeValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:30'!
validateAttributeValue: aValue
	self unparsedEntityValidator
		validateEntityReference: aValue
		inAttribute: self attribute! !

XMLAttributeValidator subclass: #XMLEnumerationAttributeValidator
	instanceVariableNames: 'allowedValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEnumerationAttributeValidator commentStamp: 'monty 2/2/2014 08:13' prior: 0!
This class validates attributes that can have any value taken from a list like "(one|two|thee)".!

!XMLEnumerationAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/9/2013 02:17'!
allowedValues
	^ allowedValues ifNil: [allowedValues := Set new]! !

!XMLEnumerationAttributeValidator methodsFor: 'accessing' stamp: 'monty 2/20/2016 01:27'!
allowedValues: aSet
	allowedValues := aSet! !

!XMLEnumerationAttributeValidator methodsFor: 'private ' stamp: 'monty 4/8/2016 23:00'!
errorInvalidValue: aValue
	XMLValidationException
		formatSignal: 'Invalid value "{1}" for attribute "{2}" of element <{3}>'
		with: aValue
		with: self attribute
		with: self element! !

!XMLEnumerationAttributeValidator methodsFor: 'validating' stamp: 'monty 4/8/2016 17:32'!
validateAttributeDefaultValue: aDefaultValue
	(self allowedValues includes: aDefaultValue)
		ifFalse: [self errorInvalidDefaultValue: aDefaultValue]! !

!XMLEnumerationAttributeValidator methodsFor: 'validating' stamp: 'monty 10/31/2014 17:30'!
validateAttributeValue: aValue
	(self allowedValues includes: aValue)
		ifFalse: [self errorInvalidValue: aValue]! !

!XMLEnumerationAttributeValidator methodsFor: 'validating' stamp: 'monty 7/13/2016 02:58'!
validateXMLSpaceAttributeValidator
	self allowedValues size > 2
		ifTrue: [self errorInvalidXMLSpaceAttributeValidator].
	self allowedValues do: [:each |
		((each = 'preserve')
			or: [each = 'default'])
			ifFalse: [self errorInvalidXMLSpaceAttributeValidator]].! !

XMLEnumerationAttributeValidator subclass: #XMLNotationEnumerationAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNotationEnumerationAttributeValidator commentStamp: 'monty 2/2/2014 08:10' prior: 0!
This class validates attributes that can have NOTATION names from a list as values.!

!XMLNotationEnumerationAttributeValidator methodsFor: 'initialization' stamp: 'monty 7/13/2016 01:15'!
initializeWithValidatorsFrom: aValidatorContainer
	aValidatorContainer unparsedEntityValidator addNotationAttributeValidator: self! !

!XMLNotationEnumerationAttributeValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:56'!
validateXMLSpaceAttributeValidator
	self errorInvalidXMLSpaceAttributeValidator! !

XMLAttributeValidator subclass: #XMLIDAttributeValidator
	instanceVariableNames: 'elementIdValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDAttributeValidator commentStamp: 'monty 2/20/2016 11:17' prior: 0!
This class validates ID attributes that must have unique (not repeated in the same document) values.!

!XMLIDAttributeValidator class methodsFor: 'private' stamp: 'monty 6/20/2015 20:30'!
classForAttribute: anAttribute
	anAttribute = 'xml:id'
		ifTrue: [^ XMLIDAttributeXMLIDValidator]
		ifFalse: [^ XMLIDAttributeValidator]! !

!XMLIDAttributeValidator class methodsFor: 'instance creation' stamp: 'monty 2/19/2016 14:53'!
element: anElement attribute: anAttribute defaultValidator: aDefaultValidator
	^ (self classForAttribute: anAttribute) new
		setElement: anElement
		attribute: anAttribute
		defaultValidator: aDefaultValidator! !

!XMLIDAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 19:49'!
elementIDValidator
	^ elementIdValidator! !

!XMLIDAttributeValidator methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:37'!
elementIDValidator: anElementIDValidator
	elementIdValidator := anElementIDValidator! !

!XMLIDAttributeValidator methodsFor: 'private' stamp: 'monty 4/9/2016 04:15'!
errorIDAttributeMustBeRequiredOrImplied
	XMLValidationException
		formatSignal:
			'Attribute "{1}" of element <{2}> is of ID type and must ',
			'be declared #REQUIRED or #IMPLIED'
		with: self attribute
		with: self element! !

!XMLIDAttributeValidator methodsFor: 'initialization' stamp: 'monty 11/14/2015 20:29'!
initializeWithValidatorsFrom: aValidatorContainer
	elementIdValidator := aValidatorContainer elementIDValidator! !

!XMLIDAttributeValidator methodsFor: 'validating' stamp: 'monty 7/13/2016 01:06'!
validateAttributeDeclaration
	super validateAttributeDeclaration.

	self elementIDValidator
		validateIDAttribute: self attribute
		forElement: self element.! !

!XMLIDAttributeValidator methodsFor: 'validating' stamp: 'monty 4/9/2016 02:36'!
validateAttributeDefaultValue: aDefaultValue
	self errorIDAttributeMustBeRequiredOrImplied! !

!XMLIDAttributeValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:32'!
validateAttributeValue: aValue
	self elementIDValidator
		validateID: aValue
		inAttribute: self attribute! !

XMLIDAttributeValidator subclass: #XMLIDAttributeXMLIDValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDAttributeXMLIDValidator commentStamp: 'monty 2/20/2016 11:17' prior: 0!
This class validates xml:id ID attributes.!

!XMLIDAttributeXMLIDValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:33'!
validateAttributeValue: aValue
	self elementIDValidator validateXMLID: aValue! !

!XMLIDAttributeXMLIDValidator methodsFor: 'validating' stamp: 'monty 3/22/2015 03:58'!
validateXMLIDAttributeValidator
! !

XMLAttributeValidator subclass: #XMLIDRefAttributeValidator
	instanceVariableNames: 'elementIdValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes that refer to an ID of another element with a ID attribute.!

!XMLIDRefAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 19:51'!
elementIDValidator
	^ elementIdValidator! !

!XMLIDRefAttributeValidator methodsFor: 'initialization' stamp: 'monty 11/14/2015 20:30'!
initializeWithValidatorsFrom: aValidatorContainer
	elementIdValidator := aValidatorContainer elementIDValidator! !

!XMLIDRefAttributeValidator methodsFor: 'validating' stamp: 'monty 4/7/2016 20:13'!
validateAttributeDefaultValue: aDefaultValue
	aDefaultValue isXMLName
		ifFalse: [self errorInvalidDefaultValue: aDefaultValue]! !

!XMLIDRefAttributeValidator methodsFor: 'validating' stamp: 'monty 1/3/2015 23:24'!
validateAttributeValue: aValue
	self elementIDValidator
		validateIDReference: aValue
		inAttribute: self attribute! !

XMLIDRefAttributeValidator subclass: #XMLIDRefsAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefsAttributeValidator commentStamp: 'monty 2/2/2014 08:13' prior: 0!
This class validates attributes with lists of ID ref values separated by spaces.!

!XMLIDRefsAttributeValidator methodsFor: 'validating' stamp: 'monty 4/7/2016 20:13'!
validateAttributeDefaultValue: aDefaultValue
	self
		spaceSeparatedListValuesIn: aDefaultValue
		do: [:each |
			each isXMLName
				ifFalse: [self errorInvalidDefaultValue: aDefaultValue]]! !

!XMLIDRefsAttributeValidator methodsFor: 'validating' stamp: 'monty 4/7/2016 03:31'!
validateAttributeValue: aValue
	self
		spaceSeparatedListValuesIn: aValue
		do: [:each |
			self elementIDValidator
				validateIDReference: each
				inAttribute: self attribute]! !

XMLAttributeValidator subclass: #XMLNmtokenAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNmtokenAttributeValidator commentStamp: 'monty 12/11/2014 04:41' prior: 0!
This class validates Nmtoken name attributes.!

!XMLNmtokenAttributeValidator methodsFor: 'private' stamp: 'monty 4/8/2016 23:01'!
errorInvalidNmtoken: aName
	XMLValidationException
		formatSignal: 'Invalid Nmtoken name "{1}" in attribute "{2}" of element <{3}>'
		with: aName
		with: self attribute
		with: self element! !

!XMLNmtokenAttributeValidator methodsFor: 'validating' stamp: 'monty 4/7/2016 20:13'!
validateAttributeDefaultValue: aDefaultValue
	aDefaultValue isXMLNmtoken
		ifFalse: [self errorInvalidDefaultValue: aDefaultValue]! !

!XMLNmtokenAttributeValidator methodsFor: 'validating' stamp: 'monty 12/11/2014 04:37'!
validateAttributeValue: aValue
	self validateNmtoken: aValue! !

!XMLNmtokenAttributeValidator methodsFor: 'validating' stamp: 'monty 3/24/2016 20:56'!
validateNmtoken: aName
	aName isXMLNmtoken
		ifFalse: [self errorInvalidNmtoken: aName]! !

XMLNmtokenAttributeValidator subclass: #XMLNmtokensAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNmtokensAttributeValidator commentStamp: 'monty 12/11/2014 04:41' prior: 0!
This class validates attributes with lists of Nmtoken name values separated by spaces.!

!XMLNmtokensAttributeValidator methodsFor: 'validating' stamp: 'monty 4/7/2016 20:13'!
validateAttributeDefaultValue: aDefaultValue
	self
		spaceSeparatedListValuesIn: aDefaultValue
		do: [:each |
			each isXMLNmtoken
				ifFalse: [self errorInvalidDefaultValue: aDefaultValue]]! !

!XMLNmtokensAttributeValidator methodsFor: 'validating' stamp: 'monty 4/7/2016 03:31'!
validateAttributeValue: aValue
	self
		spaceSeparatedListValuesIn: aValue
		do: [:each | self validateNmtoken: each]! !

Object subclass: #XMLClassFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Utils'!

!XMLClassFinder commentStamp: 'monty 7/6/2016 02:49' prior: 0!
A simple cross-platform interface for finding classes that may not be in the image.!

!XMLClassFinder class methodsFor: 'accessing' stamp: 'monty 7/6/2016 02:26'!
classNamed: aClassName
	^ self
		classNamed: aClassName
		ifAbsent: [nil]! !

!XMLClassFinder class methodsFor: 'accessing' stamp: 'monty 7/6/2016 02:26'!
classNamed: aClassName ifAbsent: aBlock
	^ Smalltalk globals
		at: aClassName asSymbol
		ifAbsent: aBlock! !

!XMLClassFinder class methodsFor: 'testing' stamp: 'monty 7/6/2016 02:30'!
hasClassNamed: aClassName
	self
		classNamed: aClassName
		ifAbsent: [^ false].
	^ true.! !

Object subclass: #XMLConfiguration
	instanceVariableNames: 'isValidating usesNamespaces decodesCharacters requiresDocumentConstraints parserHandlerClass parserLimits entityConfiguration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLConfiguration commentStamp: 'monty 7/14/2015 14:40' prior: 0!
This class stores the settings and components of a parser. For performance it uses separate inst vars instead of a dictionary and lazy initializes some components.!

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 5/8/2015 19:04'!
decodesCharacters
	^ decodesCharacters! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 5/8/2015 19:04'!
decodesCharacters: aBoolean
	decodesCharacters := aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:52'!
defaultParserHandlerClass
	^ SAXHandler! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 17:31'!
documentReadLimit
	^ self parserLimits documentReadLimit! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 4/14/2016 03:29'!
documentReadLimit: anInteger
	self parserLimits documentReadLimit: anInteger! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:31'!
entityConfiguration
	^ entityConfiguration ifNil: [entityConfiguration := DTDEntityConfiguration new]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:54'!
entityConfiguration: anEntityConfiguration
	entityConfiguration := anEntityConfiguration! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:24'!
externalEntityResolver
	^ self entityConfiguration externalEntityResolver! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:23'!
externalEntityResolver: anExternalEntityResolver
	self entityConfiguration externalEntityResolver: anExternalEntityResolver! !

!XMLConfiguration methodsFor: 'initialization' stamp: 'monty 5/8/2015 19:03'!
initialize
	super initialize.

	"always used, so explicitly initialized"
	isValidating := true.
	usesNamespaces := true.
	decodesCharacters := true.
	requiresDocumentConstraints := false.
	parserLimits := XMLParserLimits new.! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:39'!
isValidating
	^ isValidating! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:42'!
isValidating: aBoolean
	(isValidating := aBoolean)
		ifTrue: [self preservesUndeclaredEntityReferences: false]
		ifFalse: [self requiresDocumentConstraints: false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 17:34'!
maxEntityReplacementDepth
	^ self parserLimits maxEntityReplacementDepth! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 4/14/2016 03:30'!
maxEntityReplacementDepth: anInteger
	self parserLimits maxEntityReplacementDepth: anInteger! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:54'!
parserHandlerClass
	^ parserHandlerClass ifNil: [parserHandlerClass := self defaultParserHandlerClass]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass: aClass
	parserHandlerClass := aClass! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:39'!
parserLimits
	^ parserLimits! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:53'!
parserLimits: aParserLimits
	parserLimits := aParserLimits! !

!XMLConfiguration methodsFor: 'copying' stamp: 'monty 6/19/2015 20:31'!
postCopy
	super postCopy.

	parserLimits := parserLimits copy.
	entityConfiguration
		ifNotNil: [entityConfiguration := entityConfiguration copy].! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 04:12'!
preservesUndeclaredEntityReferences
	^ self entityConfiguration preservesUndeclaredEntityReferences! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/6/2015 06:12'!
preservesUndeclaredEntityReferences: aBoolean
	self entityConfiguration preservesUndeclaredEntityReferences: aBoolean.
	aBoolean
		ifTrue: [self isValidating: false].! !

!XMLConfiguration methodsFor: 'removing' stamp: 'monty 3/17/2015 17:39'!
removeLimits
	self parserLimits removeAll.
	self externalEntityResolver removeLimits.! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 23:55'!
replacesContentEntityReferences
	^ self entityConfiguration replacesContentEntityReferences! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 23:55'!
replacesContentEntityReferences: aBoolean
	self entityConfiguration replacesContentEntityReferences: aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:39'!
requiresDocumentConstraints
	^ requiresDocumentConstraints! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/25/2014 21:13'!
requiresDocumentConstraints: aBoolean
	(requiresDocumentConstraints := aBoolean)
		ifTrue: [self isValidating: true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:27'!
resolvesExternalEntities
	^ self entityConfiguration resolvesExternalEntities! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/3/2015 00:27'!
resolvesExternalEntities: aBoolean
	self entityConfiguration resolvesExternalEntities: aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:39'!
usesNamespaces
	^ usesNamespaces! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:06'!
usesNamespaces: aBoolean
	usesNamespaces := aBoolean! !

XMLConfiguration subclass: #XMLDOMConfiguration
	instanceVariableNames: 'preservesCDataNodes preservesCommentNodes preservesIgnorableWhitespace nodeFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMConfiguration commentStamp: '<historical>' prior: 0!
This class stores additional settings and components relevant to the DOM parser and DOM nodes.!

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 7/13/2015 13:10'!
defaultNodeFactoryClass
	^ XMLNodeFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/17/2015 16:54'!
defaultParserHandlerClass
	^ XMLDOMParser! !

!XMLDOMConfiguration methodsFor: 'initialization' stamp: 'monty 7/13/2015 13:10'!
initialize
	super initialize.

	"always used, so explicitly initialized"
	preservesCDataNodes := false.
	preservesCommentNodes := false.
	preservesIgnorableWhitespace := false.
	nodeFactory := self defaultNodeFactoryClass new.! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 4/20/2016 13:32'!
isValidating: aBoolean
	aBoolean
		ifFalse: [self preservesIgnorableWhitespace: false].
	super isValidating: aBoolean.! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:50'!
nodeFactory
	^ nodeFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory: aFactory
	nodeFactory := aFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 4/17/2015 10:42'!
nodeListClass
	^ self nodeFactory nodeListClass! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 4/17/2015 10:53'!
nodeListClass: aClass
	self nodeFactory:
		(self nodeFactory asPluggableNodeFactory nodeListClass: aClass)! !

!XMLDOMConfiguration methodsFor: 'copying' stamp: 'monty 6/19/2015 20:31'!
postCopy
	super postCopy.

	nodeFactory := nodeFactory copy.! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:45'!
preservesCDataNodes
	^ preservesCDataNodes! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes: aBoolean
	preservesCDataNodes := aBoolean! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:45'!
preservesCommentNodes
	^ preservesCommentNodes! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes: aBoolean
	preservesCommentNodes := aBoolean! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 3/12/2015 13:45'!
preservesIgnorableWhitespace
	^ preservesIgnorableWhitespace! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 4/21/2016 07:36'!
preservesIgnorableWhitespace: aBoolean
	(preservesIgnorableWhitespace := aBoolean)
		ifTrue: [self isValidating: true]! !

Object subclass: #XMLDFAState
	instanceVariableNames: 'nfaStates dfaStateCache dfaStateTransitions hasNonDeterminism'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This class is a DFA state made by grouping one or more NFA states.!

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 23:18'!
nfaStates: anNFAStateSet
	^ self
		nfaStates: anNFAStateSet
		dfaStateCache: nil! !

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 23:18'!
nfaStates: anNFAStateSet dfaStateCache: aDFAStateCache
	^ self new
		setNFAStates: anNFAStateSet
		dfaStateCache: aDFAStateCache! !

!XMLDFAState methodsFor: 'private' stamp: 'monty 2/19/2016 11:17'!
addDFAStateTransitionMatching: aMatcher
	| matchingNFAStates totalMatched matchingDFAState |

	self dfaStateTransitions size >= self maxDFAStateTransitions
		ifTrue: [self removeDFAStateTransitions].

	matchingNFAStates := XMLNFAStateSet new.
	totalMatched := 0.
	self nfaStatesDo: [:state |
		(state matches: aMatcher)
			ifTrue: [
				state nextState addTo: matchingNFAStates.
				totalMatched := totalMatched + 1]].

	matchingNFAStates size > 0
		ifTrue: [
			matchingDFAState :=
				dfaStateCache dfaStateForNFAStates: matchingNFAStates.
			totalMatched > 1
				ifTrue: [matchingDFAState hasNonDeterminism: true]].

	^ self dfaStateTransitions
		at: aMatcher
		put: matchingDFAState. "nil if no match"! !

!XMLDFAState methodsFor: 'private' stamp: 'monty 4/22/2015 09:27'!
dfaStateCache
	^ dfaStateCache! !

!XMLDFAState methodsFor: 'private' stamp: 'monty 4/22/2015 14:52'!
dfaStateTransitions
	^ dfaStateTransitions ifNil: [dfaStateTransitions := Dictionary new]! !

!XMLDFAState methodsFor: 'testing' stamp: 'monty 10/31/2015 19:21'!
hasNonDeterminism
	^ hasNonDeterminism! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 10/31/2015 19:21'!
hasNonDeterminism: aBoolean
	hasNonDeterminism := aBoolean! !

!XMLDFAState methodsFor: 'testing' stamp: 'monty 12/28/2013 18:54'!
isAccepting
	^ nfaStates isAccepting! !

!XMLDFAState methodsFor: 'private' stamp: 'monty 4/22/2015 14:51'!
maxDFAStateTransitions
	^ 64! !

!XMLDFAState methodsFor: 'matching' stamp: 'monty 11/13/2015 04:17'!
nextDFAStateMatching: aMatcher
	^ self
		nextDFAStateMatching: aMatcher
		ifNone: [nil]! !

!XMLDFAState methodsFor: 'matching' stamp: 'monty 2/19/2016 11:03'!
nextDFAStateMatching: aMatcher ifNone: aBlock
	^ (self dfaStateTransitions
		at: aMatcher
		ifAbsent: [self addDFAStateTransitionMatching: aMatcher])
			ifNil: [aBlock value]! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/29/2013 01:10'!
nfaStates
	^ nfaStates! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 11/10/2015 22:36'!
nfaStatesDo: aBlock
	nfaStates do: aBlock! !

!XMLDFAState methodsFor: 'printing' stamp: 'monty 4/22/2015 11:17'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self nfaStates;
		nextPut: $).! !

!XMLDFAState methodsFor: 'private' stamp: 'monty 4/22/2015 14:52'!
removeDFAStateTransitions
	dfaStateTransitions := nil! !

!XMLDFAState methodsFor: 'initialization' stamp: 'monty 10/31/2015 19:21'!
setNFAStates: anNFAStateSet dfaStateCache: aDFAStateCache
	nfaStates := anNFAStateSet.
	dfaStateCache := (aDFAStateCache ifNil: [XMLDFAStateCache firstDFAState: self]).
	hasNonDeterminism := false.! !

Object subclass: #XMLDFAStateCache
	instanceVariableNames: 'firstDFAState cachedDFAStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAStateCache commentStamp: 'monty 12/27/2014 03:19' prior: 0!
This class uses a dictionary with NFAStateSet keys to cache DFA states. (Does not use the regular XML key/value  cache for performance)!

!XMLDFAStateCache class methodsFor: 'instance creation' stamp: 'monty 4/17/2015 16:00'!
firstDFAState: aDFAState
	^ self new setFirstDFAState: aDFAState! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 8/8/2016 21:51'!
addDFAStateForNFAStates: anNFAStateSet
	cachedDFAStates size >= self maxCachedDFAStates
		ifTrue: [self removeDFAStates].
	^ cachedDFAStates
		at: anNFAStateSet
		put:  (anNFAStateSet asDFAStateWithStateCache: self).! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 4/22/2015 09:27'!
cachedDFAStates
	^ cachedDFAStates! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 8/8/2016 21:50'!
dfaStateForNFAStates: anNFAStateSet
	^ cachedDFAStates
		at: anNFAStateSet
		ifAbsent: [self addDFAStateForNFAStates: anNFAStateSet]! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 4/17/2015 15:57'!
firstDFAState
	^ firstDFAState! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 4/22/2015 10:43'!
maxCachedDFAStates
	^ 64! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 4/22/2015 14:51'!
removeDFAStates
	cachedDFAStates
		valuesDo: [:each | each removeDFAStateTransitions];
		removeAll;
		at: firstDFAState nfaStates put: firstDFAState! !

!XMLDFAStateCache methodsFor: 'initialization' stamp: 'monty 4/17/2015 16:01'!
setFirstDFAState: aDFAState
	firstDFAState := aDFAState.
	(cachedDFAStates := Dictionary new: 5)
		at: firstDFAState nfaStates
		put: firstDFAState.! !

Object subclass: #XMLDocumentValidator
	instanceVariableNames: 'elementValidatorStack defaultXMLIDValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDocumentValidator commentStamp: '<historical>' prior: 0!
This is a base class for XML document validators.!

XMLDocumentValidator subclass: #DTDDocumentValidator
	instanceVariableNames: 'doctypeDefinition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDocumentValidator commentStamp: '<historical>' prior: 0!
This class validates documents using a DTD.!

!DTDDocumentValidator class methodsFor: 'instance creation' stamp: 'monty 3/2/2015 11:13'!
doctypeDefinition: aDoctypeDefinition
	^ self new doctypeDefinition: aDoctypeDefinition! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 7/13/2016 00:22'!
attributeValidatorsFor: anElement
	^ self doctypeDefinition
		attributeValidatorsAt: anElement
		ifAbsent: [nil]! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:15'!
doctypeDefinition
	^ doctypeDefinition! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:15'!
doctypeDefinition: aDoctypeDefinition
	doctypeDefinition := aDoctypeDefinition! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 7/13/2016 00:22'!
elementValidatorFor: anElement
	^ (self doctypeDefinition
		elementValidatorsAt: anElement
		ifAbsent: [nil]) copy! !

!DTDDocumentValidator methodsFor: 'private' stamp: 'monty 8/20/2016 15:44'!
errorInvalidRootElement: anElement
	XMLValidationException
		formatSignal:
			'Root element must be <{1}> according to DOCTYPE declaration, not <{2}>'
		with: self doctypeDefinition root
		with: anElement! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 3/2/2015 11:14'!
hasAttributeValidators
	^ self doctypeDefinition hasAttributeValidators! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 3/2/2015 11:14'!
hasElementValidators
	^ self doctypeDefinition hasElementValidators! !

!DTDDocumentValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 19:48'!
unvalidatedAttributes: anAttributeCollection
	XMLValidationException
		formatSignal: 'Undeclared attribute "{1}"'
		with: anAttributeCollection anyOne! !

!DTDDocumentValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 09:20'!
unvalidatedElement: anElement
	XMLValidationException
		formatSignal: 'Undeclared element <{1}>'
		with: anElement! !

!DTDDocumentValidator methodsFor: 'validating' stamp: 'monty 3/2/2015 11:14'!
validateIDReferences
	self doctypeDefinition hasElementIDValidator
		ifTrue: [self doctypeDefinition elementIDValidator validateIDReferences]! !

!DTDDocumentValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 19:47'!
validateRootElement: anElement
	(self doctypeDefinition hasRoot
		and: [self doctypeDefinition root ~= anElement])
		ifTrue: [self errorInvalidRootElement: anElement]! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:04'!
attributeValidatorsFor: anElement
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:56'!
currentElementValidator
	^ self elementValidatorStack xmlLastOrNil! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:07'!
defaultElementValidatorFor: aName
	^ XMLAnyElementValidator element: aName! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 2/19/2016 14:47'!
defaultXMLIDValidator
	"used to validate xml:id attributes if there's no DTD or other validators"
	^ defaultXMLIDValidator
		ifNil: [
			defaultXMLIDValidator :=
				XMLIDAttributeXMLIDValidator
					element: ''
					attribute: 'xml:id'
					defaultValidator:
						(XMLAttributeDefaultImpliedValidator attribute: 'xml:id').
			defaultXMLIDValidator
				elementIDValidator: XMLElementIDValidator new;
				yourself]! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 4/13/2016 05:59'!
elementValidatorFor: anElement
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:46'!
elementValidatorStack
	^ elementValidatorStack ifNil: [elementValidatorStack := OrderedCollection new]! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 4/13/2016 10:20'!
errorUnexpectEndOfElement: anElement
	XMLValidationException
		formatSignal: 'Element <{1}> terminated prematurely'
		with: anElement! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:31'!
hasAttributeValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 4/24/2015 15:30'!
hasCurrentElementValidator
	^ self currentElementValidator notNil! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 4/13/2016 10:16'!
hasElementContent
	self hasElementValidators
		ifTrue: [
			self currentElementValidator
				ifNotNil: [:elementValidator |
					^ elementValidator hasElementContent]].
	^ false.! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:32'!
hasElementValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 3/23/2016 21:53'!
hasValidators
	^ self hasElementValidators
		or: [self hasAttributeValidators]! !

!XMLDocumentValidator methodsFor: 'printing' stamp: 'monty 4/24/2015 15:30'!
printOn: aStream
	super printOn: aStream.
	self hasCurrentElementValidator
		ifTrue: [
			aStream
				nextPut: $(;
				print: self currentElementValidator;
				nextPut: $)].! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/21/2013 20:29'!
unvalidatedAttributes: anAttributeCollection! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 09:20'!
unvalidatedElement: anElement! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 12/11/2014 05:41'!
validateAttributes: anAttributeDictionary with: aValidatorDictionary
	| unvalidatedAttributes |

	unvalidatedAttributes := Set new: anAttributeDictionary size.
	anAttributeDictionary keysDo: [:each | unvalidatedAttributes add: each].
	aValidatorDictionary
		ifNotNil: [
			aValidatorDictionary keysAndValuesDo: [:attribute :validator |
				validator validateAttributes: anAttributeDictionary.
				unvalidatedAttributes
					remove: attribute
					ifAbsent: nil]].

	unvalidatedAttributes isEmpty
		ifFalse: [self unvalidatedAttributes: unvalidatedAttributes].! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 10:15'!
validateComment: aString
	self hasElementValidators
		ifTrue: [
			self currentElementValidator
				ifNotNil: [:elementValidator |
					elementValidator validateComment: aString]]! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:28'!
validateEndDocument
	self validateIDReferences! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 10:20'!
validateEndTag: anElement
	self hasElementValidators
		ifTrue: [
			self currentElementValidator
				ifNotNil: [:elementValidator |
					elementValidator atEnd
						ifFalse: [self errorUnexpectEndOfElement: anElement].
					self elementValidatorStack removeLast]]! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 1/1/2015 21:24'!
validateIDReferences
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 11/18/2013 03:44'!
validatePCData: aString
	self hasElementValidators
		ifTrue: [self currentElementValidator validatePCData: aString]! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 4/16/2016 13:44'!
validatePI: aTargetString data: aDataString
	self hasElementValidators
		ifTrue: [
			self currentElementValidator
				ifNotNil: [:elementValidator |
					elementValidator
						validatePI: aTargetString
						data: aDataString]]! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 09:21'!
validateRootElement: anElement! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 09:18'!
validateStartTag: anElement
	self currentElementValidator
		ifNotNil: [:elementValidator |
			elementValidator validateStartTag: anElement].

	self elementValidatorStack addLast:
		((self elementValidatorFor: anElement)
			ifNil: [
				self
					unvalidatedElement: anElement;
					defaultElementValidatorFor: anElement]).! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 20:06'!
validateStartTag: anElement attributes: aDictionary
	(self hasElementValidators
		or: [self hasAttributeValidators])
		ifTrue: [
			self validateStartTag: anElement.
			self
				validateAttributes: aDictionary
				with: (self attributeValidatorsFor: anElement)]! !

!XMLDocumentValidator methodsFor: 'validating' stamp: 'monty 8/6/2016 20:07'!
validateXMLIDAttributeIn: aDictionary
	self hasAttributeValidators
		ifFalse: [self defaultXMLIDValidator validateAttributes: aDictionary]! !

Object subclass: #XMLElementClassMapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Utils'!

!XMLElementClassMapper commentStamp: 'monty 7/31/2016 12:18' prior: 0!
This is a base class for element class mappers. Subclasses need to implement the abstract methods to register and lookup classes mapped to element names, namespace URIs, and attributes.!

XMLElementClassMapper subclass: #XMLBasicElementClassMapper
	instanceVariableNames: 'elementClassMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Utils'!

!XMLBasicElementClassMapper commentStamp: 'monty 8/12/2016 20:51' prior: 0!
This class can match elements to classes based on their names, namespace URIs, and attributes.

The precedence of mapping goes:
-Qualified name, namespace URI, and attributes
-Qualified name and namespace URI
-Qualified name and attributes
-Qualified name
-Non-qualified name, namespace URI, and attributes
-Non-qualified name and namespace URI
-Non-qualified name and attributes
-Non-qualified name!

!XMLBasicElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/12/2016 18:47'!
classForElement: aQualifiedName attributes: anAssociationCollection ifNone: aBlock
	^ self
		classForElement: aQualifiedName
		namespaceURI: ''
		attributes: anAssociationCollection
		ifNone: aBlock! !

!XMLBasicElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/12/2016 19:10'!
classForElement: aQualifiedName ifNone: aBlock
	^ self
		classForElement: aQualifiedName
		namespaceURI: ''
		attributes: #()
		ifNone: aBlock! !

!XMLBasicElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/12/2016 18:45'!
classForElement: aQualifiedName namespaceURI: aURI attributes: anAssociationCollection ifNone: aBlock
	| localName |

	aURI size > 0 "optimization"
		ifTrue: [
			(self
				elementClassMapAt: aQualifiedName -> aURI
				attributes: anAssociationCollection)
				ifNotNil: [:class | ^ class].
			(self
				elementClassMapAt: aQualifiedName
				attributes: anAssociationCollection)
				ifNotNil: [:class | ^ class].
			(localName := aQualifiedName xmlLocalNameAfterPrefix) size < aQualifiedName size
				ifTrue: [
					(self
						elementClassMapAt: localName -> aURI
						attributes: anAssociationCollection)
						ifNotNil: [:class | ^ class].
					(self
						elementClassMapAt: localName
						attributes: anAssociationCollection)
						ifNotNil: [:class | ^ class]]]
		ifFalse: [
			(self
				elementClassMapAt: aQualifiedName
				attributes: anAssociationCollection)
				ifNotNil: [:class | ^ class].
			(localName := aQualifiedName xmlLocalNameAfterPrefix) size < aQualifiedName size
				ifTrue: [
					(self
						elementClassMapAt: localName
						attributes: anAssociationCollection)
						ifNotNil: [:class | ^ class]]].

	^ aBlock value.! !

!XMLBasicElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/12/2016 19:10'!
classForElement: aQualifiedName namespaceURI: aURI ifNone: aBlock
	^ self
		classForElement: aQualifiedName
		namespaceURI: aURI
		attributes: #()
		ifNone: aBlock! !

!XMLBasicElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/12/2016 18:47'!
classForElement: aQualifiedName namespaceURI: aURI localName: aLocalName attributes: anAssociationCollection ifNone: aBlock
	aURI size > 0 "optimization"
		ifTrue: [
			(self
				elementClassMapAt: aQualifiedName -> aURI
				attributes: anAssociationCollection)
				ifNotNil: [:class | ^ class].
			(self
				elementClassMapAt: aQualifiedName
				attributes: anAssociationCollection)
				ifNotNil: [:class | ^ class].
			aLocalName size < aQualifiedName size
				ifTrue: [
					(self
						elementClassMapAt: aLocalName -> aURI
						attributes: anAssociationCollection)
						ifNotNil: [:class | ^ class].
					(self
						elementClassMapAt: aLocalName
						attributes: anAssociationCollection)
						ifNotNil: [:class | ^ class]]]
		ifFalse: [
			(self
				elementClassMapAt: aQualifiedName
				attributes: anAssociationCollection)
				ifNotNil: [:class | ^ class].
			aLocalName size < aQualifiedName size
				ifTrue: [
					(self
						elementClassMapAt: aLocalName
						attributes: anAssociationCollection)
						ifNotNil: [:class | ^ class]]].

	^ aBlock value.! !

!XMLBasicElementClassMapper methodsFor: 'private' stamp: 'monty 7/26/2016 07:21'!
elementClassMap
	^ elementClassMap! !

!XMLBasicElementClassMapper methodsFor: 'private' stamp: 'monty 8/16/2016 22:03'!
elementClassMapAt: aKey attributes: anAssociationCollection
	| classOrCollection |

	classOrCollection :=
		elementClassMap
			at: aKey
			ifAbsent: [^ nil].

	classOrCollection isCollection
		ifTrue: [
			"to:do: for speed"
			1 to: classOrCollection size - 1 do: [:i |
				((classOrCollection at: i) key
					matchesAttributeAssociations: anAssociationCollection)
					ifTrue: [^ (classOrCollection at: i) value]].
			^ classOrCollection last]
		ifFalse: [^ classOrCollection].! !

!XMLBasicElementClassMapper methodsFor: 'initialization' stamp: 'monty 7/26/2016 07:21'!
initialize
	super initialize.

	elementClassMap := Dictionary new.! !

!XMLBasicElementClassMapper methodsFor: 'mapping' stamp: 'monty 8/15/2016 04:25'!
mapElement: anElement attributeSpec: anAttributeSpec toClass: aClass
	| classOrCollection |

	classOrCollection :=
		elementClassMap
			at: anElement
			ifAbsent: [nil].

	classOrCollection isCollection
		ifTrue: [
			"new mappings have precedence over old"
			classOrCollection addFirst: anAttributeSpec -> aClass]
		ifFalse: [
			elementClassMap
				at: anElement
				put:
					(OrderedCollection
						with: anAttributeSpec -> aClass
						with: classOrCollection)].! !

!XMLBasicElementClassMapper methodsFor: 'mapping' stamp: 'monty 8/15/2016 04:25'!
mapElement: anElement namespaceURI: aURI attributeSpec: anAttributeSpec toClass: aClass
	self
		mapElement:
			(aURI size > 0
				ifTrue: [anElement -> aURI]
				ifFalse: [anElement])
		attributeSpec: anAttributeSpec
		toClass: aClass! !

!XMLBasicElementClassMapper methodsFor: 'mapping' stamp: 'monty 8/15/2016 04:25'!
mapElement: anElement namespaceURI: aURI toClass: aClass
	self
		mapElement:
			(aURI size > 0
				ifTrue: [anElement -> aURI]
				ifFalse: [anElement])
		toClass: aClass! !

!XMLBasicElementClassMapper methodsFor: 'mapping' stamp: 'monty 8/15/2016 04:26'!
mapElement: anElement toClass: aClass
	| classOrCollection |

	classOrCollection :=
		elementClassMap
			at: anElement
			ifAbsent: [nil].

	classOrCollection isCollection
		ifTrue: [
			classOrCollection
				at: classOrCollection size
				put: aClass]
		ifFalse: [
			elementClassMap
				at: anElement
				put: aClass].! !

!XMLBasicElementClassMapper methodsFor: 'copying' stamp: 'monty 8/14/2016 17:25'!
postCopy
	super postCopy.

	elementClassMap := elementClassMap copy.

	"each val is either a class or an OrderedCollection, which needs
	to be copied so new mappings in either the copy or original don't
	effect the other"
	elementClassMap associationsDo: [:each |
		each value isCollection
			ifTrue: [each value: each value copy]].! !

!XMLElementClassMapper methodsFor: 'private' stamp: 'monty 7/24/2016 12:28'!
camelCasedNameFromClassNameSegments: aNameSegmentCollection
	^ String streamContents: [:stream |
		aNameSegmentCollection withIndexDo: [:each :i |
			stream nextPutAll:
				(i > 1
					ifTrue: [each]
					ifFalse: [each asLowercase])]]! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:32'!
classForElement: aQualifiedName
	^ self
		classForElement: aQualifiedName
		ifNone: [nil]! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:32'!
classForElement: aQualifiedName attributes: anAssociationCollection
	^ self
		classForElement: aQualifiedName
		attributes: anAssociationCollection
		ifNone: [nil]! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:32'!
classForElement: aQualifiedName attributes: anAssociationCollection ifNone: aBlock
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:32'!
classForElement: aQualifiedName ifNone: aBlock
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:32'!
classForElement: aQualifiedName namespaceURI: aURI
	^ self
		classForElement: aQualifiedName
		namespaceURI: aURI
		ifNone: [nil]! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:32'!
classForElement: aQualifiedName namespaceURI: aURI attributes: anAssociationCollection
	^ self
		classForElement: aQualifiedName
		namespaceURI: aURI
		attributes: anAssociationCollection
		ifNone: [nil]! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:32'!
classForElement: aQualifiedName namespaceURI: aURI attributes: anAssociationCollection ifNone: aBlock
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:32'!
classForElement: aQualifiedName namespaceURI: aURI ifNone: aBlock
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:30'!
classForElement: aQualifiedName namespaceURI: aURI localName: aLocalName attributes: anAssociationCollection
	^ self
		classForElement: aQualifiedName
		namespaceURI: aURI
		localName: aLocalName
		attributes: anAssociationCollection
		ifNone: [nil]! !

!XMLElementClassMapper methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:29'!
classForElement: aQualifiedName namespaceURI: aURI localName: aLocalName attributes: aDictionary ifNone: aBlock
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'private' stamp: 'monty 8/26/2016 00:44'!
classNameFrom: aClass withoutIgnorableSuffixes: aSuffixCollection
	| className |

	className := aClass name asString.
	aSuffixCollection do: [:each |
		(className endsWith: each)
			ifTrue: [^ className allButLast: each size]].
	^ className.! !

!XMLElementClassMapper methodsFor: 'private' stamp: 'monty 7/27/2016 08:17'!
classNameSegmentsFrom: aString
	| nameSegments startIndex |

	nameSegments := OrderedCollection new.
	startIndex := 1.
	1 to: aString size do: [:i |
		(((aString at: i) isLowercase
			and: [aString size >= (i + 1)
				and: [(aString at: i + 1) isLowercase not]])
			or: [
				((aString at: i) isUppercase
					and: [aString size >= (i + 2)
						and: [(aString at: i + 1) isUppercase
							and: [(aString at: i + 2) isUppercase not]]])])
			ifTrue: [
				nameSegments addLast:
					(aString
						copyFrom: startIndex
						to: i).
				startIndex := i + 1]].

	startIndex <= aString size
		ifTrue: [
			nameSegments addLast:
				(aString
					copyFrom: startIndex
					to: aString size)].

	^ nameSegments.! !

!XMLElementClassMapper methodsFor: 'private' stamp: 'monty 7/24/2016 12:28'!
hyphenatedNameFromClassNameSegments: aNameSegmentCollection
	^ String streamContents: [:stream |
		aNameSegmentCollection
			do: [:each | stream nextPutAll: each asLowercase]
			separatedBy: [stream nextPut: $-]]! !

!XMLElementClassMapper methodsFor: 'mapping' stamp: 'monty 8/26/2016 02:19'!
mapAllMatchingElementsToClass: aClass ignoringSuffixes: aSuffixCollection
	| classNameWithoutSuffixes classNameSegments totalClassNameSegments |

	classNameWithoutSuffixes :=
		self
			classNameFrom: aClass
			withoutIgnorableSuffixes: aSuffixCollection.
	classNameSegments := self classNameSegmentsFrom: classNameWithoutSuffixes.
	(totalClassNameSegments := classNameSegments size) > 0
		ifFalse: [^ self].

	"remove the class prefix if any"
	totalClassNameSegments > 1
		ifTrue: [classNameSegments removeFirst].
	self
		mapElement:
			(self nameFromClassNameSegments: classNameSegments)
		toClass: aClass.
	self
		mapElement:
			(self camelCasedNameFromClassNameSegments: classNameSegments)
		toClass: aClass.
	self
		mapElement:
			(self hyphenatedNameFromClassNameSegments: classNameSegments)
		toClass: aClass.
	self
		mapElement:
			(self underscoredNameFromClassNameSegments: classNameSegments)
		toClass: aClass.! !

!XMLElementClassMapper methodsFor: 'mapping' stamp: 'monty 8/26/2016 00:46'!
mapAllMatchingElementsToClasses: aClassCollection ignoringSuffixes: aSuffixCollection
	aClassCollection do: [:each |
		self
			mapAllMatchingElementsToClass: each
			ignoringSuffixes: aSuffixCollection]! !

!XMLElementClassMapper methodsFor: 'mapping' stamp: 'monty 7/26/2016 08:30'!
mapElement: anElement attributeSpec: anAttributeSpec toClass: aClass
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'mapping' stamp: 'monty 7/26/2016 08:30'!
mapElement: anElement namespaceURI: aURI attributeSpec: anAttributeSpec toClass: aClass
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'mapping' stamp: 'monty 7/26/2016 06:20'!
mapElement: anElement namespaceURI: aURI toClass: aClass
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'mapping' stamp: 'monty 7/26/2016 06:21'!
mapElement: anElement toClass: aClass
	self subclassResponsibility! !

!XMLElementClassMapper methodsFor: 'mapping' stamp: 'monty 8/15/2016 04:25'!
mapElements: anElementCollection namespaceURI: aNamespaceURI toClass: aClass
	anElementCollection do: [:each |
		self
			mapElement: each
			namespaceURI: aNamespaceURI
			toClass: aClass]! !

!XMLElementClassMapper methodsFor: 'mapping' stamp: 'monty 8/15/2016 04:25'!
mapElements: anElementCollection toClass: aClass
	anElementCollection do: [:each |
		self
			mapElement: each
			toClass: aClass]! !

!XMLElementClassMapper methodsFor: 'private' stamp: 'monty 7/27/2016 08:10'!
nameFromClassNameSegments: aNameSegmentCollection
	"avoid join: for Squeak/GS portability"
	^ String streamContents: [:stream |
		aNameSegmentCollection do: [:each | stream nextPutAll: each]]! !

!XMLElementClassMapper methodsFor: 'private' stamp: 'monty 7/28/2016 06:40'!
underscoredNameFromClassNameSegments: aNameSegmentCollection
	^ String streamContents: [:stream |
		aNameSegmentCollection
			do: [:each | stream nextPutAll: each asLowercase]
			separatedBy: [stream nextPut: $_]]! !

Object subclass: #XMLElementIDValidator
	instanceVariableNames: 'elementsAndIdAttributes idReferences ids'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLElementIDValidator commentStamp: 'monty 1/3/2015 04:41' prior: 0!
This class is shared by attribute validators to validate element IDs and ID references, including xml:id IDs.!

!XMLElementIDValidator methodsFor: 'accessing' stamp: 'monty 3/2/2015 12:53'!
elementsAndIDAttributes
	^ elementsAndIdAttributes ifNil: [elementsAndIdAttributes := Dictionary new]! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
errorDuplicateID: anID inAttribute: anAttribute
	XMLValidationException
		formatSignal: 'Duplicate element ID "{1}" used in attribute "{2}"'
		with: anID
		with: anAttribute! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
errorInvalidID: anID inAttribute: anAttribute
	XMLValidationException
		formatSignal: 'Invalid element ID value "{1}" in attribute "{2}"'
		with: anID
		with: anAttribute! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 1/1/2015 19:00'!
errorMultipleIDAttributesForElement: anElement
	XMLValidationException
		formatSignal: 'Element <{1}> has multiple ID attributes'
		with: anElement! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
errorUnknownIDReference: anID
	XMLValidationException
		formatSignal: 'Reference to unknown element ID "{1}"'
		with: anID! !

!XMLElementIDValidator methodsFor: 'accessing' stamp: 'monty 1/3/2015 23:25'!
idReferences
	^ idReferences ifNil: [idReferences := Set new]! !

!XMLElementIDValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 18:54'!
ids
	^ ids ifNil: [ids := Set new]! !

!XMLElementIDValidator methodsFor: 'removing' stamp: 'monty 11/14/2015 20:05'!
removeIDsAndIDReferences
	"Gemstone's Set does not understand removeAll, so nil reassignment
	is used instead (will be lazy initialized by accessors)"
	ids := nil.
	idReferences := nil.! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 3/24/2016 20:55'!
validateID: anID inAttribute: anAttribute
	anID isXMLName
		ifFalse: [
			self
				errorInvalidID: anID
				inAttribute: anAttribute].
	self
		validateUniquenessOfID: anID
		inAttribute: anAttribute.! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 3/2/2015 12:53'!
validateIDAttribute: anAttribute forElement: anElement
	| oldSize |

	"checking size after is faster than includes test"
	oldSize := self elementsAndIDAttributes size.
	(self elementsAndIDAttributes
		at: anElement
		put: anAttribute;
		size) > oldSize
		ifFalse: [self errorMultipleIDAttributesForElement: anElement].! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 3/24/2016 20:55'!
validateIDReference: anID inAttribute: anAttribute
	anID isXMLName
		ifFalse: [
			self
				errorInvalidID: anID
				inAttribute: anAttribute].
	self idReferences add: anID.! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 1/3/2015 23:24'!
validateIDReferences
	self idReferences do: [:each |
		(self ids includes: each)
			ifFalse: [self errorUnknownIDReference: each]]! !

!XMLElementIDValidator methodsFor: 'private' stamp: 'monty 3/22/2015 03:50'!
validateUniquenessOfID: anID inAttribute: anAttribute
	| oldSize |

	"checking size after is faster than includes test"
	oldSize := self ids size.
	(self ids
		add: anID;
		size) > oldSize
		ifFalse: [
			self
				errorDuplicateID: anID
				inAttribute: anAttribute].! !

!XMLElementIDValidator methodsFor: 'validating' stamp: 'monty 3/24/2016 20:56'!
validateXMLID: anID
	anID isXMLNCName
		ifFalse: [
			self
				errorInvalidID: anID
				inAttribute: 'xml:id'].
	self
		validateUniquenessOfID: anID
		inAttribute: 'xml:id'.! !

Object subclass: #XMLElementNester
	instanceVariableNames: 'openElements rootElement hasExternalRootElement scopedAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLElementNester commentStamp: 'monty 12/27/2014 03:06' prior: 0!
This is a base class for element nesters that are used by parser drivers to keep track of which elements are open while parsing.!

!XMLElementNester methodsFor: 'private' stamp: 'monty 1/25/2014 21:08'!
copiedScopedAttributes
	| copiedAttributes |

	copiedAttributes :=
		self hasScopedAttributes
			ifTrue: [self scopedAttributes removeLast copy]
			ifFalse: [Dictionary new].
	self scopedAttributes addLast: copiedAttributes.

	^ copiedAttributes.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:56'!
currentElement
	^ self openElements xmlLastOrNil! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:56'!
currentScopedAttributes
	^ self scopedAttributes xmlLastOrNil! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 4/25/2015 15:07'!
elementPrintLimit
	^ 5! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 8/6/2016 19:42'!
endElement: anElement
	self openElements removeLast.
	self hasScopedAttributes
		ifTrue: [self endScopedAttributes].! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 2/2/2014 02:45'!
endNesting! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 4/16/2015 01:07'!
endScopedAttributes
	scopedAttributes removeLast! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 11/3/2014 13:20'!
hasExternalRootElement
	^ hasExternalRootElement! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 8/6/2016 19:42'!
hasOpenElement: anElement
	^ self openElements includes: anElement! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 11/3/2014 13:18'!
hasOpenElements
	^ self openElements size > 0! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 3/23/2016 22:00'!
hasScopedAttributes
	"optimized to avoid lazy initialization"
	scopedAttributes == nil
		ifTrue: [^ false].
	^ scopedAttributes size > 0.! !

!XMLElementNester methodsFor: 'initialization' stamp: 'monty 11/3/2014 13:20'!
initialize
	super initialize.

	openElements := OrderedCollection new: 10.
	hasExternalRootElement := false.! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 2/10/2014 02:12'!
isCurrentElementRoot
	^ self openElements size = 1! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 2/20/2016 02:40'!
isInElement
	^ self openElements size > 0
		or: [self hasExternalRootElement]! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:07'!
openElements
	^ openElements! !

!XMLElementNester methodsFor: 'enumerating' stamp: 'monty 11/14/2015 19:53'!
openElementsDo: aBlock
	self openElements do: aBlock! !

!XMLElementNester methodsFor: 'enumerating' stamp: 'monty 11/14/2015 22:19'!
openElementsReverseDo: aBlock
	"implemented for XMLParserHTML so it's always safe to remove at
	the end while scanning backwards"
	| i |

	i := self openElements size.
	[i > 0]
		whileTrue: [
			aBlock value: (self openElements at: i).
			i := i - 1].! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 8/6/2016 20:32'!
printElementsOn: aStream
	self
		printElementsStartingAt: 1
		on: aStream! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 8/6/2016 20:34'!
printElementsStartingAt: aStart on: aStream
	self
		printElementsStartingAt: aStart
		stoppingAt: aStart + self elementPrintLimit - 1
		on: aStream! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 8/6/2016 20:34'!
printElementsStartingAt: aStart stoppingAt: aStop on: aStream
	aStart
		to: (aStop min: self openElements size)
		do: [:i |
			i > aStart
				ifTrue: [aStream nextPutAll: ', '].

			aStream
				nextPut: $<;
				nextPutAll: (self openElements at: i);
				nextPut: $>].

	(aStop < self openElements size)
		ifTrue: [aStream nextPutAll: '...'].! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 8/6/2016 20:32'!
printOn: aStream
	super printOn: aStream.
	self hasOpenElements
		ifTrue: [
			aStream nextPut: $(.
			self
				printElementsStartingAt: 1
				on: aStream.
			aStream nextPut: $)].! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 4/22/2015 11:38'!
printedElements
	^ String streamContents: [:stream | self printElementsOn: stream]! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 8/6/2016 20:32'!
printedElementsEnclosedBy: anElement
	^ String streamContents: [:stream |
		self
			printElementsStartingAt: (self openElements lastIndexOf: anElement) + 1
			on: stream]! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 8/6/2016 19:42'!
scopedAttributeAt: anAttribute
	^ self
		scopedAttributeAt: anAttribute
		ifAbsent: ['']! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 8/6/2016 19:42'!
scopedAttributeAt: anAttribute ifAbsent: aBlock
	^ self currentScopedAttributes
		ifNil: [aBlock value]
		ifNotNil: [:attributes |
			attributes
				at: anAttribute
				ifAbsent: aBlock]! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 1/20/2014 19:33'!
scopedAttributes
	^ scopedAttributes ifNil: [scopedAttributes := OrderedCollection new]! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 11/3/2014 13:20'!
setExternalRootElement: anElement
	rootElement := anElement.
	hasExternalRootElement := true.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 8/6/2016 19:43'!
setScopedAttributesNamed: anAttributeCollection from: aDictionary
	| attributes |

	anAttributeCollection do: [:each |
		aDictionary
			at: each
			ifPresent: [:value |
				"used to copy only once if needed"
				(attributes ifNil: [attributes := self copiedScopedAttributes])
					at: each
					put: value]]! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 8/6/2016 19:42'!
startElement: anElement
	self openElements addLast: anElement.
	self hasScopedAttributes
		ifTrue: [self startScopedAttributes].! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 4/16/2015 01:06'!
startScopedAttributes
	scopedAttributes addLast: self currentScopedAttributes! !

XMLElementNester subclass: #XMLWellFormedElementNester
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLWellFormedElementNester commentStamp: '<historical>' prior: 0!
The class keeps track of elements as they start and end and ensures that they nest properly.!

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 8/6/2016 19:43'!
endElement: anElement
	(self currentElement = anElement)
		ifFalse: [
			^ (self hasOpenElement: anElement)
				ifTrue: [self errorImproperlyNestedElements: anElement]
				ifFalse: [self errorUnmatchedEndTag: anElement]].

	super endElement: anElement.! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 11/3/2014 13:25'!
endNesting
	self hasOpenElements
		ifTrue: [self errorUnterminatedElements]! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 8/6/2016 19:44'!
errorImproperlyNestedElements: anElement
	XMLWellFormednessException
		formatSignal: '<{1}> element ended before enclosed element(s) {2}'
		with: anElement
		with: (self printedElementsEnclosedBy: anElement)! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 11/3/2014 13:25'!
errorMultipleRoots
	XMLWellFormednessException signal: 'Document cannot have multiple root elements'! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 8/6/2016 19:44'!
errorUnmatchedEndTag: anElement
	XMLWellFormednessException
		formatSignal: 'End </{1}> tag with no corresponding start <{1}> tag'
		with: anElement! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 4/22/2015 11:36'!
errorUnterminatedElements
	XMLWellFormednessException signal: 'Unterminated elements: ', self printedElements! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 8/6/2016 19:43'!
startElement: anElement
	rootElement
		ifNil: [rootElement := anElement]
		ifNotNil: [
			(self openElements size > 0
				or: [self hasExternalRootElement])
				ifFalse: [self errorMultipleRoots]].

	super startElement: anElement.! !

Object subclass: #XMLElementValidator
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLElementValidator commentStamp: 'monty 4/21/2016 08:08' prior: 0!
This is a base class for element validators.!

XMLElementValidator subclass: #XMLAnyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAnyElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain any content.!

!XMLAnyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:42'!
atEnd
	^ true! !

!XMLAnyElementValidator methodsFor: 'accessing' stamp: 'monty 4/13/2016 06:20'!
type
	^ 'ANY'! !

XMLElementValidator subclass: #XMLDFAElementValidator
	instanceVariableNames: 'dfa currentState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDFAElementValidator commentStamp: 'monty 4/21/2016 08:11' prior: 0!
This class validates elements using a regular expression converted to a DFA. It ignores PCDATA, comments, and PIs.!

XMLDFAElementValidator subclass: #XMLDFAElementContentElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDFAElementContentElementValidator commentStamp: 'monty 4/21/2016 08:11' prior: 0!
This class validates elements that can only contain element content (no PCDATA) using a regular expression converted to a DFA. It ignores comments, and PIs.!

!XMLDFAElementContentElementValidator methodsFor: 'testing' stamp: 'monty 4/13/2016 06:18'!
hasElementContent
	^ true! !

!XMLDFAElementContentElementValidator methodsFor: 'accessing' stamp: 'monty 4/13/2016 06:20'!
type
	^ 'element content'! !

!XMLDFAElementContentElementValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 06:24'!
validatePCData: aString
	self errorUnexpectedPCData! !

!XMLDFAElementValidator class methodsFor: 'instance creation' stamp: 'monty 4/13/2016 06:02'!
element: anElement dfa: aDFA
	^ self new
		setElement: anElement
		dfa: aDFA! !

!XMLDFAElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 05:43'!
atEnd
	^ currentState isAccepting! !

!XMLDFAElementValidator methodsFor: 'accessing' stamp: 'monty 4/22/2015 09:04'!
currentState
	^ currentState! !

!XMLDFAElementValidator methodsFor: 'accessing' stamp: 'monty 4/22/2015 09:04'!
dfa
	^ dfa! !

!XMLDFAElementValidator methodsFor: 'private' stamp: 'monty 4/13/2016 06:19'!
errorNonDeterminismInContentModelAfter: anElement
	XMLValidationException
		formatSignal:
			'Content model for <{1}> becomes non-deterministic after <{2}>. ',
			'Rewrite it deterministically; for example "{3}" instead ',
			'of "{4}"'
		with: self element
		with: anElement
		with: '(a,(b|c))'
		with: '((a,b)|(a,c))'! !

!XMLDFAElementValidator methodsFor: 'initialization' stamp: 'monty 4/13/2016 06:02'!
setElement: anElement dfa: aDFA
	element := anElement.
	dfa := aDFA.
	currentState := aDFA.! !

!XMLDFAElementValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 06:02'!
validateStartTag: anElement
	currentState :=
		currentState
			nextDFAStateMatching: anElement
			ifNone: [self errorUnexpectedElement: anElement].
	currentState hasNonDeterminism
		ifTrue: [self errorNonDeterminismInContentModelAfter: anElement].! !

!XMLElementValidator class methodsFor: 'instance creation' stamp: 'monty 4/13/2016 05:59'!
element: anElement
	^ self new element: anElement! !

!XMLElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	self subclassResponsibility! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element
	^ element! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 4/13/2016 06:00'!
element: anElement
	element := anElement! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 4/13/2016 06:34'!
errorUnexpected: aDescription
	XMLValidationException
		formatSignal: 'Unexpected {1} in {2} type element <{3}>'
		with: aDescription
		with: self type
		with: self element! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 4/13/2016 06:23'!
errorUnexpectedComment
	^ self errorUnexpected: 'comment'! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 4/13/2016 06:34'!
errorUnexpectedElement: anElement
	XMLValidationException
		formatSignal: 'Unexpected <{1}> element in {2} type element <{3}>'
		with: anElement
		with: self type
		with: self element! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 4/13/2016 06:23'!
errorUnexpectedPCData
	^ self errorUnexpected: 'PCDATA'! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 4/13/2016 06:24'!
errorUnexpectedPI
	^ self errorUnexpected: 'processing instruction'! !

!XMLElementValidator methodsFor: 'testing' stamp: 'monty 4/12/2016 04:20'!
hasElementContent
	^ false! !

!XMLElementValidator methodsFor: 'printing' stamp: 'monty 4/24/2015 08:54'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPutAll: '(<';
		nextPutAll: self element;
		nextPutAll: '>)'.! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 4/13/2016 06:20'!
type
	self subclassResponsibility! !

!XMLElementValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 05:51'!
validateComment: aString! !

!XMLElementValidator methodsFor: 'validating' stamp: 'monty 11/18/2013 05:24'!
validatePCData: aString! !

!XMLElementValidator methodsFor: 'validating' stamp: 'monty 4/16/2016 13:44'!
validatePI: aTargetString data: aDataString! !

!XMLElementValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 06:00'!
validateStartTag: anElement! !

XMLElementValidator subclass: #XMLEmptyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEmptyElementValidator commentStamp: 'monty 4/13/2016 06:48' prior: 0!
This class validates EMPTY elements, which can't contain elements, PCDATA, or even comments or PIs.!

!XMLEmptyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	^ true! !

!XMLEmptyElementValidator methodsFor: 'accessing' stamp: 'monty 4/13/2016 06:20'!
type
	^ 'EMPTY'! !

!XMLEmptyElementValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 06:24'!
validateComment: aString
	self errorUnexpectedComment! !

!XMLEmptyElementValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 06:24'!
validatePCData: aString
	self errorUnexpectedPCData! !

!XMLEmptyElementValidator methodsFor: 'validating' stamp: 'monty 4/16/2016 13:44'!
validatePI: aTargetString data: aDataString
	self errorUnexpectedPI! !

!XMLEmptyElementValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 06:25'!
validateStartTag: anElement
	self errorUnexpectedElement: anElement! !

XMLElementValidator subclass: #XMLMixedContentElementValidator
	instanceVariableNames: 'allowedElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLMixedContentElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain a mix of PCDATA and certain elements.!

!XMLMixedContentElementValidator class methodsFor: 'instance creation' stamp: 'monty 4/13/2016 06:02'!
element: anElement allowedElements: anAllowedElementsCollection
	^ self new
		setElement: anElement
		allowedElements: anAllowedElementsCollection! !

!XMLMixedContentElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:51'!
allowedElements
	^ allowedElements! !

!XMLMixedContentElementValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:14'!
atEnd
	^ true! !

!XMLMixedContentElementValidator methodsFor: 'initialization' stamp: 'monty 4/13/2016 06:02'!
setElement: anElement allowedElements: anAllowedElementsCollection
	element := anElement.
	allowedElements := anAllowedElementsCollection asSet.! !

!XMLMixedContentElementValidator methodsFor: 'accessing' stamp: 'monty 4/13/2016 06:21'!
type
	^ 'mixed content'! !

!XMLMixedContentElementValidator methodsFor: 'validating' stamp: 'monty 4/13/2016 06:02'!
validateStartTag: anElement
	(self allowedElements includes: anElement)
		ifFalse: [self errorUnexpectedElement: anElement]
	! !

Object subclass: #XMLEncodingDetector
	instanceVariableNames: 'stream'
	classVariableNames: 'EncodingHandlerTable NullCharacter UTF16BigEndianBOM UTF16LittleEndianBOM UTF32BigEndianBOM UTF32LittleEndianBOM UTF8BOM'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLEncodingDetector commentStamp: 'monty 8/1/2016 15:02' prior: 0!
An abstract class for automatic encoding detectors for streams that infers encodings from byte order marks (BOM) or implicit UTF-16/32 encodings from null bytes before or after the first ASCII character.!

!XMLEncodingDetector class methodsFor: 'private' stamp: 'monty 2/9/2016 05:54'!
handlerForByteValue: aByteValue
	aByteValue = 0
		ifTrue: [^ #handleBigEndianUTF32BOMOrImplicitEncoding].
	aByteValue = XMLUTF16BigEndianStreamConverter byteOrderMarkBytes first
		ifTrue: [^ #handleBigEndianUTF16BOM].
	aByteValue = XMLUTF8StreamConverter byteOrderMarkBytes first
		ifTrue: [^ #handleUTF8BOM].
	aByteValue = XMLUTF16LittleEndianStreamConverter byteOrderMarkBytes first
		ifTrue: [^ #handleLittleEndianUTF16BOMOrUTF32BOM].
	aByteValue < 128
		ifTrue: [^ #handleLittleEndianImplicitEncoding].
	^ nil.! !

!XMLEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 2/9/2016 03:51'!
initialize
	"self initialize"

	self
		initializeByteOrderMarks;
		initializeNullCharacter;
		initializeEncodingHandlerTable! !

XMLEncodingDetector initialize!

!XMLEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 2/9/2016 03:52'!
initializeByteOrderMarks
	UTF8BOM := XMLUTF8StreamConverter byteOrderMark.
	UTF16BigEndianBOM := XMLUTF16BigEndianStreamConverter byteOrderMark.
	UTF16LittleEndianBOM := XMLUTF16LittleEndianStreamConverter byteOrderMark.
	UTF32BigEndianBOM := XMLUTF32BigEndianStreamConverter byteOrderMark.
	UTF32LittleEndianBOM := XMLUTF32LittleEndianStreamConverter byteOrderMark.! !

!XMLEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 2/9/2016 03:52'!
initializeEncodingHandlerTable
	EncodingHandlerTable := Array new: 256.

	0 to: 255 do: [:each |
		EncodingHandlerTable
			at: each + 1
			put: (self handlerForByteValue: each)].! !

!XMLEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 2/9/2016 03:52'!
initializeNullCharacter
	NullCharacter := Character value: 0! !

!XMLEncodingDetector class methodsFor: 'instance creation' stamp: 'monty 3/8/2016 13:11'!
on: aStream
	^ self new on: aStream! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 06:20'!
atEnd
	^ stream atEnd! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/14/2016 02:03'!
canSkipDetection
	^ self atEnd! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:53'!
decodeStreamWithConverter: aStreamConverter
	self subclassResponsibility! !

!XMLEncodingDetector methodsFor: 'detecting' stamp: 'monty 2/14/2016 02:03'!
detectCurrentEncoding
	| handlerTableIndex handlerSelector |

	self canSkipDetection
		ifTrue: [^ self].

	"the index is asciiValue + 1 for null"
	((handlerTableIndex := self peek asciiValue + 1) <= EncodingHandlerTable size
		and: [(handlerSelector := EncodingHandlerTable at: handlerTableIndex) notNil])
		ifTrue: [self perform: handlerSelector].! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:52'!
handleBOM: aByteOrderMark forConverterClass: aConverterClass
	self
		handleBOM: aByteOrderMark
		forConverterClass: aConverterClass
		ifUnmatched: [:matchedPortion | self pushBack: matchedPortion]! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 4/5/2016 03:11'!
handleBOM: aByteOrderMark forConverterClass: aConverterClass ifUnmatched: aBlock
	"to:do: for speed"
	1 to: aByteOrderMark size do: [:i |
		self peek = (aByteOrderMark at: i)
			ifFalse: [
				aBlock value: (aByteOrderMark copyFrom: 1 to: i - 1).
				^ self].
		self next].

	self decodeStreamWithConverter: aConverterClass new.! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:51'!
handleBigEndianUTF16BOM
	self
		handleBOM: UTF16BigEndianBOM
		forConverterClass: XMLUTF16BigEndianStreamConverter! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 06:05'!
handleBigEndianUTF32BOMOrImplicitEncoding
	self
		handleBOM: UTF32BigEndianBOM
		forConverterClass: XMLUTF32BigEndianStreamConverter
		ifUnmatched: [:matchedPortion | | nextChar |
			(matchedPortion size = 1 "one null char"
				and: [(nextChar := self nextASCII) notNil])
				ifTrue: [
					self
						decodeStreamWithConverter: XMLUTF16BigEndianStreamConverter new;
						pushBackCharacter: nextChar]
				ifFalse: [
					(matchedPortion size = 2  "two null chars"
						and: [self peekForNull])
						ifTrue: [
							(nextChar := self nextASCII) notNil
								ifTrue: [	
									self
										decodeStreamWithConverter: XMLUTF32BigEndianStreamConverter new;
										pushBackCharacter: nextChar]
								ifFalse: [self pushBack: (matchedPortion copyWith: NullCharacter)]]
						ifFalse: [self pushBack: matchedPortion]]]! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 06:05'!
handleLittleEndianImplicitEncoding
	| nextChar |

	nextChar := self nextASCII.
	self peekForNull
		ifTrue: [
			self peekForNull
				ifTrue: [
					self next.
					self decodeStreamWithConverter:
						XMLUTF32LittleEndianStreamConverter new]
				ifFalse: [
					self decodeStreamWithConverter:
						XMLUTF16LittleEndianStreamConverter new]].
	self pushBackCharacter: nextChar.! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:52'!
handleLittleEndianUTF16BOMOrUTF32BOM
	self
		handleBOM: UTF32LittleEndianBOM
		forConverterClass: XMLUTF32LittleEndianStreamConverter
		ifUnmatched: [:matchedPortion |
			matchedPortion = UTF16LittleEndianBOM
				ifTrue: [
					self decodeStreamWithConverter:
						XMLUTF16LittleEndianStreamConverter new]
				ifFalse: [self pushBack: matchedPortion]]! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:51'!
handleUTF8BOM
	self
		handleBOM: UTF8BOM
		forConverterClass: XMLUTF8StreamConverter! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:56'!
next
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ stream next]! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 06:18'!
nextASCII
	| nextChar |

	((nextChar := self peek) notNil
		and: [nextChar asciiValue < 128])
		ifTrue: [
			self next.
			^ nextChar]
		ifFalse: [^ nil]! !

!XMLEncodingDetector methodsFor: 'initialization' stamp: 'monty 4/1/2016 00:55'!
on: aStream
	stream :=
		aStream isXMLConvertingStreamAdapter
			ifTrue: [aStream stream]
			ifFalse: [aStream]! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:56'!
peek
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ stream peek]! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 23:14'!
peekForNull
	self peek == NullCharacter
		ifTrue: [
			self next.
			^ true]
		ifFalse: [^ false]! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:53'!
pushBack: aString
	self subclassResponsibility! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:53'!
pushBackCharacter: aCharacter
	self subclassResponsibility! !

XMLEncodingDetector subclass: #XMLFileStreamEncodingDetector
	instanceVariableNames: 'decodingReadStreamAdapter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLFileStreamEncodingDetector commentStamp: 'monty 2/9/2016 06:42' prior: 0!
An automatic encoding detector for file streams wrapped by decoding read stream adapters. It assumes the wrapped stream supports reset.!

!XMLFileStreamEncodingDetector class methodsFor: 'instance creation' stamp: 'monty 3/9/2016 18:43'!
on: aDecodingReadStreamAdapter
	^ (super on: aDecodingReadStreamAdapter stream)
		decodingReadStreamAdapter: aDecodingReadStreamAdapter! !

!XMLFileStreamEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 06:36'!
decodeStreamWithConverter: aStreamConverter
	decodingReadStreamAdapter streamConverter: aStreamConverter! !

!XMLFileStreamEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 06:34'!
decodingReadStreamAdapter: aDecodingReadStreamAdapter
	decodingReadStreamAdapter := aDecodingReadStreamAdapter! !

!XMLFileStreamEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 06:42'!
pushBack: aString
	"file streams should support reset"
	stream reset! !

!XMLFileStreamEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 06:42'!
pushBackCharacter: aCharacter
	"file streams should support reset"
	stream reset! !

XMLEncodingDetector subclass: #XMLNestedStreamReaderEncodingDetector
	instanceVariableNames: 'streamReader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNestedStreamReaderEncodingDetector commentStamp: 'monty 2/9/2016 06:33' prior: 0!
An automatic encoding detector for stream reader streams.!

!XMLNestedStreamReaderEncodingDetector class methodsFor: 'instance creation' stamp: 'monty 3/8/2016 13:12'!
on: aStreamReader
	^ (super on: aStreamReader stream)
		streamReader: aStreamReader! !

!XMLNestedStreamReaderEncodingDetector methodsFor: 'private' stamp: 'monty 4/1/2016 00:55'!
canSkipDetection
	^ super canSkipDetection
		or: [stream isXMLConvertingStreamAdapter
			and: [stream hasDefaultStreamConverter not]]! !

!XMLNestedStreamReaderEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:54'!
decodeStreamWithConverter: aStreamConverter
	streamReader decodeStreamWithConverter: aStreamConverter! !

!XMLNestedStreamReaderEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:53'!
pushBack: aString
	streamReader pushBack: aString! !

!XMLNestedStreamReaderEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 05:53'!
pushBackCharacter: aCharacter
	streamReader pushBackCharacter: aCharacter! !

!XMLNestedStreamReaderEncodingDetector methodsFor: 'private' stamp: 'monty 2/9/2016 03:55'!
streamReader: aStreamReader
	streamReader := aStreamReader! !

Object subclass: #XMLIntegerReader
	instanceVariableNames: 'readStream printStream'
	classVariableNames: 'CharacterDigitValues MaxIntegerValue'
	poolDictionaries: ''
	category: 'XML-Parser-Utils'!

!XMLIntegerReader commentStamp: 'monty 1/8/2016 03:08' prior: 0!
This class reads positive hex or decimal integers up to the maximum a SmallInteger can hold from minimal streams supporting next, peek, and atEnd. It returns nil instead of raising errors for invalid integers.!

!XMLIntegerReader class methodsFor: 'class initialization' stamp: 'monty 1/7/2016 05:47'!
initialize
	"self initialize"

	self
		initializeMaxIntegerValue;
		initializeCharacterDigitValues! !

XMLIntegerReader initialize!

!XMLIntegerReader class methodsFor: 'class initialization' stamp: 'monty 3/16/2016 21:21'!
initializeCharacterDigitValues
	"array instead of dictionary for speed"
	CharacterDigitValues := Array new: 255.
	#(#($0 $9 0) #($a $f 10) #($A $Z 10)) do: [:each |
		each first asciiValue
			to: each second asciiValue
			do: [:i |
				CharacterDigitValues
					at: i
					put: i - each first asciiValue + each last]].! !

!XMLIntegerReader class methodsFor: 'class initialization' stamp: 'monty 1/7/2016 05:48'!
initializeMaxIntegerValue
	MaxIntegerValue := SmallInteger maxVal! !

!XMLIntegerReader class methodsFor: 'private' stamp: 'monty 10/24/2014 20:28'!
maxBase
	^ 16! !

!XMLIntegerReader class methodsFor: 'private' stamp: 'monty 1/7/2016 05:48'!
maxIntegerValue
	^ MaxIntegerValue! !

!XMLIntegerReader class methodsFor: 'instance creation' stamp: 'monty 1/6/2016 07:49'!
on: aStringOrStream
	^ self new on: aStringOrStream! !

!XMLIntegerReader class methodsFor: 'instance creation' stamp: 'monty 10/27/2014 08:36'!
readFrom: aStream
	^ (self on: aStream) nextInteger! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 10/27/2014 08:37'!
readFrom: aStream withBase: aBase
	^ (self on: aStream) nextIntegerWithBase: aBase! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 10/27/2014 08:43'!
readFrom: aStream withBase: aFirstBase startingWithAny: aCharacterCollection orBase: aSecondBase
	^ (self on: aStream)
		nextIntegerWithBase: aFirstBase
		startingWithAny: aCharacterCollection
		orBase: aSecondBase! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 1/21/2014 17:42'!
readFrom: aStream withBase: aBase upTo: anInteger
	^ (self on: aStream)
		nextIntegerWithBase: aBase
		upTo: anInteger! !

!XMLIntegerReader methodsFor: 'testing' stamp: 'monty 12/2/2015 20:03'!
atEnd
	^ readStream atEnd! !

!XMLIntegerReader methodsFor: 'private' stamp: 'monty 1/3/2016 21:13'!
next
	| nextChar |

	self atEnd
		ifFalse: [
			nextChar := readStream next.
			printStream
				ifNotNil: [printStream nextPut: nextChar]].
	^ nextChar.! !

!XMLIntegerReader methodsFor: 'private' stamp: 'monty 3/16/2016 21:21'!
nextDigitWithBase: aBase
	| asciiValue digit |

	((asciiValue := self peek asciiValue) > 0
		and: [asciiValue <= CharacterDigitValues size
			and: [(digit := CharacterDigitValues at: asciiValue) notNil
				and: [digit < aBase]]])
				ifTrue: [
					self next.
					^ digit]
				ifFalse: [^ nil]! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:37'!
nextInteger
	^ self nextIntegerWithBase: 10! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:35'!
nextIntegerWithBase: aBase
	^ self nextIntegerWithBase: aBase upTo: nil! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:44'!
nextIntegerWithBase: aFirstBase startingWithAny: aCharacterCollection orBase: aSecondBase
	(aCharacterCollection includes: self peek)
		ifTrue: [
			self next.
			^ self nextIntegerWithBase: aFirstBase]
		ifFalse: [^ self nextIntegerWithBase: aSecondBase]! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 1/3/2016 21:24'!
nextIntegerWithBase: aBase upTo: aReadLimit
	| integer maxValue maxBaseValue digitValue readLimit |

	(aBase > 1 and: [aBase <= self class maxBase])
		ifFalse: [Error signal: 'Unsupported base ', aBase printString].

	maxValue := self class maxIntegerValue.
	maxBaseValue := (maxValue / aBase) floor.
	readLimit := aReadLimit.
	[self atEnd
		or: [(readLimit notNil and: [readLimit <= 0])
			or: [(digitValue := self nextDigitWithBase: aBase) isNil]]]
		whileFalse: [
			integer
				ifNil: [integer := 0].
	
			integer > maxBaseValue
				ifTrue: [^ maxValue].
			integer := integer * aBase.

			integer > (maxValue - digitValue)
				ifTrue: [^ maxValue].
			integer := integer + digitValue.
			readLimit
				ifNotNil: [readLimit := readLimit - 1]].
	^ integer.! !

!XMLIntegerReader methodsFor: 'initialization' stamp: 'monty 1/6/2016 07:49'!
on: aStringOrStream
	readStream :=
		aStringOrStream isStream
			ifTrue: [aStringOrStream]
			ifFalse: [aStringOrStream readStream]! !

!XMLIntegerReader methodsFor: 'private' stamp: 'monty 1/3/2016 21:16'!
peek
	self atEnd
		ifTrue: [^ nil]
		ifFalse: [^ readStream peek]! !

!XMLIntegerReader methodsFor: 'accessing' stamp: 'monty 12/2/2015 20:04'!
printStream: aStream
	printStream := aStream! !

Object subclass: #XMLKeyValueCache
	instanceVariableNames: 'collection maxSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Utils'!

!XMLKeyValueCache commentStamp: 'monty 2/17/2016 00:41' prior: 0!
A simple key-value cache that automatically empties when it reaches a maximum size.!

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:57'!
collection: aCollection maxSize: anInteger
	^ self basicNew initialize
		setCollection: aCollection
		maxSize: anInteger! !

!XMLKeyValueCache class methodsFor: 'private' stamp: 'monty 11/20/2014 02:25'!
defaultMaxSize
	^ 16! !

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 4/23/2015 17:02'!
maxSize: anInteger
	^ self
		collection: Dictionary new
		maxSize: anInteger! !

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:56'!
new
	^ self maxSize: self defaultMaxSize! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 2/16/2016 06:27'!
at: aKey
	^ self
		at: aKey
		ifAbsent: [nil]! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 7/23/2016 10:33'!
at: aKey ifAbsent: aBlock
	^ collection
		at: aKey
		ifAbsent: aBlock! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 7/23/2016 10:35'!
at: aKey ifAbsentPut: aBlock
	^ collection
		at: aKey
		ifAbsent: [
			collection size >= maxSize
				ifTrue: [self removeAll].

			collection
				at: aKey
				put: aBlock value]! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 7/23/2016 10:33'!
at: aKey put: aValue
	(collection includesKey: aKey)
		ifFalse: [
			collection size >= self maxSize
				ifTrue: [self removeAll]].

	^ collection
		at: aKey
		put: aValue.! !

!XMLKeyValueCache methodsFor: 'clearing' stamp: 'monty 7/23/2016 10:31'!
clear
	self removeAll! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/11/2014 23:45'!
collection
	^ collection! !

!XMLKeyValueCache methodsFor: 'testing' stamp: 'monty 7/23/2016 10:33'!
includesKey: aKey
	^ collection includesKey: aKey! !

!XMLKeyValueCache methodsFor: 'testing' stamp: 'monty 7/23/2016 10:34'!
isEmpty
	^ collection isEmpty! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:57'!
maxSize
	^ maxSize! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:59'!
maxSize: anInteger
	anInteger < maxSize
		ifTrue: [self clear].

	maxSize := anInteger.! !

!XMLKeyValueCache methodsFor: 'testing' stamp: 'monty 8/17/2016 15:18'!
notEmpty
	^ collection notEmpty! !

!XMLKeyValueCache methodsFor: 'copying' stamp: 'monty 1/2/2016 23:56'!
postCopy
	super postCopy.

	collection := collection copy! !

!XMLKeyValueCache methodsFor: 'printing' stamp: 'monty 4/24/2015 18:18'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self size;
		nextPut: $/;
		print: self maxSize;
		nextPut: $).! !

!XMLKeyValueCache methodsFor: 'removing' stamp: 'monty 7/23/2016 10:34'!
removeAll
	collection removeAll! !

!XMLKeyValueCache methodsFor: 'removing' stamp: 'monty 7/23/2016 10:34'!
removeKey: aKey
	^ collection removeKey: aKey! !

!XMLKeyValueCache methodsFor: 'removing' stamp: 'monty 7/23/2016 10:34'!
removeKey: aKey ifAbsent: aBlock
	^ collection
		removeKey: aKey
		ifAbsent: aBlock! !

!XMLKeyValueCache methodsFor: 'initialization' stamp: 'monty 11/15/2014 20:57'!
setCollection: aCollection maxSize: anInteger
	collection := aCollection.
	maxSize := anInteger.! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 7/23/2016 10:34'!
size
	^ collection size! !

Object subclass: #XMLNFABuilder
	instanceVariableNames: 'fragmentStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABuilder commentStamp: 'monty 1/7/2014 19:39' prior: 0!
This class builds an NFA using a stack of NFAFragments.!

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:26'!
applyBranch
	| branchState leftFragment  rightFragment |

	leftFragment := self popFragment.
	rightFragment := self popFragment.
	branchState :=
		XMLNFABranchingState
			leftBranch: leftFragment firstState
			rightBranch: rightFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalStates: (leftFragment combinedTerminalStatesFrom: rightFragment)).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 10/24/2014 12:44'!
applyConcatenation
	| firstFragment secondFragment |

	secondFragment := self popFragment.
	firstFragment := self popFragment.

	firstFragment connectToFragment: secondFragment.

	self pushFragment: firstFragment.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 4/25/2015 09:11'!
applyOptional
	| optionalFragment branchingState |

	optionalFragment := self popFragment.
	branchingState :=
		XMLNFABranchingState branch: optionalFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchingState
			terminalStates:
				(optionalFragment combinedTerminalStatesWith: branchingState))! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:19'!
applyPlus
	| plusFragment branchState |

	plusFragment := self topFragment.
	branchState := XMLNFABranchingState branch: plusFragment firstState.
	plusFragment
		connectToState: branchState;
		terminalState: branchState.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:55'!
applyStar
	| branchState starFragment |

	starFragment := self popFragment.
	branchState := XMLNFABranchingState branch: starFragment firstState.
	starFragment connectToState: branchState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalState: branchState)! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 11/10/2015 17:11'!
buildAcceptingNFAFromFragments
	[self totalFragments > 1]
		whileTrue: [self applyConcatenation].
	^ self popFragment
		connectToState: XMLNFAAcceptingState new;
		firstState.! !

!XMLNFABuilder methodsFor: 'private' stamp: 'monty 6/17/2015 23:17'!
fragmentStack
	^ fragmentStack! !

!XMLNFABuilder methodsFor: 'testing' stamp: 'monty 6/17/2015 23:18'!
hasFragments
	^ self totalFragments > 0! !

!XMLNFABuilder methodsFor: 'initialization' stamp: 'monty 6/17/2015 23:17'!
initialize
	super initialize.
	fragmentStack := OrderedCollection new.! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
popFragment
	^ self fragmentStack removeLast! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
pushFragment: aFragment
	self fragmentStack addLast: aFragment! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 21:41'!
pushMatcher: aMatcher
	| firstState |

	firstState := XMLNFAMatchingState matcher: aMatcher.
	self pushFragment:
		(XMLNFAFragment
			firstState: firstState
			terminalState: firstState).! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 12:01'!
topFragment
	^ self fragmentStack last! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 15:04'!
totalFragments
	^ self fragmentStack size! !

Object subclass: #XMLNFAFragment
	instanceVariableNames: 'firstState terminalStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAFragment commentStamp: 'monty 12/28/2013 21:51' prior: 0!
This is a fragment used to build an NFA by the NFABuilder.!

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalState: aTerminalState
	^ self new
		firstState: aFirstState;
		terminalState: aTerminalState! !

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalStates: aTerminalStateCollection
	^ self new
		firstState: aFirstState;
		terminalStates: aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 10/24/2014 13:21'!
combinedTerminalStatesFrom: aFragment
	^ terminalStates isCollection
		ifTrue: [aFragment terminalStatesAddedTo: terminalStates]
		ifFalse: [
			aFragment terminalStates isCollection
				ifTrue: [self terminalStatesAddedTo: aFragment terminalStates]
				ifFalse: [
					self
						terminalStateCollectionWith: terminalStates
						and: aFragment terminalStates]]! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 10/24/2014 13:07'!
combinedTerminalStatesWith: aState
	^ terminalStates isCollection
		ifTrue: [
			terminalStates
				addLast: aState;
				yourself]
		ifFalse: [self terminalStateCollectionWith: terminalStates and: aState]! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 10/24/2014 12:45'!
connectToFragment: aFragment
	self connectToState: aFragment firstState.
	terminalStates := aFragment terminalStates.! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 10/24/2014 13:06'!
connectToState: aState
	terminalStates isCollection
		ifTrue: [terminalStates do: [:each | each connectToState: aState]]
		ifFalse: [terminalStates connectToState: aState]! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 10:48'!
firstState
	^ firstState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 11:51'!
firstState: aState
	firstState := aState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 10/24/2014 12:35'!
terminalState: aTerminalState
	terminalStates := aTerminalState! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 12:52'!
terminalStateCollectionWith: aFirstState and: aSecondState
	^ (OrderedCollection new: 4)
		addLast: aFirstState;
		addLast: aSecondState;
		yourself! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 12:40'!
terminalStates
	^ terminalStates! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 23:50'!
terminalStates: aTerminalStateCollection
	terminalStates := aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 13:21'!
terminalStatesAddedTo: aCollection
	terminalStates isCollection
		ifTrue: [aCollection addAll: terminalStates]
		ifFalse: [aCollection addLast: terminalStates].
	^ aCollection.! !

Object subclass: #XMLNFAState
	instanceVariableNames: 'lastAddedTo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This is a base class for NFA states.!

XMLNFAState subclass: #XMLNFAAcceptingState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAAcceptingState commentStamp: 'monty 10/7/2014 01:27' prior: 0!
An NFA in this state is considered to accept the input.!

!XMLNFAAcceptingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:23'!
isAccepting
	^ true! !

!XMLNFAAcceptingState methodsFor: 'printing' stamp: 'monty 4/25/2015 23:08'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self identityHash; "only print ids to avoid loops"
		nextPut: $).! !

XMLNFAState subclass: #XMLNFABranchingState
	instanceVariableNames: 'leftBranch rightBranch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABranchingState commentStamp: 'monty 10/7/2014 01:27' prior: 0!
This state can branch off into two different states without matching against anything.!

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:43'!
branch: aLeftBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: nil! !

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 22:04'!
leftBranch: aLeftBranch rightBranch: aRightBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: aRightBranch! !

!XMLNFABranchingState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			leftBranch addTo: aStateSet.
			rightBranch addTo: aStateSet]! !

!XMLNFABranchingState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState
	leftBranch ifNil: [leftBranch := aState].
	rightBranch ifNil: [rightBranch := aState].! !

!XMLNFABranchingState methodsFor: 'printing' stamp: 'monty 4/25/2015 23:09'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self identityHash; "only print ids to avoid loops"
		nextPutAll: ': ';
		print: leftBranch identityHash;
		nextPutAll: ' | ';
		print: rightBranch identityHash;
		nextPut: $).! !

!XMLNFABranchingState methodsFor: 'initialization' stamp: 'monty 12/28/2013 00:45'!
setLeftBranch: aLeftBranch rightBranch: aRightBranch
	leftBranch := aLeftBranch.
	rightBranch := aRightBranch.! !

XMLNFAState subclass: #XMLNFAMatchingState
	instanceVariableNames: 'matcher nextState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAMatchingState commentStamp: 'monty 10/9/2014 20:58' prior: 0!
This state can pass to a next state if the input matches against an object.!

!XMLNFAMatchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 21:15'!
matcher: aMatcher
	^ self new matcher: aMatcher! !

!XMLNFAMatchingState methodsFor: 'connecting' stamp: 'monty 12/28/2013 21:52'!
connectToState: aState
	nextState := aState! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:16'!
matcher: aMatcher
	matcher := aMatcher! !

!XMLNFAMatchingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:15'!
matches: anObject
	^ matcher = anObject! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:13'!
nextState
	^ nextState! !

!XMLNFAMatchingState methodsFor: 'printing' stamp: 'monty 4/25/2015 23:09'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self identityHash; "only print ids to avoid loops"
		nextPutAll: ': ';
		print: matcher;
		nextPutAll: ' -> ';
		print: nextState identityHash;
		nextPut: $).! !

!XMLNFAState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			aStateSet add: self]! !

!XMLNFAState methodsFor: 'converting' stamp: 'monty 4/25/2015 15:15'!
asDFAState
	| nfaStates |

	nfaStates := XMLNFAStateSet new.
	self addTo: nfaStates.
	^ nfaStates asDFAState.! !

!XMLNFAState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 1/7/2014 19:37'!
isAccepting
	^ false! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:10'!
matches: anObject
	^ false! !

Object subclass: #XMLNamespaceScope
	instanceVariableNames: 'prefixMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScope commentStamp: 'monty 12/29/2014 01:12' prior: 0!
A namespace scope is a possibly empty set of mappings from namespace prefixes to URIs and an optional default namespace URI to be used for unprefixed elements. Scopes can be created from other scopes by copying. (Uses copy-on-write to reduce memory use.)!

!XMLNamespaceScope class methodsFor: 'accessing' stamp: 'monty 10/8/2014 23:27'!
xmlNamespaceURI
	^ 'http://www.w3.org/XML/1998/namespace'! !

!XMLNamespaceScope class methodsFor: 'accessing' stamp: 'monty 10/8/2014 23:27'!
xmlnsNamespaceURI
	^ 'http://www.w3.org/2000/xmlns/'! !

!XMLNamespaceScope methodsFor: 'comparing' stamp: 'monty 3/29/2016 06:50'!
= anObject
	self == anObject
		ifTrue: [^ true].

	self species == anObject species
		ifFalse: [^ false].

	self hasNamespaces
		ifFalse: [^ anObject hasNamespaces not].

	"use hasNamespaces first to avoid initializing anObject's prefixMappings"
	^ anObject hasNamespaces
		and: [prefixMappings = anObject prefixMappings].! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 12/29/2014 01:15'!
copiedPrefixMappings
	"replaces current mappings with copy for copy-on-write"
	prefixMappings
		ifNil: [^ self prefixMappings]
		ifNotNil: [^ prefixMappings := prefixMappings copy]! !

!XMLNamespaceScope methodsFor: 'copying' stamp: 'monty 3/26/2016 23:55'!
copy
	"copy-on-write allows copies to share state so no postCopy is needed"
	^ self shallowCopy! !

!XMLNamespaceScope methodsFor: 'mapping' stamp: 'monty 10/10/2014 05:47'!
defaultNamespace
	^ prefixMappings
		ifNil: ['']
		ifNotNil: [prefixMappings at: '' ifAbsent: ['']]! !

!XMLNamespaceScope methodsFor: 'mapping' stamp: 'monty 3/2/2016 19:07'!
defaultNamespace: aURI
	self
		validatePrefix: ''
		beforeMappingTo: aURI.

	"copy-on-write"
	self copiedPrefixMappings
		at: ''
		put: aURI.! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:38'!
errorAttribute: anAttribute aliasesNamespace: aNamespace
	XMLNamespaceException
		formatSignal: 'Attribute "{1}" has alias for namespace "{2}"'
		with: anAttribute
		with: aNamespace! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 19:59'!
errorCannotAliasXMLOrXMLNSWithPrefix: aPrefix
	aPrefix
		ifEmpty: [
			XMLNamespaceException signal:
				'Cannot use "xml" or  "xmlns" namespace URI as default namespace']
		ifNotEmpty: [
			XMLNamespaceException
				formatSignal:
					'Cannot alias the "xml" or "xmlns" namespace URIs with prefix "{1}"'
				with: aPrefix]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/6/2014 12:18'!
errorCannotMapPrefixToEmptyURI
	XMLNamespaceException signal: 'Cannot map namespace prefixes to empty URIs'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 19:59'!
errorCannotMapXMLNSPrefix
	XMLNamespaceException signal:
		'Cannot map "xmlns" prefix to any namespace, even the current "xmlns" namespace'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 20:00'!
errorCannotRemapXMLPrefix
	XMLNamespaceException signal: 'Cannot map the "xml" prefix to a new namespace URI'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 2/26/2016 05:48'!
errorCannotUnmapPrefix: aPrefix
	XMLNamespaceException
		formatSignal: 'Prefix "{1}" was not mapped and cannot be unmapped'
		with: aPrefix! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/24/2015 20:00'!
errorCannotUseXMLNSPrefix	
	XMLNamespaceException signal: 'Cannot use reserved "xmlns" prefix for elements or attributes'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:42'!
errorInvalidPrefix: aPrefix	
	XMLNamespaceException
		formatSignal: 'Prefix "{1}" contains ":" character'
		with: aPrefix! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:40'!
errorInvalidUnprefixedName: aName
	XMLNamespaceException
		formatSignal: 'Name "{1}" contains ":" character'
		with: aName! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:50'!
errorUnmappedPrefix: aPrefix
	XMLNamespaceException
		formatSignal: 'Unmapped namespace prefix "{1}" in name'
		with: aPrefix! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/12/2014 03:21'!
hasDefaultNamespace
	^ self includesPrefix: ''! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 3/23/2016 22:01'!
hasNamespaces
	"optimized to avoid lazy initialization"
	prefixMappings == nil
		ifTrue: [^ false].
	^ prefixMappings size > 0.! !

!XMLNamespaceScope methodsFor: 'comparing' stamp: 'monty 3/29/2016 05:28'!
hash
	| hash |

	hash := self species hash.
	self hasNamespaces
		ifTrue: [hash := hash bitXor: self prefixMappings hash].
	^ hash.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 11/17/2014 16:45'!
includesPrefix: aPrefix
	self prefixMappingAt: aPrefix ifAbsent: [^ false].
	^ true.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 3/11/2016 15:41'!
includesPrefix: aPrefix mappedTo: aURI
	^ (self
		prefixMappingAt: aPrefix
		ifAbsent: [^ false]) = aURI! !

!XMLNamespaceScope methodsFor: 'mapping' stamp: 'monty 2/26/2016 03:31'!
inheritMappingsFrom: anEnclosingScope
	anEnclosingScope hasNamespaces
		ifFalse: [^ self].
	self hasNamespaces
		ifTrue: [| copiedPrefixMappings |
			anEnclosingScope prefixesAndURIsDo: [:prefix :uri |
				(self includesPrefix: prefix)
					ifFalse: [
						"nil test ensures we only copy the first time when needed"
						(copiedPrefixMappings
							ifNil: [copiedPrefixMappings := self copiedPrefixMappings])
							at: prefix
							put: uri]]]
		ifFalse: [
			"can safely share anEnclosingScope's prefixMappings because of
			copy-on-write behavior"
			prefixMappings := anEnclosingScope prefixMappings].! !

!XMLNamespaceScope methodsFor: 'mapping' stamp: 'monty 3/21/2015 22:37'!
mapPrefix: aPrefix to: aURI
	self
		validatePrefix: aPrefix
		beforeMappingTo: aURI.

	"copy-on-write"
	self copiedPrefixMappings
		at: aPrefix
		put: aURI.! !

!XMLNamespaceScope methodsFor: 'mapping' stamp: 'monty 2/26/2016 16:16'!
mapPrefixesAndURIs: anAssociationCollection
	| copiedPrefixMappings |

	anAssociationCollection associationsDo: [:each |
		self
			validatePrefix: each key
			beforeMappingTo: each value.
		"nil test ensures we only copy the first time when needed"
		(copiedPrefixMappings
			ifNil: [copiedPrefixMappings := self copiedPrefixMappings])
			at: each key
			put: each value]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 2/24/2016 03:11'!
mappedPrefixes
	^ prefixMappings
		ifNil: [Array new]
		ifNotNil: [prefixMappings keys]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 3/12/2015 14:15'!
prefixMappingAt: aPrefix
	^ self
		prefixMappingAt: aPrefix
		ifAbsent: ['']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 3/11/2016 15:41'!
prefixMappingAt: aPrefix ifAbsent: aBlock
	^ prefixMappings
		ifNil: [aBlock value]
		ifNotNil: [
			prefixMappings
				at: aPrefix
				ifAbsent: aBlock]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/20/2014 18:46'!
prefixMappings
	^ prefixMappings ifNil: [prefixMappings := OrderPreservingStringDictionary new: 5]! !

!XMLNamespaceScope methodsFor: 'enumerating' stamp: 'monty 11/23/2014 23:20'!
prefixesAndURIsDo: aTwoArgumentBlock
	prefixMappings
		ifNotNil: [prefixMappings keysAndValuesDo: aTwoArgumentBlock]! !

!XMLNamespaceScope methodsFor: 'printing' stamp: 'monty 4/22/2015 11:23'!
printDeclarationsOn: aStream
	| isFirst |

	isFirst := true.
	self prefixesAndURIsDo: [:prefix :uri |
		isFirst 
			ifTrue: [isFirst := false]
			ifFalse: [aStream space].
		aStream nextPutAll: 'xmlns'.
		prefix
			ifNotEmpty: [
				aStream
					nextPut: $:;
					nextPutAll: prefix].
		aStream
			nextPut: $=;
			nextPut: $";
			nextPutAll: uri;
			nextPut: $"].! !

!XMLNamespaceScope methodsFor: 'printing' stamp: 'monty 4/22/2015 11:22'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	self printDeclarationsOn: aStream.
	aStream nextPut: $).! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 3/20/2016 23:55'!
resolvePrefix: aPrefix
	"Resolves the namespace URI of aPrefix (the defualt namespace URI if aPrefix
	is empty), including the predefined 'xml' and 'xmlns' prefixes. Returns an
	empty string if aPrefix is unresolvable."

	^ self
		resolvePrefix: aPrefix
		ifUnresolvable: ''! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 3/20/2016 23:58'!
resolvePrefix: aPrefix ifUnresolvable: aBlock
	"Resolves the namespace URI of aPrefix (the defualt namespace URI if aPrefix
	is empty), including the predefined 'xml' and 'xmlns' prefixes. Returns the
	value of aBlock if aPrefix is unresolvable."

	(aPrefix size > 0 "optimization"
		and: [(aPrefix at: 1) == $x]) "optimization"
		ifTrue: [
			aPrefix = 'xml'
				ifTrue: [^ self xmlNamespaceURI].
			aPrefix = 'xmlns'
				ifTrue: [^ self xmlnsNamespaceURI]].

	^ self
		prefixMappingAt: aPrefix
		ifAbsent: aBlock.! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 4/2/2016 21:49'!
resolvePrefixesOfNamespaceURI: aURI
	| prefixes |

	"the xml and xmlns namespaces URIs can't be aliased"
	aURI = self xmlNamespaceURI
		ifTrue: [^ Set with: 'xml'].
	aURI = self xmlnsNamespaceURI
		ifTrue: [^ Set with: 'xmlns'].

	prefixes := Set new.
	self prefixesAndURIsDo: [:prefix :uri |
		uri = aURI
			ifTrue: [prefixes add: prefix]].
	^ prefixes.! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 12/26/2014 23:54'!
size
	^ prefixMappings
		ifNil: [0]
		ifNotNil: [prefixMappings size]! !

!XMLNamespaceScope methodsFor: 'unmapping' stamp: 'monty 2/26/2016 05:49'!
unmapPrefix: aPrefix
	^ self
		unmapPrefix: aPrefix
		ifAbsent: [self errorCannotUnmapPrefix: aPrefix]! !

!XMLNamespaceScope methodsFor: 'unmapping' stamp: 'monty 2/24/2016 09:03'!
unmapPrefix: aPrefix ifAbsent: aBlock
	prefixMappings
		ifNil: [^ aBlock value]
		ifNotNil: [
			"copy-on-write"
			^ self copiedPrefixMappings 
				removeKey: aPrefix
				ifAbsent: aBlock]! !

!XMLNamespaceScope methodsFor: 'unmapping' stamp: 'monty 2/26/2016 18:30'!
unmapPrefixes
	self hasNamespaces
		ifTrue: [
			"copy-on-write"
			self copiedPrefixMappings removeAll]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 2/17/2016 00:11'!
validateAndResolvePrefix: aPrefix
	^ self
		prefixMappingAt: aPrefix
		ifAbsent: [
			aPrefix = 'xml'
				ifTrue: [self xmlNamespaceURI]
				ifFalse: [					
					(aPrefix = 'xmlns'
						ifTrue: [self errorCannotUseXMLNSPrefix]
						ifFalse: [self errorUnmappedPrefix: aPrefix])]]! !

!XMLNamespaceScope methodsFor: 'validating' stamp: 'monty 6/21/2015 02:29'!
validateAndResolvePrefix: aPrefix ofElement: aName
	aPrefix size = 0 "optimization"
		ifTrue: [
			self validateUnprefixedName: aName.
			^ self defaultNamespace]
		ifFalse: [^ self validateAndResolvePrefix: aPrefix]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 3/2/2016 19:19'!
validatePrefix: aPrefix beforeMappingTo: aURI
	aPrefix size > 0 "optimization"
		ifTrue: ["non-default namespace"
			aPrefix = 'xml' "cannot be remapped"
				ifTrue: [
					aURI = self xmlNamespaceURI
						ifFalse: [self errorCannotRemapXMLPrefix].
					^ self].
			aPrefix = 'xmlns' "cannot be mapped, even to the same URI"
				ifTrue: [self errorCannotMapXMLNSPrefix].
			(aPrefix indexOf: $:) > 0 
				ifTrue: [self errorInvalidPrefix: aPrefix].
			"non-default namespaces cannot be mapped to empty uris, even
			to shadow an inherited namesapce"
			aURI size > 0
				ifFalse: [self errorCannotMapPrefixToEmptyURI]].

	"these URIs cannot be aliased, even by the default namespace"
	(aURI = self xmlNamespaceURI
		or: [aURI = self xmlnsNamespaceURI])
		ifTrue: [self errorCannotAliasXMLOrXMLNSWithPrefix: aPrefix].! !

!XMLNamespaceScope methodsFor: 'validating' stamp: 'monty 8/26/2016 03:09'!
validatePrefix: aPrefix ofAttribute: anAttribute hasNoAliasesin: aDictionaryOrSet
	| namespace oldSize |

	namespace := self validateAndResolvePrefix: aPrefix.

	"#add: and a #size check after is faster than #includes: or #includesKey:
	and works for both Sets and Dictionaries"
	oldSize := aDictionaryOrSet size.
	(aDictionaryOrSet
		add: (anAttribute xmlLocalNameAfterPrefix: aPrefix) -> namespace;
		size) > oldSize
		ifFalse: [
			self
				errorAttribute: anAttribute
				aliasesNamespace: namespace]! !

!XMLNamespaceScope methodsFor: 'validating' stamp: 'monty 8/26/2016 03:43'!
validatePrefixesOfAttributes: anAttributeCollection
	^ self
		validatePrefixesOfAttributes: anAttributeCollection asArray
		from: 1
		to: anAttributeCollection size! !

!XMLNamespaceScope methodsFor: 'validating' stamp: 'monty 8/26/2016 00:35'!
validatePrefixesOfAttributes: anAttributeCollection from: aStart to: aStop
	"performance critical method"
	| attributeLocalNamesAndURIs |

	"to:do: for speed"
	aStart to: aStop do: [:i | | attribute prefix |
		attribute := anAttributeCollection at: i.
		(prefix := attribute xmlPrefixBeforeLocalName) size > 0 "optimization"
			ifTrue: [
				"not initialized unless needed"
				attributeLocalNamesAndURIs
					ifNil: [
						attributeLocalNamesAndURIs :=
							Set new: anAttributeCollection size].
				self
					validatePrefix: prefix
					ofAttribute: attribute
					hasNoAliasesin: attributeLocalNamesAndURIs]
			ifFalse: [self validateUnprefixedName: attribute]]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:40'!
validateUnprefixedName: aName
	"catch names like ':name' with an empty prefix"
	(aName size > 0
		and: [(aName at: 1) == $:])
		ifTrue: [self errorInvalidUnprefixedName: aName]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 2/17/2016 00:10'!
xmlNamespaceURI
	^ self class xmlNamespaceURI! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 2/17/2016 00:10'!
xmlnsNamespaceURI
	^ self class xmlnsNamespaceURI! !

Object subclass: #XMLNamespaceScopeAllocator
	instanceVariableNames: 'scopes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScopeAllocator commentStamp: 'monty 6/15/2015 16:48' prior: 0!
This class allocates namespace scopes with a stack. Sending #enterScope pushes a new scope onto the stack that inherits all mappings from the previous (enclosing) scope, and #exitScop pops it off.!

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:56'!
currentScope
	^ self scopes xmlLastOrNil ifNil: [self emptyScope]! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:27'!
currentScope: aScope
	self scopes addLast: aScope! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'JAAyer 3/13/2010 13:44'!
emptyScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'monty 10/30/2014 12:27'!
enterScope
	^ self scopes addLast: self currentScope copy! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'monty 10/30/2014 12:27'!
exitScope
	^ self scopes removeLast! !

!XMLNamespaceScopeAllocator methodsFor: 'testing' stamp: 'monty 4/24/2015 15:29'!
hasCurentScope
	^ scopes notEmpty! !

!XMLNamespaceScopeAllocator methodsFor: 'initialization' stamp: 'monty 10/27/2014 09:21'!
initialize
	super initialize.
	scopes := OrderedCollection new: 10.! !

!XMLNamespaceScopeAllocator methodsFor: 'printing' stamp: 'monty 4/25/2015 15:22'!
printOn: aStream
	super printOn: aStream.
	self hasCurentScope
		ifTrue: [
			aStream
				nextPut: $(;
				print: self currentScope;
				nextPut: $)].! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'monty 10/30/2014 12:26'!
scopes
	^ scopes! !

Object subclass: #XMLNestedStreamReader
	instanceVariableNames: 'stream onClose nestedStreams peekChar normalizedLineEndingChar currentLineNumber currentPosition currentLineStartPosition readLimit isEscapingQuotes'
	classVariableNames: 'CRCharacter LFCharacter'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNestedStreamReader commentStamp: 'monty 2/14/2016 02:20' prior: 0!
This class represents a stream reader that implements pushBack: using nested streams. This enables subsitution/replacement to be performed without modifying the underlying collections streamed-over or having to copy them. It also performs line-ending normalization, transforming a CR or a CRLF sequence into a single LF character.

(The code in this class is optimized; refactor with care.)!

!XMLNestedStreamReader class methodsFor: 'class initialization' stamp: 'monty 2/10/2016 20:54'!
initialize
	"self initialize"

	"stored in class vars for faster access"
	CRCharacter := Character cr.
	LFCharacter := Character lf.! !

XMLNestedStreamReader initialize!

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'monty 11/12/2014 21:36'!
on: aStream
	^ self
		on: aStream
		readLimit: nil! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'monty 12/8/2014 08:20'!
on: aStream readLimit: aReadLimit
	^ self new
		setStream: aStream
		readLimit: aReadLimit! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 3/18/2016 03:11'!
asEscapedCharacterIfQuote: aCharacter
	aCharacter ==  $"
		ifTrue: [
			self pushBack: '#34;'.
			^ $&].
	aCharacter == $'
		ifTrue: [
			self pushBack: '#39;'.
			^ $&].
	^ aCharacter.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 3/19/2016 16:42'!
atEnd
	peekChar == nil
		ifFalse: [^ false].
	stream atEnd
		ifFalse: [^ false].
	"to:do: for speed"
	1 to: nestedStreams size do: [:i |
		(nestedStreams at: i) key atEnd
			ifFalse: [^ false]].
	^ true.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 2/6/2014 01:01'!
atQuote
	^ self peek == $"
		or: [peekChar == $']! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 4/11/2016 03:29'!
basicNext
	"Returns the next character in the stream for #next and #peek with CR/LF/CRLF
	line endings normalized (to LF by default) and quotes escaped when needed."
	| nextChar |

	stream atEnd
		ifTrue: [
			self popNestedStreamWhileAtEnd.
			stream atEnd
				ifTrue: [^ nil]].
	(nextChar := stream next) == LFCharacter
		ifTrue: [^ normalizedLineEndingChar].
	nextChar == CRCharacter
		ifTrue: [
			stream atEnd
				ifTrue: [self popNestedStreamWhileAtEnd].
			stream atEnd
				ifFalse: [
					stream peek == LFCharacter
						ifTrue: [stream next]].
			^ normalizedLineEndingChar].
	isEscapingQuotes
		ifTrue: [^ self asEscapedCharacterIfQuote: nextChar].
	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 4/6/2016 07:20'!
closeStream
	(stream respondsTo: #close)
		ifTrue: [stream close].
	onClose
		ifNotNil: [onClose value].! !

!XMLNestedStreamReader methodsFor: 'closing' stamp: 'monty 4/6/2016 07:21'!
closeStreams
	self closeStream.
	[nestedStreams size > 0]
		whileTrue: [
			self
				popNestedStream;
				closeStream].! !

!XMLNestedStreamReader methodsFor: 'decoding' stamp: 'monty 8/16/2016 23:51'!
convertFromEncoding: anEncodingName
	"encoding attributes are ignored unless we are already decoding"
	stream isXMLConvertingStreamAdapter
		ifTrue: [self decodeStreamWithConverter: anEncodingName asXMLStreamConverter]! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 3/27/2016 01:55'!
currentColumnNumber
	"recording the start pos. of the line and subtracting it from the current
	pos. is faster than an explicit column var that would need to be incremented
	each time through #next/#skipSeparators"
	^ currentLineStartPosition
		ifNotNil: [currentPosition - currentLineStartPosition]! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 7/14/2010 00:39'!
currentLineNumber
	^ currentLineNumber! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 3/27/2016 01:55'!
currentLineNumber: aLineNumber
	currentLineNumber = aLineNumber
		ifFalse: [
			currentLineNumber := aLineNumber.
			currentLineStartPosition := nil].! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 3/18/2016 02:25'!
currentPosition
	^ currentPosition! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 3/26/2016 20:04'!
currentPosition: aPosition
	currentPosition := aPosition! !

!XMLNestedStreamReader methodsFor: 'decoding' stamp: 'monty 4/1/2016 00:55'!
decodeStreamWithConverter: aStreamConverter
	stream isXMLConvertingStreamAdapter
		ifTrue: [stream streamConverter: aStreamConverter]
		ifFalse: [
			stream :=
				XMLDecodingReadStreamAdapter
					on: stream
					streamConverter: aStreamConverter]! !

!XMLNestedStreamReader methodsFor: 'decoding' stamp: 'monty 3/8/2016 13:13'!
detectCurrentEncoding
	(XMLNestedStreamReaderEncodingDetector on: self)
		detectCurrentEncoding! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 10/31/2014 21:21'!
errorNestingLimitExceeded
	XMLLimitException signal: 'Cannot further nest input streams'! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 6/23/2016 20:39'!
errorReadLimitExceeded
	XMLLimitException signal:
		'Security input read limit exceeded; ',
		'use #documentReadLimit: and #externalSubsetReadLimit: ',
		'to change/disable it, or #defaultReadLimit: ',
		'to persistently change/disable it'! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 3/18/2016 20:50'!
isBinary
	^ false! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 3/1/2015 17:12'!
isEscapingQuotes
	^ isEscapingQuotes! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 3/1/2015 17:11'!
isEscapingQuotes: aBoolean
	isEscapingQuotes := aBoolean! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 1/6/2016 08:02'!
isStream
	^ true! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/18/2014 05:43'!
maxNestedStreams
	^ 32! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 4/5/2016 03:19'!
newWriteStream
	^ (self writeStreamCollectionClass new: 128) writeStream! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 3/19/2016 00:18'!
next
	| nextChar |

	peekChar
		ifNil: [nextChar := self basicNext]
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil].

	"this is inlined in skipSeparators"
	currentPosition := currentPosition + 1.
	readLimit ifNotNil: [
		currentPosition > readLimit
			ifTrue: [self errorReadLimitExceeded]].
	nextChar == normalizedLineEndingChar
		ifTrue: [
			currentLineNumber := currentLineNumber + 1.
			currentLineStartPosition := currentPosition].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/5/2016 03:09'!
next: anInteger
	| writeStream i |

	writeStream := self newWriteStream.
	i := 0.
	[self atEnd
		or: [i >= anInteger]]
		whileFalse: [
			writeStream nextPut: self next.
			i := i + 1].
	^ writeStream contents.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 12/3/2015 07:37'!
nextInteger
	^ self nextIntegerPrintedOn: nil! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 12/2/2015 20:21'!
nextIntegerPrintedOn: aStream
	^ (XMLIntegerReader on: self)
		printStream: aStream;
		nextIntegerWithBase: 16 startingWithAny: 'x' orBase: 10! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/5/2016 03:11'!
nextMatch: aFirstCharacter followedBy: aSecondCharacter
	"a more efficient two character version of nextMatchAll:"

	| oldCurrentLineNumber oldCurrentLineStartPosition |

	"inlined peek"
	(peekChar ifNil: [peekChar := self basicNext]) = aFirstCharacter
		ifFalse: [^ false].
	oldCurrentLineNumber := currentLineNumber.
	oldCurrentLineStartPosition := currentLineStartPosition.
	self next.

	self peek = aSecondCharacter
		ifFalse: [
			self pushBackCharacter: aFirstCharacter.
			currentPosition := currentPosition - 1.
			currentLineNumber := oldCurrentLineNumber.
			currentLineStartPosition := oldCurrentLineStartPosition.
			^ false].
	self next.
	^ true.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/5/2016 03:11'!
nextMatchAll: aString
	| oldCurrentLineNumber oldCurrentPosition oldCurrentLineStartPosition |

	aString size > 0
		ifFalse: [^ true].

	"inlined peek"
	(peekChar ifNil: [peekChar := self basicNext]) = (aString at: 1)
		ifFalse: [^ false].
	oldCurrentLineNumber := currentLineNumber.
	oldCurrentPosition := currentPosition.
	oldCurrentLineStartPosition := currentLineStartPosition.
	self next.
	"the first iteration was unrolled with the above inlined peek/self next, so
	the to:do: loop starts at 2"
	2 to: aString size do: [:i |
		self peek = (aString at: i)
			ifFalse: [
				self pushBack: (aString copyFrom: 1 to: i -1).
				currentLineNumber := oldCurrentLineNumber.
				currentPosition := oldCurrentPosition.
				currentLineStartPosition := oldCurrentLineStartPosition.
				^ false].
		self next].
	^ true.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 1/16/2014 09:08'!
normalizedLineEndingChar
	^ normalizedLineEndingChar! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:25'!
normalizedLineEndingChar: aCharacter
	normalizedLineEndingChar := aCharacter! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 2/2/2014 02:43'!
peek
	^ peekChar ifNil: [peekChar := self basicNext]! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/5/2016 03:07'!
peekFor: aCharacter
	self peek = aCharacter
		ifTrue: [
			self next.
			^ true]
		ifFalse: [^ false]! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 4/6/2016 07:21'!
popNestedStream
	| nextStreamAndCallback |

	nextStreamAndCallback := nestedStreams removeLast.
	stream := nextStreamAndCallback key.
	onClose := nextStreamAndCallback value.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 4/11/2016 03:29'!
popNestedStreamWhileAtEnd
	[nestedStreams size > 0]
		whileTrue: [
			self
				closeStream;
				popNestedStream.
			stream atEnd
				ifFalse: [^ self]]! !

!XMLNestedStreamReader methodsFor: 'printing' stamp: 'monty 3/19/2016 15:09'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPutAll: '(';
		print: peekChar;
		nextPutAll: '; ';
		print: stream;
		space;
		print: nestedStreams;
		nextPut: $).! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 12/13/2014 20:15'!
pushBack: aString
	aString size > 1
		ifTrue: [^ self pushBackStream: aString readStream].
	aString size = 1
		ifTrue: [self pushBackCharacter: (aString at: 1)]! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 3/18/2016 13:26'!
pushBackCharacter: aCharacter
	self
		pushBackStream: (XMLSingleCharacterReadStream on: aCharacter)
		onClose: nil! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 12/8/2014 06:20'!
pushBackStream: aStream
	self
		pushBackStream: aStream
		onClose: nil! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 4/16/2016 14:54'!
pushBackStream: aStream onClose: aBlock
	self
		pushNestedStream: stream
		onClose: onClose.
	peekChar
		ifNotNil: [
			self
				pushNestedStream: (XMLSingleCharacterReadStream on: peekChar)
				onClose: nil.
			peekChar := nil].

	self stream: aStream.
	onClose := aBlock.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 3/19/2016 18:37'!
pushNestedStream: aStream onClose: aBlock
	nestedStreams size >= self maxNestedStreams
		ifTrue: [self errorNestingLimitExceeded].
	nestedStreams addLast: aStream -> aBlock.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/3/2015 02:01'!
readLimit
	^ readLimit! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'monty 4/16/2016 14:56'!
setStream: aStream readLimit: aReadLimit
	self stream: aStream.
	nestedStreams := OrderedCollection new: 5.
	currentLineNumber := 1.
	currentPosition := 0.
	currentLineStartPosition := 0.
	readLimit := aReadLimit.
	normalizedLineEndingChar := LFCharacter.
	isEscapingQuotes := false.! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 2/9/2016 23:21'!
skip: anInteger
	1 to: anInteger do: [:i |
		self atEnd
			ifTrue: [^ self].
		self next]! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 3/19/2016 00:18'!
skipSeparators
	"nil understands isXMLSeparator so the result of the inlined #peek
	does not need to be tested for nil first"
	[(peekChar ifNil: [peekChar := self basicNext]) "inlined #peek"
		isXMLSeparator]
		whileTrue: [
			"inlined from #next"
			currentPosition := currentPosition + 1.
			readLimit ifNotNil: [
				currentPosition > readLimit
					ifTrue: [self errorReadLimitExceeded]].
			peekChar == normalizedLineEndingChar
				ifTrue: [
					currentLineNumber := currentLineNumber + 1.
					currentLineStartPosition := currentPosition].

			peekChar := nil]! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 4/5/2016 03:08'!
skipTo: aCharacter
	[self atEnd]
		whileFalse: [
			self next = aCharacter
				ifTrue: [^ true]].
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 08:01'!
stream
	^ stream! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:55'!
stream: aStream
	"if the stream is not already a decoding adapter,
	then only assume it needs one if it's binary"
	aStream isXMLConvertingStreamAdapter
		ifTrue: [stream := aStream]
		ifFalse: [stream := XMLDecodingReadStreamAdapter onStreamIfBinary: aStream]! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 4/5/2016 03:08'!
upTo: aCharacter
	| writeStream nextChar |

	writeStream := self newWriteStream.
	[self atEnd
		or: [(nextChar := self next) = aCharacter]]
		whileFalse: [writeStream nextPut: nextChar].
	^ writeStream contents.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 2/9/2016 23:21'!
upToEnd
	| writeStream |

	writeStream := self newWriteStream.
	[self atEnd]
		whileFalse: [writeStream nextPut: self next].
	^ writeStream contents.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 4/5/2016 03:19'!
writeStreamCollectionClass
	^ String! !

Object subclass: #XMLNode
	instanceVariableNames: 'parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNode commentStamp: 'monty 6/15/2015 16:52' prior: 0!
This is a base class for XML nodes. It has testing messages, messages to access the parent, sibling, and ancestor nodes, and messages to control printing.!

XMLNode subclass: #XMLAttribute
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttribute commentStamp: 'monty 6/15/2015 22:14' prior: 0!
The class represents an attribute node stored in an attribute node list. The name of an attribute node can be namespaced using prefixes, but the prefix must be mapped to a namespace URI in the containing element node. Unprefixed attributes have no namespace, not even a default if one is in scope, per the XML namespace standard.!

!XMLAttribute class methodsFor: 'deprecated' stamp: 'monty 8/26/2016 03:17'!
name: aName
	"will be deprecated; defined here just to avoid accidentally renaming the class"

	^ self named: aName! !

!XMLAttribute class methodsFor: 'instance creation' stamp: 'monty 10/18/2014 04:03'!
name: aName value: aValue
	^ self new	
		setName: aName
		value: aValue
		element: nil! !

!XMLAttribute class methodsFor: 'instance creation' stamp: 'monty 8/26/2016 00:23'!
named: aName
	^ self
		name: aName
		value: ''! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 10/29/2014 07:42'!
element
	"reuse instance variable to save memory"
	^ parent! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 10/29/2014 07:42'!
element: anElement
	"reuse instance variable to save memory"
	parent := anElement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 4/20/2016 02:56'!
expandedName
	"returns the expanded name of the element as a string in the form of either
	'{namespaceURI}localName' (if there's a namespace URI) or 'localName'"

	^ self localName expandWithXMLNamespaceURI: self namespaceURI! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 12/23/2014 16:06'!
hasLanguage: aLanguage
	| language xmlLang |

	language := aLanguage asLowercase.
	^ (xmlLang := self value asLowercase) = language
		or: [(xmlLang copyUpTo: $-) = language].! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 2/13/2015 22:20'!
hasNamespaceURI
	^ self namespaceURI notEmpty! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 3/28/2016 23:16'!
hasPrefix
	^ (self name
		indexOf: $:
		startingAt: 2) > 0! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 1/29/2015 00:30'!
isAttribute
	^ true! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 4/21/2016 07:40'!
isInLanguage: aLanguage
	"Returns true if the receiver is an xml:lang attribute or if its element
	or an ancestor element has an xml:lang attribute, and if the value is an IETF
	language tag that matches aLanguage or if its first ('-' separated) subtag
	matches aLanguage."

	self name = 'xml:lang'
		ifTrue: [^ self hasLanguage: aLanguage]
		ifFalse: [^ super isInLanguage: aLanguage]! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 4/20/2016 02:55'!
isNamed: aName
	"Does the receiver's qualified or local name match the argument?"

	^ self name isXMLQualifiedOrLocalName: aName! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 4/20/2016 02:55'!
isNamedAny: aNameCollection
	"Does the receiver's qualified or local name match any of the names
	in aNameCollection?"

	^ aNameCollection includesXMLName: self name! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:56'!
localName
	^ self name xmlLocalNameAfterPrefix! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 17:36'!
name
	^ name ifNil: [name := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 18:01'!
name: aName
	self renameWith: [name := aName]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 1/29/2015 00:52'!
namespaceURI
	| prefix |

	"attributes must have prefixes to be namespaced"
	(self hasParent
		and: [(prefix := self prefix) notEmpty])
		ifTrue: [^ self parent resolvePrefix: prefix]
		ifFalse: [^ '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:57'!
prefix
	^ self name xmlPrefixBeforeLocalName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:57'!
prefix: aPrefix
	self renameWith: [name := self localName qualifyWithXMLPrefix:  aPrefix]! !

!XMLAttribute methodsFor: 'private' stamp: 'monty 4/18/2016 07:37'!
renameWith: aBlock
	| oldName |

	oldName := self name.
	aBlock value.
	(self name ~= oldName
		and: [self hasParent])
			ifTrue: [
				self parent
					renamedAttribute: self
					from: oldName
					to: self name].! !

!XMLAttribute methodsFor: 'initialization' stamp: 'monty 11/16/2014 17:35'!
setName: aName value: aValue element: anElement
	name := aName.
	value := aValue.
	parent := anElement.! !

!XMLAttribute methodsFor: 'private' stamp: 'monty 12/2/2014 22:56'!
sortKey
	^ self name! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
value
	^ value ifNil: [value := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 08:18'!
value: aString
	value := aString! !

!XMLAttribute methodsFor: 'printing' stamp: 'monty 2/9/2014 04:31'!
writeXMLOn: aWriter
	aWriter
		writeName: self name
		escapedContentValue: self value! !

XMLNode subclass: #XMLComment
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLComment commentStamp: 'monty 6/15/2015 22:14' prior: 0!
This class represents comment nodes. Comments are ignored during parsing by default. If you want them preserved as comment nodes, use #preservesCommentNodes: with the DOM parser before parsing.!

!XMLComment class methodsFor: 'instance creation' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	^ self new string: aString! !

!XMLComment methodsFor: 'visiting' stamp: 'monty 7/27/2016 05:31'!
acceptNodeVisitor: aNodeVisitor
	aNodeVisitor visitComment: self! !

!XMLComment methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ true! !

!XMLComment methodsFor: 'private' stamp: 'monty 12/3/2014 01:08'!
sortKey
	^ self string! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string
	^ string ifNil: [string := '']! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	string := aString! !

!XMLComment methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter comment: self string! !

XMLNode subclass: #XMLDeclarationNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDeclarationNode commentStamp: 'monty 6/15/2015 16:54' prior: 0!
This is an abstract class for markup declaration nodes in the internal subset.!

XMLDeclarationNode subclass: #XMLAttributeDeclaration
	instanceVariableNames: 'element attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeDeclaration commentStamp: 'monty 2/2/2014 16:47' prior: 0!
This is a class for <!!ATTLIST> declarations with definitions for single attributes. (An ATTLIST with multiple attribute defs is the same as a series of separate ATTLISTs for each def)!

!XMLAttributeDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:11'!
element: anElementName attribute: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		element: anElementName;
		attribute: anAttributeName;
		type: aType;
		defaultPragma: aDefaultPragma;
		defaultValue: aDefaultValue! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:53'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:07'!
attribute: anAttributeName
	attribute := anAttributeName! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:14'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
defaultPragma: aDefaultPragma
	defaultPragma := aDefaultPragma! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:11'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
element: anElementName
	element := anElementName! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
type
	^ type ifNil: [type := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:09'!
type: aType 
	type := aType! !

!XMLAttributeDeclaration methodsFor: 'printing' stamp: 'monty 5/7/2015 18:51'!
writeXMLOn: aWriter
	aWriter attributeDeclaration
		element: self element;
		attribute: self attribute;
		type: self type;
		defaultPragma: self defaultPragma;
		defaultValue: self defaultValue;
		write! !

!XMLDeclarationNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ true! !

XMLDeclarationNode subclass: #XMLElementDeclaration
	instanceVariableNames: 'element contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElementDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is a class for <!!ELEMENT> declarations.!

!XMLElementDeclaration class methodsFor: 'instance creation' stamp: 'monty 6/7/2015 19:39'!
element: anElementName contentModel: aContentModel
	^ self new
		element: anElementName;
		contentModel: aContentModel! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:03'!
contentModel: aContentModel
	contentModel := aContentModel! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 6/7/2015 19:43'!
element: anElementName
	element := anElementName! !

!XMLElementDeclaration methodsFor: 'printing' stamp: 'monty 5/7/2015 18:51'!
writeXMLOn: aWriter
	aWriter elementDeclaration
		element: self element;
		contentModel: self contentModel;
		write! !

XMLDeclarationNode subclass: #XMLEntityDeclaration
	instanceVariableNames: 'name replacement publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLEntityDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is an abstract class for general and parameter <!!ENTITY> declarations.!

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
name: aName publicID: aPublicID systemID: aSystemID
	^ self new
		name: aName;
		publicID: aPublicID;
		systemID: aSystemID! !

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:06'!
name: aName replacement: aReplacement
	^ self new
		name: aName;
		replacement: aReplacement! !

!XMLEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/30/2013 21:53'!
isParameter
	^ false! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:01'!
name: aName
	name := aName! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:05'!
replacement: aReplacement
	replacement := aReplacement! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
systemID: aSystemID
	systemID := aSystemID! !

XMLEntityDeclaration subclass: #XMLGeneralEntityDeclaration
	instanceVariableNames: 'ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLGeneralEntityDeclaration commentStamp: 'monty 2/2/2014 16:47' prior: 0!
This is a class for internal/external parsed/unparsed general <!!ENTITY> declarations.!

!XMLGeneralEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
name: aName publicID: aPublicID systemID: aSystemID ndata: aNotationName
	^ self new
		name: aName;
		publicID: aPublicID;
		systemID: aSystemID;
		ndata: aNotationName! !

!XMLGeneralEntityDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:54'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLGeneralEntityDeclaration methodsFor: 'accessing' stamp: 'monty 6/7/2015 19:32'!
ndata: aNotationName
	ndata := aNotationName! !

!XMLGeneralEntityDeclaration methodsFor: 'printing' stamp: 'monty 5/7/2015 18:51'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: false;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		ndata: self ndata;
		write! !

XMLEntityDeclaration subclass: #XMLParameterEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLParameterEntityDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is class for internal/external parameter <!!ENTITY> declarations.!

!XMLParameterEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/13/2013 19:34'!
isParameter
	^ true! !

!XMLParameterEntityDeclaration methodsFor: 'printing' stamp: 'monty 5/7/2015 18:52'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: true;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		write! !

XMLDeclarationNode subclass: #XMLNotationDeclaration
	instanceVariableNames: 'name publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNotationDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is class for <!!NOTATION> declarations.
!

!XMLNotationDeclaration class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:37'!
name: aName publicID: aPublicID systemID: aSystemID
	^ self new
		name: aName;
		publicID: aPublicID;
		systemID: aSystemID! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 10:35'!
name: aName
	name := aName! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:20'!
systemID: aSystemID
	systemID := aSystemID! !

!XMLNotationDeclaration methodsFor: 'printing' stamp: 'monty 5/7/2015 18:52'!
writeXMLOn: aWriter
	aWriter notationDeclaration
		name: self name;
		publicID: self publicID;
		systemID: self systemID;
		write! !

!XMLNode methodsFor: 'visiting' stamp: 'monty 7/27/2016 07:52'!
acceptNodeVisitor: aNodeVisitor! !

!XMLNode methodsFor: 'deprecated' stamp: 'monty 8/23/2016 05:50'!
addContent: aStringOrStringNode
	"will be deprecated"

	XMLDOMException signal: 'Node cannot contain string content'! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 4/19/2016 09:08'!
allNodesDo: aBlock
	"evaluates aBlock for the receiver and all of its descendant nodes"

	aBlock value: self! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/19/2016 09:01'!
ancestorElements
	"returns a node list with all ancestor elements of the receiver"
	| ancestorElements |

	ancestorElements := self newListForSelect.
	self ancestorElementsDo: [:each | ancestorElements addLast: each].
	^ ancestorElements.! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:19'!
ancestorElementsDo: aBlock
	| ancestor |

	ancestor := self parent.
	[ancestor notNil
		and: [ancestor isElement]]
		whileTrue: [
			aBlock value: ancestor.
			ancestor := ancestor parent].! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/19/2016 09:02'!
ancestorNodes
	"returns a node list with all ancestor nodes of the receiver"
	| ancestors |

	ancestors := self newListForSelect.
	self ancestorNodesDo: [:each | ancestors addLast: each].
	^ ancestors.! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:18'!
ancestorNodesDo: aBlock
	| ancestor |

	ancestor := self parent.
	[ancestor notNil]
		whileTrue: [
			aBlock value: ancestor.
			ancestor := ancestor parent].! !

!XMLNode methodsFor: 'converting' stamp: 'monty 4/6/2015 08:05'!
asString
	"for compatibility with GS where Object>>asString isn't defined"
	^ self printString! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/19/2016 09:21'!
canHaveChildren
	"returns true if the receiver is a type of node that can have child nodes"

	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/21/2016 07:50'!
canonicallyEquals: aNode
	"returns true if the receiver and aNode have the same canonical
	representation (http://www.w3.org/TR/xml-c14n)"

	^ self canonicallyPrinted = aNode canonicallyPrinted! !

!XMLNode methodsFor: 'printing' stamp: 'monty 4/19/2016 09:16'!
canonicallyPrintOn: aStream
 	"Prints on aStream as canonical XML (http://www.w3.org/TR/xml-c14n)"

	self
		printOn: aStream
		beforeWritingDo: [:writer | writer isCanonical: true]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 7/4/2016 07:18'!
canonicallyPrintToFileNamed: aFileName
 	"Prints to specified file as canonical XML (http://www.w3.org/TR/xml-c14n)"

	self
		printToFileNamed: aFileName
		beforeWritingDo: [:writer | writer isCanonical: true]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 4/19/2016 09:11'!
canonicallyPrinted
 	"Prints as canonical XML (http://www.w3.org/TR/xml-c14n)"

	^ String streamContents: [:stream | self canonicallyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/1/2015 19:09'!
configuration
	^ self parent ifNotNil: [:node | node configuration]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 8/7/2016 12:51'!
contentString
 	"Returns the combined content strings of all descendants with space inserted
	when one string has no trailing space and the next has no leading space.

	See also #rawContentString."

	^ String streamContents: [:stream | self printContentOn: stream]! !

!XMLNode methodsFor: 'copying' stamp: 'monty 8/15/2016 03:28'!
copy
	"returns a deep copy of the receiver, including copies of any descendant nodes,
	and copies the config"

	^ self shallowCopy postCopy postCopyConfiguration! !

!XMLNode methodsFor: 'copying' stamp: 'monty 8/15/2016 03:28'!
copySharingConfiguration	
	"returns a deep copy of the receiver, including copies of any descendant nodes,
	but does not copy the config"

	^ self shallowCopy postCopy! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 2/27/2016 13:38'!
descendantNodesDo: aBlock! !

!XMLNode methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentNodesDo: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantNodesDo: instead"

	^ self descendantNodesDo: aBlock! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/19/2016 09:15'!
document
	"returns the document node that the receiver belongs to (the receiver if it's a
	document) or nil if there isn't one"

	^ self parent ifNotNil: [:node | node document]! !

!XMLNode methodsFor: 'private' stamp: 'monty 10/31/2014 20:56'!
errorXMLWritingUnsupported
	XMLDOMException signal: 'The XMLWriter package is required for writng DOM objects'! !

!XMLNode methodsFor: 'private' stamp: 'monty 1/7/2015 02:16'!
escapesContentEntitiesWhenWriting
	self hasParent
		ifTrue: [^ self parent escapesContentEntitiesWhenWriting]
		ifFalse: [^ true] "default to escaping"! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/19/2016 09:20'!
hasChildren
	"returns true if the receiver has child nodes"

	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/19/2016 09:20'!
hasParent
	"returns true if the receiver has a parent node"

	^ parent notNil! !

!XMLNode methodsFor: 'private' stamp: 'monty 8/16/2016 23:52'!
initializeFileWriteStream: aWriteStream
	"This sets a stream converter on aWriteStream based on the document's encoding
	attribute and writes the encoding's byte order mark."

	| document |

	aWriteStream
		streamConverter:
			(((document := self document) notNil
				and: [document hasEncoding])
				ifTrue: [document encoding asXMLStreamConverter]
				ifFalse: [XMLStreamConverter default]);
		nextPutByteOrderMark! !

!XMLNode methodsFor: 'testing' stamp: 'monty 1/29/2015 00:30'!
isAttribute
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ false! !

!XMLNode methodsFor: 'private' stamp: 'monty 8/6/2016 17:54'!
isCoalescingStringNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/19/2016 09:20'!
isContentNode
	"returns true if the receiver is a content node (an element or string node)"

	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:50'!
isDocument
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/19/2016 09:24'!
isElementNamed: aName
	"returns true if the receiver is an element whose qualified name or local
	name matches aName"

	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/19/2016 09:24'!
isElementNamedAny: aNameCollection
	"returns true if the receiver is an element whose qualified name or local
	name matches any name in aNameCollection"

	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/20/2016 10:59'!
isInLanguage: aLanguage
	"Returns true if one of the ancestor elements of the receiver has an xml:lang
	attribute, and if the value is an IETF language tag that matches aLanguage
	or if its first ('-' separated) subtag matches aLanguage."

	^ self hasParent
		and: [self parent isInLanguage: aLanguage]! !

!XMLNode methodsFor: 'testing' stamp: 'monty 4/19/2016 09:18'!
isPI
	"returns true if the receiver is a processing instruction node"

	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:33'!
isStringNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/5/2014 12:09'!
isStringNode: aStringOrStringNode
	^ false! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/9/2016 21:01'!
newCData: aString
	^ self nodeFactory newCData: aString! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/9/2016 21:01'!
newComment: aString
	^ self nodeFactory newComment: aString! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/9/2016 21:01'!
newDocument
	^ self nodeFactory newDocument
		configuration: self configuration! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/9/2016 21:01'!
newElement
	^ self nodeFactory newElement
		configuration: self configuration! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/9/2016 21:01'!
newElementNamed: aName
	^ (self nodeFactory newElementNamed: aName)
		configuration: self configuration! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/16/2016 23:53'!
newElementNamed: aName attributes: anAssociationCollection
	^ (self nodeFactory
		newElementNamed: aName
		attributes: anAssociationCollection)
			configuration: self configuration! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/16/2016 23:53'!
newElementNamed: aName namespaceURI: aURI
	^ (self nodeFactory
		newElementNamed: aName
		namespaceURI: aURI)
			configuration: self configuration! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/16/2016 23:54'!
newElementNamed: aName namespaceURI: aURI attributes: anAssociationCollection
	^ (self nodeFactory
		newElementNamed: aName
		namespaceURI: aURI
		attributes: anAssociationCollection)
			configuration: self configuration! !

!XMLNode methodsFor: 'private' stamp: 'monty 3/8/2015 09:20'!
newListForCollect
	"collect can return non-nodes, so a basic ordered list is used"
	^ XMLOrderedList new: 10! !

!XMLNode methodsFor: 'private' stamp: 'monty 3/8/2015 09:20'!
newListForSelect
	"select returns only nodes, so a node list is used"
	^ XMLNodeList new: 10! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/9/2016 21:01'!
newPI
	^ self nodeFactory newPI! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/9/2016 21:01'!
newPITarget: aTargetString data: aDataString
	^ self nodeFactory
		newPITarget: aTargetString
		data: aDataString! !

!XMLNode methodsFor: 'instance creation' stamp: 'monty 8/9/2016 21:01'!
newStringNode: aString
	^ self nodeFactory newStringNode: aString! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/19/2016 09:06'!
nextNode
	"returns the first node after this one that is a child of the same parent (the next
	sibling) or nil if there isn't one"

	^ self parent ifNotNil: [:node | node nodeAfter: self]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 8/9/2016 21:01'!
nodeFactory
	^ self configuration
		ifNil: [XMLNodeFactory new]
		ifNotNil: [:config | config nodeFactory]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/19/2016 09:06'!
parent
	"returns the parent node of this node or nil if there isn't one"

	^ parent! !

!XMLNode methodsFor: 'private' stamp: 'monty 4/19/2016 09:07'!
parent: aNode
	parent := aNode! !

!XMLNode methodsFor: 'copying' stamp: 'monty 1/2/2016 23:56'!
postCopy
	super postCopy.

	parent := nil.! !

!XMLNode methodsFor: 'copying' stamp: 'monty 3/2/2016 05:48'!
postCopyConfiguration! !

!XMLNode methodsFor: 'printing' stamp: 'monty 4/19/2016 09:10'!
prettyPrintOn: aStream
 	"Prints XML on aStream with whitespace formatting for readability"

	self
		printOn: aStream
		beforeWritingDo: [:writer | writer enablePrettyPrinting]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 7/4/2016 07:18'!
prettyPrintToFileNamed: aFileName
 	"Prints XML to specified file with whitespace formatting for readability"

	self
		printToFileNamed: aFileName
		beforeWritingDo: [:writer | writer enablePrettyPrinting]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 4/19/2016 09:10'!
prettyPrinted
 	"Prints XML with whitespace formatting for readability"

	^ String streamContents: [:stream | self prettyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/19/2016 09:06'!
previousNode
	"returns the first node before this one that is a child of the same parent
	(the preivous sibling) or nil if there isn't one"

	^ self parent ifNotNil: [:node | node nodeBefore: self]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 4/20/2016 12:53'!
printContentOn: aStream
 	"Prints the combined content strings of all descendants with space inserted
	when one string has no trailing space and the next has no leading space"

	self printRawContentOn: (XMLNodeContentWriteStreamAdapter on: aStream)! !

!XMLNode methodsFor: 'printing' stamp: 'monty 4/19/2016 09:10'!
printOn: aStream
	"Uses XMLWriter if supported to print XML on aStream, uses the
	basic Object>>printOn: otherwise"

	self
		withNewXMLWriterOn: aStream
		do: [:writer |
			self writeXMLOn: writer.
			writer write]
		whenAbsent: [super printOn: aStream]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 7/8/2016 00:57'!
printOn: aStream beforeWritingDo: aBlock
	"Prints XML on aStream using an XMLWriter object it creates but first
	evaluates aBlock with the XMLWriter so it can be configured before writing.

	Raises an error without XMLWriter support."

	self
		withNewXMLWriterOn: aStream
		do: [:writer |
			aBlock value: writer.
			self writeXMLOn: writer.
			writer write]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/19/2014 23:54'!
printRawContentOn: aStream 
 	"Prints on aStream the content strings of all descendants without spaces inserted."! !

!XMLNode methodsFor: 'printing' stamp: 'monty 7/4/2016 07:33'!
printToFileNamed: aFileName
	"Prints XML to specified file. Uses the encoding specified in the
	document's encoding attribute if supported or UTF-8 otherwise."

	self
		withNewWriteStreamOnFileNamed: aFileName
		do: [:writeStream |
			self
				printOn: writeStream
				beforeWritingDo: [:writer |
					"use the platform's line break because we're writing to a file"
					writer enablePlatformSpecificLineBreak]]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 7/4/2016 07:32'!
printToFileNamed: aFileName beforeWritingDo: aBlock
	"Prints XML to specified file after evaluating aBlock with the XMLWriter.
	Uses the encoding specified in the document's encoding attribute if
	supported or UTF-8 otherwise."

	self
		withNewWriteStreamOnFileNamed: aFileName
		do: [:writeStream |
			self
				printOn: writeStream
				beforeWritingDo: [:writer |
					"use the platform's line break because we're writing to a file"
					writer enablePlatformSpecificLineBreak.
					aBlock value: writer]]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 5/10/2015 05:46'!
printWithoutSelfClosingTagsOn: aStream
	self
		printOn: aStream
		beforeWritingDo: [:writer | writer outputsSelfClosingTags: false]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 7/4/2016 07:15'!
printWithoutSelfClosingTagsToFileNamed: aFileName
	self
		printToFileNamed: aFileName
		beforeWritingDo: [:writer | writer outputsSelfClosingTags: false]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:27'!
printedWithoutSelfClosingTags
	^ String streamContents: [:stream | self printWithoutSelfClosingTagsOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 4/19/2016 09:02'!
rawContentString
 	"Returns the combined content string of all descendants without
	spaces inserted."

	^ String streamContents: [:stream | self printRawContentOn: stream]! !

!XMLNode methodsFor: 'private' stamp: 'monty 12/4/2014 14:00'!
sortKey
	"used to compare nodes when sorting"

	^ self class name asString! !

!XMLNode methodsFor: 'validating' stamp: 'monty 8/23/2016 05:53'!
validate
	"revalidate against the DTD it was parsed with (if any)"

	self document ifNotNil: [:document | document validate]! !

!XMLNode methodsFor: 'validating' stamp: 'monty 8/23/2016 05:53'!
validateWith: aValidator
	"revalidate against a custom DTD or other validator"

	self document ifNotNil: [:document | document validateWith: aValidator]! !

!XMLNode methodsFor: 'private' stamp: 'monty 3/4/2016 08:53'!
withNewWriteStreamOnFileNamed: aFileName do: aBlock
	aFileName asXMLFileHandle writeStreamDo: [:writeStream |
		self initializeFileWriteStream: writeStream.
		aBlock value: writeStream]! !

!XMLNode methodsFor: 'private' stamp: 'monty 10/31/2014 20:56'!
withNewXMLWriterOn: aStream do: aOneArgBlock
	^ self
		withNewXMLWriterOn: aStream
		do: aOneArgBlock
		whenAbsent: [self errorXMLWritingUnsupported]! !

!XMLNode methodsFor: 'private' stamp: 'monty 5/10/2015 00:22'!
withNewXMLWriterOn: aStream do: aOneArgBlock whenAbsent: aZeroArgBlock
	| writer |

	writer :=
		(self xmlWriterClassOrNil
			ifNil: [^ aZeroArgBlock value]) on: aStream.
	self escapesContentEntitiesWhenWriting
		ifFalse: [writer escapesContentEntityReferences: false].
	^ aOneArgBlock value: writer.! !

!XMLNode methodsFor: 'printing' stamp: 'monty 7/5/2016 21:24'!
writeXMLOn: aWriter
	"Template method used by nodes to write themselves on XMLWriters.

	Sending with a customized XMLWriter instance as the argument can be used
	to customize printing, but #printOn:beforeWritingDo: should be preferred."! !

!XMLNode methodsFor: 'private' stamp: 'monty 7/6/2016 02:48'!
xmlWriterClassOrNil
	"can be overriden to return a custom XMLWriter subclass"

	^ XMLClassFinder classNamed: #XMLWriter! !

XMLNode subclass: #XMLNodeWithChildren
	instanceVariableNames: 'nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithChildren commentStamp: 'monty 2/27/2016 13:48' prior: 0!
This is an abstract class for nodes that can contain child nodes.

It has messages to access, add and remove child nodes. The nodes are stored in a kind of XMLObservableList returned by #nodes, which can be modified directly to add or remove nodes from the owner of #nodes (copy it first if that isn't what you want).

There are three types of "enumerating" messages: the #nodes* messages enumerate child nodes of the receiver, the #allNode* forms enumerate (using depth-first traversal) the receiver and all descendant nodes, and the #descendantNode* forms  enumerate only descendant nodes.!

XMLNodeWithChildren subclass: #XMLDoctypeDeclaration
	instanceVariableNames: 'root publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDoctypeDeclaration commentStamp: 'monty 6/15/2015 01:39' prior: 0!
This class represents a <!!DOCTYPE ...> declaration. It stores the root element name, public/system ID, and its nodes are the internal subset.!

!XMLDoctypeDeclaration class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:36'!
root: aRootElementName publicID: aPublicID systemID: aSystemID
	^ self new
		root: aRootElementName;
		publicID: aPublicID;
		systemID: aSystemID! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'monty 3/8/2015 09:17'!
newListForSelect
	^ XMLInternalSubsetList new: 10! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'monty 11/26/2013 21:51'!
nodeListClass
	^ XMLInternalSubsetList! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 20:01'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 20:01'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 10/10/2014 07:12'!
root
	^ root ifNil: [root := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 04:42'!
root: aRootElementName
	root := aRootElementName! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 20:01'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 3/24/2015 20:01'!
systemID: aSystemID
	systemID := aSystemID! !

!XMLDoctypeDeclaration methodsFor: 'printing' stamp: 'monty 5/7/2015 18:51'!
writeXMLOn: aWriter
	aWriter doctype
		root: self root;
		publicID: self publicID;
		systemID: self systemID;
		with: [self writeInnerXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'monty 8/9/2016 21:20'!
addComment: aString
	^ self addNode: (self newComment: aString)! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'monty 4/20/2016 12:54'!
addNode: aNode
	"Adds aNode to the end of the receiver's node list. Can be used for elements,
	comments, and other types of nodes."

	^ self nodes addLast: aNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'monty 4/20/2016 12:56'!
addNode: aNode after: aPreviousNode
	"Adds aNode to the receiver's node list after aPreviousNode raising an
	error if aPreviousNode is absent"

	^ self nodes
		add: aNode
		after: aPreviousNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'monty 4/20/2016 12:56'!
addNode: aNode before: aNextNode
	"Adds aNode to the receiver's node list before aNextNode raising an
	error if aNextNode is absent"

	^ self nodes
		add: aNode
		before: aNextNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'monty 4/20/2016 12:54'!
addNodeFirst: aNode
	"Adds aNode to the begining of the receiver's node list. Can be used for
	elements, comments, and other types of nodes."

	^ self nodes addFirst: aNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 05:12'!
addNodes: aNodeCollection
	^ self nodes addAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'monty 8/9/2016 21:21'!
addPITarget: aTargetString data: aDataString
	^ self addNode:
		(self
			newPITarget: aTargetString
			data: aDataString)! !

!XMLNodeWithChildren methodsFor: 'notifying' stamp: 'monty 2/2/2014 02:40'!
addedNode: aNode
	aNode parent: self! !

!XMLNodeWithChildren methodsFor: 'notifying' stamp: 'monty 10/31/2014 17:34'!
addingNode: aNode
	aNode hasParent
		ifTrue: [self errorAddingNodeWithParent]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:14'!
allNodes
	"returns a node list with the receiver and all of its descendant nodes"
	| allNodes |

	allNodes := self newListForSelect.
	self allNodesDo: [:each | allNodes addLast: each].
	^ allNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:29'!
allNodesCollect: aBlock
	"returns an ordered list of the receiver and all of its descendant nodes
	evaluated with aBlock"
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self allNodesDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:17'!
allNodesDetect: aBlock
	^ self allNodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:40'!
allNodesDetect: aBlock ifNone: anExceptionBlock
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/19/2016 09:08'!
allNodesDo: aBlock
	"evaluates aBlock for the receiver and all of its descendant nodes"

	aBlock value: self.
	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 3/8/2015 09:17'!
allNodesSelect: aBlock
	| selectedNodes |

	selectedNodes := self newListForSelect.
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes addLast: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'monty 11/23/2014 20:53'!
canHaveChildren
	^ true! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:15'!
descendantNodes
	"returns a node list with all descendant nodes of the receiver"
	| descendentNodes |

	descendentNodes := self newListForSelect.
	self descendantNodesDo: [:each | descendentNodes addLast: each].
	^ descendentNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 13:16'!
descendantNodesCollect: aBlock
	"returns an ordered list of the values of all descendant nodes of the
	receiver evaluated with aBlock"
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self descendantNodesDo: [:each |
		collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:33'!
descendantNodesDetect: aBlock
	"returns the first descendant node of the receiver that aBlock evaluates
	true for or nil if there isn't one"

	^ self
		descendantNodesDetect: aBlock
		ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:33'!
descendantNodesDetect: aBlock ifNone: anExceptionBlock
	"returns the first descendant node of the receiver that aBlock evaluates
	true for or the value of anExceptionBlock if there isn't one"

	self descendantNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:28'!
descendantNodesDo: aBlock
	"evaluates aBlock for all descendant nodes of the receiver"

	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:33'!
descendantNodesSelect: aBlock
	"returns a node list of all descendant nodes of the receiver that aBlock
	evaluates true for"
	| selectedNodes |

	selectedNodes := self newListForSelect.
	self descendantNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes addLast: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentNodes
	"less common alternate spelling, will be deprecated;
	use #descendantNodes instead"

	^ self descendantNodes! !

!XMLNodeWithChildren methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentNodesCollect: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantNodesCollect: instead"

	^ self descendantNodesCollect: aBlock! !

!XMLNodeWithChildren methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentNodesDetect: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantNodesDetect: instead"

	^ self descendantNodesDetect: aBlock! !

!XMLNodeWithChildren methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentNodesDetect: aBlock ifNone: anExceptionBlock
	"less common alternate spelling, will be deprecated;
	use #descendantNodesDetect:ifNone: instead"

	^ self
		descendantNodesDetect: aBlock
		ifNone: anExceptionBlock! !

!XMLNodeWithChildren methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentNodesSelect: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantNodesSelect: instead"

	^ self descendantNodesSelect: aBlock! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 10/31/2014 17:34'!
errorAddingNodeWithParent
	XMLDOMException signal: 'Cannot add child node that already has a parent'! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:25'!
firstNode
	"returns the first child node of the receiver if it has one, nil otherwise"

	^ self nodes first! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'monty 7/15/2016 01:58'!
hasChildren
	"optimized with nil test and size check to avoid lazy initializing"
	nodes == nil
		ifTrue: [^ false].
	^ nodes size > 0.! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:23'!
includesNode: aNode
	^ self nodes includes: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:38'!
innerXML
	"Returns a string containing the XML source of all descendent nodes.
	
	See #innerXML:"

	^ String streamContents: [:stream | self printInnerXMLOn: stream]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:38'!
innerXMLPrettyPrinted
	"Pretty print version of #innerXML.
	
	See #innerXML:"

	^ String streamContents: [:stream |
		self
			printInnerXMLOn: stream
			beforeWritingDo: [:writer | writer enablePrettyPrinting]]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:25'!
lastNode
	"returns the last child node of the receiver if it has one, nil otherwise"

	^ self nodes last! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:16'!
nodeAfter: aNode
	"returns the first child node of the receiver after aNode or nil if there
	isn't one"

	^ self nodes after: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:44'!
nodeAt: anIndex
	"returns the child node at position anIndex or nil if there isn't one"

	^ self nodes at: anIndex! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:27'!
nodeAt: anIndex ifAbsent: aBlock
	"returns the child node at position anIndex or the value of aBlock
	if there isn't one"

	^ self nodes
		at: anIndex
		ifAbsent: aBlock! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/21/2016 07:54'!
nodeAt: anIndex put: aNode
	"replaces the child node at anIndex with aNode, raising an error if
	anIndex is outside the current range of the node list"

	^ self nodes
		at: anIndex
		put: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:16'!
nodeBefore: aNode
	"returns the first child node of the receiver before aNode or nil if there
	isn't one"

	^ self nodes before: aNode! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 11/4/2014 19:04'!
nodeListClass
	^ XMLNodeList! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:18'!
nodes
	"returns the internal node list; safe to modify, but adding or removing nodes
	from it will have the effect of adding or removing them from the receiver"

	^ nodes ifNil: [nodes := self nodeListClass observer: self]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:35'!
nodesCollect: aBlock
	"returns an ordered list of the values of all node children of the
	receiver evaluated with aBlock"

	^ self nodes collect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/28/2014 19:46'!
nodesDetect: aBlock
	^ self nodes detect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:14'!
nodesDetect: aBlock ifNone: anExceptionBlock
	^ self nodes detect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:32'!
nodesDo: aBlock
	"evaluates aBlock for all child nodes of the receiver"

	self nodes do: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:34'!
nodesSelect: aBlock
	"returns a node list of all node children of the receiver that aBlock
	evaluates true for"

	^ self nodes select: aBlock! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 11/1/2014 15:02'!
parserHandlerClass
	^ XMLDOMParser! !

!XMLNodeWithChildren methodsFor: 'copying' stamp: 'monty 3/2/2016 12:21'!
postCopy
	super postCopy.

	nodes
		ifNotNil: [| copiedNodes |
			copiedNodes := nodes copyEmpty observer: self.
			nodes do: [:each |
				"copySharingConfiguration is used so the config is only ever copied
				once when an element or doc is sent #copy and is done after all of the
				descendants have been copied"
				copiedNodes addLast: each copySharingConfiguration].
			nodes := copiedNodes].! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'monty 4/16/2016 13:05'!
printInnerXMLOn: aStream
	self hasChildren
		ifTrue: [
			self
				withNewXMLWriterOn: aStream
				do: [:writer |
					self writeInnerXMLOn: writer]]! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'monty 5/10/2015 06:19'!
printInnerXMLOn: aStream beforeWritingDo: aBlock
	self hasChildren
		ifTrue: [
			self
				withNewXMLWriterOn: aStream
				do: [:writer |
					aBlock value: writer.
					self writeInnerXMLOn: writer.
					writer write]]! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNode: aNode
	^ self nodes remove: aNode! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'monty 11/19/2014 20:24'!
removeNode: aNode ifAbsent: aBlock
	^ self nodes remove: aNode ifAbsent: aBlock! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes
	self nodes removeAll! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes: aNodeCollection
	^ self nodes removeAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifying' stamp: 'monty 2/2/2014 02:38'!
removedNode: aNode
	aNode parent: nil! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:43'!
replaceNode: aNode with: aReplacementNode
	"replaces aNode in the receiver with aReplacementNode, or if aReplacementNode
	is already a child of the receiver, just swaps their locations"
	| firstIndex secondIndex |

	firstIndex := self nodes indexOf: aNode.
	(secondIndex := self nodes indexOf: aReplacementNode) > 0
		ifTrue: [self nodes swap: firstIndex with: secondIndex]
		ifFalse: [self nodes at: firstIndex put: aReplacementNode].
	^ aReplacementNode.! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeInnerXMLOn: aWriter
	self nodesDo: [:each | each writeXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	self writeInnerXMLOn: aWriter! !

XMLNodeWithChildren subclass: #XMLNodeWithElements
	instanceVariableNames: 'configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithElements commentStamp: 'monty 2/27/2016 13:48' prior: 0!
This is an abstract class for nodes with elements.

Instances provide "accessing" messages to retrieve child elements by their name and namespace information. The #elementAt: forms return the first matching element, while the #elementsAt: forms return all matching child elements.

There are three different modes of enumeration: the #elements* enumerating messages enumerate child elements, the #allElements* forms enumerate the receiver (if it's an element) and all descendant elements, and the #descendantElement* forms enumerate descendant elements only.

The #findElementNamed:* forms search the receicer (if it's an element) and descendants for a specific element.

Element name matching is done the qualified and local name, so 'prefix:element-name' will only match 'prefix:element-name' while 'element-name' will match 'element-name', 'prefix:element-name' or 'different-prefix:element-name' and so on.

The inner XML can be accessed as a string using #innerXML and set (reparsed) using #innerXML:.!

XMLNodeWithElements subclass: #XMLDocument
	instanceVariableNames: 'version encoding isStandalone doctypeDeclaration doctypeDefinition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDocument commentStamp: 'monty 6/16/2015 01:46' prior: 0!
This class represents a document node, which is often the root of a DOM tree. Nodes can access their document ancestor with #document.!

!XMLDocument class methodsFor: 'instance creation' stamp: 'monty 3/24/2015 18:26'!
root: aRoot
	^ self new root: aRoot! !

!XMLDocument methodsFor: 'visiting' stamp: 'monty 7/27/2016 05:36'!
acceptNodeVisitor: aNodeVisitor
	aNodeVisitor visitDocument: self! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration
	^ doctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 4/11/2015 01:01'!
doctypeDeclaration: aDoctypeDeclaration
	aDoctypeDeclaration
		ifNotNil: [
			aDoctypeDeclaration hasParent
				ifTrue: [self errorAddingDoctypeDeclarationWithParent].
			aDoctypeDeclaration parent: self].
	doctypeDeclaration
		ifNotNil: [doctypeDeclaration parent: nil].

	doctypeDeclaration := aDoctypeDeclaration.! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/8/2015 10:09'!
doctypeDefinition
	^ doctypeDefinition! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/8/2015 10:09'!
doctypeDefinition: aDoctypeDefinition
	doctypeDefinition := aDoctypeDefinition! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 13:34'!
document
	^ self! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/24/2015 18:28'!
encoding	
	^ encoding ifNil: [encoding := '']! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
encoding: aString	
	encoding := aString! !

!XMLDocument methodsFor: 'private' stamp: 'monty 4/11/2015 01:00'!
errorAddingDoctypeDeclarationWithParent
	XMLDOMException signal: 'Cannot add doctype declaration node with parent'! !

!XMLDocument methodsFor: 'private' stamp: 'monty 3/6/2016 09:24'!
errorCannotHaveNonElementRoot
	XMLDOMException signal: 'Cannot have non-element root'! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 5/9/2015 03:10'!
hasDoctypeDeclaration
	^ self doctypeDeclaration notNil! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 5/9/2015 03:11'!
hasDoctypeDefinition
	^ self doctypeDefinition notNil! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 3/6/2016 09:12'!
hasEncoding
	^ self encoding notEmpty! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 3/6/2016 09:35'!
hasRoot
	^ self root notNil! !

!XMLDocument methodsFor: 'private' stamp: 'monty 5/9/2015 23:23'!
innerXMLContext
	^ XMLPostDoctypeDeclarationContext new! !

!XMLDocument methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:51'!
isDocument
	^ true! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 11/19/2013 05:38'!
isStandalone
	^ isStandalone ifNil: [isStandalone := false]! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:40'!
isStandalone: aBoolean
	isStandalone := aBoolean! !

!XMLDocument methodsFor: 'copying' stamp: 'monty 11/22/2014 08:20'!
postCopy
	super postCopy.

	doctypeDeclaration
		ifNotNil: [doctypeDeclaration := doctypeDeclaration copy].! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/24/2015 18:38'!
root
	"returns root element or nil"
	^ self firstElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 3/6/2016 09:24'!
root: anElement
	"replaces root element with new element"
	anElement isElement
		ifFalse: [self errorCannotHaveNonElementRoot].
	self hasChildren
		ifTrue: [self removeNodes: self elements].
	self addNode: anElement.! !

!XMLDocument methodsFor: 'validating' stamp: 'monty 3/27/2016 23:10'!
validate
	"revalidate against the DTD it was parsed with (if any)"

	self doctypeDefinition
		ifNotNil: [:dtd |
			dtd hasValidators
				ifTrue: [dtd removeIDsAndIDReferences].
			self validateWith: dtd newValidator]! !

!XMLDocument methodsFor: 'validating' stamp: 'monty 7/27/2016 07:51'!
validateWith: aValidator
	"revalidate against a custom DTD or other validator"

	(XMLValidatingNodeVisitor validator: aValidator) visitDocument: self! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
version	
	^ version! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 6/15/2015 04:17'!
version: aVersionFloat
	version := aVersionFloat! !

!XMLDocument methodsFor: 'printing' stamp: 'monty 5/9/2015 03:11'!
writeDoctypeDeclarationOn: aWriter
	self hasDoctypeDeclaration
		ifTrue: [self doctypeDeclaration writeXMLOn: aWriter]! !

!XMLDocument methodsFor: 'printing' stamp: 'monty 5/7/2015 18:53'!
writeXMLDeclarationOn: aWriter
	self version
		ifNotNil: [
			aWriter xml
				version: self version;
				encoding: self encoding;
				standalone: self isStandalone;
				write]! !

!XMLDocument methodsFor: 'printing' stamp: 'monty 5/7/2015 18:53'!
writeXMLOn: aWriter
	self
		writeXMLDeclarationOn: aWriter;
		writeDoctypeDeclarationOn: aWriter;
		writeInnerXMLOn: aWriter! !

XMLNodeWithElements subclass: #XMLElement
	instanceVariableNames: 'name localName attributes namespaceScope'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElement commentStamp: 'monty 4/6/2016 22:20' prior: 0!
The class represents an element node, which has a qualified or unqualified name and optionally attributes, namespace declarations and child nodes.

Element names can be tested using #isNamed: and #isNamedAny:, which test both the qualified and local name.

If the name is qualified and namespace support is enabled (the default), then the prefix must be mapped to a namespace URI in the element or an ancestor. The class-side instance creation #name:namespaceURI:* and #name:namespaces:* messages and the instance-side #name:namespaceURI: message can set both simultaneously. If namespace support is disabled, prefixes are not checked.

The #attribute* messages provide a Dictionary-like protocol for manipulating attribute nodes. Unlike the #elementAt:* messages, they match qualified names only, and attribute value accessors return empty strings if the attribute is absent. The underlying attribute node list can be accessed using #attributeNodes (copy before modifying if you don't want to change the element's attributes), and the names/values can be obtained as an (order-preserving) dictionary using #attributes.

See the superclasses for more info.!

!XMLElement class methodsFor: 'deprecated' stamp: 'monty 8/26/2016 03:18'!
name: aLocalName
	"will be deprecated; defined here to avoid accidentally renaming the class"

	^ self named: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'monty 8/20/2016 19:42'!
name: aLocalName attributes: anAssociationCollection
	"Returns a new element with the specified name and attributes.

	Both aLocalName and the attributes must not be prefixed or a namespace
	exception will be thrown.

	The anAssociationCollection argument can be a dictionary or any collection
	of associations."

	^ (self named: aLocalName)
		setAttributes: anAssociationCollection! !

!XMLElement class methodsFor: 'instance creation' stamp: 'monty 8/20/2016 19:44'!
name: aQualifiedName namespaceURI: aURI
	"Returns a new element with the specified name and namespace URI.

	If aQualifiedName has a prefix, then aURI must be non-empty and the prefix
	will be bound to it. Otherwise aURI will be declared as the default namespace."

	^ self new
		name: aQualifiedName
		namespaceURI: aURI! !

!XMLElement class methodsFor: 'instance creation' stamp: 'monty 8/20/2016 19:45'!
name: aQualifiedName namespaceURI: aURI attributes: anAssociationCollection
	"Returns a new element with the specified name, namespace URI, and attributes.

	If aQualifiedName has a prefix, then aURI must be non-empty and the prefix
	will be bound to it. Otherwise aURI will be declared as the default namespace.
	If the attributes are prefixed, then aQualifiedName must have the same prefix
	or a namespace exception will be thrown.

	The anAssociationCollection argument can be a dictionary or any collection
	of associations."

	^ (self
		name: aQualifiedName
		namespaceURI: aURI)
			setAttributes: anAssociationCollection! !

!XMLElement class methodsFor: 'instance creation' stamp: 'monty 8/26/2016 03:12'!
name: aQualifiedName namespaces: anAssociationCollection
	"Returns a new element with the specified name and namespaces.

	If aQualifiedName has a prefix, then anAssociationCollection must contain
	an association for it or a namespace exception will be thrown.

	The anAssociationCollection argument can be a dictionary or any collection
	of associations. An association with an empty string key sets the default
	namespace."

	^ self new
		declareNamespaces: anAssociationCollection;
		name: aQualifiedName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'monty 8/26/2016 03:13'!
name: aQualifiedName namespaces: aNamespaceAssociationCollection attributes: anAttributeAssociationCollection
	"Returns a new element with the specified name, namespaces, and attributes.

	If aQualifiedName or one of the attributes has a prefix, then
	aNamespaceAssociationCollection must contain an association for it or a
	namespace exception will be thrown.

	The aNamespaceAssociationCollection and anAttributeAssociationCollection
	arguments can be dictionaries or any collection of associations. An
	association with an empty string key sets the default namespace."

	^ (self
		name: aQualifiedName
		namespaces: aNamespaceAssociationCollection)
			setAttributes: anAttributeAssociationCollection! !

!XMLElement class methodsFor: 'instance creation' stamp: 'monty 8/26/2016 00:24'!
named: aLocalName
	"Returns a new element with the specified name.

	The argument must not be prefixed or a namespace exception will be thrown."

	^ self new name: aLocalName! !

!XMLElement methodsFor: 'visiting' stamp: 'monty 7/27/2016 05:35'!
acceptNodeVisitor: aNodeVisitor
	aNodeVisitor visitElement: self! !

!XMLElement methodsFor: 'adding' stamp: 'monty 8/16/2016 23:41'!
addElementNamed: aName
	"added first to inherit the config and namespaces before setting the name"
	^ (self addNode: 
		(self nodeFactory
			classForElement: aName
			namespaceURI: (self resolvePrefix: aName xmlPrefixBeforeLocalName)) new)
				name: aName! !

!XMLElement methodsFor: 'adding' stamp: 'monty 8/16/2016 23:41'!
addElementNamed: aName attributes: anAssociationCollection
	"added first to inherit the config and namespaces before setting the name
	and attributes"
	^ (self addNode:
		(self nodeFactory
			classForElement: aName
			namespaceURI: (self resolvePrefix: aName xmlPrefixBeforeLocalName)
			attributes: anAssociationCollection) new)
				name: aName;
				setAttributes: anAssociationCollection! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 10/18/2014 00:39'!
addedAttribute: anAttribute
	anAttribute element: self.
	self validateAttributes.! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 3/2/2016 10:02'!
addedElement: anElement
	super addedElement: anElement.
	(self hasNamespaces
		and: [self namespaceScope ~= anElement namespaceScope])
		ifTrue: [
			anElement allNamespaceScopesDo: [:each |
				each inheritMappingsFrom: self namespaceScope]]! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 4/18/2016 08:47'!
addingAttribute: anAttribute
	anAttribute element
		ifNotNil: [:element |
			element == self
				ifFalse: [self errorAddingAttributeFromOtherElement]]! !

!XMLElement methodsFor: 'enumerating' stamp: 'monty 4/19/2016 08:57'!
allElementsDo: aBlock
	"evaluates aBlock for the receiver and all descendant elements"

	aBlock value: self.
	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]].! !

!XMLElement methodsFor: 'private' stamp: 'monty 3/2/2016 13:51'!
allNamespaceScopesDo: aBlock
	self allElementsDo: [:each |
		aBlock value: each namespaceScope]! !

!XMLElement methodsFor: 'private' stamp: 'monty 2/26/2016 08:05'!
allNamespaceScopesDo: aBlock whileTrue: aConditionalBlock
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true but does not validate the namespaces after"

	(aConditionalBlock value: self namespaceScope)
		ifTrue: [
			aBlock value: self namespaceScope.
			self elementsDo: [:each |
				each
					allNamespaceScopesDo: aBlock
					whileTrue: aConditionalBlock]]! !

!XMLElement methodsFor: 'private' stamp: 'monty 2/29/2016 08:38'!
allNamespaceScopesWithValidationDo: aBlock whileTrue: aConditionalBlock
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true and validates the namespaces after"

	(aConditionalBlock value: self namespaceScope)
		ifTrue: [
			aBlock value: self namespaceScope.
			self
				validateName;
				validateAttributes.
			self elementsDo: [:each |
				each
					allNamespaceScopesWithValidationDo: aBlock
					whileTrue: aConditionalBlock]]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 03:52'!
attributeAssociations
	"returns attribute name->value associations"

	^ self attributeNodes attributeAssociations! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:20'!
attributeAt: aName
	"returns the value of the attribute whose full name matches aName or an
	empty string if there isn't any"

	^ self attributeNodes attributeValueAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:21'!
attributeAt: aName ifAbsent: aBlock
	"returns the value of the attribute whose full name matches aName or the
	value of aBlock if there isn't any"

	^ self attributeNodes
		attributeValueAt: aName
		ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:22'!
attributeAt: aName ifAbsentPut: aBlock
	"sets the value of the attribute whose full name matches aName to the value
	of aBlock if there isn't one already"

	^ self attributeNodes
		attributeValueAt: aName
		ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:22'!
attributeAt: aName put: aValue
	"sets the value of the attribute whose full name matches aName to the value
	aValue"

	^ self attributeNodes
		attributeValueAt: aName
		put: aValue! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/28/2014 18:38'!
attributeListClass
	^ XMLAttributeList! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self attributeNodes attributeNames! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self attributeNodes attributeNamesAndValuesDo: aTwoArgumentBlock! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self attributeNodes attributeNamesDo: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:24'!
attributeNodeAt: aName
	"returns the attribute node whose qualified or local name matches aName
	(unlike attributeAt:, which matches the full name exactly), or nil if there
	isn't any"

	^ self attributeNodes attributeNodeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:24'!
attributeNodeAt: aName ifAbsent: aBlock
	"returns the attribute node whose qualified or local name matches aName
	(unlike attributeAt:ifAbsent:, which matches the full name exactly), or
	the value of aBlock if there isn't any"

	^ self attributeNodes
		attributeNodeAt: aName
		ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/21/2016 07:38'!
attributeNodeAt: aName namespaceURI: aURI
	"returns the attribute node whose qualified or local name matches aName
	(unlike attributeAt:, which matches the full name exactly) and whose
	namespace URI matches aURI, or nil if there isn't any"

	^ self attributeNodes
		attributeNodeAt: aName
		namespaceURI: aURI! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/21/2016 07:39'!
attributeNodeAt: aName namespaceURI: aURI ifAbsent: aBlock
	"returns the attribute node whose qualified or local name matches aName
	(unlike attributeAt:ifAbsent:, which matches the full name exactly) and
	whose namespace URI matches aURI, or the value of aBlock if there isn't any"

	^ self attributeNodes
		attributeNodeAt: aName
		namespaceURI: aURI
		ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:29'!
attributeNodes
	"Returns the attribute node list. Can be modified, but adding or removing
	attributes also adds or removes them from the element. Copy before
	modifying if you don't want that."

	^ attributes ifNil: [attributes := self attributeListClass observer: self]! !

!XMLElement methodsFor: 'enumerating' stamp: 'monty 11/26/2014 19:13'!
attributeNodesDo: aBlock
	self attributeNodes do: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:29'!
attributes
	"returns a new order preserving dictionary of attribute names and values"

	^ self attributeNodes attributes! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 2/29/2016 08:22'!
declareNamespaces: anAssociationCollection
	anAssociationCollection associationsDo: [:each |
		self
			declarePrefix: each key
			uri: each value]! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 3/2/2016 10:20'!
declarePrefix: aPrefix uri: aURI
	| oldMapping |

	oldMapping :=
		self namespaceScope
			prefixMappingAt: aPrefix
			ifAbsent: [nil].
	(oldMapping notNil
		and: [oldMapping = aURI])
		ifFalse: [
			self
				allNamespaceScopesDo: [:scope |
					scope
						mapPrefix: aPrefix
						to: aURI]
				whileTrue: [:scope |
					(scope includesPrefix: aPrefix) not
						or: [
							scope
								includesPrefix: aPrefix
								mappedTo: oldMapping]]]! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 2/23/2016 10:47'!
declaredPrefixes
	^ self namespaceScope mappedPrefixes! !

!XMLElement methodsFor: 'testing' stamp: 'monty 7/20/2015 21:18'!
declaresDefaultNamespace
	"Did the receiver declare a new default namespace rather than inheriting one?"

	^ self namespaceScope hasDefaultNamespace
		and: [self isRoot
			or: [self defaultNamespace ~= self parent defaultNamespace]]! !

!XMLElement methodsFor: 'testing' stamp: 'monty 7/19/2015 20:47'!
declaresPrefix: aPrefix uri: aURI
	"Did the receiver declare a new prefix/uri pair rather than inheriting one?"

	^ (self isDeclaredPrefix: aPrefix uri: aURI)
		and: [self isRoot
			or: [(self parent isDeclaredPrefix: aPrefix uri: aURI) not]]! !

!XMLElement methodsFor: 'namespacing' stamp: 'JAAyer 6/20/2010 23:08'!
defaultNamespace
	^ self namespaceScope defaultNamespace! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 3/2/2016 10:21'!
defaultNamespace: aURI
	"unlike non-default namespaces, the uri can be empty to shadow an
	inherited default namesapce"
	| oldNamespace |

	oldNamespace := self defaultNamespace.
	self
		allNamespaceScopesDo: [:scope |
			scope defaultNamespace: aURI]
		whileTrue: [:scope |
			scope defaultNamespace = oldNamespace].! !

!XMLElement methodsFor: 'private' stamp: 'monty 4/18/2016 08:47'!
errorAddingAttributeFromOtherElement
	XMLDOMException signal: 'Cannot add attribute node from other element'! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/20/2016 02:57'!
expandedName
	"returns the expanded name of the element as a string in the form of either
	'{namespaceURI}localName' (if there's a namespace URI) or 'localName'"

	^ self localName expandWithXMLNamespaceURI: self namespaceURI! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/20/2016 10:55'!
hasAttributes
	"returns true if this element has attribute"

	"optimized to avoid lazy initialization"
	attributes == nil
		ifTrue: [^ false].
	^ attributes size > 0.! !

!XMLElement methodsFor: 'testing' stamp: 'monty 2/13/2015 22:20'!
hasNamespaceURI
	^ self namespaceURI notEmpty! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/20/2016 10:56'!
hasNamespaces
	"returns true if the receiver declared or inherited namespace declarations"

	"optimized to avoid lazy initialization"
	namespaceScope == nil
		ifTrue: [^ false].
	^ namespaceScope hasNamespaces.! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 09:25'!
hasPrefix
	"returns true if the receiver's name is a qualified name with a prefix"

	^ self name size > self localName size! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributeNodes includesAttribute: aName.! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 10:35'!
includesAttributeNode: aName
	^ self attributeNodes includesAttributeNode: aName.! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 10:35'!
includesAttributeNode: aName namespaceURI: aURI
	^ self attributeNodes
		includesAttributeNode: aName
		namespaceURI: aURI! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 03:55'!
isDeclaredPrefix: aPrefix
	"is the namespace prefix aPrefix declared for the receiver and its descendants?"

	^ self namespaceScope includesPrefix: aPrefix! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 03:55'!
isDeclaredPrefix: aPrefix uri: aURI
	"is the namespace prefix aPrefix mapped to aURI for the receiver and
	its descendants?"

	^ self namespaceScope
		includesPrefix: aPrefix
		mappedTo: aURI! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ true! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 09:24'!
isElementNamed: aName
	"returns true if the receiver's qualified name or local name matches aName"

	^ self isNamed: aName! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 09:24'!
isElementNamedAny: aNameCollection
	"returns true if the receiver's qualified name or local name matches any
	name in aNameCollection"

	^ self isNamedAny: aNameCollection! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/20/2016 10:57'!
isInLanguage: aLanguage
	"Returns true if the receiver or one of its ancestors has an xml:lang
	attribute, and if the value is an IETF language tag that matches aLanguage
	or if its first ('-' separated) subtag matches aLanguage."

	^ (self
		attributeNodeAt: 'xml:lang'
		ifAbsent: [
			^ self hasParent
				and: [self parent isInLanguage: aLanguage]])
			hasLanguage: aLanguage! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 03:56'!
isNamed: aName
	"Does the receiver's qualified or local name match the argument?"

	^ self name isXMLQualifiedOrLocalName: aName! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 09:25'!
isNamedAny: aNameCollection
	"Does the receiver's qualified or local name match any of the names
	in aNameCollection?"

	^ aNameCollection includesXMLName: self name! !

!XMLElement methodsFor: 'testing' stamp: 'monty 4/19/2016 03:57'!
isRoot
	"Is the receiver the root element of a DOM tree?"

	^ (self hasParent and: [self parent isElement]) not! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:37'!
localName
	"returns the local name of the receiver, which is diffrent from #name if
	#name is a qualified name"

	^ localName ifNil: [localName := self name xmlLocalNameAfterPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:32'!
name
	"returns the full name of the element, either a qualified name if it has a
	prefix or just a local name if it doesn't"

	^ name ifNil: [name := '']! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:33'!
name: aName
	"Renames the element. If aName is a qualified name and namespace checking is
	enabled (default), then the prefix must already have been declared. This can
	be done while renaming using name:namespaceURI."

	self renameWith: [
		name := aName.
		localName := aName xmlLocalNameAfterPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:34'!
name: aName namespaceURI: aURI
	"Renames the element and also declares a namespace URI in one step. If aName
	is qualified, then it declares the prefix to be mapped to aURI. If aName has
	no prefix, then aURI is declared as the default namespace URI."

	self
		declarePrefix: (aName xmlPrefixBeforeLocalName) uri: aURI;
		name: aName! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:10'!
namespaceScope
	^ namespaceScope ifNil: [namespaceScope := XMLNamespaceScope new]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 3/14/2010 12:52'!
namespaceURI
	^ self resolvePrefix: self prefix! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:36'!
nextElement
	"returns the first element after the receiver that is a child of the same
	parent if there is one, nil otherwise"

	^ self parent ifNotNil: [:node | node elementAfter: self]! !

!XMLElement methodsFor: 'copying' stamp: 'monty 3/1/2016 14:00'!
postCopy
	super postCopy.

	namespaceScope
		ifNotNil: [namespaceScope := namespaceScope copy].
	attributes
		ifNotNil: [| copiedAttributes |
			copiedAttributes := attributes copyEmpty observer: self.
			attributes do: [:each | copiedAttributes addLast: each copy].
			attributes := copiedAttributes].! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:38'!
prefix
	"returns the prefix of the name of the receiver or an empty string if
	there isn't one"

	^ self name xmlPrefixBeforeLocalName: self localName! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:39'!
prefix: aPrefix
	"Changes the prefix of the name of the receiver to aPrefix. If namespace
	checking is enabled (default) and aPrefix is non-empty, then aPrefix must
	have been declared."

	self renameWith: [name := self localName qualifyWithXMLPrefix:  aPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 4/19/2016 08:36'!
previousElement
	"returns the first element before the receiver that is a child of the same
	parent if there is one, nil otherwise"

	^ self parent ifNotNil: [:node | node elementBefore: self]! !

!XMLElement methodsFor: 'removing' stamp: 'monty 4/6/2016 22:30'!
removeAttribute: aName
	^ (self attributeNodes removeAttribute: aName) value.! !

!XMLElement methodsFor: 'removing' stamp: 'monty 4/6/2016 22:30'!
removeAttribute: aName ifAbsent: aBlock
	^ self attributeNodes
		removeAttribute: aName
		ifAbsent: aBlock! !

!XMLElement methodsFor: 'removing' stamp: 'JAAyer 7/22/2010 08:54'!
removeAttributeNode: aNode
	^ self attributeNodes remove: aNode! !

!XMLElement methodsFor: 'removing' stamp: 'monty 12/9/2015 09:06'!
removeAttributeNode: aNode ifAbsent: aBlock
	^ self attributeNodes
		remove: aNode
		ifAbsent: aBlock! !

!XMLElement methodsFor: 'removing' stamp: 'monty 11/23/2014 00:29'!
removeAttributes
	self attributeNodes removeAll! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 10/18/2014 00:43'!
removedAttribute: anAttribute
	anAttribute element: nil! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/18/2014 01:35'!
renameWith: aBlock
	| oldName |

	oldName := self name.
	aBlock value.
	self name = oldName
		ifFalse: [
			self validateName.
			self hasParent
				ifTrue: [
					self parent
						renamedElement: self
						from: oldName
						to: self name]].! !

!XMLElement methodsFor: 'notifying' stamp: 'monty 4/18/2016 07:37'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributeNodes	
		renamedAttribute: anAttribute
		from: anOldName
		to: aNewName.
	self validateAttributes.! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 3/20/2016 23:55'!
resolvePrefix: aPrefix
	"Resolves the namespace URI of aPrefix (the defualt namespace URI if aPrefix
	is empty), including the predefined 'xml' and 'xmlns' prefixes. Returns an
	empty string if aPrefix is unresolvable."

	^ self
		resolvePrefix: aPrefix
		ifUnresolvable: ''! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 3/20/2016 23:55'!
resolvePrefix: aPrefix ifUnresolvable: aBlock
	"Resolves the namespace URI of aPrefix (the defualt namespace URI if aPrefix
	is empty), including the predefined 'xml' and 'xmlns' prefixes. Returns the
	value of aBlock if aPrefix is unresolvable."

	self hasNamespaces "avoid lazy initializing the scope"
		ifTrue: [
			^ self namespaceScope
				resolvePrefix: aPrefix
				ifUnresolvable: aBlock]
		ifFalse: [^ aBlock value]! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 2/25/2016 16:10'!
resolvePrefixesOfNamespaceURI: aURI
	"returns a set of all the namespace prefixes of aURI"
	^ self namespaceScope resolvePrefixesOfNamespaceURI: aURI! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 4/11/2016 03:25'!
setAttributes: anAssociationCollection
	self attributeNodes setAttributes: anAssociationCollection! !

!XMLElement methodsFor: 'private' stamp: 'monty 12/27/2013 03:42'!
setNamespaceScope: aNamespaceScope
	namespaceScope := aNamespaceScope! !

!XMLElement methodsFor: 'private' stamp: 'monty 8/6/2016 20:10'!
setQualifiedName: aQualifiedName localName: aLocalName attributes: aDictionary
	name := aQualifiedName.
	localName := aLocalName.
	aDictionary size > 0 "optimization"
		ifTrue: [
			"setWithOrderPreservingDictionary: is used to initialize the attribute
			node list with aDictionary without revalidation"
			(attributes := self attributeListClass new: aDictionary size)
				observer: self;
				setWithOrderPreservingDictionary: aDictionary].! !

!XMLElement methodsFor: 'private' stamp: 'monty 12/2/2014 22:57'!
sortKey
	^ self name! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 11/24/2014 06:54'!
undeclareDefaultNamespace
	self undeclarePrefix: ''! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 3/2/2016 10:21'!
undeclarePrefix: aPrefix
	| currentNamespaceURI |

	currentNamespaceURI :=
		self namespaceScope
			prefixMappingAt: aPrefix
			ifAbsent: [^ self].
	self
		allNamespaceScopesWithValidationDo: [:scope |
			scope unmapPrefix: aPrefix]
		whileTrue: [:scope |
			scope
				includesPrefix: aPrefix
				mappedTo: currentNamespaceURI].! !

!XMLElement methodsFor: 'namespacing' stamp: 'monty 3/2/2016 10:19'!
undeclarePrefixes
	self declaredPrefixes do: [:each |
		self undeclarePrefix: each]! !

!XMLElement methodsFor: 'private' stamp: 'monty 8/26/2016 00:36'!
validateAttributes
	| attributeLocalNamesAndURIs |

	self usesNamespaces
		ifFalse: [^ self].

	self attributeNodesDo: [:attribute | | prefix |
		(prefix := attribute prefix) notEmpty
			ifTrue: [
				"not initialized unless needed"
				attributeLocalNamesAndURIs
					ifNil: [
						attributeLocalNamesAndURIs :=
							Set new: self attributeNodes size].
				self namespaceScope
					validatePrefix: prefix
					ofAttribute: attribute name
					hasNoAliasesin: attributeLocalNamesAndURIs]
			ifFalse: [self namespaceScope validateUnprefixedName: attribute name]].! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/9/2014 06:42'!
validateName
	self usesNamespaces
		ifTrue: [
			self namespaceScope
				validateAndResolvePrefix: self prefix
				ofElement: self name]! !

!XMLElement methodsFor: 'printing' stamp: 'monty 3/2/2016 12:18'!
writeXMLOn: aWriter
	| tagWriter |

	tagWriter := aWriter tag: self name.
	self hasNamespaces
		ifTrue: [
			"non-standard setXMLNSDeclarations: use for performance"
			tagWriter setXMLNSDeclarations: self namespaceScope prefixMappings].
	self hasAttributes
		ifTrue: [
			"non-standard setAttributes: use for performance"
			tagWriter setAttributes: self attributeNodes].
	self hasChildren
		ifTrue: [tagWriter with: [self writeInnerXMLOn: aWriter]]
		ifFalse: [tagWriter write].! !

!XMLNodeWithElements methodsFor: 'visiting' stamp: 'monty 7/28/2016 10:54'!
acceptNodeVisitor: aNodeVisitor
	aNodeVisitor visitNodeWithElements: self! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 8/17/2016 00:10'!
addCData: aString
	^ self addNode: (self newCData: aString)! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/17/2016 00:09'!
addContent: aStringOrStringNode
	"will be deprecated; use #addString: instead"

	(self hasChildren
		and: [self lastNode isCoalescingStringNode])
		ifTrue: [self lastNode append: aStringOrStringNode]
		ifFalse: [self addNode:
			(aStringOrStringNode isString
				ifTrue: [self nodeFactory newStringNode: aStringOrStringNode]
				ifFalse: [aStringOrStringNode])].
	"returns self for backwards compatibility"! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/14/2016 17:13'!
addElement: anElement
	self deprecated:
		'Use #addNode: with an element argument or #addElementNamed: instead'.

	self addNode: anElement.
	"returns self for backwards compatibility"! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 8/16/2016 21:52'!
addElementNamed: aName
	"adds and returns a new element with the specified name"

	"added first to inherit the config and namespaces
	before setting the name"
	^ (self addNode:
		(self nodeFactory classForElement: aName) new)
			name: aName! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 8/16/2016 21:52'!
addElementNamed: aName attributes: anAssociationCollection
	"adds and returns a new element with the specified name and attributes"

	"added first to inherit the config and namespaces
	before setting the name and attributes"
	^ (self addNode:
		(self nodeFactory
			classForElement: aName
			attributes: anAssociationCollection) new)
				name: aName;
				setAttributes: anAssociationCollection! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 8/16/2016 21:53'!
addElementNamed: aName namespaceURI: aURI
	"adds and returns a new element with the specified name and namespace
	URI"

	"added first to inherit the config and namespaces
	before setting the name and namespace URI"
	^ (self addNode:
		(self nodeFactory
			classForElement: aName
			namespaceURI: aURI) new)
				name: aName
				namespaceURI: aURI! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 8/16/2016 21:53'!
addElementNamed: aName namespaceURI: aURI attributes: anAssociationCollection
	"adds and returns a new element with the specified name, namespace URI
	and attributes"

	"added first to inherit the config and namespaces
	before setting the name, namespace URI, and attributes"
	^ (self addNode:
		(self nodeFactory
			classForElement: aName
			namespaceURI: aURI
			attributes: anAssociationCollection) new)
				name: aName namespaceURI: aURI;
				setAttributes: anAssociationCollection! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 8/14/2016 21:46'!
addString: aString
	"Adds a new string node for aString or just appends aString to the
	last node if it's a coalescing string node (not CDATA).

	Returns the new string node or the string node appeneded to."

	| lastNode |

	(self hasChildren
		and: [(lastNode := self lastNode) isCoalescingStringNode])
		ifTrue: [^ lastNode append: aString]
		ifFalse: [^ self addNode: (self nodeFactory newStringNode: aString)]! !

!XMLNodeWithElements methodsFor: 'notifying' stamp: 'monty 10/28/2014 17:59'!
addedElement: anElement
	self addedNode: anElement.
	anElement configuration: self configuration.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:06'!
allElements
	"returns a node list with all descendant elements (and the receiver if
	it's an element)"
	| allElements |

	allElements := self newListForSelect.
	self allElementsDo: [:each | allElements addLast: each].
	^ allElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 12:07'!
allElementsCollect: aBlock
	"returns an ordered list of the values of evaluating all descendant
	elements (and the receiver if it's an element) with aBlock"
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self allElementsDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:47'!
allElementsDetect: aBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	^ self allElementsDetect: aBlock ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:15'!
allElementsDetect: aBlock ifNone: anExceptionBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return the value of anExceptionBlock."

	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 12:08'!
allElementsDo: aBlock
	"evaluates all descendant elements (and the receiver if it's an
	element) with aBlock"

	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:06'!
allElementsNamed: aName
	"returns a node list with all descendant elements (and the receiver if
	it's an element) whose qualified or local names match aName"

	^ self allElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 12:08'!
allElementsNamed: aName do: aBlock
	"evaluates all descendant elements (and the receiver if it's an
	element) whose qualified or local names match aName with aBlock"

	self allElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:12'!
allElementsNamed: aName namespaceURI: aURI
	"returns a node list with all descendant elements (and the receiver if
	it's an element) whose qualified or local names match aName and whose
	namespace URIs matches aURI"

	^ self allElementsSelect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI]]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 12:09'!
allElementsSelect: aBlock
	"returns a new node list with all descendant elements (and the receiver
	if it's an element) that aBlock evaluates true for"

	| selectedElements |

	selectedElements := self newListForSelect.
	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:42'!
configuration
	^ configuration ifNil: [
		self configuration: self configurationClass new.
		configuration]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:43'!
configuration: aConfiguration
	configuration := aConfiguration.

	self hasChildren
		ifTrue: [self elementsDo: [:each | each configuration: aConfiguration]]! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/2/2014 18:52'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:40'!
contentNodes
	"returns a node list of the receiver's element and string node children"

	^ self nodes contentNodes! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 12:41'!
contentNodesDo: aBlock
	"evaluates aBlock for all element and string node children of the receiver"

	self nodes contentNodesDo: aBlock! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/20/2016 01:09'!
contentString: aString
	"will be deprecated; use #removeNodes and #addString: instead"

	self
		removeNodes;
		addString: aString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 8/7/2016 12:51'!
contentStringAt: aName
	"Returns the content string of the first element child of the receiver
	whose qualified or local name matches aName or an empty string if there
	isn't one.

	See also #rawContentStringAt:"

	^ self nodes contentStringAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/19/2016 03:47'!
descendantElements
	"returns a node list of all descendant elements"
	| descendentElements |

	descendentElements := self newListForSelect.
	self descendantElementsDo: [:each | descendentElements addLast: each].
	^ descendentElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:18'!
descendantElementsCollect: aBlock
	"returns an ordered list of the values of evaluating all descendant
	elements with aBlock"
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self descendantElementsDo: [:each |
		collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:36'!
descendantElementsDetect: aBlock
	"returns the first descendant element that aBlock evaluates true for or
	nil if there isn't one"

	^ self
		descendantElementsDetect: aBlock
		ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:36'!
descendantElementsDetect: aBlock ifNone: anExceptionBlock
	"returns the first descendant element that aBlock evaluates true for or
	the value of anExceptionBlock if there isn't one"

	self descendantElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:18'!
descendantElementsDo: aBlock
	"evaluates all descendant elements with aBlock"

	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:11'!
descendantElementsNamed: aName
	"returns a node list of all descendant elements whose qualified
	or local names match aName"

	^ self descendantElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:18'!
descendantElementsNamed: aName do: aBlock
	"evaluates all descendant elements whose qualified or local names
	match aName with aBlock"

	self descendantElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:12'!
descendantElementsNamed: aName namespaceURI: aURI
	"returns a node list of all descendant elements whose qualified
	or local names match aName and whose namespace URIs match aURI"

	^ self descendantElementsSelect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI]]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:19'!
descendantElementsSelect: aBlock
	"returns a new node list with all descendant elements that aBlock
	evaluates true for"
	| selectedElements |

	selectedElements := self newListForSelect.
	self descendantElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElements
	"less common alternate spelling, will be deprecated;
	use #descendantElements instead"

	^ self descendantElements! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElementsCollect: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantElementsCollect: instead"

	^ self descendantElementsCollect: aBlock! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElementsDetect: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantElementsDetect: instead"

	^ self descendantElementsDetect: aBlock! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElementsDetect: aBlock ifNone: anExceptionBlock
	"less common alternate spelling, will be deprecated;
	use #descendantElementsDetect:ifNone: instead"

	^ self
		descendantElementsDetect: aBlock
		ifNone: anExceptionBlock! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElementsDo: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantElementsDo: instead"

	^ self descendantElementsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElementsNamed: aName
	"less common alternate spelling, will be deprecated;
	use #descendantElementsNamed: instead"

	^ self descendantElementsNamed: aName! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElementsNamed: aName do: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantElementsNamed:do: instead"

	^ self
		descendantElementsNamed: aName
		do: aBlock! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElementsNamed: aName namespaceURI: aURI
	"less common alternate spelling, will be deprecated;
	use #descendantElementsNamed:namespaceURI: instead"

	^ self
		descendantElementsNamed: aName
		namespaceURI: aURI! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/6/2016 20:56'!
descendentElementsSelect: aBlock
	"less common alternate spelling, will be deprecated;
	use #descendantElementsSelect: instead"

	^ self descendantElementsSelect: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:01'!
elementAfter: anElement
	"returns the first element child of the receiver after anElement or nil if
	there isn't one"

	^ self nodes elementAfter: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:07'!
elementAt: aName
	"returns the first element child of the receiver whose qualified or
	local name matches aName or nil if there isn't one"

	^ self nodes elementAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:07'!
elementAt: aName ifAbsent: aBlock
	"returns the first element child of the receiver whose qualified or
	local name matches aName or the value of aBlock if there isn't one"

	^ self nodes
		elementAt: aName
		ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:08'!
elementAt: aName namespaceURI: aURI
	"returns the first element child of the receiver whose qualified or
	local name matches aName and whose namespace URI matches aURI or nil
	if there isn't one"

	^ self nodes
		elementAt: aName
		namespaceURI: aURI! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:08'!
elementAt: aName namespaceURI: aURI ifAbsent: aBlock
	"returns the first element child of the receiver whose qualified or
	local name matches aName and whose namespace URI matches aURI or the
	value of aBlock if there isn't one"

	^ self nodes
		elementAt: aName
		namespaceURI: aURI
		ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 12:01'!
elementBefore: anElement
	"returns the first element child of the receiver before anElement or
	nil if there isn't one"

	^ self nodes elementBefore: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/26/2014 08:18'!
elementNames
	^ self nodes elementNames! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:06'!
elements
	"returns a new node list of all of the receiver's elements"

	^ self nodes elements! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:13'!
elementsAt: aName
	"returns a node list of all the receiver's elements whose qualified
	or local names match aName"

	^ self nodes elementsAt: aName! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:20'!
elementsAt: aName do: aBlock
	"evaluates aBlock for all element children of the receiver whose
	qualified or local names match aName"

	self nodes
		elementsAt: aName
		do: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:13'!
elementsAt: aName namespaceURI: aURI
	"returns a node list of all the receiver's elements whose qualified
	or local names match aName and whose namespace URIs match aURI"

	^ self nodes
		elementsAt: aName
		namespaceURI: aURI! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:23'!
elementsCollect: aBlock
	"returns an ordered list of the values of evaluating the element
	children of the receiver with aBlock"

	^ self nodes elementsCollect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:24'!
elementsDetect: aBlock
	"returns the first element child of the receiver that aBlock evaluates
	true for or nil if there isn't one"

	^ self nodes elementsDetect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:24'!
elementsDetect: aBlock ifNone: anExceptionBlock
	"returns the first element child of the receiver that aBlock evaluates
	true for or the value of anExceptionBlock if there isn't one"

	^ self nodes
		elementsDetect: aBlock
		ifNone: anExceptionBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:21'!
elementsDo: aBlock
	"evaluates aBlock for all element children of the receiver"

	self nodes elementsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:21'!
elementsSelect: aBlock
	"returns a node list of all element children of the receiver that
	aBlock evaluates true for"

	^ self nodes elementsSelect: aBlock! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 1/5/2015 09:12'!
escapesContentEntitiesWhenWriting
	"avoid initializing config; defualt to true"
	^ configuration isNil
		or: [configuration replacesContentEntityReferences]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 4/20/2016 11:48'!
findElementNamed: aName
	"searches the receiver and all descendants for the first element whose
	qualified or local name matches aName"

	^ self allElementsDetect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 4/20/2016 11:48'!
findElementNamed: aName namespaceURI: aURI
	"searches the receiver and all descendants for the first element whose
	qualified or local name matches aName and whose namespace URI matches aURI"

	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 4/20/2016 11:48'!
findElementNamed: aName namespaceURI: aURI with: aBlock
	"searches the receiver and all descendants for the first element whose
	qualified or local name matches aName and whose namespace URI matches
	aURI and that aBlock evaluates true for"

	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aURI
				and: [aBlock value: each]]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 4/20/2016 11:49'!
findElementNamed: aName with: aBlock
	"searches the receiver and all descendants for the first element whose
	qualified or local name matches aName and aBlock evaluates true for"

	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 4/20/2016 11:44'!
findElementWithID: anID
	"Works like XML DOM Level 2's getElementByID(). Unlike HTML, it requires
	a DTD with ID declarations to identify ID attributes."

	| document doctypeDefinition elementsAndIDAttributes |
	
	((document := self document) notNil
		and: [(doctypeDefinition := document doctypeDefinition) notNil
			and: [doctypeDefinition hasElementIDValidator]])
		ifFalse: [^ nil].
	elementsAndIDAttributes :=
		doctypeDefinition elementIDValidator elementsAndIDAttributes.

	self allElementsDo: [:each |
		each hasAttributes
			ifTrue: [
				elementsAndIDAttributes
					at: each name
					ifPresent: [:idAttribute |
						(each attributeAt: idAttribute) = anID
							ifTrue: [^ each]]]].
	^ nil.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:12'!
firstElement
	"returns the first child element of the receiver or nil if there
	isn't one"

	^ self nodes firstElement! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 10/25/2014 21:02'!
hasContentNodes
	^ self nodes hasContentNodes! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 4/19/2016 10:42'!
hasElements
	"returns true if the receiver has element children"

	"optimized with a nil test to avoid lazy initializing nodes"
	nodes == nil
		ifTrue: [^ false].
	^ nodes hasElements.! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasStringNodes
	^ self nodes hasStringNodes! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 4/19/2016 10:41'!
includesElement: aName
	"returns true if the receiver has a child element whose qualified or local name
	matches aName"

	^ self nodes includesElement: aName! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 4/19/2016 10:41'!
includesElement: aName namespaceURI: aURI
	"returns true if the receiver has a child element whose qualified or local name
	matches aName and whose namespace URI matches aURI"

	^ self nodes
		includesElement: aName
		namespaceURI: aURI! !

!XMLNodeWithElements methodsFor: 'parsing' stamp: 'monty 8/7/2016 14:45'!
innerXML: aStringOrStream
	"Replaces the node children by parsing new inner XML from a string or stream
	argument. By default the same parser configuration will be used to parse the
	new inner XML."

	self innerXMLParsedWith: (self parserHandlerClass on: aStringOrStream)! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 5/8/2015 00:40'!
innerXMLContext
	^ XMLContentContext new! !

!XMLNodeWithElements methodsFor: 'parsing' stamp: 'monty 5/8/2015 00:10'!
innerXMLParsedWith: aParser
	| removedNodes |

	removedNodes := self nodes asArray.
	self removeNodes.
	[aParser
		pushInnerXMLRootNode: self;
		parseDocument]
		on: Error
		do: [:error |
			self
				removeNodes;
				addNodes: removedNodes.
			error pass].! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:15'!
isContentNode
	^ true! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:12'!
lastElement
	"returns the last child element of the receiver or nil if there
	isn't one"

	^ self nodes lastElement! !

!XMLNodeWithElements methodsFor: 'deprecated' stamp: 'monty 8/5/2016 22:22'!
newString: aString
	"will be deprecated; use #newStringNode: instead"

	^ self newStringNode: aString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:02'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:37'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 4/17/2015 10:43'!
nodeListClass
	^ self configuration nodeFactory nodeListClass! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/1/2014 15:03'!
parserHandlerClass
	^ self configuration parserHandlerClass! !

!XMLNodeWithElements methodsFor: 'copying' stamp: 'monty 3/2/2016 05:44'!
postCopyConfiguration
	configuration
		ifNotNil: [self configuration: configuration copy]! !

!XMLNodeWithElements methodsFor: 'printing' stamp: 'monty 11/19/2014 23:44'!
printRawContentOn: aStream
	self contentNodesDo: [:each | each printRawContentOn: aStream]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:54'!
rawContentStringAt: aName
	"returns the raw content string (without space inserted) of the first
	element child of the receiver whose qualified or local name matches aName
	or an empty string if there isn't one"

	^ self nodes rawContentStringAt: aName! !

!XMLNodeWithElements methodsFor: 'notifying' stamp: 'monty 10/28/2014 17:59'!
removedElement: anElement
	self removedNode: anElement! !

!XMLNodeWithElements methodsFor: 'notifying' stamp: 'monty 10/18/2014 01:08'!
renamedElement: anElement from: anOldName to: aNewName
	self nodes
		renamedElement: anElement
		from: anOldName
		to: aNewName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:53'!
stringNodes
	"returns a new node list of all string node children of the receiver"

	^ self nodes stringNodes! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:37'!
stringNodesDo: aBlock
	"evaluates each string node child of the receiver with aBlock"

	self nodes stringNodesDo: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/20/2016 11:53'!
strings
	"returns an ordered list of the strings of the string nodes of the receiver"

	^ self nodes strings! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 4/20/2016 11:37'!
stringsDo: aBlock
	"evaluates each string of each string node child of the receiver with
	aBlock"

	self nodes stringsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 4/20/2016 11:03'!
usesNamespaces
	"returns true if namespace checking is enabled (default)"

	"optimized to avoid lazy initializing the config"
	configuration == nil
		ifTrue: [^ true].
	^ configuration usesNamespaces.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 4/19/2016 03:50'!
usesNamespaces: aBoolean
	"can be used to disable namespace checking, which is enabled by default"

	self configuration usesNamespaces: aBoolean! !

XMLNode subclass: #XMLPI
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPI commentStamp: 'JAAyer 6/25/2010 14:06' prior: 0!
This class represents a processing instruction.
!

!XMLPI class methodsFor: 'instance creation' stamp: 'monty 3/21/2015 22:52'!
target: aTargetString data: aDataString
	^ self new
		target: aTargetString;
		data: aDataString! !

!XMLPI methodsFor: 'visiting' stamp: 'monty 7/27/2016 05:32'!
acceptNodeVisitor: aNodeVisitor
	aNodeVisitor visitPI: self! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
data
	^ data ifNil: [data := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
data: aString
	data := aString! !

!XMLPI methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ true! !

!XMLPI methodsFor: 'private' stamp: 'monty 12/3/2014 01:08'!
sortKey
	^ self target! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 12:42'!
target
	^ target ifNil: [target := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
target: aString
	target := aString! !

!XMLPI methodsFor: 'printing' stamp: 'monty 5/7/2015 18:52'!
writeXMLOn: aWriter
	aWriter pi
		target: self target;
		data: self data;
		write! !

XMLNode subclass: #XMLString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLString commentStamp: 'monty 8/5/2016 16:45' prior: 0!
This class represents string (character data) nodes. Instances are mutable, but the string is treated as immutable. String nodes can be created using the class-side #string: message or by sending a String #asXMLStringNode. 

When printing, special characters XML-escaped.!

XMLString subclass: #XMLCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCData commentStamp: 'monty 6/15/2015 22:05' prior: 0!
This class represents a preserved <!![CDATA[...]]> section. By default these are handled as XMLString nodes and merged with adjacent string nodes during parsing.  To preserve CDATA nodes, use #preservesCDataNodes: with the DOM parser before parsing.!

!XMLCData methodsFor: 'visiting' stamp: 'monty 7/27/2016 05:32'!
acceptNodeVisitor: aNodeVisitor
	aNodeVisitor visitCData: self! !

!XMLCData methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ true! !

!XMLCData methodsFor: 'private' stamp: 'monty 8/6/2016 21:02'!
isCoalescingStringNode
	"this returns false to stop preserved CDATA nodes from coalescing
	together or with ordinary string nodes"

	^ false! !

!XMLCData methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter cdata: self string! !

!XMLString class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
string: aString
	^ self new string: aString! !

!XMLString methodsFor: 'visiting' stamp: 'monty 8/5/2016 13:17'!
acceptNodeVisitor: aNodeVisitor
	aNodeVisitor visitStringNode: self! !

!XMLString methodsFor: 'deprecated' stamp: 'monty 8/14/2016 03:10'!
addContent: aStringOrStringNode
	"will be deprecated"

	self append: aStringOrStringNode! !

!XMLString methodsFor: 'appending/prepending' stamp: 'monty 8/14/2016 03:10'!
append: aStringOrStringNode
	string :=
		string
			ifNil: [aStringOrStringNode asString]
			ifNotNil: [string, aStringOrStringNode asString]! !

!XMLString methodsFor: 'converting' stamp: 'monty 1/1/2014 07:14'!
asString
	^ self string! !

!XMLString methodsFor: 'converting' stamp: 'monty 11/4/2014 14:51'!
asXMLEscapedString
	^ self string asXMLEscapedString! !

!XMLString methodsFor: 'deprecated' stamp: 'monty 8/16/2016 21:57'!
asXMLString
	"will be deprecated; use #asXMLStringNode instead"

	^ self asXMLStringNode! !

!XMLString methodsFor: 'converting' stamp: 'monty 8/5/2016 16:39'!
asXMLStringNode
	^ self! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 21:43'!
beginsWith: aStringOrStringNode
	^ self string beginsWith: aStringOrStringNode asString! !

!XMLString methodsFor: 'deprecated' stamp: 'monty 8/20/2016 01:09'!
contentString: aString
	"will be deprecated; use #string: instead"

	self string: aString! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 12:38'!
deleteFrom: aStart to: anEnd
	self replaceFrom: aStart to: anEnd with: ''! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 21:43'!
endsWith: aStringOrStringNode
	^ self string endsWith: aStringOrStringNode asString! !

!XMLString methodsFor: 'testing' stamp: 'monty 12/9/2015 10:55'!
includesSubstring: aStringOrStringNode
	"avoid String>>includesSubstring: for backwards compatibility"
	^ aStringOrStringNode isEmpty
		or: [(self indexOfSubstring: aStringOrStringNode) > 0]! !

!XMLString methodsFor: 'accessing' stamp: 'monty 2/2/2015 17:29'!
indexOfSubstring: aStringOrStringNode
	^ self string
		findString: aStringOrStringNode asString
		startingAt: 1! !

!XMLString methodsFor: 'private' stamp: 'monty 8/6/2016 17:52'!
isCoalescingStringNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:26'!
isContentNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 22:28'!
isEmpty
	^ self string isEmpty! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/5/2014 12:05'!
isString: aStringOrStringNode
	^ self string = aStringOrStringNode asString! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:35'!
isStringNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/5/2014 12:09'!
isStringNode: aStringOrStringNode
	^ self isString: aStringOrStringNode! !

!XMLString methodsFor: 'testing' stamp: 'monty 4/13/2016 11:35'!
isWhitespace
	^ self string isXMLWhitespace! !

!XMLString methodsFor: 'splitting/merging' stamp: 'monty 2/2/2015 17:30'!
mergeAdjacent
	"merges a string node with adjacent string nodes in parent"
	| siblingNodes selfIndex adjacentNode |

	self hasParent
		ifFalse: [^ self].

	siblingNodes := self parent nodes.
	selfIndex := siblingNodes indexOf: self.
	[(adjacentNode := siblingNodes at: selfIndex + 1) notNil
		and: [adjacentNode isStringNode]]
		whileTrue: [
			self append: adjacentNode.
			siblingNodes removeAt: selfIndex + 1].

	[(adjacentNode := siblingNodes at: selfIndex - 1) notNil
		and: [adjacentNode isStringNode]]
		whileTrue: [
			self prepend: adjacentNode.
			siblingNodes removeAt: selfIndex - 1.
			selfIndex := selfIndex - 1].! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/14/2015 20:06'!
notEmpty
	^ self string notEmpty! !

!XMLString methodsFor: 'appending/prepending' stamp: 'monty 8/14/2016 03:10'!
prepend: aStringOrStringNode
	string :=
		string
			ifNil: [aStringOrStringNode asString]
			ifNotNil: [aStringOrStringNode asString, string]! !

!XMLString methodsFor: 'printing' stamp: 'monty 11/19/2014 23:44'!
printRawContentOn: aStream
	aStream nextPutAll: self string! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 18:51'!
replaceAll: aSource with: aReplacement
	"works like copyReplaceAll:with:"
	self string:
		(self string
			copyReplaceAll: aSource asString
			with: aReplacement asString)! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 18:54'!
replaceFrom: aStart to: aStop with: aReplacement
	"works like copyReplaceFrom:to:with: and
	can be used to insert, delete, and replace"
	| start stop |

	start := (aStart max: 1) min: self size + 1.
	stop := (aStop max: start - 1) min: self size.
	self string:
		(self string
			copyReplaceFrom: start
			to: stop
			with: aReplacement asString)! !

!XMLString methodsFor: 'accessing' stamp: 'monty 11/3/2014 23:41'!
size
	^ self string size! !

!XMLString methodsFor: 'private' stamp: 'monty 12/2/2014 22:56'!
sortKey
	^ self string! !

!XMLString methodsFor: 'splitting/merging' stamp: 'monty 11/3/2014 22:02'!
splitAt: anOffset
	| newNode |

	newNode := self copy.
	self replaceFrom: anOffset to: self size with: ''.
	newNode replaceFrom: 1 to: anOffset - 1 with: ''.

	self hasParent
		ifTrue: [self parent addNode: newNode after: self].

	^ newNode.! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:19'!
string
	^ string ifNil: [string := '']! !

!XMLString methodsFor: 'accessing' stamp: 'monty 1/1/2014 07:23'!
string: aString
	string := aString asString! !

!XMLString methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter string: self string! !

Object subclass: #XMLNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeFactory commentStamp: 'monty 6/16/2015 02:41' prior: 0!
This class is the default node factory. Its accessor messages return the classes used by the DOM parser to build a node tree. You can override these messages to return different classes and inject an instance with the DOM parser's #nodeFactory: message before parsing.!

!XMLNodeFactory methodsFor: 'converting' stamp: 'monty 4/17/2015 10:36'!
asPluggableNodeFactory
	^ XMLPluggableNodeFactory new! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:58'!
attributeDeclarationClass
	^ XMLAttributeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
cdataClass
	^ XMLCData! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 7/26/2016 13:57'!
classForElement: aQualifiedName
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 8/10/2016 20:01'!
classForElement: aQualifiedName attributes: anAssociationCollection
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 7/26/2016 13:58'!
classForElement: aQualifiedName namespaceURI: aURI
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 8/10/2016 20:01'!
classForElement: aQualifiedName namespaceURI: aURI attributes: anAssociationCollection
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:48'!
classForElement: aQualifiedName namespaceURI: aURI localName: aLocalName attributes: anAssociationCollection
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
commentClass
	^ XMLComment! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
doctypeDeclarationClass
	^ XMLDoctypeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
documentClass
	^ XMLDocument! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
elementClass
	^ XMLElement! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
elementDeclarationClass
	^ XMLElementDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
generalEntityDeclarationClass
	^ XMLGeneralEntityDeclaration! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:57'!
newCData: aString
	^ self cdataClass string: aString! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:57'!
newComment: aString
	^ self commentClass string: aString! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:00'!
newDocument
	^ self documentClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:00'!
newElement
	^ self elementClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:01'!
newElementNamed: aName
	^ (self classForElement: aName)
		named: aName! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 8/15/2016 23:53'!
newElementNamed: aName attributes: anAssociationCollection
	^ (self
		classForElement: aName
		attributes: anAssociationCollection)
			name: aName
			attributes: anAssociationCollection! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 8/15/2016 23:53'!
newElementNamed: aName namespaceURI: aURI
	^ (self
		classForElement: aName
		namespaceURI: aURI)
			name: aName
			namespaceURI: aURI! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 8/15/2016 23:53'!
newElementNamed: aName namespaceURI: aURI attributes: anAssociationCollection
	^ (self
		classForElement: aName
		namespaceURI: aURI
		attributes: anAssociationCollection)
			name: aName
			namespaceURI: aURI
			attributes: anAssociationCollection! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:55'!
newPI
	^ self piClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 8/9/2016 17:16'!
newPITarget: aTargetString data: aDataString
	^ self newPI
		target: aTargetString;
		data: aDataString! !

!XMLNodeFactory methodsFor: 'deprecated' stamp: 'monty 8/5/2016 22:22'!
newString: aString
	"will be deprecated; use #newStringNode: instead"

	^ self newStringNode: aString! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 8/6/2016 12:56'!
newStringNode: aString
	^ self stringNodeClass new string: aString! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 4/17/2015 10:32'!
nodeListClass
	^ XMLNodeList! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
notationDeclarationClass
	^ XMLNotationDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
parameterEntityDeclarationClass
	^ XMLParameterEntityDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
piClass
	^ XMLPI! !

!XMLNodeFactory methodsFor: 'deprecated' stamp: 'monty 8/5/2016 22:24'!
stringClass
	"will be deprecated; use #stringNodeClass instead"

	^ XMLString! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 8/5/2016 22:23'!
stringNodeClass
	"for now this delegates to the soft-deprecated #stringClass to not
	break custom subclasses that override it expecting it to be sent"

	^ self stringClass! !

XMLNodeFactory subclass: #XMLPluggableNodeFactory
	instanceVariableNames: 'documentClass elementClass stringNodeClass cdataClass commentClass piClass nodeListClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPluggableNodeFactory commentStamp: 'monty 8/26/2016 03:16' prior: 0!
This is a pluggable factory that can be used to change which classes the DOM parser uses to build the DOM tree. Here is an example:

	(XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableFactory new
				documentClass: MyDocumentClass;
				elementClass: MyElementClass;
				stringNodeClass: MyStringNodeClass)
		parseDocument.

Instances can be saved and resused for performance and safely modified after copying.!

XMLPluggableNodeFactory subclass: #XMLPluggableElementFactory
	instanceVariableNames: 'elementClassMapper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPluggableElementFactory commentStamp: 'monty 8/26/2016 03:16' prior: 0!
This class is a pluggable factory that can map elements to different XMLElement subclasses based on the name, namespace information, and attributes of those elements. You have to create an instance, configure it to handle certain elements with certain classes, and then inject your instance into a DOM parser using #nodeFactory: before parsing. Here is an example of its use:

	doc := (XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableElementFactory new
				elementClass: GenericElement;
				handleElement: 'user' withClass: UserElement;
				handleElement: 'report' withClass: ReportElement;
				handleElement: 'report' namespaceURI: 'urn:specialreprot' withClass: SpecialReportElement)
		parseDocument.
		
The #handleElementsMatchingClass*: forms try to match elements to the specified classes based on the names of those classes when stripped of any prefix (like XML) and "Element" suffix and converted to both camel case and a hyphenated form. So this:
	XMLPluggableElementFactory new
		handleElementsMatchingClass: MYRootUserElement
		....
will handle 'rootUser' and 'root-user' elements with the MYRootUserElement class.

Instances can be saved and resused for performance and safely modified after copying.!

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:45'!
classForElement: aQualifiedName
	^ elementClassMapper
		classForElement: aQualifiedName
		ifNone: [self elementClass]! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:45'!
classForElement: aQualifiedName attributes: anAssociationCollection
	^ elementClassMapper
		classForElement: aQualifiedName
		attributes: anAssociationCollection
		ifNone: [self elementClass]! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:45'!
classForElement: aQualifiedName namespaceURI: aURI
	^ elementClassMapper
		classForElement: aQualifiedName
		namespaceURI: aURI
		ifNone: [self elementClass]! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:45'!
classForElement: aQualifiedName namespaceURI: aURI attributes: anAssociationCollection
	^ elementClassMapper
		classForElement: aQualifiedName
		namespaceURI: aURI
		attributes: anAssociationCollection
		ifNone: [self elementClass]! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'monty 8/10/2016 23:44'!
classForElement: aQualifiedName namespaceURI: aURI localName: aLocalName attributes: anAssociationCollection
	^ elementClassMapper
		classForElement: aQualifiedName
		namespaceURI: aURI
		localName: aLocalName
		attributes: anAssociationCollection
		ifNone: [self elementClass]! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'monty 7/23/2016 05:24'!
elementClassMapper
	^ elementClassMapper! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'monty 7/24/2016 09:18'!
elementClassMapper: anElementClassMapper
	elementClassMapper := anElementClassMapper! !

!XMLPluggableElementFactory methodsFor: 'private ' stamp: 'monty 8/26/2016 02:16'!
elementClassMapperClass
	^ XMLBasicElementClassMapper! !

!XMLPluggableElementFactory methodsFor: 'handling' stamp: 'monty 8/18/2016 23:10'!
handleElement: anElementName attributes: aCollectionOrAttributeSpec withClass: aClass
	"aCollectionOrAttributeSpec can be some kind of XMLAttributeSpec or a
	collection of associations of required attribute names and values, and the
	required values can be strings, blocks, or nil meaning the attribute is
	required but can take any value"

	elementClassMapper
		mapElement: anElementName
		attributeSpec: aCollectionOrAttributeSpec asXMLAttributeSpec
		toClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'handling' stamp: 'monty 8/18/2016 23:10'!
handleElement: anElementName namespaceURI: aURI attributes: aCollectionOrAttributeSpec withClass: aClass
	"aCollectionOrAttributeSpec can be some kind of XMLAttributeSpec or a
	collection of associations of required attribute names and values, and the
	required values can be strings, blocks, or nil meaning the attribute is
	required but can take any value"

	elementClassMapper
		mapElement: anElementName
		namespaceURI: aURI
		attributeSpec: aCollectionOrAttributeSpec asXMLAttributeSpec
		toClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'handling' stamp: 'monty 7/26/2016 06:20'!
handleElement: anElementName namespaceURI: aURI withClass: aClass
	elementClassMapper
		mapElement: anElementName
		namespaceURI: aURI
		toClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'handling' stamp: 'monty 7/26/2016 06:21'!
handleElement: anElementName withClass: aClass
	elementClassMapper
		mapElement: anElementName
		toClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'handling' stamp: 'monty 7/26/2016 06:25'!
handleElements: anElementNameCollection namespaceURI: aNamespaceURI withClass: aClass
	elementClassMapper
		mapElements: anElementNameCollection
		namespaceURI: aNamespaceURI
		toClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'handling' stamp: 'monty 7/26/2016 06:24'!
handleElements: anElementNameCollection withClass: aClass
	elementClassMapper
		mapElements: anElementNameCollection
		toClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'handling' stamp: 'monty 8/26/2016 01:41'!
handleElementsMatchingClass: aClass
	"This will handle all elements matching the name of a class with that
	class, but igoring the class's prefix and any 'Element' suffix.

	For example, it will match MyUserDefinedElement to:
		<UserDefined>
		<userDefined>
		<user-defined>
		<user_defined>"

	elementClassMapper
		mapAllMatchingElementsToClass: aClass
		ignoringSuffixes: self ignorableClassNameSuffixes! !

!XMLPluggableElementFactory methodsFor: 'handling' stamp: 'monty 8/26/2016 01:41'!
handleElementsMatchingClasses: aClassCollection
	"See the comment in #handleElementsMatchingClass:"

	elementClassMapper
		mapAllMatchingElementsToClasses: aClassCollection
		ignoringSuffixes: self ignorableClassNameSuffixes! !

!XMLPluggableElementFactory methodsFor: 'private ' stamp: 'monty 8/26/2016 00:47'!
ignorableClassNameSuffixes
	^ #('Element')! !

!XMLPluggableElementFactory methodsFor: 'initialization' stamp: 'monty 8/2/2016 12:38'!
initialize
	super initialize.

	elementClassMapper := self elementClassMapperClass new.! !

!XMLPluggableElementFactory methodsFor: 'copying' stamp: 'monty 7/23/2016 04:50'!
postCopy
	super postCopy.

	elementClassMapper := elementClassMapper copy.! !

!XMLPluggableNodeFactory methodsFor: 'converting' stamp: 'monty 4/17/2015 10:37'!
asPluggableNodeFactory
	^ self! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:08'!
cdataClass
	^ cdataClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
cdataClass: aClass
	cdataClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:08'!
commentClass
	^ commentClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:45'!
commentClass: aClass
	commentClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:08'!
documentClass
	^ documentClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:42'!
documentClass: aClass
	documentClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:08'!
elementClass
	^ elementClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:42'!
elementClass: aClass
	elementClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'initialization' stamp: 'monty 8/5/2016 16:47'!
initialize
	super initialize.

	documentClass := XMLDocument.
	elementClass := XMLElement.
	stringNodeClass := XMLString.
	cdataClass := XMLCData.
	commentClass := XMLComment.
	piClass := XMLPI.
	nodeListClass := XMLNodeList.! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:08'!
nodeListClass
	^ nodeListClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 4/17/2015 10:33'!
nodeListClass: aClass
	nodeListClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 8/4/2016 20:09'!
piClass
	^ piClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
piClass: aClass
	piClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'deprecated' stamp: 'monty 8/5/2016 22:24'!
stringClass
	"will be deprecated; use #stringNodeClass instead"

	^ stringNodeClass! !

!XMLPluggableNodeFactory methodsFor: 'deprecated' stamp: 'monty 8/5/2016 22:24'!
stringClass: aClass
	"will be deprecated; use #stringNodeClass: instead"

	stringNodeClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 8/5/2016 16:48'!
stringNodeClass
	^ stringNodeClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 8/5/2016 16:47'!
stringNodeClass: aClass
	stringNodeClass := aClass! !

Object subclass: #XMLNodeVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeVisitor commentStamp: 'monty 8/16/2016 23:04' prior: 0!
This is a base class for DOM node tree visitors. Subclasses can implement any of the "visiting" messages, and instances can be used by sending a node #acceptNodeVisitor: with it as the argument.!

!XMLNodeVisitor methodsFor: 'visiting' stamp: 'monty 8/5/2016 13:17'!
visitCData: aCData
	self visitStringNode: aCData! !

!XMLNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/27/2016 05:33'!
visitComment: aComment! !

!XMLNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/28/2016 10:54'!
visitDocument: aDocument
	self visitNodeWithElements: aDocument! !

!XMLNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/28/2016 10:54'!
visitElement: anElement
	self visitNodeWithElements: anElement! !

!XMLNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/27/2016 07:48'!
visitNode: aNode
	aNode acceptNodeVisitor: self! !

!XMLNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/28/2016 10:54'!
visitNodeWithElements: aNode
	aNode hasChildren
		ifTrue: [aNode nodesDo: [:each | each acceptNodeVisitor: self]]! !

!XMLNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/27/2016 05:36'!
visitPI: aPI! !

!XMLNodeVisitor methodsFor: 'visiting' stamp: 'monty 8/5/2016 13:17'!
visitStringNode: aStringNode! !

XMLNodeVisitor subclass: #XMLValidatingNodeVisitor
	instanceVariableNames: 'validator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLValidatingNodeVisitor commentStamp: 'monty 8/16/2016 23:03' prior: 0!
A validating node visitor that can validate DOM node trees using XMLDocumentValidators.!

!XMLValidatingNodeVisitor class methodsFor: 'instance creation' stamp: 'monty 7/27/2016 06:21'!
validator: aValidator
	^ self new validator: aValidator! !

!XMLValidatingNodeVisitor methodsFor: 'accessing' stamp: 'monty 7/27/2016 06:21'!
validator
	^ validator! !

!XMLValidatingNodeVisitor methodsFor: 'accessing' stamp: 'monty 7/27/2016 06:21'!
validator: aValidator
	validator := aValidator! !

!XMLValidatingNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/27/2016 06:21'!
visitComment: aComment
	self validator validateComment: aComment! !

!XMLValidatingNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/27/2016 06:25'!
visitDocument: aDocument
	aDocument root
		ifNotNil: [:rootElement |
			self validator validateRootElement: rootElement name].
	super visitDocument: aDocument.
	self validator validateEndDocument.! !

!XMLValidatingNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/27/2016 07:22'!
visitElement: anElement
	| attributeDictionary |

	attributeDictionary := 
		anElement hasAttributes
			ifTrue: [anElement attributeNodes asDictionary]
			ifFalse: [Dictionary new].

	self validator
		validateStartTag: anElement name
		attributes: attributeDictionary.
	"validation can change/add attribute values"
	anElement setAttributes: attributeDictionary.

	super visitElement: anElement.

	self validator validateEndTag: anElement name.! !

!XMLValidatingNodeVisitor methodsFor: 'visiting' stamp: 'monty 7/27/2016 06:22'!
visitPI: aPI
	self validator
		validatePI: aPI target
		data: aPI data! !

!XMLValidatingNodeVisitor methodsFor: 'visiting' stamp: 'monty 8/5/2016 13:17'!
visitStringNode: aStringNode
	self validator validatePCData: aStringNode string! !

Object subclass: #XMLParser
	instanceVariableNames: 'tokenizer driver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLParser commentStamp: '<historical>' prior: 0!
This class is a facade that parses XML using a driver and tokenizer. See SAXHandler and XMLDOMParser for ready-made front ends to it.!

!XMLParser class methodsFor: 'instance creation' stamp: 'monty 3/26/2016 23:49'!
driver: aDriver on: aStringOrStream
	^ self new
		setDriver: aDriver
		tokenizer: (aDriver newTokenizerOn: aStringOrStream)! !

!XMLParser methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:17'!
atEnd
	^ self tokenizer atEnd! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 5/3/2015 16:55'!
context
	^ self tokenizer context! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 5/3/2015 17:08'!
context: aTokenContext
	self tokenizer context: aTokenContext! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 3/18/2016 01:47'!
currentColumnNumber
	^ self tokenizer currentColumnNumber! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:34'!
currentLineNumber
	^ self tokenizer currentLineNumber! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 3/18/2016 02:20'!
currentPosition
	^ self tokenizer currentPosition! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 07:07'!
driver
	^ driver! !

!XMLParser methodsFor: 'private' stamp: 'monty 3/30/2016 05:31'!
handleParserError: anError
	self tokenizer closeStreams.

	(anError isKindOf: XMLParserException)
		ifFalse: [anError pass].

	(anError hasLineNumber
		or: [self currentLineNumber isNil])
		ifFalse: [anError lineNumber: self currentLineNumber].
	(anError hasColumnNumber
		or: [self currentColumnNumber isNil])
		ifFalse: [anError columnNumber: self currentColumnNumber].

	anError shouldPassOnWhenCaught
		ifTrue: [anError pass].! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseDocument
	[[self tokenizer atEnd]
		whileFalse: [self tokenizer nextToken]]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/26/2014 21:27'!
parseDocumentUntil: aBlock
	[[self tokenizer atEnd or: [aBlock value]]
		whileFalse: [self tokenizer nextToken]]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseDocumentWhile: aBlock
	[[self tokenizer atEnd not and: [aBlock value]]
		whileTrue: [self tokenizer nextToken]]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseToken
	^ [self tokenizer nextToken]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'initialization' stamp: 'JAAyer 11/28/2010 07:09'!
setDriver: aDriver tokenizer: aTokenizer
	driver := aDriver.
	tokenizer := aTokenizer.! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 07:16'!
tokenizer
	^ tokenizer! !

Object subclass: #XMLParserDriver
	instanceVariableNames: 'entityURIStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLParserDriver commentStamp: 'monty 7/15/2015 10:29' prior: 0!
This is a base class for parser drivers. Subclasses should override any of the "handling" or "handling - dtd" messages sent by tokenizers to parse tokens.!

XMLParserDriver subclass: #DTDExternalSubsetParserDriver
	instanceVariableNames: 'saxDriver doctypeDefinition documentURI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!DTDExternalSubsetParserDriver commentStamp: 'monty 7/15/2015 10:32' prior: 0!
This class is used by SAXParserDriver for parsing the external subset of a DTD.!

!DTDExternalSubsetParserDriver class methodsFor: 'instance creation' stamp: 'monty 3/30/2016 07:30'!
saxDriver: aSAXDriver documentURI: anExternalSubsetURIString
	^ self new
		setSAXDriver: aSAXDriver
		documentURI: anExternalSubsetURIString! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 5/8/2015 19:05'!
decodesCharacters
	^ self saxDriver decodesCharacters! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 3/2/2015 11:14'!
doctypeDefinition
	^ doctypeDefinition! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 3/21/2015 23:23'!
documentURI
	^ documentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
externalEntityResolver
	^ self saxDriver externalEntityResolver! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 6/7/2015 20:05'!
handleAttributeDeclaration: anElement name: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self doctypeDefinition
				addValidatorForAttributeIn: anElement
				named: anAttribute
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self doctypeDefinition
				addValidatorForElement: aName
				contentModel: aContentModel]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/8/2016 02:18'!
handleGeneralEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotation
	aNotation isEmpty
		ifTrue: [	
			self doctypeDefinition
				addExternalGeneralEntity: aName
				resolver: self externalEntityResolver
				publicID: aPublicID
				systemID: aSystemID
				baseURI: self baseURI]
		ifFalse: [
			self doctypeDefinition
				addUnparsedEntity: aName
				ndata: aNotation]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/8/2016 02:45'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self doctypeDefinition
		addInternalGeneralEntity: aName
		replacement: aReplacement
		baseURI: self baseURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/6/2016 05:17'!
handleImproperParameterEntityNesting
	self saxDriver handleImproperParameterEntityNesting! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 7/13/2016 01:23'!
handleNotationDeclaration: aName publicID: aPublicID systemID: aSystemID
	self isValidating
		ifTrue: [
			self doctypeDefinition unparsedEntityValidator
				validateNotation: aName].
	self doctypeDefinition addNotation: aName.! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/8/2016 02:18'!
handleParameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID
	self doctypeDefinition
		addExternalParameterEntity: aName
		resolver: self externalEntityResolver
		publicID: aPublicID
		systemID: aSystemID
		baseURI: self baseURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/8/2016 02:45'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self doctypeDefinition
		addInternalParameterEntity: aName
		replacement:aReplacement
		baseURI: self baseURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling - dtd' stamp: 'monty 1/4/2015 00:11'!
handleParameterEntityReference: aName
	^ self saxDriver handleParameterEntityReference: aName! !

!DTDExternalSubsetParserDriver methodsFor: 'handling' stamp: 'monty 3/25/2016 05:11'!
handleParseError: aString
	XMLWellFormednessException signal: aString! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:58'!
isValidating
	^ self saxDriver isValidating! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 19:10'!
maxEntityReplacementDepth
	^ self saxDriver maxEntityReplacementDepth! !

!DTDExternalSubsetParserDriver methodsFor: 'instance creation' stamp: 'monty 3/26/2016 23:48'!
newInitialContext
	^ XMLTextDeclarationContext new! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 1/6/2015 04:12'!
preservesUndeclaredEntityReferences
	^ self saxDriver preservesUndeclaredEntityReferences! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 1/27/2015 22:51'!
readLimit
	^ self externalEntityResolver externalSubsetReadLimit! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 1/4/2015 00:15'!
replacesContentEntityReferences
	^ self saxDriver replacesContentEntityReferences! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
saxDriver
	^ saxDriver! !

!DTDExternalSubsetParserDriver methodsFor: 'initialization' stamp: 'monty 3/30/2016 07:30'!
setSAXDriver: aSAXDriver documentURI: anExternalSubsetURIString
	saxDriver := aSAXDriver.
	doctypeDefinition := aSAXDriver doctypeDefinition.
	documentURI := anExternalSubsetURIString.! !

XMLParserDriver subclass: #SAXParserDriver
	instanceVariableNames: 'saxHandler elementNester scopeAllocator doctypeDefinition validator isStandalone hasUnhandledReservedAttributes externalSubsetSystemID externalSubsetPublicID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!SAXParserDriver commentStamp: 'monty 6/16/2015 00:07' prior: 0!
SAXParserDriver together with SAXHandler provides a SAX-like API for XML parsing. The driver processes XML tokens and signals SAX events to a SAXHandler. See SAXHandler for more info.!

!SAXParserDriver class methodsFor: 'instance creation' stamp: 'monty 10/27/2014 09:12'!
saxHandler: aHandler
	^ self new setSAXHandler: aHandler! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 3/30/2016 07:39'!
baseURI
	"returns a string with the current base URI at this stage of parsing"
	self elementNester hasScopedAttributes
		ifTrue: [
			^ self elementNester
				scopedAttributeAt: 'xml:base'
				ifAbsent: [super baseURI]].
	^ super baseURI.! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 8/6/2016 20:02'!
checkForReservedAttributesIn: anOrderPreservingDictionary afterIndex: anIndex
	"validation may add attributes, so this is used to check if reserved
	attributes were added that need handling when there weren't any before"

	self hasUnhandledReservedAttributes
		ifTrue: [^ self].
	anIndex + 1
		to: anOrderPreservingDictionary size
		do: [:i |
			(anOrderPreservingDictionary keyAtIndex: i) isXMLReservedAttributeName
				ifTrue: [^ self haveUnhandledReservedAttributes]].! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:13'!
currentElement
	^ self elementNester currentElement! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 4/13/2016 04:04'!
declareNamespaces: aSequenceableCollection inScope: aScope
	1 to: aSequenceableCollection size do: [:i | | association |
		association := aSequenceableCollection at: i.
		(aScope resolvePrefix: association key) = association value
			ifFalse: [
				self saxHandler
					startPrefixMapping: association key
					uri: association value]].
	aScope mapPrefixesAndURIs: aSequenceableCollection.! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 5/8/2015 19:05'!
decodesCharacters
	^ self saxHandler decodesCharacters! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 3/18/2016 00:51'!
doctypeDefinition
	^ doctypeDefinition! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:44'!
documentReadLimit
	^ self saxHandler documentReadLimit! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ self saxHandler documentURI! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:11'!
elementNester
	^ elementNester! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 12/27/2013 04:08'!
elementNesterClass
	^ XMLWellFormedElementNester! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/2/2014 18:39'!
errorCharacterDataOutsideElement
	XMLWellFormednessException signal: 'Character data outside of element'! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 17:23'!
errorDocumentConstraintsRequired
	XMLValidationException signal: 'Document validation constraints required'! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 4/6/2016 05:18'!
errorImproperParameterEntityNesting
	XMLValidationException signal:
		'Improper nesting of parameter entity replacement within DTD subset'! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 3/28/2016 23:44'!
errorStandaloneUndeclaredGeneralEntity: aName
	XMLWellFormednessException
		formatSignal:
			'Undeclared general entity reference &{1}; in standalone document'
		with: aName! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 21:41'!
errorUndeclaredGeneralEntity: aName
	XMLValidationException
		formatSignal: 'Undeclared general entity reference &{1};'
		with: aName! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 21:14'!
errorUndeclaredParameterEntity: aName
	XMLValidationException
		formatSignal: 'Undeclared parameter entity reference %{1};'
		with: aName! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 7/13/2016 03:06'!
errorUnparsedGeneralEntityReference: aName
	XMLWellFormednessException
		formatSignal: 'Illegal unparsed general entity reference &{1};'
		with: aName! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 17:42'!
externalEntityResolver
	^ self saxHandler externalEntityResolver! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 5/23/2015 03:02'!
externalSubsetParserOn: aStringOrStream documentURI: aURIString
	^ self saxHandler parserClass
		driver:
			(DTDExternalSubsetParserDriver
				saxDriver: self
				documentURI: aURIString)
		on: aStringOrStream! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 3/31/2016 01:32'!
externalSubsetPublicID
	^ externalSubsetPublicID! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 3/31/2016 01:32'!
externalSubsetSystemID
	^ externalSubsetSystemID! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 8/6/2016 20:01'!
extractNamespacesFromAttributes: anOrderPreservingDictionary
	"returns an ordered collection of namespace prefix/uri associations
	removed from the attribute dictionary or nil if there aren't any"
	| namespaces keysToRemove |

	self haveNoUnhandledReservedAttributes.
	anOrderPreservingDictionary keysDo: [:name |
		"optimized comparison ordering"
		(name at: 1) == $x
			ifTrue: [
				name isXMLNSAttributeName
					ifTrue: [
						"only initialize when needed"
						(namespaces ifNil: [namespaces := OrderedCollection new: 5])
							addLast:
								(name size > 6 "'xmlns:' size"
									ifTrue: [name copyFrom: 7 to: name size]
									ifFalse: ['']) ->
										(anOrderPreservingDictionary at: name).
						"only initialize when needed"
						(keysToRemove ifNil: [keysToRemove := OrderedCollection new: 5])
							addLast: name]
					ifFalse: [
						name isXMLReservedAttributeName
							ifTrue: [
								"at least one xml attr is not an xmlns attr, so there
								will still be reserved attrs to handle after extracting
								the namespaces"
								self haveUnhandledReservedAttributes]]]].

	keysToRemove
		ifNotNil: [
			"use removeKeys: for speed"
			anOrderPreservingDictionary removeKeys: keysToRemove].

	^ namespaces.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 6/7/2015 20:05'!
handleAttributeDeclaration: anElement name: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self doctypeDefinition
				addValidatorForAttributeIn: anElement
				named: anAttribute
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue].
	self saxHandler
		attributeDeclaration: anElement
		name: anAttribute
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 3/11/2016 20:00'!
handleCData: aString
	self elementNester isInElement
		ifFalse: [self errorCharacterDataOutsideElement].
	self isValidating
		ifTrue: [self validator validatePCData: aString].

	self saxHandler
		isInCDataSection: true;
		startCData;
		characters: aString;
		endCData;
		isInCDataSection: false.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 4/13/2016 08:53'!
handleComment: aString
	(self elementNester isInElement
		and: [self isValidating])
		ifTrue: [self validator validateComment: aString].

	self saxHandler comment: aString.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self doctypeDefinition
				addValidatorForElement: aName
				contentModel: aContentModel].

	self saxHandler
		elementDeclaration: aName
		contentModel: aContentModel! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
	self saxHandler endEntity: aName! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 7/13/2016 02:54'!
handleEndDTD
	self hasExternalSubsetToParse
		ifTrue: [self parseExternalSubset].

	self isValidating
		ifTrue: [self doctypeDefinition validateCompletedDefinition].

	self saxHandler
		doctypeDefinition: self doctypeDefinition;
		endDTD.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 2/2/2014 02:44'!
handleEndDocument
	self elementNester endNesting.
	self isValidating
		ifTrue: [self validator validateEndDocument].

	self saxHandler endDocument.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 8/6/2016 19:49'!
handleEndTag: anElement
	"performance critical method"
	| prefix currentScope |

	self elementNester endElement: anElement.

	prefix := anElement xmlPrefixBeforeLocalName.
	self saxHandler 
		endElement: anElement
		prefix: prefix
		uri:
			(self usesNamespaces
				ifTrue: [
					(currentScope := self scopeAllocator exitScope)
						resolvePrefix: prefix]
				ifFalse: [''])
		localName: (anElement xmlLocalNameAfterPrefix: prefix).

	currentScope ifNotNil: [
		currentScope hasNamespaces
			ifTrue: [self undeclareNamespacesInScope: currentScope]].
	self isValidating
		ifTrue: [self validator validateEndTag: anElement].! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 7/18/2016 07:26'!
handleGeneralEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotation
	aNotation isEmpty
		ifTrue: [
			self resolvesExternalEntities
				ifTrue: [
					self doctypeDefinition
						addExternalGeneralEntity: aName
						resolver: self externalEntityResolver
						publicID: aPublicID
						systemID: aSystemID
						baseURI: self baseURI]
				ifFalse: [
					(self isValidating
						or: [self isStandalone])
						ifTrue: [
							self doctypeDefinition
								addUnresolvableExternalGeneralEntity: aName]]]
		ifFalse: [
			self doctypeDefinition
				addUnparsedEntity: aName
				ndata: aNotation].

	self saxHandler
		generalEntityDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID
		ndata: aNotation.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/8/2016 02:45'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self doctypeDefinition
		addInternalGeneralEntity: aName
		replacement: aReplacement
		baseURI: self baseURI.

	self saxHandler
		generalEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 7/18/2016 05:35'!
handleGeneralEntityReference: aName
	| entity |

	entity :=
		self doctypeDefinition
			generalEntityAt: aName
			ifAbsent: [
				self isValidating
					ifTrue: [self errorUndeclaredGeneralEntity: aName].
				self isStandalone
					ifTrue: [self errorStandaloneUndeclaredGeneralEntity: aName].
				^ nil].

	entity isUnparsed
		ifTrue: [self errorUnparsedGeneralEntityReference: aName].

	^ entity.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/10/2016 07:42'!
handleImproperParameterEntityNesting
	"this is a validity constraint and is only checked when validating"
	self isValidating
		ifTrue: [self errorImproperParameterEntityNesting]! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 7/13/2016 01:23'!
handleNotationDeclaration: aName publicID: aPublicID systemID: aSystemID
	self isValidating
		ifTrue: [
			self doctypeDefinition unparsedEntityValidator
				validateNotation: aName].
	self doctypeDefinition addNotation: aName.

	self saxHandler
		notationDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 11/3/2014 13:09'!
handlePCData: aString
	self elementNester isInElement
		ifFalse: [self errorCharacterDataOutsideElement].
	self isValidating
		ifTrue: [self validator validatePCData: aString].

	self saxHandler characters: aString.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 4/16/2016 13:44'!
handlePI: aTargetString data: aDataString
	(self elementNester isInElement
		and: [self isValidating])
		ifTrue: [
			self validator
				validatePI: aTargetString
				data: aDataString].

	self saxHandler 
		processingInstruction: aTargetString
		data: aDataString.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 7/18/2016 07:26'!
handleParameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID
	self resolvesExternalEntities
		ifTrue: [
			self doctypeDefinition
				addExternalParameterEntity: aName
				resolver: self externalEntityResolver
				publicID: aPublicID
				systemID: aSystemID
				baseURI: self baseURI]
		ifFalse: [
			(self isValidating
				or: [self isStandalone])
				ifTrue: [
					self doctypeDefinition
						addUnresolvableExternalParameterEntity: aName]].

	self saxHandler
		parameterEntityDeclaration: aName
		publicID: aPublicID
		systemID: aSystemID.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 4/8/2016 02:45'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self doctypeDefinition
		addInternalParameterEntity: aName
		replacement:aReplacement
		baseURI: self baseURI.

	self saxHandler
		parameterEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/2/2015 11:14'!
handleParameterEntityReference: aName
	^ self doctypeDefinition
		parameterEntityAt: aName
		ifAbsent: [
			self isValidating
				ifTrue: [self errorUndeclaredParameterEntity: aName].
			nil]! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 3/25/2016 05:11'!
handleParseError: aString
	XMLWellFormednessException signal: aString! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 8/6/2016 20:03'!
handleReservedAttributes: aDictionary
	aDictionary
		at: 'xml:base'
		ifPresent: [:uri |
			"expand the new base URI using the current"
			aDictionary
				at: 'xml:base'
				put: (self baseURI asXMLURI / uri) asString].
	self isValidating
		ifTrue: [self validator validateXMLIDAttributeIn: aDictionary].

	self elementNester
		setScopedAttributesNamed: self scopedAttributeNames
		from: aDictionary.

	self haveNoUnhandledReservedAttributes.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 3/24/2015 20:28'!
handleStartContent
	self saxHandler startContent! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 12/18/2013 22:08'!
handleStartContentEntityReplacement: aName
	self saxHandler startEntity: aName! !

!SAXParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/31/2016 01:33'!
handleStartDTD: aRoot publicID: aPublicID systemID: aSystemID
	externalSubsetPublicID := aPublicID.
	externalSubsetSystemID := aSystemID.

	self doctypeDefinition root: aRoot.
	self saxHandler
		startDTD: aRoot
		publicID: aPublicID
		systemID: aSystemID.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'mir 8/14/2000 18:29'!
handleStartDocument
	self saxHandler startDocument! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 8/26/2016 03:07'!
handleStartTag: anElement attributes: aDictionary
	"performance critical method"
	| prefix totalAttributes namespaceURI currentScope |

	self elementNester startElement: anElement.

	prefix := anElement xmlPrefixBeforeLocalName.
	totalAttributes := aDictionary size.
	self isValidating
		ifTrue: [
			self elementNester isCurrentElementRoot
				ifTrue: [self validateRootElement: anElement].
			self validator
				validateStartTag: anElement
				attributes: aDictionary.
			aDictionary size > totalAttributes
				ifTrue: [
					self
						checkForReservedAttributesIn: aDictionary
						afterIndex: totalAttributes.
					totalAttributes := aDictionary size]].
	self usesNamespaces
		ifTrue: [
			currentScope := self scopeAllocator enterScope.
			self hasUnhandledReservedAttributes
				ifTrue: [
					(self extractNamespacesFromAttributes: aDictionary)
						ifNotNil: [:namespaces |
							self
								declareNamespaces: namespaces
								inScope: currentScope.
							totalAttributes := aDictionary size]].
			namespaceURI :=
				currentScope
					validateAndResolvePrefix: prefix
					ofElement: anElement.
			totalAttributes > 0
				ifTrue: [
					currentScope
						validatePrefixesOfAttributes: aDictionary orderedKeys
						from: 1
						to: totalAttributes]].
	self hasUnhandledReservedAttributes
		ifTrue: [self handleReservedAttributes: aDictionary].

	self saxHandler
		startElement: anElement
		prefix: prefix
		uri: (namespaceURI ifNil: [''])
		localName: (anElement xmlLocalNameAfterPrefix: prefix)
		attributes: aDictionary.

	currentScope
		ifNotNil: [self saxHandler currentScope: currentScope].! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 1/4/2015 00:01'!
handleUnreplacedGeneralEntityReference: aName
	self saxHandler skippedEntity: aName! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 4/12/2016 04:37'!
handleWhitespace: aString
	self elementNester isInElement
		ifFalse: [^ self].

	self isValidating
		ifTrue: [
			self validator hasElementContent
				ifTrue: [
					(self elementNester hasScopedAttributes
						and: [(self elementNester scopedAttributeAt: 'xml:space') = 'preserve'])
						ifTrue: [self saxHandler characters: aString]
						ifFalse: [self saxHandler ignorableWhitespace: aString].
					^ self].
			self validator validatePCData: aString].
	self saxHandler characters: aString.! !

!SAXParserDriver methodsFor: 'handling' stamp: 'monty 3/9/2015 13:59'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo
	isStandalone := aYesOrNo = 'yes'.

	self saxHandler
		xmlVersion: (Float readFrom: aVersion readStream)
		encoding: anEncoding
		standalone: isStandalone.! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 3/31/2016 01:36'!
hasExternalSubsetToParse
	self resolvesExternalEntities
		ifFalse: [^ false].

	(self externalSubsetPublicID notEmpty
		or: [self externalSubsetSystemID notEmpty])
		ifFalse: [^ false].
	
	^ self isValidating
		or: [self isStandalone not].! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/3/2014 11:19'!
hasUnhandledReservedAttributes
	^ hasUnhandledReservedAttributes! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 7/9/2016 00:15'!
haveNoUnhandledReservedAttributes
	hasUnhandledReservedAttributes := false! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/3/2014 16:16'!
haveUnhandledReservedAttributes
	hasUnhandledReservedAttributes := true! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 3/12/2015 14:06'!
isStandalone
	^ isStandalone! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:43'!
isValidating
	^ self saxHandler isValidating! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
maxEntityReplacementDepth
	^ self saxHandler maxEntityReplacementDepth! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 4/3/2016 00:48'!
parseExternalSubset
	| externalSubsetURI externalSubsetStream |

	externalSubsetURI :=
		self externalEntityResolver
			externalEntityURIFromPublicID: self externalSubsetPublicID
			systemID: self externalSubsetSystemID
			baseURI: self baseURI.

	"External subsets are stream-resolved so if they are local (stored on disk),
	they aren't kept entirely in memory during parsing. HTTP entities will be
	kept in memory regardless and this will just return a read stream on them."
	externalSubsetStream :=
		self externalEntityResolver streamResolveExternalEntityURI:
			externalSubsetURI.

	(self
		externalSubsetParserOn:
			(XMLDecodingReadStreamAdapter on: externalSubsetStream)
		documentURI: externalSubsetURI asString)
		parseDocument.! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 1/6/2015 04:12'!
preservesUndeclaredEntityReferences
	^ self saxHandler preservesUndeclaredEntityReferences! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
readLimit
	^ self documentReadLimit! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 1/4/2015 00:14'!
replacesContentEntityReferences
	^ self saxHandler replacesContentEntityReferences! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:28'!
resolvesExternalEntities
	^ self saxHandler resolvesExternalEntities! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 08:00'!
saxHandler
	^ saxHandler! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 03:40'!
scopeAllocator
	^ scopeAllocator ifNil: [scopeAllocator := XMLNamespaceScopeAllocator new]! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 7/1/2016 05:22'!
scopedAttributeNames
	"the scoped attributes tracked by this driver"
	^ #('xml:space' 'xml:base')! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 8/6/2016 19:45'!
setRootElement: anElement currentScope: aNamespaceScope
	self elementNester setExternalRootElement: anElement.
	self usesNamespaces
		ifTrue: [self scopeAllocator currentScope: aNamespaceScope].! !

!SAXParserDriver methodsFor: 'initialization' stamp: 'monty 3/31/2016 01:32'!
setSAXHandler: aHandler
	saxHandler := aHandler.
	elementNester := self elementNesterClass new.
	doctypeDefinition := DTDDoctypeDefinition new.
	validator := (aHandler schema ifNil: [doctypeDefinition]) newValidator.
	externalSubsetPublicID := ''.
	externalSubsetSystemID := ''.
	isStandalone := false.
	hasUnhandledReservedAttributes := false.! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 3/13/2016 19:42'!
undeclareNamespacesInScope: anExitedScope
	| currentScope |

	currentScope := self scopeAllocator currentScope.
	anExitedScope prefixesAndURIsDo: [:prefix :uri |
		(currentScope resolvePrefix: prefix) = uri
			ifFalse: [self saxHandler endPrefixMapping: prefix]].! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:43'!
usesNamespaces
	^ self saxHandler usesNamespaces! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 4/13/2016 09:21'!
validateRootElement: anElement
	(self saxHandler requiresDocumentConstraints
		and: [self validator hasElementValidators not])
		ifTrue: [self errorDocumentConstraintsRequired].

	self validator validateRootElement: anElement.! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 3/18/2016 00:52'!
validator
	^ validator! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 4/1/2016 00:56'!
baseURI
	"returns a string with the current base URI at this stage of parsing"
	^ self entityURIStack xmlLastOrNil ifNil: [self documentURI]! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 3/30/2016 07:43'!
decodesCharacters
	^ true! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ nil! !

!XMLParserDriver methodsFor: 'private' stamp: 'monty 3/30/2016 07:35'!
entityURIStack
	^ entityURIStack ifNil: [entityURIStack := OrderedCollection new]! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 6/7/2015 20:05'!
handleAttributeDeclaration: anElement name: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleCData: aString! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleComment: aString! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/14/2013 03:46'!
handleElementDeclaration: aName contentModel: aContentModel! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/12/2013 17:36'!
handleEndDTD! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndDocument! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 8/6/2016 19:55'!
handleEndTag: anElement! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:37'!
handleGeneralEntityDeclaration: aName publicID: aPublicID systemID: aSystemID ndata: aNotation! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/14/2013 03:10'!
handleGeneralEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 11/14/2013 20:44'!
handleGeneralEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:37'!
handleNotationDeclaration: aName publicID: aPublicID systemID: aSystemID! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handlePCData: aString! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 3/21/2015 23:27'!
handlePI: aTargetString data: aDataString! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:37'!
handleParameterEntityDeclaration: aName publicID: aPublicID systemID: aSystemID! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/14/2013 03:10'!
handleParameterEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 11/14/2013 20:44'!
handleParameterEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 3/25/2016 05:11'!
handleParseError: aString! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 3/24/2015 20:27'!
handleStartContent! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 11/16/2013 05:10'!
handleStartContentEntityReplacement: aName! !

!XMLParserDriver methodsFor: 'handling - dtd' stamp: 'monty 3/21/2015 22:37'!
handleStartDTD: aRoot publicID: aPublicID systemID: aSystemID! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartDocument! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 8/6/2016 19:52'!
handleStartTag: anElementName attributes: aDictionary! !

!XMLParserDriver methodsFor: 'handling' stamp: 'monty 1/4/2015 00:01'!
handleUnreplacedGeneralEntityReference: aName! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleWhitespace: aString! !

!XMLParserDriver methodsFor: 'handling' stamp: 'JAAyer 11/27/2010 07:23'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo! !

!XMLParserDriver methodsFor: 'instance creation' stamp: 'monty 3/26/2016 23:48'!
newInitialContext
	^ XMLXMLDeclarationContext new! !

!XMLParserDriver methodsFor: 'instance creation' stamp: 'monty 3/26/2016 23:49'!
newTokenizerOn: aStringOrStream
	^ self tokenizerClass
		driver: self
		on: aStringOrStream
		readLimit: self readLimit! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 3/30/2016 07:35'!
popEntityURI: aURI
	"remove: removes the first match"
	self entityURIStack removeAt:
		(self entityURIStack
			lastIndexOf: aURI
			ifAbsent: [^ aURI]).
	^ aURI.! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 3/30/2016 07:35'!
pushEntityURI: aURIString
	^ self entityURIStack addLast: aURIString! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 19:45'!
readLimit
	^ nil! !

!XMLParserDriver methodsFor: 'private' stamp: 'monty 2/10/2014 02:43'!
tokenizerClass
	^ XMLWellFormedParserTokenizer! !

Object subclass: #XMLParserLimits
	instanceVariableNames: 'documentReadLimit maxEntityReplacementDepth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

XMLParserLimits class
	instanceVariableNames: 'defaultReadLimit'!

!XMLParserLimits commentStamp: 'monty 7/13/2015 13:17' prior: 0!
This class places security limits on input. Setting a limit to nil disables it, and removeAll disables all.!

XMLParserLimits class
	instanceVariableNames: 'defaultReadLimit'!

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 12/9/2014 07:41'!
defaultDocumentReadLimit
	^ self defaultReadLimit! !

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 7/13/2015 13:14'!
defaultMaxEntityReplacementDepth
	^ 10! !

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 3/19/2016 18:38'!
defaultReadLimit
	^ defaultReadLimit ifNil: [defaultReadLimit := 10000000]! !

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 3/19/2016 18:38'!
defaultReadLimit: aDefaultLimit
	"can be used to change the default read limit instances start with"
	defaultReadLimit := aDefaultLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:43'!
documentReadLimit
	^ documentReadLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 4/14/2016 03:29'!
documentReadLimit: anInteger
	documentReadLimit := anInteger! !

!XMLParserLimits methodsFor: 'initialization' stamp: 'monty 7/13/2015 13:12'!
initialize
	super initialize.

	documentReadLimit := self class defaultDocumentReadLimit.
	maxEntityReplacementDepth := self class defaultMaxEntityReplacementDepth.! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:45'!
maxEntityReplacementDepth
	^ maxEntityReplacementDepth! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 4/14/2016 03:30'!
maxEntityReplacementDepth: anInteger
	maxEntityReplacementDepth := anInteger! !

!XMLParserLimits methodsFor: 'removing' stamp: 'monty 1/27/2015 22:46'!
removeAll
	documentReadLimit := nil.
	maxEntityReplacementDepth := nil.! !

Object subclass: #XMLParserTokenizer
	instanceVariableNames: 'context driver streamReader writeStream nameStream'
	classVariableNames: 'PredefinedEntities'
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLParserTokenizer commentStamp: 'monty 6/16/2015 01:48' prior: 0!
This is a base class for parser tokenizers that read tokens from a stream using an XMLNestedStreamReader and context objects. Sending #nextToken causes a token to be read and handler messages to be sent to a driver.

Be careful changing the code in this class or subclasses because it's optimized.!

!XMLParserTokenizer class methodsFor: 'instance creation' stamp: 'monty 1/17/2014 01:27'!
driver: aDriver on: aStringOrStream
	^self
		driver: aDriver
		on: aStringOrStream
		readLimit: nil! !

!XMLParserTokenizer class methodsFor: 'instance creation' stamp: 'monty 1/17/2014 01:26'!
driver: aDriver on: aStringOrStream readLimit: aReadLimit
	^self new
		setDriver: aDriver
		stream:
			(aStringOrStream isStream
				ifTrue: [aStringOrStream]
				ifFalse: [aStringOrStream readStream])
		readLimit: aReadLimit! !

!XMLParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 3/23/2016 23:10'!
initialize
	"self initialize"

	self initializePredefinedEntities! !

XMLParserTokenizer initialize!

!XMLParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 3/8/2016 16:37'!
initializePredefinedEntities
	(PredefinedEntities := Dictionary new)
		at: 'lt' put: $<;
		at: 'gt' put: $>;
		at: 'amp' put: $&;
		at: 'apos' put: $';
		at: 'quot' put: $"! !

!XMLParserTokenizer class methodsFor: 'accessing' stamp: 'monty 11/18/2014 07:11'!
predefinedEntities
	^ PredefinedEntities! !

!XMLParserTokenizer methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
atEnd
	^ context isTerminatedContext! !

!XMLParserTokenizer methodsFor: 'decoding' stamp: 'monty 3/24/2016 21:01'!
characterFromCodePoint: aCodePoint
	(aCodePoint notNil
		and: [aCodePoint > 0
			and: [aCodePoint <= 16r10FFFF]])
		ifTrue: [
			^ [Character value: aCodePoint]
				on: Error
				do: [:error | nil]]
		ifFalse: [^ nil]! !

!XMLParserTokenizer methodsFor: 'closing' stamp: 'monty 11/24/2014 19:27'!
closeStreams
	streamReader closeStreams! !

!XMLParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 11/10/2014 00:46'!
constructReplacementForGeneralEntity: anEntity! !

!XMLParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 11/10/2014 00:37'!
constructReplacementForParameterEntity: anEntity! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 5/3/2015 16:35'!
context
	^ context! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 5/3/2015 16:35'!
context: aTokenContext
	context := aTokenContext! !

!XMLParserTokenizer methodsFor: 'decoding' stamp: 'monty 3/5/2016 06:45'!
convertFromEncoding: anEncodingName
	driver decodesCharacters
		ifTrue: [streamReader convertFromEncoding: anEncodingName]! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 3/18/2016 01:46'!
currentColumnNumber
	^ streamReader currentColumnNumber! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
currentLineNumber
	^ streamReader currentLineNumber! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 3/18/2016 02:19'!
currentPosition
	^ streamReader currentPosition! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
driver
	^ driver! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:50'!
errorExpected: aDescription
	self parseError: 'Expected ', aDescription! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:50'!
errorExpected: aDescription butGot: aCharacter
	self
		formatParseError: 'Expected {1} but got "{2}"'
		with: aDescription
		with: (aCharacter ifNil: [''])! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:50'!
errorExpectedLiteral: aCharacterOrString
	self
		errorExpectedLiteral: aCharacterOrString
		butGot: nil! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 09:14'!
errorExpectedLiteral: anExpectedCharacterOrString butGot: aReceivedCharacterOrString
	self
		formatParseError: 'Expected "{1}" but got "{2}"' 
		with: anExpectedCharacterOrString
		with: (aReceivedCharacterOrString ifNil: [''])! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:35'!
expectNext: aCharacter
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:37'!
expectNextAll: aString
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/3/2015 17:50'!
expectSeparators
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/3/2015 01:32'!
expectTerminator
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 7/3/2016 18:38'!
formatParseError: aString with: aFirstValue
	self
		formatParseError: aString
		withArguments: (Array with: aFirstValue)! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 7/3/2016 18:38'!
formatParseError: aString with: aFirstValue with: aSecondValue
	self
		formatParseError: aString
		withArguments:
			(Array
				with: aFirstValue
				with: aSecondValue)! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 7/3/2016 18:38'!
formatParseError: aString with: aFirstValue with: aSecondValue with: aThirdValue
	self
		formatParseError: aString
		withArguments:
			(Array
				with: aFirstValue
				with: aSecondValue
				with: aThirdValue)! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 7/3/2016 18:38'!
formatParseError: aString with: aFirstValue with: aSecondValue with: aThirdValue with: aFourthValue
	self
		formatParseError: aString
		withArguments:
			(Array
				with: aFirstValue
				with: aSecondValue
				with: aThirdValue
				with: aFourthValue)! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 7/3/2016 18:38'!
formatParseError: aString withArguments: aValueCollection
	self parseError: (aString format: aValueCollection)! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/25/2016 05:20'!
nextCDataSection
	self subclassResponsibility
! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/12/2014 06:52'!
nextContentMarkupToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/12/2014 06:48'!
nextContentToken
	streamReader peek == $<
		ifTrue: [
			streamReader next.
			^ self nextContentMarkupToken].
	self nextPCDataToken.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/23/2016 00:20'!
nextDoctypeDeclaration
	| root publicID systemID |

	context := context doctypeDeclarationContext.
	self
		expectNextAll: 'DOCTYPE';
		expectSeparators.

	root := self nextName.
	(streamReader peek == $[
		or: [streamReader peek == $>])
		ifFalse: [
			self expectSeparators.
			streamReader peek == $P
				ifTrue: [
					publicID := self nextPublicID.
					self expectSeparators.
					systemID := self nextSystemIDLiteral]
				ifFalse: [
					streamReader peek == $S
						ifTrue: [systemID := self nextSystemID]]].
	driver
		handleStartDTD: root
		publicID: (publicID ifNil: [''])
		systemID: (systemID ifNil: ['']).

	streamReader skipSeparators.
	streamReader peek == $[
		ifTrue: [self nextStartInternalSubset]
		ifFalse: [self nextEndDoctypeDeclaration].! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2015 04:43'!
nextEndDoctypeDeclaration
	self expectTerminator.
	context := context postDoctypeDeclarationContext.

	driver handleEndDTD.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/24/2014 21:11'!
nextEndDocument
	context isTerminatedContext
		ifFalse: [
			self closeStreams.
			context := context terminatedContext.
			driver handleEndDocument]! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/23/2016 00:20'!
nextEndInternalSubset
	"skip ]"
	streamReader
		next;
		skipSeparators.
	self nextEndDoctypeDeclaration.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/23/2016 00:20'!
nextInternalSubsetToken
	streamReader skipSeparators.
	streamReader peek == $%
		ifTrue: [
			streamReader next.
			^ self nextParameterEntityReference].
	streamReader peek == $]
		ifTrue: [^ self nextEndInternalSubset].
	self nextSubsetMarkupToken.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 01:14'!
nextName
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/23/2016 23:13'!
nextNonPIPrologOrContentToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/12/2014 06:49'!
nextPCDataToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/6/2015 07:16'!
nextPI
	"skip ?"
	streamReader next.
	self nextPIWithTarget: self nextPITarget.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:16'!
nextPITarget
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/25/2016 05:19'!
nextPIValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/23/2016 23:01'!
nextPIWithTarget: aTarget
	streamReader peek == $?
		ifTrue: [
			streamReader next.
			self expectTerminator.
			driver
				handlePI: aTarget
				data: '']
		ifFalse: [
			self expectSeparators.
			driver
				handlePI: aTarget
				data: self nextPIValue].! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/7/2015 02:28'!
nextParameterEntityReference
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:19'!
nextPrologToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:42'!
nextPublicID
	^ self
		expectNextAll: 'PUBLIC';
		expectSeparators;
		nextPublicIDLiteral! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/10/2015 10:20'!
nextPublicIDLiteral
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/24/2015 20:38'!
nextStartContent
	context := context contentContext.
	driver handleStartContent.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/9/2016 01:53'!
nextStartDocument
	((context := driver newInitialContext) supportsEncodingDetection
		and: [driver decodesCharacters])
		ifTrue: [streamReader detectCurrentEncoding].
	driver handleStartDocument.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/23/2016 00:20'!
nextStartInternalSubset
	"skip ["
	streamReader next.
	context := context internalSubsetContext.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:21'!
nextSubsetMarkupToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:41'!
nextSystemID
	^ self
		expectNextAll: 'SYSTEM';
		expectSeparators;
		nextSystemIDLiteral! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 03:41'!
nextSystemIDLiteral
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 5/3/2015 16:12'!
nextToken
	(streamReader atEnd
		and: [context isInitializedContext])
		ifTrue: [^ self nextEndDocument].
	context nextTokenFrom: self.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:33'!
nextXMLAttributeName: aName
	self expectNextAll: aName.
	streamReader skipSeparators.
	self expectNext: $=.
	streamReader skipSeparators.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 5/8/2015 21:27'!
nextXMLDeclaration
	| version encoding |

	self expectSeparators.
	version := self nextXMLVersionAttributeRequired: true.
	streamReader peek == $?
		ifFalse: [self expectSeparators].
	(encoding := self nextXMLEncodingAttributeRequired: false) isEmpty
		ifFalse: [
			self convertFromEncoding: encoding.
			streamReader peek == $?
				ifFalse: [self expectSeparators]].
	driver
		handleXMLVersion: version
		encoding: encoding
		standalone: self nextXMLStandaloneAttribute.
	streamReader skipSeparators.
	self
		expectNext: $?;
		expectTerminator.

	context := context prologContext.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/13/2016 20:43'!
nextXMLDeclarationOrPrologToken
	streamReader peek == $<
		ifTrue: [
			streamReader next.
			streamReader peek == $?
				ifTrue: [| target |
					streamReader next.
					(target := self nextPITarget) = 'xml'
						ifTrue: [^ self nextXMLDeclaration]
						ifFalse: [
							context := context prologContext.
							^ self nextPIWithTarget: target]].
			context := context prologContext.
			^ self nextNonPIPrologOrContentToken].

	"other prolog tokens can have whitespace before them, so the doc
	does not need to start with an < for them"
	context := context prologContext.
	self nextPrologToken.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 18:12'!
nextXMLEncodingAttributeRequired: aBoolean
	(aBoolean or: [streamReader peek == $e])
		ifTrue: [	
			^ self
				nextXMLAttributeName: 'encoding';
				nextXMLEncodingAttributeValue]
		ifFalse: [^ '']! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLEncodingAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 18:13'!
nextXMLStandaloneAttribute
	"always optional"
	(streamReader peek == $s)
		ifTrue: [
			^ self
				nextXMLAttributeName: 'standalone';
				nextXMLStandaloneAttributeValue]
		ifFalse: [^ '']! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 5/8/2015 21:15'!
nextXMLTextDeclaration
	(streamReader nextMatchAll: '<?xml')
		ifTrue: [| encoding |
			self expectSeparators.
			(self nextXMLVersionAttributeRequired: false) isEmpty
				ifFalse: [self expectSeparators].
			(encoding := self nextXMLEncodingAttributeRequired: true) isEmpty
				ifFalse: [self convertFromEncoding: encoding].

			streamReader skipSeparators.
			self
				expectNext: $?;
				expectTerminator].
	context := context previousContext.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 18:13'!
nextXMLVersionAttributeRequired: aBoolean
	(aBoolean or: [streamReader peek == $v])
		ifTrue: [
			^ self
				nextXMLAttributeName: 'version';
				nextXMLVersionAttributeValue]
		ifFalse: [^ '']! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:34'!
nextXMLVersionAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/16/2014 09:08'!
normalizedLineEndingChar
	^ streamReader normalizedLineEndingChar! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:26'!
normalizedLineEndingChar: aCharacter
	streamReader normalizedLineEndingChar: aCharacter! !

!XMLParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:38'!
parseError: aString
	driver handleParseError: aString! !

!XMLParserTokenizer methodsFor: 'printing' stamp: 'monty 4/25/2015 15:13'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: context;
		nextPutAll: '; ';
		print: streamReader;
		nextPut: $).! !

!XMLParserTokenizer methodsFor: 'initialization' stamp: 'monty 2/18/2016 04:36'!
setDriver: aDriver stream: aStream readLimit: aReadLimit
	driver := aDriver.
	streamReader :=
		(XMLNestedStreamReader
			on: aStream
			readLimit: aReadLimit).
	"must use #writeStream instead of WriteStream on: to get a 0-based stream
	on Gemstone"
	writeStream := (String new: 128) writeStream.
	nameStream := (String new: 16) writeStream.
	context := XMLUnitializedContext new.! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
streamReader
	^ streamReader! !

XMLParserTokenizer subclass: #XMLWellFormedParserTokenizer
	instanceVariableNames: 'pushedBackEntities openIncludeSections'
	classVariableNames: 'EncodingNameChars EncodingNameStartChars NormalizedSpace PublicIDChars VersionDigitChars'
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLWellFormedParserTokenizer commentStamp: 'monty 6/15/2015 23:55' prior: 0!
This is a tokenizer for well-formed XML documents that supports DTD subsets and entity replacement.!

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 4/1/2016 06:45'!
initialize
	"self initialize"

	self
		initializeNormalizedSpace;
		initializePublicIDChars;
		initializeEncodingNameStartChars;
		initializeEncodingNameChars;
		initializeVersionDigitChars! !

XMLWellFormedParserTokenizer initialize!

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 3/16/2016 16:58'!
initializeEncodingNameChars
	"EncName ::= [A-Za-z] ([A-Za-z0-9._] | '-')*"

	(EncodingNameChars := BitmapCharacterSet new)
		addRangeFrom: $a to: $z;
		addRangeFrom: $A to: $Z;
		addRangeFrom: $0 to: $9;
		addAll: '-_.'! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 4/1/2016 06:43'!
initializeEncodingNameStartChars
	"Character>>isLetter is not used because it may return true for non-ASCII
	letters depending on the locale.
	
	EncName ::= [A-Za-z] ([A-Za-z0-9._] | '-')*"

	(EncodingNameStartChars := BitmapCharacterSet new)
		addRangeFrom: $a to: $z;
		addRangeFrom: $A to: $Z! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 2/10/2014 01:18'!
initializeNormalizedSpace
	NormalizedSpace := Character space! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 3/25/2016 23:58'!
initializePublicIDChars
	"PubidChar ::= #x20 | #xD | #xA | [a-zA-Z0-9] | [-'()+,./:=?;!!*#@$_%]"

	(PublicIDChars := BitmapCharacterSet new)
		add: 16r20 asCharacter;
		add: 16rD asCharacter;
		add: 16rA asCharacter;
		addRangeFrom: $a to: $z;
		addRangeFrom: $A to: $Z;
		addRangeFrom: $0 to: $9;
		addAll: '-''()+,./:=?;!!*#@$_%'! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 4/1/2016 06:44'!
initializeVersionDigitChars
	"Character>>isDigit is not used because it may return true for non-ASCII
	digits depending on the locale.
	
	VersionNum ::= '1.' [0-9]+"

	(VersionDigitChars := BitmapCharacterSet new)
		addRangeFrom: $0 to: $9! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 4/3/2016 04:23'!
addPushedBackEntity: aPushedBackEntity
	(self hasPushedBackEntityNamed: aPushedBackEntity name)
		ifTrue: [self errorSelfReferentialEntity: aPushedBackEntity].

	self maxEntityReplacementDepth
		ifNotNil: [:maxDepth |
			self pushedBackEntities size >= maxDepth
				ifTrue: [self errorEntityDepthLimitExceeded]].
	
	self pushedBackEntities addLast: aPushedBackEntity.
	aPushedBackEntity isExternal
		ifTrue: [driver pushEntityURI: aPushedBackEntity uri].

	^ aPushedBackEntity.! !

!XMLWellFormedParserTokenizer methodsFor: 'decoding' stamp: 'monty 3/24/2016 21:02'!
characterFromCodePoint: aCodePoint
	| character |

	"nil understands isXMLChar so it does not need to be tested for"
	(character := super characterFromCodePoint: aCodePoint) isXMLChar
		ifTrue: [^ character]
		ifFalse: [^ nil].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 4/6/2016 06:44'!
checkNestingOfParameterEntity: aParameterEntity
	aParameterEntity
		ifNil: [
			self topPushedBackEntity
				ifNotNil: [driver handleImproperParameterEntityNesting]]
		ifNotNil: [
			self topPushedBackEntity == aParameterEntity
				ifFalse: [driver handleImproperParameterEntityNesting]].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 4/3/2016 04:44'!
constructReplacementForGeneralEntity: anEntity
	context isLiteralValueContext
		ifTrue: [
			anEntity isExternal
				ifTrue: [self errorLiteralContextExternalEntity: anEntity].
			self constructReplacementForLiteralContextEntity: anEntity]
		ifFalse: [
			driver handleStartContentEntityReplacement: anEntity name.
			self
				pushBackEntity: anEntity
				onClose: [driver handleEndContentEntityReplacement: anEntity name]]! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 4/6/2016 06:43'!
constructReplacementForLiteralContextEntity: anEntity
	| wasEscapingQuotes |

	wasEscapingQuotes := streamReader isEscapingQuotes.
	self
		pushBackEntity: anEntity
		onClose: [streamReader isEscapingQuotes: wasEscapingQuotes].

	"wait till after the text declaration was read to start escaping"
	self topPushedBackEntity
		ifNotNil: [:pushedBackEntity |
			pushedBackEntity == anEntity
				ifTrue: [streamReader isEscapingQuotes: true]].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 3/20/2016 17:39'!
constructReplacementForParameterEntity: anEntity
	context isLiteralValueContext
		ifTrue: [self constructReplacementForLiteralContextEntity: anEntity]
		ifFalse: [
			streamReader pushBackCharacter: NormalizedSpace.
			self
				pushBackEntity: anEntity
				onClose: nil.
			streamReader pushBackCharacter: NormalizedSpace]! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/11/2015 00:01'!
errorDocumentTerminatedPrematurely
	self parseError: 'Document terminated prematurely'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 8/6/2016 19:57'!
errorDuplicateAttribute: anAttribute
	self
		formatParseError: 'Duplicate attribute "{1}" in start tag'
		with: anAttribute! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 15:40'!
errorEntityDepthLimitExceeded
	XMLLimitException signal: 
		'Cannot further nest entity replacements; ',
		'use #maxEntityReplacementDepth: to change or disable'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 08:20'!
errorExpectedAttributeNameValueSeparatorButGot: aCharacter
	self
		errorExpected: 'attribute name-value "=" separator'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 21:47'!
errorExpectedAttributeValueEnd: aQuote butGot: aCharacter
	aCharacter == $<
		ifTrue: [self parseError: 'Attribute value contains illegal "<" character']
		ifFalse: [
			self
				errorExpectedQuote: aQuote
				ending: 'attribute value'
				butGot: aCharacter]! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 08:18'!
errorExpectedAttributeValueStartButGot: aCharacter
	self
		errorExpectedQuoteStarting: 'attribute value'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/23/2016 22:57'!
errorExpectedCDataSectionTerminator
	self errorExpected: 'end of CDATA section "]]>" terminator'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/1/2016 06:40'!
errorExpectedEncodingName
	self errorExpected: 'encoding name in <?xml ...?> encoding attribute'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 21:47'!
errorExpectedEntityValueEnd: aQuote butGot: aCharacter
	self
		errorExpectedQuote: aQuote
		ending: 'entity value'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/11/2016 12:00'!
errorExpectedIntegerCharacterReference
	self errorExpected:
		'decimal or hex integer after "&#" character reference start'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/27/2016 01:52'!
errorExpectedLiteral: aString butGot: aCharacter at: aStringPosition
	| oldPosition receivedString |

	oldPosition := streamReader currentPosition.
	receivedString :=
		(aString
			copyReplaceFrom: aStringPosition
			to: aString size
			with:
				(aCharacter ifNil: ['']) asString,
				(streamReader next: aString size - aStringPosition)).

	"more accurately estimate where the error occured"
	streamReader
		currentPosition: oldPosition - aStringPosition + 1;
		currentLineNumber:
			streamReader currentLineNumber -
				(receivedString occurrencesOf: streamReader normalizedLineEndingChar).

	self
		errorExpectedLiteral: aString
		butGot: receivedString.! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:46'!
errorExpectedName
	self errorExpected: 'an XML name'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:46'!
errorExpectedNmtoken
	self errorExpected: 'an XML Nmtoken name'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 21:47'!
errorExpectedPublicIDLiteralEnd: aQuote butGot: aCharacter
	self
		errorExpectedQuote: aQuote
		ending: 'public ID literal'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 09:08'!
errorExpectedPublicIDLiteralStartButGot: aCharacter
	self
		errorExpectedQuoteStarting: 'public ID literal'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 21:47'!
errorExpectedQuote: aQuote ending: aDescription butGot: aCharacter
	self
		formatParseError: 'Expected end of {1} ({2}) quoted {3} but got "{4}"'
		with:
			(aQuote == $"
				ifTrue: ['double']
				ifFalse: ['single'])
		with: aQuote
		with: aDescription
		with: (aCharacter ifNil: [''])! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 09:15'!
errorExpectedQuoteStarting: aDescription butGot: aCharacter
	self
		formatParseError:
			'Expected start of double (") or single ('') quoted {2} but got "{2}"'
		with: aDescription
		with: (aCharacter ifNil: [''])! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/3/2015 17:50'!
errorExpectedSeparators
	self errorExpected: 'one or more whitespace characters'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/11/2016 12:03'!
errorExpectedSubsetDeclaration
	self errorExpected:
		'DTD subset ELEMENT, ENTITY, ATTLIST, or NOTATION declaration'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 21:47'!
errorExpectedSystemIDLiteralEnd: aQuote butGot: aCharacter
	self
		errorExpectedQuote: aQuote
		ending: 'system ID literal'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 09:11'!
errorExpectedSystemIDLiteralStartButGot: aCharacter
	self
		errorExpectedQuoteStarting: 'system ID literal'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/25/2015 20:25'!
errorExpectedTerminatorButGot: aCharacter
	self
		errorExpected: '">" terminator'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/15/2016 07:38'!
errorExpectedVersionDigitButGot: aCharacter
	self
		errorExpected: 'XML version digit'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 21:47'!
errorExpectedXMLEncodingValueEnd: aQuote butGot: aCharacter
	self
		errorExpectedQuote: aQuote
		ending: '<?xml ...?> encoding value'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 08:33'!
errorExpectedXMLEncodingValueStartButGot: aCharacter
	self
		errorExpectedQuoteStarting: '<?xml ...?> encoding value'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 21:47'!
errorExpectedXMLStandaloneValueEnd: aQuote butGot: aCharacter
	self
		errorExpectedQuote: aQuote
		ending: '<?xml ...?> standalone value'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 08:33'!
errorExpectedXMLStandaloneValueStartButGot: aCharacter
	self
		errorExpectedQuoteStarting: '<?xml ...?> standalone value'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 21:47'!
errorExpectedXMLVersionValueEnd: aQuote butGot: aCharacter
	self
		errorExpectedQuote: aQuote
		ending: '<?xml ...?> version value'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 7/16/2016 08:42'!
errorExpectedXMLVersionValueStartButGot: aCharacter
	self
		errorExpectedQuoteStarting: '<?xml ...?> version value'
		butGot: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/6/2016 08:02'!
errorImproperGeneralEntityNesting
	self parseError:
		'Markup tokens must start and end in the same general entity replacement'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/10/2016 02:53'!
errorInternalDeclarationParameterEntityReference
	self parseError: 'Parameter entity reference within internal subset declaration'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/23/2015 23:21'!
errorInternalSubsetConditionalSection
	self parseError:
		'Cannot have <!![INCLUDE[...]> or <!![IGNORE[...]> sections in internal subset'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:47'!
errorInvalidCharacterReferenceCodePoint: aCodePoint
	self
		formatParseError: 'Invalid character reference to U+{1} code point'
		with: aCodePoint printStringHex! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:47'!
errorLiteralContextExternalEntity: anEntity
	self
		formatParseError: 'External general entity reference {1} in literal context'
		with: anEntity asReference! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/6/2015 04:38'!
errorMultipleDoctypeDeclarations
	self parseError: 'Multiple <!!DOCTYPE ...> declarations'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/26/2016 21:57'!
errorNonChar: aCharacter in: aDescription
	self
		formatParseError: 'Invalid character U+{1} in {2}'
		with: aCharacter asciiValue printStringHex
		with: aDescription! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/23/2016 22:41'!
errorNonCharInAttributeValue: aCharacter
	self
		errorNonChar: aCharacter
		in: 'attribute value'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/26/2016 21:58'!
errorNonCharInCDataSection: aCharacter
	self
		errorNonChar: aCharacter
		in: '<!![CDATA[...]]> section'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/23/2016 22:42'!
errorNonCharInComment: aCharacter
	self
		errorNonChar: aCharacter
		in: 'comment'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 6/27/2016 23:03'!
errorNonCharInEntityValue: aCharacter
	self
		errorNonChar: aCharacter
		in: 'entity value'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/23/2016 22:38'!
errorNonCharInPCData: aCharacter
	self
		errorNonChar: aCharacter
		in: 'PCDATA'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/23/2016 23:04'!
errorNonCharInPIValue: aCharacter
	self
		errorNonChar: aCharacter
		in: 'processing instruction value'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 02:56'!
errorNonCharInSystemID: aCharacter
	self
		errorNonChar: aCharacter
		in: 'system ID'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 20:43'!
errorParameterEntityTerminatingSubset
	self parseError: 'Parameter entity replacement cannot terminate internal subset'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:49'!
errorReservedPITarget: aTarget
	self
		formatParseError: 'Processing instruction target "{1}" is reserved'
		with: aTarget! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/25/2016 03:49'!
errorSelfReferentialEntity: anEntity
	self
		formatParseError: 'Illegal self-referential {1} entity'
		with: anEntity asReference! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/4/2016 00:17'!
errorUnexpectedCDataTerminator
	self parseError: 'Unexpected CDATA "]]>" terminator in PCDATA'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/3/2016 23:51'!
errorUnexpectedEndConditionalSection
	self parseError: 'Unexpected "]" end of conditional section in external entity'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/23/2015 23:22'!
errorUnterminatedIgnoreSections
	self parseError: 'Unterminated <!![IGNORE[...]]> section(s) in external entity'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 4/23/2015 20:33'!
errorUnterminatedIncludeSections
	self parseError: 'Unterminated <!![INCLUDE[...]]> section(s) in external entity'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 3/30/2016 19:52'!
errorXMLDeclarationNotAtStart
	self parseError: 'XML declaration can only occur at the start of a document'! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/5/2016 03:12'!
expectNext: aCharacter
	| nextChar |

	(nextChar := streamReader next) = aCharacter
		ifFalse: [
			self
				errorExpectedLiteral: aCharacter
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/5/2016 03:12'!
expectNextAll: aString
	"uses to:do: and not implemented in terms of expectNextAll:startingAt:
	for speed"
	1 to: aString size do: [:i | | nextChar |
		(nextChar := streamReader next) = (aString at: i)
			ifFalse: [
				self
					errorExpectedLiteral: aString
					butGot: nextChar
					at: i]].
	^ aString.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/5/2016 03:12'!
expectNextAll: aString startingAt: aPosition
	"to:do: for speed"
	aPosition to: aString size do: [:i | | nextChar |
		(nextChar := streamReader next) = (aString at: i)
			ifFalse: [
				self
					errorExpectedLiteral: aString
					butGot: nextChar
					at: i]].
	^ aString.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/1/2016 06:57'!
expectSeparators
	"nil understands isXMLSeparator so no nil test is needed"
	streamReader next isXMLSeparator
		ifTrue: [streamReader skipSeparators]
		ifFalse: [self errorExpectedSeparators]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/1/2016 06:54'!
expectSeparatorsInDeclaration
	"this can be used to replace entity references within declarations"

	[streamReader peek == $%]
		whileTrue: [
			streamReader next.
			self nextParameterEntityReferenceInDeclaration].
	self expectSeparators.
	[streamReader peek == $%]
		whileTrue: [
			streamReader next.
			self nextParameterEntityReferenceInDeclaration.
			streamReader skipSeparators].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 4/3/2015 02:18'!
expectTerminator
	| nextChar |

	(nextChar := streamReader next) == $>
		ifFalse: [self errorExpectedTerminatorButGot: nextChar]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 7/17/2016 04:40'!
expectVersionDigit
	| nextChar |

	((nextChar := streamReader next) == nil
		or: [(VersionDigitChars includes: nextChar) not])
		ifTrue: [self errorExpectedVersionDigitButGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 4/7/2016 02:23'!
hasOpenIncludeSections
	openIncludeSections == nil
		ifTrue: [^ false].
	^ openIncludeSections size > 0.! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 3/30/2016 20:26'!
hasPushedBackEntities
	pushedBackEntities == nil
		ifTrue: [^ false].
	^ pushedBackEntities size > 0.! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 3/30/2016 20:27'!
hasPushedBackEntityNamed: aName
	^ self hasPushedBackEntities
		and: [self pushedBackEntities anySatisfy: [:each | each name = aName]]! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 4/14/2016 08:51'!
isInExternalSubsetOrExternallyDefinedEntity
	"this is used to check for conditional sections and
	parameter entity refs within declarations within the
	internal subset, which are disallowed"

	context isExternalSubsetContext
		ifTrue: [^ true].
	self hasPushedBackEntities
		ifTrue: [
			self pushedBackEntities do: [:each |
				(each isInternalTo: driver documentURI)
					ifFalse: [^ true]]].
	^ false.! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/17/2014 05:28'!
maxEntityReplacementDepth
	^ driver maxEntityReplacementDepth! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttlistDeclaration
	| element defaultPragma |

	self
		expectNextAll: 'ATTLIST';
		expectSeparatorsInDeclaration.

	element := self nextName.
	[streamReader peek == $>
		or: [
			self expectSeparatorsInDeclaration.
			streamReader peek == $>]]
		whileFalse: [
			driver
				handleAttributeDeclaration: element
				name: self nextName
				type: self nextAttributeType
				defaultPragma: (defaultPragma := self nextAttributeDefaultPragma)
				defaultValue: (self nextAttributeDefaultValueForPragma: defaultPragma)].
	self expectTerminator.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeDefaultPragma
	self expectSeparatorsInDeclaration.
	streamReader peek == $#
		ifFalse: [^ ''].
	streamReader next.

	streamReader peek == $R
		ifTrue: [^ self expectNextAll: 'REQUIRED'].
	streamReader peek == $I
		ifTrue: [^ self expectNextAll: 'IMPLIED'].
	^ self expectNextAll: 'FIXED'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeDefaultValueForPragma: aPragma
	| previousContext attributeValue |

	aPragma isEmpty
		ifFalse: [
			(aPragma first == $F) "FIXED"
				ifFalse: [^ ''].
			self expectSeparatorsInDeclaration].
	previousContext := context.
	context := context contentContext.
	attributeValue := self nextAttributeValue.
	context := previousContext.
	^ attributeValue.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/27/2016 00:56'!
nextAttributeEntityType
	self expectNextAll: 'ENTIT'.
	streamReader peek == $Y
		ifFalse: [
			^ self
				expectNextAll: 'ENTITIES'
				startingAt: 6 "'ENTITIES' indexOfSubCollection: 'IES'"].
	streamReader next.
	^ 'ENTITY'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeEnumerationType
	writeStream
		reset;
		nextPut: (self expectNext: $().
	self
		skipSeparatorsInDeclaration;
		nextNmtokenOnWriteStream;
		skipSeparatorsInDeclaration.
	[streamReader peek == $)]
		whileFalse: [
			writeStream nextPut: (self expectNext: $|).
			self
				skipSeparatorsInDeclaration;
				nextNmtokenOnWriteStream;
				skipSeparatorsInDeclaration].
	writeStream nextPut: streamReader next.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/27/2016 00:42'!
nextAttributeIDOrIDRefType
	self expectNextAll: 'ID'.
	streamReader peek == $R
		ifFalse: [^ 'ID'].	
	self
		expectNextAll: 'IDREF'
		startingAt: 3.
	streamReader peek == $S
		ifFalse: [^ 'IDREF'].
	streamReader next.
	^ 'IDREFS'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/27/2016 00:48'!
nextAttributeNotationEnumerationOrNmtokenType
	"skip the N"
	streamReader next.
	streamReader peek == $O
		ifTrue: [^ self nextAttributeNotationEnumerationType].
	self
		expectNextAll: 'NMTOKEN'
		startingAt: 2.
	streamReader peek == $S
		ifFalse: [^ 'NMTOKEN'].
	streamReader next.
	^ 'NMTOKENS'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:25'!
nextAttributeNotationEnumerationType
	writeStream
		reset;
		nextPutAll:
			(self
				expectNextAll: 'NOTATION'
				startingAt: 2). "the 'N' was already read"
	self
		expectSeparatorsInDeclaration;
		expectNext: $(.
	writeStream nextPutAll: ' ('.

	self
		skipSeparatorsInDeclaration;
		nextNameOnWriteStream;
		skipSeparatorsInDeclaration;
		nextNameListOnWriteStream.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextAttributeType
	| nextChar |

	self expectSeparatorsInDeclaration.
	(nextChar := streamReader peek) == $C
		ifTrue: [^ self expectNextAll: 'CDATA'].
	nextChar == $I
		ifTrue: [^ self nextAttributeIDOrIDRefType].
	nextChar == $E
		ifTrue: [^ self nextAttributeEntityType].
	nextChar == $N
		ifTrue: [^ self nextAttributeNotationEnumerationOrNmtokenType].
	^ self nextAttributeEnumerationType.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/16/2016 08:03'!
nextAttributeValue
	| quote nextChar |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [self errorExpectedAttributeValueStartButGot: quote].
	
	context enterLiteralValue.
	writeStream reset.
	[(nextChar := streamReader next) == quote
		or: [nextChar == $<
			or: [nextChar == nil]]]
		whileFalse: [
			nextChar == $&
				ifTrue: [
					"char refs should be ignored by whitespace normalization"
					self nextGeneralEntityOrCharacterReferenceOnWriteStream]
				ifFalse: [
					nextChar isXMLSeparator
						ifTrue: [nextChar := NormalizedSpace]
						ifFalse: [
							nextChar isXMLChar
								ifFalse: [self errorNonCharInAttributeValue: nextChar]].
					writeStream nextPut: nextChar]].
	context exitLiteralValue.

	nextChar == quote
		ifFalse: [
			self
				errorExpectedAttributeValueEnd: quote
				butGot: nextChar].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/1/2016 06:17'!
nextCDataSection
	| wasTerminated nextChar |

	self expectNextAll: '[CDATA['.
	writeStream reset.
	wasTerminated := false.
	[streamReader atEnd
		or: [(nextChar := streamReader next) == $]
			and: [wasTerminated := streamReader nextMatch: $] followedBy: $>]]]
		whileFalse: [
			nextChar isXMLChar
				ifFalse: [self errorNonCharInCDataSection: nextChar].
			writeStream nextPut: nextChar].
	wasTerminated
		ifFalse: [self errorExpectedCDataSectionTerminator].
	driver handleCData: writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/8/2016 00:21'!
nextCharacterReferenceOnWriteStream
	"returns the character written to the stream or $& if it wasn't replaced"
	| codePoint |

	"skip #"
	streamReader next.
	(context isContentContext
		and: [driver replacesContentEntityReferences not])
		ifTrue: [
			nameStream reset.
			(codePoint := streamReader nextIntegerPrintedOn: nameStream)
				ifNil: [self errorExpectedIntegerCharacterReference].
			(self characterFromCodePoint: codePoint)
				ifNil: [self errorInvalidCharacterReferenceCodePoint: codePoint].
			self expectNext: $;.
			writeStream
				nextPutAll: '&#';
				nextPutAll: nameStream contents;
				nextPut: $;.
			^ $&]
		ifFalse: [| character |
			(codePoint := streamReader nextInteger)
				ifNil: [self errorExpectedIntegerCharacterReference].
			(character := self characterFromCodePoint: codePoint)
				ifNil: [self errorInvalidCharacterReferenceCodePoint: codePoint].
			self expectNext: $;.
			writeStream nextPut: character.
			^ character].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/26/2016 00:06'!
nextComment
	| nextChar |

	"Skip first -"
	streamReader next.
	self expectNext: $-.

	"xml comments can't have '--' unless followed by the terminating '>'"
	writeStream reset.
	[(nextChar := streamReader next) == nil
		or: [nextChar == $-
			and: [streamReader peek == $-]]]
		whileFalse: [
			nextChar isXMLChar
				ifFalse: [self errorNonCharInComment: nextChar].
			writeStream nextPut: nextChar].
	self
		expectNext: $-;
		expectTerminator.

	driver handleComment: writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:40'!
nextConditionalSectionFromEntity: aParameterEntity
	"skip ["
	streamReader next.
	self
		skipSeparatorsInDeclaration;
		expectNext: $I.
	streamReader peek == $N
		ifTrue: [self nextStartIncludeSectionFromEntity: aParameterEntity]
		ifFalse: [self nextIgnoreSectionFromEntity: aParameterEntity].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/7/2016 01:54'!
nextContentMarkupToken
	| generalEntity |

	generalEntity := self topPushedBackEntity.
	streamReader peek == $?
		ifTrue: [self nextPI]
		ifFalse: [
			streamReader peek == $!!
				ifTrue: [
					streamReader next.
					streamReader peek == $-
						ifTrue: [self nextComment]
						ifFalse: [self nextCDataSection]]
				ifFalse: [self nextTag]].
	generalEntity
		ifNotNil: [
			self topPushedBackEntity == generalEntity
				ifFalse: [self errorImproperGeneralEntityNesting]].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:57'!
nextContentModel
	| parameterEntity |

	self expectSeparatorsInDeclaration.
	streamReader peek == $A
		ifTrue: [^ self expectNextAll: 'ANY'].
	streamReader peek == $E
		ifTrue: [^ self expectNextAll: 'EMPTY'].
	
	writeStream
		reset;
		nextPut: (self expectNext: $().
	parameterEntity := self topPushedBackEntity.
	self skipSeparatorsInDeclaration.
	streamReader peek == $#
		ifTrue: [
			self nextMixedContentListOnWriteStreamFromEntity:
				parameterEntity]
		ifFalse: [
			self nextContentModelChoiceOrSequenceOnWriteStreamFromEntity:
				parameterEntity].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:57'!
nextContentModelChoiceOrSequenceOnWriteStreamFromEntity: aParameterEntity
	self
		nextContentModelParticleOnWriteStream;
		skipSeparatorsInDeclaration.
	streamReader peek == $|
		ifTrue: [self nextContentModelChoiceValuesOnWriteStream]
		ifFalse: [self nextContentModelSequenceOnWriteStream].
	self
		checkNestingOfParameterEntity: aParameterEntity;
		nextContentModelUnaryOperatorOnWriteStream.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextContentModelChoiceValuesOnWriteStream
	[streamReader peek == $)]
		whileFalse: [
			writeStream nextPut: (self expectNext: $|).
			self
				nextContentModelParticleOnWriteStream;
				skipSeparatorsInDeclaration].
	writeStream nextPut: streamReader next.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:57'!
nextContentModelParticleOnWriteStream
	self skipSeparatorsInDeclaration.
	streamReader peek == $(
		ifTrue: [
			writeStream nextPut: streamReader next.
			self nextContentModelChoiceOrSequenceOnWriteStreamFromEntity:
				self topPushedBackEntity]
		ifFalse: [
			self
				nextNameOnWriteStream;
				nextContentModelUnaryOperatorOnWriteStream]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:25'!
nextContentModelSequenceOnWriteStream
	[streamReader peek == $)]
		whileFalse: [
			writeStream nextPut: (self expectNext: $,).
			self
				nextContentModelParticleOnWriteStream;
				skipSeparatorsInDeclaration].
	writeStream nextPut: streamReader next.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/17/2015 11:33'!
nextContentModelUnaryOperatorOnWriteStream
	| nextChar |

	((nextChar := streamReader peek) == $?
		or: [nextChar == $*
			or: [nextChar == $+]])
		ifTrue: [writeStream nextPut: streamReader next]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/10/2016 22:16'!
nextDoctypeDeclarationOrComment
	"skip !!"
	streamReader next.
	streamReader peek == $-
		ifTrue: [^ self nextComment].
	context isPostDoctypeDeclarationContext
		ifTrue: [self errorMultipleDoctypeDeclarations].
	self nextDoctypeDeclaration.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/27/2016 01:07'!
nextElementDeclaration
	self
		expectNextAll: 'ELEMENT'
		startingAt: 2. "the 'E' was already read"

	self expectSeparatorsInDeclaration.
	driver
		handleElementDeclaration: self nextName
		contentModel: self nextContentModel.
	self
		skipSeparatorsInDeclaration;
		expectTerminator.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/23/2015 17:15'!
nextEndDocument
	context canTerminateDocument
		ifFalse: [self errorDocumentTerminatedPrematurely].
	self hasOpenIncludeSections
		ifTrue: [self errorUnterminatedIncludeSections].
	super nextEndDocument.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/7/2016 02:24'!
nextEndIncludeSection
	"skip ]"
	streamReader next.
	self
		expectNext: $];
		expectTerminator;
		checkNestingOfParameterEntity: self openIncludeSections removeLast.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2016 00:30'!
nextEndInternalSubset
	self hasOpenIncludeSections
		ifTrue: [^ self nextEndIncludeSection].
	"skip ]"
	streamReader next.
	self hasPushedBackEntities
		ifTrue: [self errorParameterEntityTerminatingSubset].
	streamReader skipSeparators.
	self nextEndDoctypeDeclaration.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/3/2015 01:33'!
nextEndTag
	"Skip /"
	streamReader next.
	driver handleEndTag: self nextName.
	streamReader skipSeparators.
	self expectTerminator.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/27/2016 01:07'!
nextEntityDeclaration
	| isParameterEntity name |

	self
		expectNextAll: 'ENTITY'
		startingAt: 2. "the 'E' was already read"

	context isExternalSubsetContext
		ifTrue: [
			"this is to distinguish a parameter reference from declaration"
			[isParameterEntity isNil]
				whileTrue: [
					streamReader skipSeparators.
					streamReader peek == $%
						ifTrue: [
							streamReader next.
							"nil understands isXMLSeparator so no nil test is needed"
							streamReader peek isXMLSeparator
								ifTrue: [
									isParameterEntity := true.
									self expectSeparatorsInDeclaration]
								ifFalse: [self nextParameterEntityReference]]
						ifFalse: [isParameterEntity := false]]]
		ifFalse: [
			self expectSeparators.
			(isParameterEntity := (streamReader peek == $%))
				ifTrue: [
					streamReader next.
					self expectSeparators]].

	name := self nextName.
	self expectSeparatorsInDeclaration.
	streamReader atQuote
		ifTrue: [
			self
				nextInternalEntityDeclaration: name
				isParameterEntity: isParameterEntity]
		ifFalse: [
			self
				nextExternalEntityDeclaration: name
				isParameterEntity: isParameterEntity].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 7/16/2016 08:24'!
nextEntityValue
	| quote nextChar |

	quote := streamReader next.

	context enterLiteralValue.
	writeStream reset.
	[(nextChar := streamReader next) == quote
		or: [nextChar == nil]]
		whileFalse: [
			nextChar == $&
				ifTrue: [
					streamReader peek == $#
						ifTrue: [
							"character refs are replaced in entity values"
							self nextCharacterReferenceOnWriteStream]
						ifFalse: [
							"general entity refs aren't replaced but must be complete"
							writeStream nextPut: $&.
							self nextNameOnWriteStream.
							writeStream nextPut: (self expectNext: $;)]]
				ifFalse: [
					nextChar == $%
						ifTrue: [
							self isInExternalSubsetOrExternallyDefinedEntity
								ifFalse: [
									self errorInternalDeclarationParameterEntityReference].
							self nextParameterEntityReferenceOnWriteStream]
						ifFalse: [
							nextChar isXMLChar
								ifFalse: [self errorNonCharInEntityValue: nextChar].
							writeStream nextPut: nextChar]]].
	context exitLiteralValue.

	nextChar == quote
		ifFalse: [
			self
				errorExpectedEntityValueEnd: quote
				butGot: nextChar].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 22:15'!
nextExternalEntityDeclaration: aName isParameterEntity: aBoolean
	| publicID systemID notation |

	streamReader peek == $P
		ifTrue: [
			publicID := self nextPublicIDInDeclaration.
			self expectSeparatorsInDeclaration.
			systemID := self nextSystemIDLiteral]
		ifFalse: [
			publicID := ''.
			systemID := self nextSystemIDInDeclaration].
	aBoolean
		ifTrue: [
			self
				skipSeparatorsInDeclaration;
				expectTerminator.
			driver
				handleParameterEntityDeclaration: aName
				publicID: publicID
				systemID: systemID]
		ifFalse: [
			(streamReader peek == $>
				or: [
					self expectSeparatorsInDeclaration.
					streamReader peek == $>])
				ifTrue: [	
					streamReader next.
					notation := '']
				ifFalse: [
					self
						expectNextAll: 'NDATA';
						expectSeparatorsInDeclaration.
					notation := self nextName.
					self
						skipSeparatorsInDeclaration;
						expectTerminator].
			driver
				handleGeneralEntityDeclaration: aName
				publicID: publicID
				systemID: systemID
				ndata: notation].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/3/2016 23:51'!
nextExternalSubsetToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].
	streamReader peek == $%
		ifTrue: [
			streamReader next.
			^ self nextParameterEntityReference].
	streamReader peek == $]
		ifTrue: [
			self hasOpenIncludeSections
				ifFalse: [self errorUnexpectedEndConditionalSection].
			^ self nextEndIncludeSection].
	self nextSubsetMarkupToken.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/8/2016 00:22'!
nextGeneralEntityOrCharacterReferenceOnWriteStream
	"returns the (first) character written to 'writeStream' if any, nil otherwise"
	| generalEntity result |

	generalEntity := self topPushedBackEntity.
	result :=
		streamReader peek == $#
			ifTrue: [self nextCharacterReferenceOnWriteStream]
			ifFalse: [self nextGeneralEntityReferenceOnWriteStream].
	generalEntity
		ifNotNil: [
			(self pushedBackEntities includes: generalEntity)
				ifFalse: [self errorImproperGeneralEntityNesting]].
	^ result.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/8/2016 02:45'!
nextGeneralEntityReferenceOnWriteStream
	"returns the (first) character written to 'writeStream' if any, nil otherwise"
	| name |

	name := self nextName.
	self expectNext: $;.
	PredefinedEntities
		at: name
		ifPresent: [:character |
			driver replacesContentEntityReferences
				ifFalse: [^ self nextUnreplacedGeneralEntityOnWriteStreamNamed: name].
			writeStream nextPut: character.
			^ character].

	(driver handleGeneralEntityReference: name)
		ifNil: [
			(driver replacesContentEntityReferences not
				or: [driver preservesUndeclaredEntityReferences])
				ifTrue: [^ self nextUnreplacedGeneralEntityOnWriteStreamNamed: name].
			driver handleUnreplacedGeneralEntityReference: name]
		ifNotNil: [:entity |
			driver replacesContentEntityReferences
				ifFalse: [^ self nextUnreplacedGeneralEntityOnWriteStreamNamed: name].
			self constructReplacementForGeneralEntity: entity].
	^ nil.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/14/2016 08:32'!
nextIgnoreSectionFromEntity: aParameterEntity
	| openSections |

	self
		expectNextAll: 'IGNORE'
		startingAt: 2. "the 'I' was already read"
	streamReader skipSeparators.
	self expectNext: $[.
	self topPushedBackEntity == aParameterEntity
		ifFalse: [driver handleImproperParameterEntityNesting].

	"From the spec:
	The contents of an ignored conditional section MUST be parsed by
	ignoring all characters after the '[' following the keyword, except
	conditional section starts '<!![' and ends ']]>', until the matching
	conditional section end is found. Parameter entity references MUST
	NOT be recognized in this process."
	openSections := 1.
	[openSections > 0
		and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: ']]>')
				ifTrue: [openSections := openSections - 1]
				ifFalse: [
					(streamReader nextMatchAll: '<!![')
						ifTrue: [openSections := openSections + 1]
						ifFalse: [streamReader next]]].

	openSections > 0
		ifTrue: [self errorUnterminatedIgnoreSections].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:23'!
nextInternalEntityDeclaration: aName isParameterEntity: aBoolean
	| value |

	value := self nextEntityValue.
	self
		skipSeparatorsInDeclaration;
		expectTerminator.

	aBoolean
		ifTrue: [driver handleParameterEntityDeclaration: aName replacement: value]
		ifFalse: [driver handleGeneralEntityDeclaration: aName replacement: value].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:54'!
nextMixedContentListOnWriteStreamFromEntity: aParameterEntity
	| hasElements |

	writeStream nextPutAll: (self expectNextAll: '#PCDATA').

	self skipSeparatorsInDeclaration.
	hasElements := streamReader peek == $|.

	self
		nextNameListOnWriteStream;
		checkNestingOfParameterEntity: aParameterEntity.

	streamReader peek == $*
		ifTrue: [writeStream nextPut: streamReader next]
		ifFalse: [
			hasElements
				ifTrue: [self errorExpectedLiteral: $*]].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/24/2016 20:45'!
nextName
	| nextChar |

	nameStream reset.
	"nil understands isXMLNameStartChar so it does not need to be tested for"
	(nextChar := streamReader next) isXMLNameStartChar
		ifFalse: [self errorExpectedName].
	nameStream nextPut: nextChar.

	"nil understands isXMLNameChar so it does not need to be tested for"
	[(nextChar := streamReader peek) isXMLNameChar]
		whileTrue: [nameStream nextPut: streamReader next].
	^ nameStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:25'!
nextNameListOnWriteStream
	[streamReader peek == $)]
		whileFalse: [
			writeStream nextPut: (self expectNext: $|).
			self
				skipSeparatorsInDeclaration;
				nextNameOnWriteStream;
				skipSeparatorsInDeclaration].
	writeStream nextPut: streamReader next.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/24/2016 20:45'!
nextNameOnWriteStream
	| nextChar |

	"nil understands isXMLNameStartChar so it does not need to be tested for"
	(nextChar := streamReader next) isXMLNameStartChar
		ifFalse: [self errorExpectedName].
	writeStream nextPut: nextChar.

	"nil understands isXMLNameChar so it does not need to be tested for"
	[(nextChar := streamReader peek) isXMLNameChar]
		whileTrue: [writeStream nextPut: streamReader next].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/25/2016 23:40'!
nextNmtokenOnWriteStream
	"matches the Nmtoken production"
	| nextChar |

	"nil understands isXMLNameChar so it does not need to be tested for"
	(nextChar := streamReader next) isXMLNameChar
		ifFalse: [self errorExpectedNmtoken].
	writeStream nextPut: nextChar.

	"nil understands isXMLNameChar so it does not need to be tested for"
	[(nextChar := streamReader peek) isXMLNameChar]
		whileTrue: [writeStream nextPut: streamReader next].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/10/2016 22:36'!
nextNonPIPrologOrContentToken
	streamReader peek == $!!
		ifTrue: [^ self nextDoctypeDeclarationOrComment].
	self
		nextStartContent;
		nextTag.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 22:15'!
nextNotationDeclaration
	| name publicID systemID |

	self
		expectNextAll: 'NOTATION';
		expectSeparatorsInDeclaration.

	name := self nextName.
	self expectSeparatorsInDeclaration.
	streamReader peek == $P
		ifTrue: [
			publicID := self nextPublicIDInDeclaration.
			(streamReader peek == $>
				or: [
					self expectSeparatorsInDeclaration.
					streamReader peek == $>])
				ifFalse: [systemID := self nextSystemIDLiteral]]
		ifFalse: [systemID := self nextSystemIDInDeclaration].
	self
		skipSeparatorsInDeclaration;
		expectTerminator.

	driver
		handleNotationDeclaration: name
		publicID: (publicID ifNil: [''])
		systemID: (systemID ifNil: ['']).! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/7/2016 20:26'!
nextPCDataToken
	| isWhitespace nextChar |

	isWhitespace := true.
	writeStream reset.
	[(nextChar := streamReader peek) == $<
		or: [nextChar == nil]]
		whileFalse: [
			streamReader next == $&
				ifTrue: [
					self nextGeneralEntityOrCharacterReferenceOnWriteStream
						ifNotNil: [:writtenChar |
							isWhitespace
								ifTrue: [isWhitespace := writtenChar isXMLSeparator]]]
				ifFalse: [
					nextChar == $]
						ifTrue: [
							(streamReader nextMatch: $] followedBy: $>)
								ifTrue: [self errorUnexpectedCDataTerminator].
							isWhitespace := false]
						ifFalse: [
							(isWhitespace
								and: [isWhitespace := nextChar isXMLSeparator])
								ifFalse: [
									nextChar isXMLChar
										ifFalse: [self errorNonCharInPCData: nextChar]]].
					writeStream nextPut: nextChar]].
	isWhitespace
		ifTrue: [
			writeStream position > 0
				ifTrue: [driver handleWhitespace: writeStream contents]]
		ifFalse: [driver handlePCData: writeStream contents].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/15/2016 07:37'!
nextPITarget
	| target |

	(target := self nextName) size = 3 "optimization"
		ifTrue: [
			target = 'xml'
				ifTrue: [
					context isXMLDeclarationContext
						ifFalse: [self errorXMLDeclarationNotAtStart]]
				ifFalse: [
					target asLowercase = 'xml'
						ifTrue: [self errorReservedPITarget: target]]].
	^ target.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/24/2016 21:05'!
nextPIValue
	| nextChar |

	writeStream reset.
	[(nextChar := streamReader next) == nil
		or: [nextChar == $?
			and: [streamReader peek == $>]]]
		whileFalse: [
			nextChar isXMLChar
				ifFalse: [self errorNonCharInPIValue: nextChar].
			writeStream nextPut: nextChar].
	self expectTerminator.
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/10/2016 02:52'!
nextParameterEntityReference
	| parameterEntity name |

	parameterEntity := self topPushedBackEntity.
	name := self nextName.
	self
		expectNext: $;;
		checkNestingOfParameterEntity: parameterEntity.

	(driver handleParameterEntityReference: name)
		ifNotNil: [:entity | self constructReplacementForParameterEntity: entity].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/10/2016 02:52'!
nextParameterEntityReferenceInDeclaration
	self isInExternalSubsetOrExternallyDefinedEntity
		ifFalse: [self errorInternalDeclarationParameterEntityReference].
	self nextParameterEntityReference.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/10/2016 02:55'!
nextParameterEntityReferenceOnWriteStream
	| parameterEntity name entity |

	parameterEntity := self topPushedBackEntity.
	name := self nextName.
	self
		expectNext: $;;
		checkNestingOfParameterEntity: parameterEntity.
	
	(entity := driver handleParameterEntityReference: name)
		ifNil: [
			driver preservesUndeclaredEntityReferences
				ifTrue: [
					writeStream
						nextPut: $%;
						nextPutAll: name;
						nextPut: $;].
			^ true].
	self constructReplacementForParameterEntity: entity.
	^ false.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 3/11/2016 11:09'!
nextPrologToken
	streamReader skipSeparators.
	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPI].
	self nextNonPIPrologOrContentToken.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 22:14'!
nextPublicIDInDeclaration
	^ self
		expectNextAll: 'PUBLIC';
		expectSeparatorsInDeclaration;
		nextPublicIDLiteral! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 7/17/2016 04:41'!
nextPublicIDLiteral
	| quote nextChar |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [self errorExpectedPublicIDLiteralStartButGot: quote].

	writeStream reset.
	"it's not enough to test for nil and PublicIDChars inclusion because
	(PublicIDChars includes: $') = true (double quoted pubids can contain
	single quotes), so the terminating quote must be tested for first to
	handle single quoted pubids correctly"
	[(nextChar := streamReader next) == nil
		or: [nextChar == quote
			or: [(PublicIDChars includes: nextChar) not]]]
		whileFalse: [writeStream nextPut: nextChar].

	nextChar == quote
		ifFalse: [
			self
				errorExpectedPublicIDLiteralEnd: quote
				butGot: nextChar].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 4/9/2016 01:53'!
nextStartExternalEntity
	context := context textDeclarationContext.
	driver decodesCharacters
		ifTrue: [streamReader detectCurrentEncoding]
		ifFalse: [streamReader decodeStreamWithConverter: XMLStreamConverter null].
	context nextTokenFrom: self.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/7/2016 02:12'!
nextStartIncludeSectionFromEntity: aParameterEntity
	self
		expectNextAll: 'INCLUDE'
		startingAt: 2. "the 'I' was already read"
	streamReader skipSeparators.
	self expectNext: $[.
	self topPushedBackEntity == aParameterEntity
		ifFalse: [driver handleImproperParameterEntityNesting].
	self openIncludeSections addLast: aParameterEntity.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 3/11/2016 12:03'!
nextSubsetDeclaration
	streamReader peek == $E
		ifTrue: [
			streamReader next.
			streamReader peek == $N
				ifTrue: [^ self nextEntityDeclaration]
				ifFalse: [^ self nextElementDeclaration]].
	streamReader peek == $A
		ifTrue: [^ self nextAttlistDeclaration].
	streamReader peek == $N
		ifTrue: [^ self nextNotationDeclaration].

	self errorExpectedSubsetDeclaration.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/6/2016 05:45'!
nextSubsetMarkupToken
	| parameterEntity |

	self expectNext: $<.
	parameterEntity := self topPushedBackEntity.

	streamReader peek == $?
		ifTrue: [self nextPI]
		ifFalse: [
			self expectNext: $!!.
			streamReader peek == $-
				ifTrue: [self nextComment]
				ifFalse: [
					streamReader peek == $[
						ifTrue: [
							self isInExternalSubsetOrExternallyDefinedEntity
								ifFalse: [self errorInternalSubsetConditionalSection].
							self nextConditionalSectionFromEntity: parameterEntity]
						ifFalse: [self nextSubsetDeclaration]]].

	self checkNestingOfParameterEntity: parameterEntity.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 22:14'!
nextSystemIDInDeclaration
	^ self
		expectNextAll: 'SYSTEM';
		expectSeparatorsInDeclaration;
		nextSystemIDLiteral! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 7/16/2016 09:10'!
nextSystemIDLiteral
	| quote nextChar |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [self errorExpectedSystemIDLiteralStartButGot: quote].

	writeStream reset.
	[(nextChar := streamReader next) == nil
		or: [nextChar == quote]]
		whileFalse: [
			nextChar isXMLChar
				ifFalse: [self errorNonCharInSystemID: nextChar].
			writeStream nextPut: nextChar].

	nextChar == quote
		ifFalse: [
			self
				errorExpectedSystemIDLiteralEnd: quote
				butGot: nextChar].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/16/2016 08:20'!
nextTag
	"performance critical method"
	| tagName attributes nextChar |

	(streamReader peek == $/)
		ifTrue: [^ self nextEndTag].
	tagName := self nextName.

	"nil understands isXMLSeparator so is not tested for"
	[(nextChar := streamReader peek) isXMLSeparator
		and: [
			streamReader
				next;
				skipSeparators.
			((nextChar := streamReader peek) == $>
				or: [nextChar == $/]) not]]
		whileTrue: [| attributeName |
			attributeName := self nextName.
			streamReader skipSeparators.
			(nextChar := streamReader next) == $=
				ifFalse: [
					self errorExpectedAttributeNameValueSeparatorButGot: nextChar].
			streamReader skipSeparators.

			"delay initializing attributes until now to know for sure if it
			needs a non-zero capacity, otherwise default initialize to zero
			later for performance"
			((attributes
				ifNil: [attributes := OrderPreservingStringDictionary new: 10])
				xmlAt: attributeName
				newPut: self nextAttributeValue)
				ifFalse: [self errorDuplicateAttribute: attributeName].

			"the ordering of these tests is optimized"
			(attributeName at: 1) == $x
				ifTrue: [
					driver hasUnhandledReservedAttributes
						ifFalse: [
							attributeName isXMLReservedAttributeName
								ifTrue: [driver haveUnhandledReservedAttributes]]]].

	driver
		handleStartTag: tagName
		attributes:
			(attributes
				ifNil: [OrderPreservingStringDictionary new: 0]).

	nextChar == $/
		ifTrue: [
			streamReader next.
			driver handleEndTag: tagName].
	self expectTerminator.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/7/2016 06:59'!
nextUnreplacedGeneralEntityOnWriteStreamNamed: aName
	driver handleUnreplacedGeneralEntityReference: aName.
	writeStream
		nextPut: $&;
		nextPutAll: aName;
		nextPut: $;.
	^ $&.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/16/2016 08:35'!
nextXMLEncodingAttributeValue
	| quote nextChar |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [self errorExpectedXMLEncodingValueStartButGot: quote].

	((nextChar := streamReader next) notNil
		and: [EncodingNameStartChars includes: nextChar])
		ifFalse: [self errorExpectedEncodingName].
	writeStream
		reset;
		nextPut: nextChar.
	[(nextChar := streamReader next) == nil
		or: [(EncodingNameChars includes: nextChar) not]]
		whileFalse: [writeStream nextPut: nextChar].

	nextChar == quote
		ifFalse: [
			self
				errorExpectedXMLEncodingValueEnd: quote
				butGot: nextChar].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/16/2016 08:39'!
nextXMLStandaloneAttributeValue
	| quote yesOrNo nextChar |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [self errorExpectedXMLStandaloneValueStartButGot: quote].

	yesOrNo := 
		streamReader peek == $y
			ifTrue: [self expectNextAll: 'yes']
			ifFalse: [self expectNextAll: 'no'].

	(nextChar := streamReader next) == quote
		ifFalse: [
			self
				errorExpectedXMLStandaloneValueEnd: quote
				butGot: nextChar].

	^ yesOrNo.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 7/16/2016 08:46'!
nextXMLVersionAttributeValue
	| quote nextChar |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [self errorExpectedXMLVersionValueStartButGot: quote].

	writeStream
		reset;
		nextPut: self expectVersionDigit;
		nextPut: (self expectNext: $.);
		nextPut: self expectVersionDigit.

	[(nextChar := streamReader next) == nil
		or: [(VersionDigitChars includes: nextChar) not]]
		whileFalse: [writeStream nextPut: nextChar].

	nextChar == quote
		ifFalse: [
			self
				errorExpectedXMLVersionValueEnd: quote
				butGot: nextChar].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 4/7/2016 02:23'!
openIncludeSections
	^ openIncludeSections
		ifNil: [openIncludeSections := OrderedCollection new: 5]! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 4/6/2016 06:45'!
pushBackEntity: anEntity onClose: aBlock
	anEntity hasReplacement
		ifTrue: [
			self addPushedBackEntity: anEntity.
			streamReader
				pushBackStream: anEntity replacementStream
				onClose: [
					self removePushedBackEntity: anEntity.
					aBlock
						ifNotNil: [aBlock value]].
			anEntity isExternal
				ifTrue: [self nextStartExternalEntity]]
		ifFalse: [
			aBlock
				ifNotNil: [aBlock value]]! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 3/30/2016 20:25'!
pushedBackEntities
	^ pushedBackEntities ifNil: [pushedBackEntities := OrderedCollection new]! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacing' stamp: 'monty 4/6/2016 06:45'!
removePushedBackEntity: aPushedBackEntity
	self pushedBackEntities remove: aPushedBackEntity.
	aPushedBackEntity isExternal
		ifTrue: [driver popEntityURI: aPushedBackEntity uri].
	^ aPushedBackEntity.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 4/4/2015 21:26'!
skipSeparatorsInDeclaration
	"this can be used to replace references within declarations"

	streamReader skipSeparators.
	[streamReader peek == $%]
		whileTrue: [
			streamReader next.
			self nextParameterEntityReferenceInDeclaration.
			streamReader skipSeparators]! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 4/6/2016 05:45'!
topPushedBackEntity
	"heavily optimized"
	pushedBackEntities == nil
		ifTrue: [^ nil].
	pushedBackEntities size > 0
		ifFalse: [^ nil].
	^ pushedBackEntities at: pushedBackEntities size.! !

Object subclass: #XMLSingleCharacterReadStream
	instanceVariableNames: 'character'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLSingleCharacterReadStream commentStamp: 'monty 1/27/2015 23:39' prior: 0!
A read stream for single characters. (faster and less memory than converting a char to a string and creating a read stream on it)!

!XMLSingleCharacterReadStream class methodsFor: 'instance creation' stamp: 'monty 11/16/2014 04:43'!
on: aCharacter
	^ self new on: aCharacter! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/16/2014 04:44'!
atEnd
	^ character == nil! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/20/2014 04:18'!
basicNext
	^ self next! !

!XMLSingleCharacterReadStream methodsFor: 'closing' stamp: 'monty 4/13/2016 01:51'!
close
	character := nil! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/20/2014 02:02'!
isBinary
	^ false! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/20/2014 01:56'!
isStream
	^ true! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/16/2014 04:44'!
next
	^ character ifNotNil: [:nextChar |
		character := nil.
		nextChar]! !

!XMLSingleCharacterReadStream methodsFor: 'initialization' stamp: 'monty 11/16/2014 04:50'!
on: aCharacter
	character := aCharacter! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/16/2014 04:44'!
peek
	^ character! !

!XMLSingleCharacterReadStream methodsFor: 'printing' stamp: 'monty 4/24/2015 09:03'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: character;
		nextPut: $).! !

Object subclass: #XMLStreamAdapter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLStreamAdapter commentStamp: 'monty 1/26/2016 22:58' prior: 0!
An abstract class for read and write stream adapters.!

XMLStreamAdapter subclass: #XMLReadStreamAdapter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLReadStreamAdapter commentStamp: 'monty 7/30/2016 13:06' prior: 0!
An abstract class for read stream adapters for basic streams that support #next, #peek, and #atEnd.!

XMLReadStreamAdapter subclass: #XMLBinaryReadStreamAdapter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLBinaryReadStreamAdapter commentStamp: 'monty 5/24/2015 21:39' prior: 0!
A read stream adapter for binary streams so that next and peek return characters instead of integers.!

!XMLBinaryReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 7/30/2016 12:51'!
onStreamIfBinary: aStream
	"consider any stream that returns ints to be binary, because not every
	stream implements isBinary and it isn't portable to GS"
	(aStream atEnd not
		and: [aStream peek isInteger])
		ifTrue: [^ self on: aStream]
		ifFalse: [^ aStream]! !

!XMLBinaryReadStreamAdapter methodsFor: 'private' stamp: 'monty 1/27/2016 01:00'!
basicNext
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ Character value: stream next]! !

!XMLBinaryReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/27/2016 19:02'!
next
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ Character value: stream next]! !

!XMLBinaryReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/27/2016 01:01'!
peek
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ Character value: stream peek]! !

XMLReadStreamAdapter subclass: #XMLDecodingReadStreamAdapter
	instanceVariableNames: 'peekChar streamConverter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLDecodingReadStreamAdapter commentStamp: 'monty 7/30/2016 13:07' prior: 0!
A read stream adapter that decodes stream contents using a stream converter. Used to handle encoded external streams.

The class  #on: and #on:streamConverter: instance creation messages accept character and binary streams.!

!XMLDecodingReadStreamAdapter class methodsFor: 'private' stamp: 'monty 7/30/2016 12:53'!
basicOn: aStream streamConverter: aStreamConverter
	^ self new
		on: aStream;
		streamConverter: aStreamConverter! !

!XMLDecodingReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 1/25/2016 22:39'!
on: aStream
	^ self
		on: aStream
		streamConverter: XMLStreamConverter default! !

!XMLDecodingReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 7/30/2016 12:53'!
on: aStream streamConverter: aStreamConverter
	^ self
		basicOn: (XMLBinaryReadStreamAdapter onStreamIfBinary: aStream)
		streamConverter: aStreamConverter! !

!XMLDecodingReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 7/30/2016 12:53'!
onStreamIfBinary: aStream
	| binaryReadStreamAdapter |

	binaryReadStreamAdapter :=
		XMLBinaryReadStreamAdapter onStreamIfBinary: aStream.
	binaryReadStreamAdapter == aStream
		ifTrue: [^ aStream]
		ifFalse: [
			^ self
				basicOn: binaryReadStreamAdapter
				streamConverter: XMLStreamConverter default].! !

!XMLDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 1/27/2016 00:59'!
atEnd
	^ peekChar == nil and: [stream atEnd]! !

!XMLDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 1/31/2016 22:49'!
hasDefaultStreamConverter
	^ streamConverter isDefault! !

!XMLDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 3/5/2016 12:26'!
hasNullStreamConverter
	^ streamConverter isNull! !

!XMLDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 4/1/2016 00:55'!
isXMLConvertingStreamAdapter
	^ true! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/23/2016 16:49'!
next
	peekChar
		ifNil: [^ streamConverter nextFromStream: self]
		ifNotNil: [:nextChar |
			peekChar := nil.
			^ nextChar]! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/27/2016 00:59'!
peek
	^ peekChar
		ifNil: [peekChar := streamConverter nextFromStream: self]! !

!XMLDecodingReadStreamAdapter methodsFor: 'printing' stamp: 'monty 1/23/2016 16:49'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: streamConverter;
		nextPut: $).! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/23/2016 16:49'!
streamConverter
	^ streamConverter! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 3/6/2016 09:08'!
streamConverter: aStreamConverter
	streamConverter :=
		aStreamConverter ifNil: [XMLStreamConverter null]! !

XMLDecodingReadStreamAdapter subclass: #XMLPercentDecodingReadStreamAdapter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLPercentDecodingReadStreamAdapter commentStamp: 'monty 2/9/2016 02:41' prior: 0!
A read stream adapter that decodes streams encoded with  sequences of "%HH" byte hex values for parsing URIs.!

!XMLPercentDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 1/27/2016 00:56'!
atEnd
	^ super atEnd
		or: [stream peek ~~ $%]! !

!XMLPercentDecodingReadStreamAdapter methodsFor: 'private' stamp: 'monty 1/27/2016 18:51'!
basicNext
	| byteValue |

	(stream atEnd not
		and: [stream peek == $%])
		ifFalse: [^ nil].
	stream next.

	byteValue :=
		XMLIntegerReader
			readFrom: stream
			withBase: 16
			upTo: 2.
	^ (byteValue ifNil: [$%]) asCharacter.! !

!XMLReadStreamAdapter methodsFor: 'testing' stamp: 'monty 2/11/2016 03:22'!
atEnd
	^ stream atEnd! !

!XMLReadStreamAdapter methodsFor: 'private' stamp: 'monty 2/11/2016 03:26'!
basicNext
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ stream next]! !

!XMLReadStreamAdapter methodsFor: 'private' stamp: 'monty 2/10/2016 20:04'!
newWriteStream
	^ (self writeStreamCollectionClass new: 128) writeStream! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/11/2016 03:24'!
next
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ stream next]! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 4/5/2016 03:10'!
next: anInteger
	| writeStream i |

	writeStream := self newWriteStream.
	i := 0.
	[self atEnd
		or: [i >= anInteger]]
		whileFalse: [
			writeStream nextPut: self next.
			i := i + 1].
	^ writeStream contents.! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/11/2016 03:25'!
peek
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ stream peek]! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/10/2016 19:01'!
peekFor: anObject
	self peek = anObject
		ifTrue: [
			self next.
			^ true]
		ifFalse: [^ false]! !

!XMLReadStreamAdapter methodsFor: 'positioning' stamp: 'monty 2/14/2016 02:06'!
skip: anInteger
	"only supports skipping forwards"
	1 to: anInteger do: [:i |
		self atEnd
			ifTrue: [^ self].
		self next]! !

!XMLReadStreamAdapter methodsFor: 'positioning' stamp: 'monty 3/12/2016 19:15'!
skipSeparators
	"isXMLWhitespace matches the Squeak/Pharo isSeparator range, so
	it is used here instead of isXMLSeparator to give expected behavior
	when reading non-XML files"
	[self atEnd not
		and: [self peek isXMLWhitespace]]
		whileTrue: [self next]! !

!XMLReadStreamAdapter methodsFor: 'positioning' stamp: 'monty 2/10/2016 19:01'!
skipTo: anObject
	[self atEnd]
		whileFalse: [
			self next = anObject
				ifTrue: [^ true]].
	^ false.! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 3/18/2016 23:08'!
upTo: anObject
	| writeStream next |

	writeStream := self newWriteStream.
	[self atEnd
		or: [(next := self next) = anObject]]
		whileFalse: [writeStream nextPut: next].
	^ writeStream contents.! !

!XMLReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/9/2016 20:49'!
upToEnd
	| writeStream |

	writeStream := self newWriteStream.
	[self atEnd]
		whileFalse: [writeStream nextPut: self next].
	^ writeStream contents.! !

!XMLReadStreamAdapter methodsFor: 'private' stamp: 'monty 2/11/2016 03:23'!
writeStreamCollectionClass
	^ String! !

!XMLStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 1/22/2016 23:46'!
on: aStream
	^ self new on: aStream! !

!XMLStreamAdapter methodsFor: 'closing' stamp: 'monty 1/22/2016 23:46'!
close
	(stream respondsTo: #close)
		ifTrue: [stream close]! !

!XMLStreamAdapter methodsFor: 'testing' stamp: 'monty 1/22/2016 23:46'!
isBinary
	^ false! !

!XMLStreamAdapter methodsFor: 'testing' stamp: 'monty 1/22/2016 23:46'!
isStream
	^ true! !

!XMLStreamAdapter methodsFor: 'initialization' stamp: 'monty 1/22/2016 23:47'!
on: aStream
	stream := aStream! !

!XMLStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/22/2016 23:46'!
stream
	^ stream! !

XMLStreamAdapter subclass: #XMLWriteStreamAdapter
	instanceVariableNames: ''
	classVariableNames: 'CRCharacter LFCharacter SpaceCharacter TabCharacter'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLWriteStreamAdapter commentStamp: 'monty 2/7/2016 06:37' prior: 0!
An abstract class for write stream adapters for basic streams that support nextPut:, nextPutAll:, contents, reset, positioning.!

XMLWriteStreamAdapter subclass: #XMLEncodingWriteStreamAdapter
	instanceVariableNames: 'streamConverter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLEncodingWriteStreamAdapter commentStamp: 'monty 1/26/2016 23:31' prior: 0!
A write stream adapter that encodes stream contents using a stream converter.!

!XMLEncodingWriteStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 1/25/2016 22:39'!
on: aStream
	^ self
		on: aStream
		streamConverter: XMLStreamConverter default! !

!XMLEncodingWriteStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 2/14/2016 04:28'!
on: aStream streamConverter: aStreamConverter
	^ self new
		on: aStream;
		streamConverter: aStreamConverter! !

!XMLEncodingWriteStreamAdapter methodsFor: 'private' stamp: 'monty 1/26/2016 02:35'!
basicNextPut: aCharacter
	stream nextPut: aCharacter! !

!XMLEncodingWriteStreamAdapter methodsFor: 'private' stamp: 'monty 1/29/2016 15:16'!
basicNextPutByte: aByteValue
	self basicNextPut: (Character value: aByteValue)! !

!XMLEncodingWriteStreamAdapter methodsFor: 'testing' stamp: 'monty 2/3/2016 02:54'!
hasDefaultStreamConverter
	^ streamConverter isDefault! !

!XMLEncodingWriteStreamAdapter methodsFor: 'testing' stamp: 'monty 3/5/2016 12:26'!
hasNullStreamConverter
	^ streamConverter isNull! !

!XMLEncodingWriteStreamAdapter methodsFor: 'testing' stamp: 'monty 4/1/2016 00:55'!
isXMLConvertingStreamAdapter
	^ true! !

!XMLEncodingWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/29/2016 20:51'!
nextPut: aCharacter
	streamConverter
		nextPut: aCharacter
		toStream: self.
	^ aCharacter.! !

!XMLEncodingWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/17/2016 12:02'!
nextPutAll: aCollection
	"to:do: for speed"
	1 to: aCollection size do: [:i |
		streamConverter
			nextPut: (aCollection at: i)
			toStream: self].
	^ aCollection.! !

!XMLEncodingWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/11/2016 23:57'!
nextPutByteOrderMark
	streamConverter nextPutByteOrderMarkToStream: self! !

!XMLEncodingWriteStreamAdapter methodsFor: 'printing' stamp: 'monty 1/26/2016 04:03'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		print: streamConverter;
		nextPut: $).! !

!XMLEncodingWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/23/2016 22:21'!
streamConverter
	^ streamConverter! !

!XMLEncodingWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 3/6/2016 09:09'!
streamConverter: aStreamConverter
	streamConverter :=
		aStreamConverter ifNil: [XMLStreamConverter null]! !

XMLEncodingWriteStreamAdapter subclass: #XMLPercentEncodingWriteStreamAdapter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLPercentEncodingWriteStreamAdapter commentStamp: 'monty 2/9/2016 02:41' prior: 0!
A write stream adapter that encodes streams with sequences of "%HH" hex byte values for printing URIs.!

!XMLPercentEncodingWriteStreamAdapter methodsFor: 'private' stamp: 'monty 3/8/2016 15:06'!
basicNextPut: aByteCharacter
	| asciiValue |

	stream nextPut: $%.

	"pad it if needed"
	(asciiValue := aByteCharacter asciiValue) < 16
		ifTrue: [stream nextPut: $0].

	"printOn:base: on GS shows the radix, so printOn:base:showRadix:
	is used instead"
	asciiValue
		printOn: stream
		base: 16
		showRadix: false.! !

!XMLPercentEncodingWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/17/2016 12:01'!
nextPutByteOrderMark
	self shouldNotImplement! !

XMLWriteStreamAdapter subclass: #XMLNodeContentWriteStreamAdapter
	instanceVariableNames: 'lastChar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNodeContentWriteStreamAdapter commentStamp: 'monty 2/14/2016 02:09' prior: 0!
A write stream adapter for writing node content with whitespace insertion if needed between writes.!

!XMLNodeContentWriteStreamAdapter methodsFor: 'private' stamp: 'monty 3/12/2016 19:09'!
insertSpaceBefore: aCharacter
	(lastChar == nil
		or: [lastChar isXMLWhitespace
			or: [aCharacter isXMLWhitespace]])
		ifFalse: [stream nextPut: SpaceCharacter]! !

!XMLNodeContentWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 12/11/2014 10:13'!
nextPut: aCharacter
	self insertSpaceBefore: aCharacter.
	stream nextPut: aCharacter.
	^ lastChar := aCharacter.! !

!XMLNodeContentWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/5/2016 02:25'!
nextPutAll: aString
	aString size > 0 "optimization"
		ifTrue: [
			self insertSpaceBefore: aString first.
			stream nextPutAll: aString.
			lastChar := aString last].
	^ aString.! !

!XMLWriteStreamAdapter class methodsFor: 'class initialization' stamp: 'monty 2/10/2016 20:54'!
initialize
	"self initialize"

	"stored in class vars for faster access"
	SpaceCharacter := Character space.
	TabCharacter := Character tab.
	CRCharacter := Character cr.
	LFCharacter := Character lf.! !

XMLWriteStreamAdapter initialize!

!XMLWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 1/22/2016 23:57'!
contents
	^ stream contents! !

!XMLWriteStreamAdapter methodsFor: 'character writing' stamp: 'monty 2/10/2016 20:54'!
cr
	^ self nextPut: CRCharacter! !

!XMLWriteStreamAdapter methodsFor: 'character writing' stamp: 'monty 2/10/2016 20:54'!
lf
	^ self nextPut: LFCharacter! !

!XMLWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:23'!
nextPut: anObject
	stream nextPut: anObject.
	^ anObject.! !

!XMLWriteStreamAdapter methodsFor: 'accessing' stamp: 'monty 2/11/2016 22:23'!
nextPutAll: aCollection
	stream nextPutAll: aCollection.
	^ aCollection.! !

!XMLWriteStreamAdapter methodsFor: 'positioning' stamp: 'monty 2/7/2016 00:25'!
position
	^ stream position! !

!XMLWriteStreamAdapter methodsFor: 'positioning' stamp: 'monty 2/7/2016 00:25'!
position: anInteger
	stream position: anInteger! !

!XMLWriteStreamAdapter methodsFor: 'positioning' stamp: 'monty 1/22/2016 23:57'!
reset
	stream reset! !

!XMLWriteStreamAdapter methodsFor: 'character writing' stamp: 'monty 2/5/2016 01:57'!
space
	^ self nextPut: SpaceCharacter! !

!XMLWriteStreamAdapter methodsFor: 'character writing' stamp: 'monty 2/10/2016 20:48'!
tab
	^ self nextPut: TabCharacter! !

Object subclass: #XMLStreamConverter
	instanceVariableNames: 'encoding'
	classVariableNames: 'EncodingNamesAndClassesCache PreferredClasses'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

XMLStreamConverter class
	instanceVariableNames: 'encodingNames'!

!XMLStreamConverter commentStamp: 'monty 2/14/2016 03:54' prior: 0!
An abstract class for stream converters.

Subclasses need to implement nextFromStream: to use basicNext on the argument to read byte characters and decode them, and implement nextPut:toStream: to use basicNextPut:/basicNextPutByte: on the second argument to encode the first.

The class basicEncodingNames method should return a basic list of names for the encoding handled by the class, and byteOrderMarkBytes can return the bytes of a BOM if the encoding has one.!

XMLStreamConverter class
	instanceVariableNames: 'encodingNames'!

XMLStreamConverter subclass: #XMLByteStreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLByteStreamConverter commentStamp: 'monty 2/14/2016 03:54' prior: 0!
A stream converter for ISO Latin-1, ASCII,  and other compatible byte character encodings.!

!XMLByteStreamConverter class methodsFor: 'private' stamp: 'monty 3/4/2016 16:20'!
basicEncodingNames
	^ #('ASCII' 'Latin-1' 'Latin1' 'CP819' 'IBM819' 'ISO-8859-1' 'ISO-IR-100' 'L1' 'csISOLatin1')! !

!XMLByteStreamConverter methodsFor: 'decoding' stamp: 'monty 3/9/2016 09:02'!
nextFromStream: aStream
	| character |

	((character := aStream basicNext) notNil
		and: [character asciiValue > 255])
		ifTrue: [self errorBadCharacterValue: character asciiValue].

	^ character.! !

!XMLByteStreamConverter methodsFor: 'encoding' stamp: 'monty 3/9/2016 09:02'!
nextPut: aCharacter toStream: aStream
	aCharacter asciiValue > 255
		ifTrue: [self errorBadCharacterValue: aCharacter asciiValue].

	aStream basicNextPut: aCharacter.! !

XMLStreamConverter subclass: #XMLNullStreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNullStreamConverter commentStamp: 'monty 2/14/2016 02:29' prior: 0!
A stream converter that does no encoding or decoding.!

!XMLNullStreamConverter methodsFor: 'testing' stamp: 'monty 3/5/2016 06:37'!
isNull
	^ true! !

!XMLNullStreamConverter methodsFor: 'decoding' stamp: 'monty 2/14/2016 02:28'!
nextFromStream: aStream
	^ aStream basicNext! !

!XMLNullStreamConverter methodsFor: 'encoding' stamp: 'monty 2/14/2016 02:29'!
nextPut: aCharacter toStream: aStream
	aStream basicNextPut: aCharacter! !

!XMLStreamConverter class methodsFor: 'accessing' stamp: 'monty 3/5/2016 12:57'!
anyEncodingName
	^ self basicEncodingNames
		at: 1
		ifAbsent: ['']! !

!XMLStreamConverter class methodsFor: 'private' stamp: 'monty 4/26/2015 23:57'!
basicEncodingNames
	"encodingNames will have these names + upper/lowercase versions"
	^ #()! !

!XMLStreamConverter class methodsFor: 'accessing' stamp: 'monty 2/11/2016 23:58'!
byteOrderMark
	"avoid asString for portability"
	^ String withAll: (self byteOrderMarkBytes collect: [:each | each asCharacter])! !

!XMLStreamConverter class methodsFor: 'private' stamp: 'monty 2/11/2016 23:58'!
byteOrderMarkBytes
	^ #()! !

!XMLStreamConverter class methodsFor: 'testing' stamp: 'monty 1/25/2016 23:54'!
canConvertEncoding: anEncodingName
	^ self encodingNames includes: anEncodingName! !

!XMLStreamConverter class methodsFor: 'accessing' stamp: 'monty 1/25/2016 23:58'!
classForEncoding: anEncodingName
	^ self encodingNamesAndClassesCache
		at: anEncodingName
		ifAbsentPut: [self findClassForEncoding: anEncodingName]! !

!XMLStreamConverter class methodsFor: 'instance creation' stamp: 'monty 1/25/2016 22:39'!
default
	^ self defaultClass new! !

!XMLStreamConverter class methodsFor: 'accessing' stamp: 'monty 1/25/2016 22:39'!
defaultClass
	^ XMLUTF8StreamConverter! !

!XMLStreamConverter class methodsFor: 'accessing' stamp: 'monty 4/25/2015 16:32'!
encodingNames
	^ encodingNames ifNil: [
		encodingNames := Set new.
		self basicEncodingNames do: [:each |
			encodingNames
				add: each;
				add: each asLowercase;
				add: each asUppercase].
		encodingNames]! !

!XMLStreamConverter class methodsFor: 'accessing' stamp: 'monty 4/25/2015 16:33'!
encodingNames: aSet
	encodingNames := aSet! !

!XMLStreamConverter class methodsFor: 'private' stamp: 'monty 1/25/2016 23:59'!
encodingNamesAndClassesCache
	^ EncodingNamesAndClassesCache! !

!XMLStreamConverter class methodsFor: 'private' stamp: 'monty 3/4/2016 12:48'!
findClassForEncoding: anEncodingName
	"check preferred classes first"
	self preferredClasses do: [:each |
		(each canConvertEncoding: anEncodingName)
			ifTrue: [^ each]].

	(self isNonPreferredClassForEncoding: anEncodingName)
		ifTrue: [^ self].
	self allSubclassesDo: [:each |
		(each isNonPreferredClassForEncoding: anEncodingName)
			ifTrue: [^ each]].

	^ nil.! !

!XMLStreamConverter class methodsFor: 'class initialization' stamp: 'monty 3/6/2016 10:29'!
initialize
	"self initialize"

	self
		initializeEncodingNamesAndClassesCache;
		initializePreferredClasses.! !

XMLStreamConverter initialize!

!XMLStreamConverter class methodsFor: 'class initialization' stamp: 'monty 3/22/2016 04:56'!
initializeEncodingNamesAndClassesCache
	"a class var because it should be shared by all subclasses"
	EncodingNamesAndClassesCache := XMLKeyValueCache maxSize: 128! !

!XMLStreamConverter class methodsFor: 'class initialization' stamp: 'monty 3/22/2016 04:56'!
initializePreferredClasses
	"a class var because it should be shared by all subclasses"
	(PreferredClasses := IdentitySet new)
		addAll: XMLUTFStreamConverter allSubclasses;
		add: XMLByteStreamConverter! !

!XMLStreamConverter class methodsFor: 'testing' stamp: 'monty 1/25/2016 22:39'!
isDefaultClass
	^ self == self defaultClass! !

!XMLStreamConverter class methodsFor: 'testing' stamp: 'monty 3/4/2016 12:48'!
isNonPreferredClassForEncoding: anEncoding
	^ self isPreferredClass not
		and: [self canConvertEncoding: anEncoding]! !

!XMLStreamConverter class methodsFor: 'testing' stamp: 'monty 3/4/2016 12:48'!
isPreferredClass
	^ self preferredClasses includes: self! !

!XMLStreamConverter class methodsFor: 'instance creation' stamp: 'monty 3/6/2016 09:55'!
newForEncoding: anEncodingName
	(self classForEncoding: anEncodingName)
		ifNil: [^ self null]
		ifNotNil: [:converterClass | ^ converterClass new encoding: anEncodingName]! !

!XMLStreamConverter class methodsFor: 'instance creation' stamp: 'monty 3/5/2016 06:34'!
null
	^ XMLNullStreamConverter new! !

!XMLStreamConverter class methodsFor: 'accessing' stamp: 'monty 1/26/2016 00:09'!
preferredClasses
	^ PreferredClasses! !

!XMLStreamConverter methodsFor: 'converting' stamp: 'monty 3/3/2016 21:00'!
asXMLStreamConverter
	^ self! !

!XMLStreamConverter methodsFor: 'accessing' stamp: 'monty 3/4/2016 08:26'!
encoding
	^ encoding ifNil: [encoding := self class anyEncodingName]! !

!XMLStreamConverter methodsFor: 'accessing' stamp: 'monty 3/4/2016 07:42'!
encoding: anEncodingName
	encoding := anEncodingName! !

!XMLStreamConverter methodsFor: 'private' stamp: 'monty 3/10/2016 14:44'!
errorBadCharacterEncoding
	XMLEncodingException
		formatSignal: 'Invalid {1} character encoding'
		with: self encoding! !

!XMLStreamConverter methodsFor: 'private' stamp: 'monty 3/25/2016 01:55'!
errorBadCharacterValue: aValue
	XMLEncodingException
		formatSignal: 'Invalid character 0x{1} for {2} encoding'
		with: aValue printStringHex
		with: self encoding! !

!XMLStreamConverter methodsFor: 'testing' stamp: 'monty 1/25/2016 22:39'!
isDefault
	^ self class isDefaultClass! !

!XMLStreamConverter methodsFor: 'testing' stamp: 'monty 3/5/2016 06:37'!
isNull
	^ false! !

!XMLStreamConverter methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	self subclassResponsibility! !

!XMLStreamConverter methodsFor: 'encoding' stamp: 'monty 1/26/2016 01:48'!
nextPut: aCharacter toStream: aStream
	self subclassResponsibility! !

!XMLStreamConverter methodsFor: 'encoding' stamp: 'monty 2/11/2016 23:59'!
nextPutByteOrderMarkToStream: aStream
	self class byteOrderMarkBytes do: [:each | aStream basicNextPutByte: each]! !

!XMLStreamConverter methodsFor: 'printing' stamp: 'monty 3/4/2016 07:42'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self encoding;
		nextPut: $).! !

XMLStreamConverter subclass: #XMLTextConverterStreamConverter
	instanceVariableNames: 'textConverter'
	classVariableNames: 'EncodingNamesAndTextConverterClassesCache'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLTextConverterStreamConverter commentStamp: 'monty 6/22/2016 18:48' prior: 0!
A stream converter that delegates to a Pharo/Squeak TextConverter subclass. This will only be used on Pharo/Squeak if none of the other XML stream converters support the specified encoding but a TextConverter subclass does.!

!XMLTextConverterStreamConverter class methodsFor: 'testing' stamp: 'monty 1/26/2016 00:08'!
canConvertEncoding: anEncodingName
	^ (self textConverterClassForEncoding: anEncodingName) notNil! !

!XMLTextConverterStreamConverter class methodsFor: 'private' stamp: 'monty 1/26/2016 00:19'!
encodingNamesAndTextConverterClassesCache
	^ EncodingNamesAndTextConverterClassesCache
		ifNil: [
			EncodingNamesAndTextConverterClassesCache :=
				XMLKeyValueCache maxSize: 32]! !

!XMLTextConverterStreamConverter class methodsFor: 'private' stamp: 'monty 7/6/2016 02:32'!
findTextConverterClassForEncoding: anEncoding
	^ (XMLClassFinder
		classNamed: #TextConverter
		ifAbsent: [^ nil])
			defaultConverterClassForEncoding: anEncoding asLowercase! !

!XMLTextConverterStreamConverter class methodsFor: 'accessing' stamp: 'monty 1/26/2016 00:12'!
textConverterClassForEncoding: anEncodingName
	^ self encodingNamesAndTextConverterClassesCache
		at: anEncodingName
		ifAbsentPut: [self findTextConverterClassForEncoding: anEncodingName]! !

!XMLTextConverterStreamConverter methodsFor: 'accessing' stamp: 'monty 3/4/2016 07:41'!
encoding: anEncodingName
	super encoding: anEncodingName.
	self textConverter:
		(self class textConverterClassForEncoding: anEncodingName) new.! !

!XMLTextConverterStreamConverter methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	^ textConverter nextFromStream: aStream! !

!XMLTextConverterStreamConverter methodsFor: 'encoding' stamp: 'monty 1/26/2016 19:08'!
nextPut: aCharacter toStream: aStream
	textConverter
		nextPut: aCharacter
		toStream: aStream! !

!XMLTextConverterStreamConverter methodsFor: 'accessing' stamp: 'monty 11/12/2014 01:55'!
textConverter
	^ textConverter! !

!XMLTextConverterStreamConverter methodsFor: 'accessing' stamp: 'monty 11/12/2014 01:54'!
textConverter: aTextConverter
	textConverter := aTextConverter! !

XMLStreamConverter subclass: #XMLUTFStreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTFStreamConverter commentStamp: 'monty 2/14/2016 02:13' prior: 0!
An abstract class for Unicode stream converters.!

XMLUTFStreamConverter subclass: #XMLUTF16StreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16StreamConverter commentStamp: 'monty 2/14/2016 02:13' prior: 0!
An abstract class for UTF-16 stream converters.!

XMLUTF16StreamConverter subclass: #XMLUTF16BigEndianStreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16BigEndianStreamConverter commentStamp: 'monty 6/22/2016 22:12' prior: 0!
A stream converter for big endian UTF-16 streams, and the default converter class used when a UTF-16 stream converter is created with newForEncoding: or asXMLStreamConverter and no endian is specified by the encoding name.!

!XMLUTF16BigEndianStreamConverter class methodsFor: 'private' stamp: 'monty 4/24/2015 11:10'!
basicEncodingNames
	^ #('UTF-16' 'UTF-16BE' 'UTF-16-BE' 'UTF16' 'UTF16-BE' 'UTF16BE')! !

!XMLUTF16BigEndianStreamConverter class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFE 16rFF)! !

!XMLUTF16BigEndianStreamConverter methodsFor: 'private' stamp: 'monty 11/11/2014 14:28'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	^ (aFirstByte bitShift: 8) + aSecondByte! !

!XMLUTF16BigEndianStreamConverter methodsFor: 'testing' stamp: 'monty 11/11/2014 14:47'!
isBigEndian
	^ true! !

!XMLUTF16BigEndianStreamConverter methodsFor: 'private' stamp: 'monty 1/29/2016 00:02'!
nextPutFirstByte: aFirstByte secondByte: aSecondByte toStream: aStream
	aStream
		basicNextPutByte: aFirstByte;
		basicNextPutByte: aSecondByte! !

XMLUTF16StreamConverter subclass: #XMLUTF16LittleEndianStreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16LittleEndianStreamConverter commentStamp: 'monty 2/14/2016 02:14' prior: 0!
A stream converter for little endian UTF-16 streams.!

!XMLUTF16LittleEndianStreamConverter class methodsFor: 'private' stamp: 'monty 4/24/2015 11:10'!
basicEncodingNames
	^ #('UTF-16LE' 'UTF-16-LE' 'UTF16-LE' 'UTF16LE')! !

!XMLUTF16LittleEndianStreamConverter class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFF 16rFE)! !

!XMLUTF16LittleEndianStreamConverter methodsFor: 'private' stamp: 'monty 11/11/2014 14:28'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	^ (aSecondByte bitShift: 8) + aFirstByte! !

!XMLUTF16LittleEndianStreamConverter methodsFor: 'testing' stamp: 'monty 11/11/2014 14:47'!
isBigEndian
	^ false! !

!XMLUTF16LittleEndianStreamConverter methodsFor: 'private' stamp: 'monty 1/29/2016 00:02'!
nextPutFirstByte: aFirstByte secondByte: aSecondByte toStream: aStream
	aStream
		basicNextPutByte: aSecondByte;
		basicNextPutByte: aFirstByte! !

!XMLUTF16StreamConverter methodsFor: 'private' stamp: 'monty 11/11/2014 14:44'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	self subclassResponsibility! !

!XMLUTF16StreamConverter methodsFor: 'testing' stamp: 'monty 11/11/2014 14:46'!
isBigEndian
	self subclassResponsibility! !

!XMLUTF16StreamConverter methodsFor: 'decoding' stamp: 'monty 3/9/2016 09:03'!
nextFromStream: aStream
	| codePoint |

	codePoint :=
		self
			codePointFromFirstByte:
				(aStream basicNext
					ifNil: [^ nil]) asciiValue
			secondByte:
				(aStream basicNext
					ifNil: [self errorBadCharacterEncoding]) asciiValue.

	(codePoint < 16rD800 or: [codePoint > 16rDBFF])
		ifTrue: [^ self characterFromCodePoint: codePoint]
		ifFalse: [
			^ self
				surrogatePairFromStream: aStream
				withHighSurrogate: codePoint]! !

!XMLUTF16StreamConverter methodsFor: 'encoding' stamp: 'monty 3/8/2016 16:21'!
nextPut: aCharacter toStream: aStream
	| codePoint |

	(codePoint := self codePointFromCharacter: aCharacter) < 16r10000
		ifTrue: [
			self
				nextPutFirstByte: ((codePoint bitShift: -8) bitAnd: 16rFF)
				secondByte: (codePoint bitAnd: 16rFF)
				toStream: aStream]
		ifFalse: [| difference |
			difference := codePoint - 16r10000.
			"the 10 high-order bits (2 + 16rD8 then the next 8 high-order bits)"
			self
				nextPutFirstByte: 16rD8 + ((difference bitShift: -18) bitAnd: 16r03)
				secondByte: ((difference bitShift: -10) bitAnd: 16rFF)
				toStream: aStream.
			"the 10 low-order bits (2 + 16rDC then the next 8 low-order bits)"
			self
				nextPutFirstByte: 16rDC + ((difference bitShift: -8) bitAnd: 16r03)
				secondByte: (difference bitAnd: 16rFF)
				toStream: aStream]! !

!XMLUTF16StreamConverter methodsFor: 'private' stamp: 'monty 1/26/2016 23:25'!
nextPutFirstByte: aFirstByte secondByte: aSecondByte toStream: aStream
	self subclassResponsibility! !

!XMLUTF16StreamConverter methodsFor: 'private' stamp: 'monty 3/9/2016 09:03'!
surrogatePairFromStream: aStream withHighSurrogate: aHighSurrogate
	| lowSurrogate |

	lowSurrogate :=
		self
			codePointFromFirstByte:
				(aStream basicNext
					ifNil: [self errorBadCharacterEncoding]) asciiValue
			secondByte:
				(aStream basicNext
					ifNil: [self errorBadCharacterEncoding]) asciiValue.

	^ self characterFromCodePoint:
		((aHighSurrogate bitAnd: 16r3FF) bitShift: 10) +
		(lowSurrogate bitAnd: 16r3FF) +
		16r10000.! !

XMLUTFStreamConverter subclass: #XMLUTF32StreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32StreamConverter commentStamp: 'monty 2/14/2016 02:13' prior: 0!
An abstract class for UTF-32 stream converters.!

XMLUTF32StreamConverter subclass: #XMLUTF32BigEndianStreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32BigEndianStreamConverter commentStamp: 'monty 6/22/2016 22:12' prior: 0!
A stream converter for big endian UTF-32 streams, and the default converter class used when a UTF-32 stream converter is created with newForEncoding: or asXMLStreamConverter and no endian is specified by the encoding name.!

!XMLUTF32BigEndianStreamConverter class methodsFor: 'private' stamp: 'monty 4/24/2015 11:11'!
basicEncodingNames
	^ #('UTF-32' 'UTF-32BE' 'UTF-32-BE' 'UTF32' 'UTF32-BE' 'UTF32BE')! !

!XMLUTF32BigEndianStreamConverter class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16r00 16r00 16rFE 16rFF)! !

!XMLUTF32BigEndianStreamConverter methodsFor: 'private' stamp: 'monty 3/8/2016 16:21'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	^ self characterFromCodePoint:
		(aFirstByte bitShift: 24) +
		(aSecondByte bitShift: 16) +
		(aThirdByte bitShift: 8) +
		aFourthByte! !

!XMLUTF32BigEndianStreamConverter methodsFor: 'testing' stamp: 'monty 11/11/2014 14:49'!
isBigEndian
	^ true! !

!XMLUTF32BigEndianStreamConverter methodsFor: 'private' stamp: 'monty 1/29/2016 00:00'!
nextPutFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte toStream: aStream
	aStream
		basicNextPutByte: aFirstByte;
		basicNextPutByte: aSecondByte;
		basicNextPutByte: aThirdByte;
		basicNextPutByte: aFourthByte! !

XMLUTF32StreamConverter subclass: #XMLUTF32LittleEndianStreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32LittleEndianStreamConverter commentStamp: 'monty 2/14/2016 02:14' prior: 0!
A stream converter for little endian UTF-32 streams.!

!XMLUTF32LittleEndianStreamConverter class methodsFor: 'private' stamp: 'monty 4/24/2015 11:12'!
basicEncodingNames
	^ #('UTF-32LE' 'UTF-32-LE' 'UTF32-LE' 'UTF32LE')! !

!XMLUTF32LittleEndianStreamConverter class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFF 16rFE 16r00 16r00)! !

!XMLUTF32LittleEndianStreamConverter methodsFor: 'private' stamp: 'monty 3/8/2016 16:21'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	^ self characterFromCodePoint:
		(aFourthByte bitShift: 24) +
		(aThirdByte bitShift: 16) +
		(aSecondByte bitShift: 8) +
		aFirstByte! !

!XMLUTF32LittleEndianStreamConverter methodsFor: 'testing' stamp: 'monty 11/11/2014 14:50'!
isBigEndian
	^ false! !

!XMLUTF32LittleEndianStreamConverter methodsFor: 'private' stamp: 'monty 1/29/2016 00:01'!
nextPutFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte toStream: aStream
	aStream
		basicNextPutByte: aFourthByte;
		basicNextPutByte: aThirdByte;
		basicNextPutByte: aSecondByte;
		basicNextPutByte: aFirstByte! !

!XMLUTF32StreamConverter methodsFor: 'private' stamp: 'monty 11/11/2014 14:58'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	self subclassResponsibility! !

!XMLUTF32StreamConverter methodsFor: 'testing' stamp: 'monty 11/11/2014 14:49'!
isBigEndian
	self subclassResponsibility! !

!XMLUTF32StreamConverter methodsFor: 'decoding' stamp: 'monty 3/9/2016 09:04'!
nextFromStream: aStream
	^ self
		codePointFromFirstByte:
			(aStream basicNext
				ifNil: [^ nil]) asciiValue
		secondByte:
			(aStream basicNext
				ifNil: [self errorBadCharacterEncoding]) asciiValue
		thirdByte:
			(aStream basicNext
				ifNil: [self errorBadCharacterEncoding]) asciiValue
		fourthByte:
			(aStream basicNext
				ifNil: [self errorBadCharacterEncoding]) asciiValue! !

!XMLUTF32StreamConverter methodsFor: 'encoding' stamp: 'monty 3/8/2016 16:21'!
nextPut: aCharacter toStream: aStream
	| codePoint |

	codePoint := self codePointFromCharacter: aCharacter.
	self
		nextPutFirstByte: ((codePoint bitShift: -24) bitAnd: 16rFF)
		secondByte: ((codePoint bitShift: -16) bitAnd: 16rFF)
		thirdByte: ((codePoint bitShift: -8) bitAnd: 16rFF)
		fourthByte: (codePoint bitAnd: 16rFF)
		toStream: aStream.! !

!XMLUTF32StreamConverter methodsFor: 'private' stamp: 'monty 1/26/2016 23:23'!
nextPutFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte toStream: aStream
	self subclassResponsibility! !

XMLUTFStreamConverter subclass: #XMLUTF8StreamConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF8StreamConverter commentStamp: 'monty 2/14/2016 02:14' prior: 0!
A stream converter for UTF-8 streams. (accepts overlong encodings)!

!XMLUTF8StreamConverter class methodsFor: 'private' stamp: 'monty 4/24/2015 11:12'!
basicEncodingNames
	^ #('UTF-8' 'UTF8')! !

!XMLUTF8StreamConverter class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rEF 16rBB 16rBF)! !

!XMLUTF8StreamConverter methodsFor: 'decoding' stamp: 'monty 3/9/2016 09:04'!
nextFromStream: aStream
	| firstChar firstByte secondByte thirdByte |

	((firstChar := aStream basicNext) == nil
		or: [(firstByte := firstChar asciiValue) < 16r80]) "single byte character"
		ifTrue: [^ firstChar].

	firstByte < 16rC0 "second bit must be set"
		ifTrue: [self errorBadCharacterEncoding].

	secondByte :=
		(aStream basicNext
			ifNil: [self errorBadCharacterEncoding]) asciiValue.
	(firstByte bitAnd: 16rF0) < 16rE0 "two bytes"
		ifTrue: [
			^ self characterFromCodePoint: 
				((firstByte bitAnd: 16r1F) bitShift: 6) +
				(secondByte bitAnd: 16r3F)].

	thirdByte :=
		(aStream basicNext
			ifNil: [self errorBadCharacterEncoding]) asciiValue.
	(firstByte bitAnd: 16rF0) = 16rE0 "three bytes"
		ifTrue: [
			^ self characterFromCodePoint: 
				((firstByte bitAnd: 16r0F) bitShift: 12) +
				((secondByte bitAnd: 16r3F) bitShift: 6) +
				(thirdByte bitAnd: 16r3F)].

	^ self characterFromCodePoint: "four"
		((firstByte bitAnd: 16r07) bitShift: 18) +
		((secondByte bitAnd: 16r3F) bitShift: 12) +
		((thirdByte bitAnd: 16r3F) bitShift: 6) +
		((aStream basicNext
			ifNil: [self errorBadCharacterEncoding]) asciiValue bitAnd: 16r3F).! !

!XMLUTF8StreamConverter methodsFor: 'encoding' stamp: 'monty 3/8/2016 16:21'!
nextPut: aCharacter toStream: aStream
	| codePoint |

	(codePoint := self codePointFromCharacter: aCharacter) < 16r80
		ifTrue: [^ aStream basicNextPut: aCharacter].

	codePoint < 16r0800
		ifTrue: [
			^ aStream
				basicNextPutByte: 16rC0 + (codePoint bitShift: -6);
				basicNextPutByte: 16r80 + (codePoint bitAnd: 16r003F)].

	codePoint < 16r010000
		ifTrue: [
			^ aStream
				basicNextPutByte: 16rE0 + (codePoint bitShift: -12);
				basicNextPutByte: 16r80 + ((codePoint bitShift: -6) bitAnd: 16r3F);
				basicNextPutByte: 16r80 + (codePoint bitAnd: 16r3F)].

	aStream
		basicNextPutByte: 16rF0 + (codePoint bitShift: -18);
		basicNextPutByte: 16r80 + ((codePoint bitShift: -12) bitAnd: 16r3F);
		basicNextPutByte: 16r80 + ((codePoint bitShift: -6) bitAnd: 16r3F);
		basicNextPutByte: 16r80 + (codePoint bitAnd: 16r3F).! !

!XMLUTFStreamConverter methodsFor: 'converting' stamp: 'monty 3/23/2016 23:27'!
characterFromCodePoint: aCodePoint
	aCodePoint > 16r10FFFF "max code point"
		ifTrue: [self errorBadCharacterValue: aCodePoint].
	^ Character value: aCodePoint.! !

!XMLUTFStreamConverter methodsFor: 'converting' stamp: 'monty 3/23/2016 23:27'!
codePointFromCharacter: aCharacter
	| codePoint |

	(codePoint := aCharacter asciiValue) > 16r10FFFF "max code point"
		ifTrue: [self errorBadCharacterValue: codePoint].
	^ codePoint.! !

Object subclass: #XMLTokenContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLTokenContext commentStamp: 'monty 7/13/2016 03:04' prior: 0!
This is an abstract class for token contexts to control tokenization. Subclasses must implement #nextTokenFrom: to send a tokenizing message to the argument tokenizer.!

XMLTokenContext subclass: #XMLContentContext
	instanceVariableNames: 'isLiteralValueContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLContentContext commentStamp: 'monty 7/28/2016 13:05' prior: 0!
A context for content tokens, everything from the first stat tag to the end of the document.!

!XMLContentContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
enterLiteralValue
	isLiteralValueContext := true! !

!XMLContentContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
exitLiteralValue
	isLiteralValueContext := false! !

!XMLContentContext methodsFor: 'testing' stamp: 'monty 1/4/2015 04:32'!
isContentContext
	^ true! !

!XMLContentContext methodsFor: 'testing' stamp: 'monty 4/6/2015 02:51'!
isLiteralValueContext
	^ isLiteralValueContext ifNil: [isLiteralValueContext := false]! !

!XMLContentContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 15:41'!
nextTokenFrom: aTokenizer
	aTokenizer nextContentToken! !

XMLTokenContext subclass: #XMLExternalSubsetContext
	instanceVariableNames: 'isLiteralValueContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLExternalSubsetContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for external subset tokens.!

!XMLExternalSubsetContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
enterLiteralValue
	isLiteralValueContext := true! !

!XMLExternalSubsetContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
exitLiteralValue
	isLiteralValueContext := false! !

!XMLExternalSubsetContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isExternalSubsetContext
	^ true! !

!XMLExternalSubsetContext methodsFor: 'testing' stamp: 'monty 4/6/2015 02:53'!
isLiteralValueContext
	^ isLiteralValueContext ifNil: [isLiteralValueContext := false]! !

!XMLExternalSubsetContext methodsFor: 'tokenizing' stamp: 'monty 3/11/2016 10:50'!
nextTokenFrom: aTokenizer
	aTokenizer nextExternalSubsetToken! !

XMLTokenContext subclass: #XMLPrologContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLPrologContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for prolog tokens.!

XMLPrologContext subclass: #XMLDoctypeDeclarationContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLDoctypeDeclarationContext commentStamp: 'monty 4/6/2015 09:20' prior: 0!
A context for <!!DOCTYPE ...> and internal subset tokens.!

!XMLDoctypeDeclarationContext methodsFor: 'testing' stamp: 'monty 4/6/2015 04:31'!
isDoctypeDeclarationContext
	^ true! !

XMLDoctypeDeclarationContext subclass: #XMLInternalSubsetContext
	instanceVariableNames: 'isLiteralValueContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLInternalSubsetContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for internal subset tokens.!

!XMLInternalSubsetContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
enterLiteralValue
	isLiteralValueContext := true! !

!XMLInternalSubsetContext methodsFor: 'changing' stamp: 'monty 4/6/2015 02:51'!
exitLiteralValue
	isLiteralValueContext := false! !

!XMLInternalSubsetContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isInternalSubsetContext
	^ true! !

!XMLInternalSubsetContext methodsFor: 'testing' stamp: 'monty 4/6/2015 02:53'!
isLiteralValueContext
	^ isLiteralValueContext ifNil: [isLiteralValueContext := false]! !

!XMLInternalSubsetContext methodsFor: 'tokenizing' stamp: 'monty 3/11/2016 10:50'!
nextTokenFrom: aTokenizer
	aTokenizer nextInternalSubsetToken! !

XMLPrologContext subclass: #XMLPostDoctypeDeclarationContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLPostDoctypeDeclarationContext commentStamp: 'monty 4/6/2015 09:20' prior: 0!
A context for parsing prolog tokens after <!!DOCTYPE ...> declarations.!

!XMLPostDoctypeDeclarationContext methodsFor: 'testing' stamp: 'monty 4/6/2015 04:36'!
isPostDoctypeDeclarationContext
	^ true! !

!XMLPrologContext methodsFor: 'testing' stamp: 'monty 3/7/2016 20:03'!
canTerminateDocument
	^ false! !

!XMLPrologContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isPrologContext
	^ true! !

!XMLPrologContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 15:41'!
nextTokenFrom: aTokenizer
	aTokenizer nextPrologToken! !

XMLPrologContext subclass: #XMLXMLDeclarationContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLXMLDeclarationContext commentStamp: 'monty 4/6/2015 09:19' prior: 0!
A context for parsing the <?xml ...?> token.!

!XMLXMLDeclarationContext methodsFor: 'testing' stamp: 'monty 4/6/2015 01:38'!
isXMLDeclarationContext
	^ true! !

!XMLXMLDeclarationContext methodsFor: 'tokenizing' stamp: 'monty 3/10/2016 22:10'!
nextTokenFrom: aTokenizer
	aTokenizer nextXMLDeclarationOrPrologToken! !

!XMLXMLDeclarationContext methodsFor: 'testing' stamp: 'monty 5/3/2015 17:02'!
supportsEncodingDetection
	^ true! !

XMLTokenContext subclass: #XMLTerminatedContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLTerminatedContext commentStamp: 'monty 8/16/2016 22:36' prior: 0!
A context for when tokenizing has stopped.!

!XMLTerminatedContext methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
isTerminatedContext
	^ true! !

!XMLTerminatedContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:27'!
nextTokenFrom: aTokenizer
	aTokenizer nextEndDocument! !

XMLTokenContext subclass: #XMLTextDeclarationContext
	instanceVariableNames: 'previousContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLTextDeclarationContext commentStamp: 'monty 7/28/2016 13:09' prior: 0!
A context for <?xml ...?> text declaration tokens at the start of external parsed entities and the external DTD subset.!

!XMLTextDeclarationContext class methodsFor: 'instance creation' stamp: 'monty 11/9/2014 17:13'!
previousContext: aTokenContex
	^ self new previousContext: aTokenContex! !

!XMLTextDeclarationContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 17:15'!
nextTokenFrom: aTokenizer
	aTokenizer nextXMLTextDeclaration! !

!XMLTextDeclarationContext methodsFor: 'accessing' stamp: 'monty 4/13/2015 01:29'!
previousContext
	^ previousContext ifNil: [self externalSubsetContext]! !

!XMLTextDeclarationContext methodsFor: 'accessing' stamp: 'monty 11/9/2014 17:13'!
previousContext: aTokenContex
	previousContext := aTokenContex! !

!XMLTextDeclarationContext methodsFor: 'testing' stamp: 'monty 5/3/2015 17:02'!
supportsEncodingDetection
	^ true! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/11/2015 00:00'!
canTerminateDocument
	^ true! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
contentContext
	^ XMLContentContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 4/6/2015 04:31'!
doctypeDeclarationContext
	^ XMLDoctypeDeclarationContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
externalSubsetContext
	^ XMLExternalSubsetContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 3/1/2015 17:20'!
internalSubsetContext
	^ XMLInternalSubsetContext new! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 1/4/2015 04:32'!
isContentContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/6/2015 04:31'!
isDoctypeDeclarationContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isExternalSubsetContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:15'!
isInitializedContext
	^ true! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isInternalSubsetContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/6/2015 02:27'!
isLiteralValueContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/6/2015 04:37'!
isPostDoctypeDeclarationContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isPrologContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
isTerminatedContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 4/6/2015 01:38'!
isXMLDeclarationContext
	^ false! !

!XMLTokenContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:40'!
nextTokenFrom: aTokenizer
	self subclassResponsibility! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 4/6/2015 04:37'!
postDoctypeDeclarationContext
	^ XMLPostDoctypeDeclarationContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
prologContext
	^ XMLPrologContext new! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 5/3/2015 17:02'!
supportsEncodingDetection
	^ false! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:03'!
terminatedContext
	^ XMLTerminatedContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:01'!
textDeclarationContext
	^ XMLTextDeclarationContext previousContext: self! !

XMLTokenContext subclass: #XMLUnitializedContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Drivers'!

!XMLUnitializedContext commentStamp: 'monty 8/16/2016 22:35' prior: 0!
A context for before tokenizing has started.!

!XMLUnitializedContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:15'!
isInitializedContext
	^ false! !

!XMLUnitializedContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:27'!
nextTokenFrom: aTokenizer
	aTokenizer nextStartDocument! !

Object subclass: #XMLURI
	instanceVariableNames: 'uriString scheme hasAuthorityPrefix rawUserInfo rawHost port rawPathSegments rawQuery rawFragment'
	classVariableNames: 'UnreservedCharacters'
	poolDictionaries: ''
	category: 'XML-Parser-URI'!

!XMLURI commentStamp: 'monty 1/8/2016 11:14' prior: 0!
A class for XML URIs. Instances are immutable and support combining with other XMLURIs or URI strings using / (to resolve relative URIs in the context of the receiver), accessing URI components, creating copies with replaced components, and retrieving the contents of http:// and file:// URLs using XMLHTTPRequest and XMLFileHandle.

Uses XMLURIParser to parse URIs lazily (for example, when a URI compoenent is accessed with an "accessing" message) when created from a string and eagerly when created from a stream.!

!XMLURI class methodsFor: 'instance creation' stamp: 'monty 3/17/2016 22:48'!
fromStream: aStream
	"streams must be parsed eagerly (because the stream might be read from
	or closed after), unlike strings"
	^ self basicNew initialize parseURIStream: aStream! !

!XMLURI class methodsFor: 'instance creation' stamp: 'monty 1/3/2016 20:55'!
fromString: aString
	^ self basicNew initialize uriString: aString! !

!XMLURI class methodsFor: 'class initialization' stamp: 'monty 3/28/2016 01:14'!
initialize
	"self initialize"

	self initializeUnreservedCharacters.! !

XMLURI initialize!

!XMLURI class methodsFor: 'class initialization' stamp: 'monty 3/28/2016 01:14'!
initializeUnreservedCharacters
	(UnreservedCharacters := BitmapCharacterSet new)
		addRangeFrom: $a to: $z;
		addRangeFrom: $A to: $Z;
		addRangeFrom: $0 to: $9;
		addAll: '-._~'! !

!XMLURI class methodsFor: 'instance creation' stamp: 'monty 2/9/2015 06:40'!
new
	^ self fromString: ''! !

!XMLURI class methodsFor: 'instance creation' stamp: 'monty 1/5/2016 09:07'!
readFrom: aStringOrStream
	aStringOrStream isStream
		ifTrue: [^ self fromStream: aStringOrStream]
		ifFalse: [^ self fromString: aStringOrStream]! !

!XMLURI methodsFor: 'combining' stamp: 'monty 3/31/2016 04:18'!
/ aURIOrURIString
	"Returns the argument URI if it is absolute or the argument resolved in the
	context of the receiver if it is relative, to do relative path resolution
	or replace hosts, query strings, and other components."

	^ self combinedWith: aURIOrURIString! !

!XMLURI methodsFor: 'comparing' stamp: 'monty 3/30/2016 20:32'!
= anObject
	self == anObject
		ifTrue: [^ true].

	self species == anObject species
		ifFalse: [^ false].

	self scheme = anObject scheme
		ifFalse: [^ false].
	((self hasUserInfo = anObject hasUserInfo)
		and: [self userInfo = anObject userInfo])
		ifFalse: [^ false].
	((self hasHost = anObject hasHost)
		and: [self host = anObject host])
		ifFalse: [^ false].
	self port = anObject port
		ifFalse: [^ false].
	(self hasPath = anObject hasPath
		and: [self pathSegments = anObject pathSegments])
		ifFalse: [^ false].
	((self hasQuery = anObject hasQuery)
		and: [self query = anObject query])
		ifFalse: [^ false].
	((self hasFragment = anObject hasFragment)
		and: [self fragment = anObject fragment])
		ifFalse: [^ false].

	^ true.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 05:50'!
addRawDotPathSegment
	self hasNonDirectoryPath
		ifTrue: [self haveDirectoryPath]! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 05:50'!
addRawDoubleDotPathSegment
	self hasPath
		ifFalse: [^ self].

	self hasRootPath
		ifTrue: [self removeRawPathSegments]
		ifFalse: [| hadAbsolutePath |
			hadAbsolutePath := self hasAbsolutePath.
			self hasDirectoryPath
				ifTrue: [self removeLastRawPathSegment].
			self hasPath
				ifTrue: [
					self removeLastRawPathSegment.
					(self hasPath
						or: [hadAbsolutePath])
						ifTrue: [self haveDirectoryPath]]].! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 05:50'!
addRawPathSegment: aString
	aString = '.'
		ifTrue: [^ self addRawDotPathSegment].
	aString = '..'
		ifTrue: [^ self addRawDoubleDotPathSegment].

	self hasPath
		ifTrue: [
			(self hasDirectoryPath
				and: [self rawPathSegments size > 1])
				ifTrue: [self removeLastRawPathSegment]]
		ifFalse: [
			self hasHost
				ifTrue: [self rawPathSegments addLast: '']].

	self rawPathSegments addLast: aString.! !

!XMLURI methodsFor: 'private' stamp: 'monty 4/2/2016 01:15'!
addRawPathSegmentsFrom: aURI
	"copy before modifying"
	self rawPathSegments: self rawPathSegments copy.

	aURI hasAbsolutePath
		ifTrue: [self removeRawPathSegments]
		ifFalse: [
			self hasNonDirectoryPath
				ifTrue: [
					self removeLastRawPathSegment.
					self hasPath
						ifTrue: [self haveDirectoryPath]]].

	aURI rawPathSegments do: [:each | self addRawPathSegment: each].! !

!XMLURI methodsFor: 'converting' stamp: 'monty 2/13/2016 00:36'!
asRetrievableResource
	^ self isLocalURI
		ifTrue: [self path asXMLFileHandle]
		ifFalse: [self asString asXMLHTTPRequest]! !

!XMLURI methodsFor: 'converting' stamp: 'monty 4/6/2015 08:05'!
asString
	"for compatibility with GS where Object>>asString isn't defined"
	^ self printString! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/10/2014 00:56'!
asXMLURI
	^ self! !

!XMLURI methodsFor: 'combining' stamp: 'monty 3/31/2016 04:18'!
combinedWith: aURIOrURIString
	"Returns the argument URI if it is absolute or the argument resolved in the
	context of the receiver if it is relative, to do relative path resolution
	or replace hosts, query strings, and other components."

	| uriToCombineWith |

	(uriToCombineWith := aURIOrURIString asXMLURI) isEmpty
		ifFalse: [
			(uriToCombineWith isRelative)
				ifTrue: [^ self combinedWithRelativeURI: uriToCombineWith]
				ifFalse: [^ uriToCombineWith]]! !

!XMLURI methodsFor: 'private' stamp: 'monty 4/2/2016 01:14'!
combinedWithRelativeURI: aRelativeURI
	| copy |

	copy := self copy.
	aRelativeURI hasUserInfo
		ifTrue: [
			^ copy
				rawUserInfo: aRelativeURI rawUserInfo;
				rawHost: aRelativeURI rawHost;
				port: aRelativeURI port;
				rawPathSegments: aRelativeURI rawPathSegments;
				rawQuery: aRelativeURI rawQuery;
				rawFragment: aRelativeURI rawFragment].
	aRelativeURI hasHost
		ifTrue: [
			^ copy
				rawHost: aRelativeURI rawHost;
				port: aRelativeURI port;
				rawPathSegments: aRelativeURI rawPathSegments;
				rawQuery: aRelativeURI rawQuery;
				rawFragment: aRelativeURI rawFragment].
	aRelativeURI hasPort
		ifTrue: [
			^ copy
				port: aRelativeURI port;
				rawPathSegments: aRelativeURI rawPathSegments;
				rawQuery: aRelativeURI rawQuery;
				rawFragment: aRelativeURI rawFragment].
	aRelativeURI hasPath
		ifTrue: [
			^ copy
				addRawPathSegmentsFrom: aRelativeURI;
				rawQuery: aRelativeURI rawQuery;
				rawFragment: aRelativeURI rawFragment].
	aRelativeURI hasQuery
		ifTrue: [
			^ copy
				rawQuery: aRelativeURI rawQuery;
				rawFragment: aRelativeURI rawFragment].
	aRelativeURI hasFragment
		ifTrue: [^ copy rawFragment: aRelativeURI rawFragment].! !

!XMLURI methodsFor: 'copying' stamp: 'monty 3/30/2016 05:40'!
copyWithFragment: aString
	^ self copy rawFragment: (self percentEncoded: aString)! !

!XMLURI methodsFor: 'copying' stamp: 'monty 3/30/2016 02:34'!
copyWithHost: aString
	| copy |

	copy := self copy.
	(aString size > 0
		and: [aString first == $[
			and: [aString last == $]]])
		ifTrue: [copy rawHost: aString]
		ifFalse: [copy rawHost: (self percentEncoded: aString)].
	^ copy.! !

!XMLURI methodsFor: 'copying' stamp: 'monty 3/30/2016 05:56'!
copyWithPath: aString
	| newRawPathSegments |

	newRawPathSegments := self rawPathSegments copyEmpty.
	$/
		xmlSplit: aString
		do: [:each | newRawPathSegments addLast: (self percentEncoded: each)].
	^ self copy rawPathSegments: newRawPathSegments.! !

!XMLURI methodsFor: 'copying' stamp: 'monty 3/30/2016 05:53'!
copyWithPathSegments: aCollection
	| newRawPathSegments |

	newRawPathSegments := self rawPathSegments copyEmpty.
	aCollection do: [:each |
		newRawPathSegments addLast: (self percentEncoded: each)].
	^ self copy rawPathSegments: newRawPathSegments.! !

!XMLURI methodsFor: 'copying' stamp: 'monty 1/3/2016 20:57'!
copyWithPort: anInteger
	^ self copy port: anInteger! !

!XMLURI methodsFor: 'copying' stamp: 'monty 3/30/2016 05:53'!
copyWithQuery: aDictionary
	| newRawQuery |

	newRawQuery := self rawQuery copyEmpty.
	aDictionary keysAndValuesDo: [:key :value |
		newRawQuery
			at: (self percentAndPlusEncoded: key)
			put: (self percentAndPlusEncoded: value)].
	^ self copy rawQuery: newRawQuery.! !

!XMLURI methodsFor: 'copying' stamp: 'monty 1/13/2016 10:48'!
copyWithScheme: aString
	^ self copy
		scheme: aString;
		hasAuthorityPrefix:
			(self hasAuthorityPrefix
				or: [self hasScheme not])! !

!XMLURI methodsFor: 'copying' stamp: 'monty 3/30/2016 05:54'!
copyWithUserInfo: aString
	^ self copy rawUserInfo: (self percentEncoded: aString)! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 8/20/2016 23:56'!
fragment
	"Returns the percent-decoded URI fragment (everything after the '#') or
	an empty string if there isn't one"

	^ self percentDecoded: self rawFragment! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 5/17/2015 00:05'!
get
	^ self asRetrievableResource get! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 11/29/2015 22:39'!
getUpToLimit: aMaxSize decoding: aBoolean
	^ self asRetrievableResource
		getUpToLimit: aMaxSize
		decoding: aBoolean! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/28/2016 01:07'!
hasAbsolutePath
	^ self hasPath
		and: [self rawPathSegments first isEmpty]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 2/9/2015 01:43'!
hasAuthority
	^ self hasUserInfo
		or: [self hasHost
			or: [self hasPort]]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/2/2016 20:45'!
hasAuthorityPrefix
	self parseURIStringIfUnparsed.
	^ hasAuthorityPrefix.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/3/2016 20:56'!
hasAuthorityPrefix: aBoolean
	hasAuthorityPrefix := aBoolean! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/28/2016 01:07'!
hasDirectoryPath
	^ self hasPath
		and: [self rawPathSegments last isEmpty]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/30/2016 02:08'!
hasFragment
	^ self rawFragment notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/30/2016 02:08'!
hasHost
	^ self rawHost notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/30/2016 02:34'!
hasIPv6Host
	^ self hasHost
		and: [self rawHost first == $[
			and: [self rawHost last == $]]]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/1/2016 07:24'!
hasNonDirectoryPath
	^ self hasPath
		and: [self hasDirectoryPath not]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/28/2016 01:05'!
hasPath
	^ self rawPathSegments notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/1/2016 07:24'!
hasPort
	^ self port notNil! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/30/2016 02:08'!
hasQuery
	^ self rawQuery notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/28/2016 01:05'!
hasRootPath
	^ self rawPathSegments size = 2
		and: [self rawPathSegments allSatisfy: [:each | each isEmpty]]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/1/2016 07:24'!
hasScheme
	^ self scheme notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/30/2016 02:09'!
hasUserInfo
	^ self rawUserInfo notEmpty! !

!XMLURI methodsFor: 'comparing' stamp: 'monty 1/1/2016 07:31'!
hash
	^ self host hash bitXor: self pathSegments hash! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/28/2016 01:06'!
haveDirectoryPath
	self hasPath
		ifFalse: [self rawPathSegments addLast: ''].
	self rawPathSegments addLast: ''.! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 8/20/2016 23:57'!
host
	"Returns the percent-decoded host or IPv6-style host or an empty string
	if there isn't one"

	self hasIPv6Host
		ifTrue: [^ self rawHost]
		ifFalse: [^ (self percentDecoded: self rawHost) asLowercase]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/19/2016 13:24'!
isEmpty
	self hasScheme
		ifTrue: [^ false].
	self hasAuthority
		ifTrue: [^ false].
	self hasPath
		ifTrue: [^ false].
	self hasQuery
		ifTrue: [^ false].
	self hasFragment
		ifTrue: [^ false].

	^ true.! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/1/2016 07:24'!
isFileURI
	^ self scheme = 'file'! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/6/2016 00:12'!
isLocalURI
	^ self isFileURI
		or: [self isRelative
			and: [self hasHost not]]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/6/2016 05:19'!
isRelative
	^ self hasScheme not! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/7/2016 02:24'!
notEmpty
	^ self isEmpty not! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/28/2016 00:06'!
parseURIStream: aStream
	| parser |

	parser := XMLURIParser on: aStream.
	self
		scheme: parser parseScheme;
		hasAuthorityPrefix: parser parseAuthorityPrefix.
	(self hasScheme
		or: [self hasAuthorityPrefix])
		ifTrue: [
			self
				rawUserInfo: parser parseUserInfo;
				rawHost: parser parseHost;
				port: parser parsePort]
		ifFalse: [
			self
				rawUserInfo: '';
				rawHost: ''].
	self
		rawPathSegments: parser parsePathSegments;
		rawQuery: parser parseQuery;
		rawFragment: parser parseFragment.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/5/2016 09:05'!
parseURIStringIfUnparsed
	scheme
		ifNil: [self parseURIStream: self uriString readStream]! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 8/20/2016 23:53'!
path
	"Returns the percent-decoded path as a string"

	self hasPath
		ifTrue: [| writeStream |
			writeStream := (String new: self rawPathSegments size * 10) writeStream.
			self rawPathSegments
				do: [:each | writeStream nextPutAll: (self percentDecoded: each)]
				separatedBy: [writeStream nextPut: $/].
			^ writeStream contents]
		ifFalse: [^ '']! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 8/20/2016 23:59'!
pathSegments
	"Returns a collection of percent-decoded path segments"

	^ self rawPathSegments collect: [:each | self percentDecoded: each]! !

!XMLURI methodsFor: 'enumerating' stamp: 'monty 3/28/2016 01:00'!
pathSegmentsDo: aBlock
	self rawPathSegments do: [:each |
		aBlock value: (self percentDecoded: each)]! !

!XMLURI methodsFor: 'private' stamp: 'monty 4/2/2016 01:30'!
percentAndPlusDecoded: aString
	| readStream writeStream |

	((aString includes: $%)
		or: [aString includes: $+])
		ifFalse: [^ aString].

	readStream := aString readStream.
	writeStream := (String new: aString size) writeStream.
	[readStream atEnd]
		whileFalse: [
			readStream peek == $%
				ifTrue: [
					self
						percentDecodeNextFrom: readStream
						on: writeStream]
				ifFalse: [| nextChar |
					(nextChar := readStream next) == $+
						ifTrue: [writeStream nextPut: Character space]
						ifFalse: [writeStream nextPut: nextChar]]].
	^ writeStream contents.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/28/2016 00:31'!
percentAndPlusEncoded: aString
	| writeStream |

	writeStream := (String new: aString size) writeStream.
	"to:do: for speed"
	1 to: aString size do: [:i | | nextChar |
		(UnreservedCharacters includes: (nextChar := aString at: i))
			ifTrue: [	
				nextChar asciiValue = 32 "Character space asciiValue"
					ifTrue: [writeStream nextPut: $+]
					ifFalse: [writeStream nextPut: nextChar]]
			ifFalse: [
				(XMLPercentEncodingWriteStreamAdapter on: writeStream)
					nextPut: nextChar]].
	^ writeStream contents.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/27/2016 23:34'!
percentDecodeNextFrom: aReadStream on: aWriteStream
	| readStreamAdapter |

	readStreamAdapter := XMLPercentDecodingReadStreamAdapter on: aReadStream.
	[readStreamAdapter atEnd]
		whileFalse: [aWriteStream nextPut: readStreamAdapter next].! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/27/2016 23:32'!
percentDecoded: aString
	| readStream writeStream |

	(aString includes: $%)
		ifFalse: [^ aString].

	readStream := aString readStream.
	writeStream := (String new: aString size) writeStream.
	[readStream atEnd]
		whileFalse: [
			readStream peek == $%
				ifTrue: [
					self
						percentDecodeNextFrom: readStream
						on: writeStream]
				ifFalse: [writeStream nextPut: readStream next]].
	^ writeStream contents.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/28/2016 00:13'!
percentEncoded: aString
	| writeStream |

	writeStream := (String new: aString size) writeStream.
	"to:do: for speed"
	1 to: aString size do: [:i | | nextChar |
		(UnreservedCharacters includes: (nextChar := aString at: i))
			ifTrue: [writeStream nextPut: nextChar]
			ifFalse: [
				(XMLPercentEncodingWriteStreamAdapter on: writeStream)
					nextPut: nextChar]].
	^ writeStream contents.! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 8/20/2016 23:54'!
port
	"Returns the integer port or nil if there isn't one"

	self parseURIStringIfUnparsed.
	^ port.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/3/2016 20:57'!
port: anInteger
	port := anInteger! !

!XMLURI methodsFor: 'copying' stamp: 'monty 1/15/2016 13:41'!
postCopy
	"the copyWith* methods modify copies, so any URI string of a copy must be parsed
	if it hasn't been already and then cleared because it will be obsolete after a
	URI component is changed"
	self
		parseURIStringIfUnparsed;
		uriString: nil! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/12/2016 07:29'!
printAuthorityOn: aStream
	self hasAuthority
		ifTrue: [
			self
				printUserInfoOn: aStream;
				printHostOn: aStream;
				printPortOn: aStream]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/12/2016 07:50'!
printAuthorityPrefixOn: aStream
	(self hasAuthorityPrefix
		or: [self hasScheme not and: [self hasAuthority]])
		ifTrue: [aStream nextPutAll: '//']! !

!XMLURI methodsFor: 'printing' stamp: 'monty 3/28/2016 00:27'!
printFragmentOn: aStream
	self hasFragment
		ifTrue: [
			aStream
				nextPut: $#;
				nextPutAll: self rawFragment]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 3/28/2016 00:34'!
printHostOn: aStream
	self hasHost
		ifTrue: [aStream nextPutAll: self rawHost]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/12/2016 09:30'!
printOn: aStream
	self uriString
		ifNil: [| writeStream |
			writeStream := (String new: 64) writeStream.
			self
				printSchemeOn: writeStream;
				printAuthorityPrefixOn: writeStream;
				printAuthorityOn: writeStream;
				printPathPrefixOn: writeStream;
				printPathOn: writeStream;
				printQueryOn: writeStream;
				printFragmentOn: writeStream.
			self uriString: writeStream contents].
	aStream nextPutAll: self uriString.! !

!XMLURI methodsFor: 'printing' stamp: 'monty 3/28/2016 00:29'!
printPathOn: aStream
	self rawPathSegments
		do: [:each | aStream nextPutAll: each]
		separatedBy: [aStream nextPut: $/]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/12/2016 11:22'!
printPathPrefixOn: aStream
	(self hasPath
		and: [self hasAbsolutePath not
			and: [self hasScheme or: [self hasAuthority]]])
		ifTrue: [aStream nextPut: $/]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/1/2016 23:53'!
printPortOn: aStream
	self hasPort
		ifTrue: [
			aStream nextPut: $:.
			port printOn: aStream]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 3/28/2016 00:33'!
printQueryOn: aStream
	| atBeginning |

	self hasQuery
		ifFalse: [^ self].
	
	aStream nextPut: $?.
	atBeginning := true.
	self rawQuery keysAndValuesDo: [:key :value |
		atBeginning
			ifTrue: [atBeginning := false]
			ifFalse: [aStream nextPut: $&].
		aStream
			nextPutAll: key;
			nextPut: $=;
			nextPutAll: value].! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/12/2016 07:29'!
printSchemeOn: aStream
	self hasScheme
		ifTrue: [
			aStream "do not percent encode scheme"
				nextPutAll: self scheme;
				nextPut: $:]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 3/28/2016 00:27'!
printUserInfoOn: aStream
	self hasUserInfo
		ifTrue: [
			aStream
				nextPutAll: self rawUserInfo;
				nextPut: $@]! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 8/21/2016 00:00'!
query
	"Returns a (temporary) order-preserving dictionary of percent and plus
	decoded query string key=value pairs"

	| query |

	query := self rawQuery copyEmpty.
	self queryKeysAndValuesDo: [:key :value |
		query
			at: key
			put: value].
	^ query.! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 8/26/2016 03:14'!
queryAt: aKey
	"Returns the percent and plus decoded value of the query string parameter at
	aKey or an empty string if there isn't one"

	^ self percentAndPlusDecoded: (self rawQuery at: aKey)! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 3/28/2016 00:30'!
queryAt: aKey ifAbsent: aBlock
	^ self percentAndPlusDecoded:
		(self rawQuery
			at: aKey
			ifAbsent: [^ aBlock value])! !

!XMLURI methodsFor: 'testing' stamp: 'monty 3/30/2016 02:09'!
queryIncludesKey: aKey
	self
		queryAt: aKey
		ifAbsent: [^ false].
	^ true.! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 3/28/2016 00:30'!
queryKeys
	^ self rawQuery keys collect: [:each | self percentAndPlusDecoded: each]! !

!XMLURI methodsFor: 'enumerating' stamp: 'monty 3/28/2016 00:46'!
queryKeysAndValuesDo: aTwoArgumentBlock
	self rawQuery keysAndValuesDo: [:key :value |
		aTwoArgumentBlock
			value: (self percentAndPlusDecoded: key)
			value: (self percentAndPlusDecoded: value)]! !

!XMLURI methodsFor: 'enumerating' stamp: 'monty 3/28/2016 00:45'!
queryKeysDo: aBlock
	self rawQuery keysDo: [:each |
		aBlock value: (self percentAndPlusDecoded: each)]! !

!XMLURI methodsFor: 'private' stamp: 'monty 4/2/2016 01:58'!
rawFragment
	self parseURIStringIfUnparsed.
	^ rawFragment.! !

!XMLURI methodsFor: 'private' stamp: 'monty 4/2/2016 01:58'!
rawFragment: aString
	rawFragment := aString! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 01:45'!
rawHost
	self parseURIStringIfUnparsed.
	^ rawHost.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 01:45'!
rawHost: aString
	rawHost := aString! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 01:45'!
rawPathSegments
	self parseURIStringIfUnparsed.
	^ rawPathSegments.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 01:45'!
rawPathSegments: aPathSegmentCollection
	rawPathSegments := aPathSegmentCollection! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 01:46'!
rawQuery
	self parseURIStringIfUnparsed.
	^ rawQuery.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 01:46'!
rawQuery: aDictionary
	rawQuery := aDictionary! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 01:45'!
rawUserInfo
	self parseURIStringIfUnparsed.
	^ rawUserInfo.! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 01:45'!
rawUserInfo: aString
	rawUserInfo := aString! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 05:48'!
removeLastRawPathSegment
	^ self rawPathSegments removeLast! !

!XMLURI methodsFor: 'private' stamp: 'monty 3/30/2016 05:48'!
removeRawPathSegments
	self rawPathSegments removeAll! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/2/2016 20:45'!
scheme
	self parseURIStringIfUnparsed.
	^ scheme.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/3/2016 20:56'!
scheme: aString
	scheme := aString! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 5/17/2015 00:05'!
streamGet
	^ self asRetrievableResource streamGet! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 11/29/2015 22:38'!
streamGetUpToLimit: aMaxSize decoding: aBoolean
	^ self asRetrievableResource
		streamGetUpToLimit: aMaxSize
		decoding: aBoolean! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/3/2016 20:58'!
uriString
	^ uriString! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/3/2016 20:55'!
uriString: aString
	uriString := aString! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 3/27/2016 23:24'!
userInfo
	^ self percentDecoded: self rawUserInfo! !

Object subclass: #XMLURIParser
	instanceVariableNames: 'writeStream readStream'
	classVariableNames: 'HostDelimiters PathDelimiters QueryKeyDelimiters QueryValueDelimiters SchemeDelimiters UserInfoDelimiters'
	poolDictionaries: ''
	category: 'XML-Parser-URI'!

!XMLURIParser commentStamp: 'monty 3/28/2016 00:01' prior: 0!
This class is a parser that can extract the scheme, user info, host, port, path segments, query (as a dictionary), and fragment of a URI. It does not handle percent decoding and raises no errors.!

!XMLURIParser class methodsFor: 'class initialization' stamp: 'monty 1/10/2016 00:03'!
initialize
	"self initialize"

	self
		initializeSchemeDelimiters;
		initializeUserInfoDelimiters;
		initializeHostDelimiters;
		initializePathDelimiters;
		initializeQueryKeyDelimiters;
		initializeQueryValueDelimiters! !

XMLURIParser initialize!

!XMLURIParser class methodsFor: 'class initialization' stamp: 'monty 1/11/2016 02:16'!
initializeHostDelimiters
	HostDelimiters := BitmapCharacterSet newFrom: ':/?#'! !

!XMLURIParser class methodsFor: 'class initialization' stamp: 'monty 1/11/2016 02:16'!
initializePathDelimiters
	PathDelimiters := BitmapCharacterSet newFrom: '/?#'! !

!XMLURIParser class methodsFor: 'class initialization' stamp: 'monty 1/11/2016 02:17'!
initializeQueryKeyDelimiters
	QueryKeyDelimiters := BitmapCharacterSet newFrom: '=&#'! !

!XMLURIParser class methodsFor: 'class initialization' stamp: 'monty 1/11/2016 02:17'!
initializeQueryValueDelimiters
	QueryValueDelimiters := BitmapCharacterSet newFrom: '&#'! !

!XMLURIParser class methodsFor: 'class initialization' stamp: 'monty 1/11/2016 03:52'!
initializeSchemeDelimiters
	"including % disallows percent encoding in schemes"
	SchemeDelimiters := BitmapCharacterSet newFrom: ':@/?#%'! !

!XMLURIParser class methodsFor: 'class initialization' stamp: 'monty 1/11/2016 02:19'!
initializeUserInfoDelimiters
	UserInfoDelimiters := BitmapCharacterSet newFrom: '@/?#'! !

!XMLURIParser class methodsFor: 'instance creation' stamp: 'monty 1/2/2016 23:42'!
on: aStringOrStream
	^ self new on: aStringOrStream! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/3/2016 20:32'!
atEnd
	^ readStream atEnd! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/3/2016 21:16'!
next
	self atEnd
		ifTrue: [^ nil]
		ifFalse: [^ readStream next]! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 3/27/2016 22:28'!
nextDelimitedBy: aCollection
	writeStream reset.
	[self atEnd
		or: [aCollection includes: self peek]]
		whileFalse: [writeStream nextPut: self next].
	^ writeStream contents.! !

!XMLURIParser methodsFor: 'initialization' stamp: 'monty 2/14/2016 02:36'!
on: aStringOrStream
	readStream :=
		aStringOrStream isStream
			ifTrue: [aStringOrStream]
			ifFalse: [aStringOrStream readStream].
	writeStream := (String new: 64) writeStream.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/3/2016 20:44'!
parseAuthorityPrefix
	| oldPosition |

	oldPosition := self position.
	((self peekFor: $/)
		and: [self peekFor: $/])
		ifTrue: [^ true]
		ifFalse: [
			self position: oldPosition.
			^ false].! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 3/27/2016 22:32'!
parseFragment
	(self peekFor: $#)
		ifTrue: [^ readStream upToEnd]
		ifFalse: [^ '']! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 3/30/2016 01:59'!
parseHost
	| host |

	self peek == $[
		ifTrue: [
			host := (self nextDelimitedBy: ']') copyWith: $].
			self peekFor: $]]
		ifFalse: [host := self nextDelimitedBy: HostDelimiters].
	^ host.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 3/27/2016 22:31'!
parsePathSegments
	| pathSegments segment |

	pathSegments := OrderedCollection new.

	segment := self nextDelimitedBy: PathDelimiters.
	[self peekFor: $/]
		whileTrue: [
			pathSegments addLast: segment.
			segment := self nextDelimitedBy: PathDelimiters].
	(pathSegments isEmpty
		and: [segment isEmpty])
		ifFalse: [pathSegments addLast: segment].

	^ pathSegments.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 2/4/2016 20:31'!
parsePort
	| port |

	(self peekFor: $:)
		ifTrue: [
			port :=
				XMLIntegerReader
					readFrom: readStream
					withBase: 10.
			[self atEnd
				or: [PathDelimiters includes: self peek]]
				whileFalse: [self next]].
	^ port.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 3/27/2016 22:30'!
parseQuery
	| query key value |

	query := OrderPreservingStringDictionary new.
	(self peekFor: $?)
		ifFalse: [^ query].

	[self atEnd
		or: [(key := self nextDelimitedBy: QueryKeyDelimiters) isEmpty
			and: [self peek == $#]]]
		whileFalse: [| hasValue |
			(hasValue := self peekFor: $=)
				ifTrue: [value := self nextDelimitedBy: QueryValueDelimiters]
				ifFalse: [value := ''].
			(key notEmpty
				or: [hasValue])
				ifTrue: [
					query
						at: key
						put: value].
			self peekFor: $&].

	^ query.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 3/27/2016 22:30'!
parseScheme
	| oldPosition scheme |

	oldPosition := self position.
	scheme := self nextDelimitedBy: SchemeDelimiters.
	(self peekFor: $:)
		ifTrue: [^ scheme asLowercase]
		ifFalse: [
			self position: oldPosition.
			^ ''].! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 3/27/2016 22:30'!
parseUserInfo
	| userInfo oldPosition |

	oldPosition := self position.
	userInfo := self nextDelimitedBy: UserInfoDelimiters.
	(self peekFor: $@)
		ifTrue: [^ userInfo]
		ifFalse: [
			self position: oldPosition.
			^ ''].! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/3/2016 21:16'!
peek
	self atEnd
		ifTrue: [^ nil]
		ifFalse: [^ readStream peek]! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 4/5/2016 03:07'!
peekFor: aCharacter
	self peek = aCharacter
		ifTrue: [
			self next.
			^ true]
		ifFalse: [^ false]! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/3/2016 20:35'!
position
	^ readStream position! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/3/2016 20:35'!
position: anInteger
	readStream position: anInteger! !

Object subclass: #XMLUnparsedEntityValidator
	instanceVariableNames: 'unparsedEntities notations notationAttributeValidators'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLUnparsedEntityValidator commentStamp: 'monty 1/1/2015 21:00' prior: 0!
Used by attribute validators to validate attribute references to unparsed entities and notations.!

!XMLUnparsedEntityValidator class methodsFor: 'instance creation' stamp: 'monty 1/1/2015 21:12'!
unparsedEntities: anUnparsedEntityDictionary notations: aNotationSet
	^ self new
		setUnparsedEntities: anUnparsedEntityDictionary
		notations: aNotationSet! !

!XMLUnparsedEntityValidator methodsFor: 'accessing' stamp: 'monty 7/13/2016 01:18'!
addNotationAttributeValidator: aNotationAttributeValidator
	| oldSize |

	"checking size after is faster than includes"
	oldSize := self notationAttributeValidators size.
	(self notationAttributeValidators
		at: aNotationAttributeValidator element
		put: aNotationAttributeValidator;
		size) > oldSize
		ifFalse: [
			self errorMultipleNotationAttributesForElement:
				aNotationAttributeValidator element].! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 4/10/2016 05:24'!
errorMultipleDeclarationsForNotation: aNotation
	XMLValidationException
		formatSignal: 'Multiple declarations for notation "{1}"'
		with: aNotation! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 1/1/2015 20:45'!
errorMultipleNotationAttributesForElement: anElement
	XMLValidationException
		formatSignal: 'Element <{1}> has multiple NOTATION list attributes'
		with: anElement! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 7/13/2016 03:05'!
errorUndeclaredNotation: aNotation inAttribute: anAttribute
	XMLValidationException
		formatSignal: 'Undeclared notation "{1}" in NOTATION (...) attribute "{2}"'
		with: aNotation
		with: anAttribute! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 7/13/2016 03:05'!
errorUndeclaredNotation: aNotation inEntity: anEntity
	XMLValidationException
		formatSignal: 'Undeclared notation "{1}" referenced by <!!ENTITY {2} ...> declaration'
		with: aNotation
		with: anEntity! !

!XMLUnparsedEntityValidator methodsFor: 'private' stamp: 'monty 4/13/2016 06:01'!
errorUndeclaredUnparsedEntityReference: anEntity inAttribute: anAttriubute
	XMLValidationException
		formatSignal: 'Undeclared unparsed entity "{1}" in "{2}" attribute'
		with: anEntity
		with: anAttriubute! !

!XMLUnparsedEntityValidator methodsFor: 'testing' stamp: 'monty 7/13/2016 01:58'!
hasNotationAttributeValidators
	"direct access to avoid lazy initialization"
	^ notationAttributeValidators notNil
		and: [notationAttributeValidators notEmpty]! !

!XMLUnparsedEntityValidator methodsFor: 'accessing' stamp: 'monty 7/13/2016 01:18'!
notationAttributeValidators
	^ notationAttributeValidators ifNil: [notationAttributeValidators := Dictionary new]! !

!XMLUnparsedEntityValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:45'!
notations
	^ notations! !

!XMLUnparsedEntityValidator methodsFor: 'initialization' stamp: 'monty 1/1/2015 21:12'!
setUnparsedEntities: anUnparsedEntityDictionary notations: aNotationSet
	unparsedEntities := anUnparsedEntityDictionary.
	notations := aNotationSet.! !

!XMLUnparsedEntityValidator methodsFor: 'accessing' stamp: 'monty 1/1/2015 20:34'!
unparsedEntities
	^ unparsedEntities! !

!XMLUnparsedEntityValidator methodsFor: 'validating' stamp: 'monty 7/13/2016 00:27'!
validateEntityReference: anEntity inAttribute: anAttribute
	(self unparsedEntities includesKey: anEntity)
		ifFalse: [
			self
				errorUndeclaredUnparsedEntityReference: anEntity
				inAttribute: anAttribute]! !

!XMLUnparsedEntityValidator methodsFor: 'validating' stamp: 'monty 7/13/2016 03:04'!
validateNotation: aNotation
	"Unlike redundant entity declarations which are just ignored, redundant
	notation declarations are invalid"
	(self notations includes: aNotation)
		ifTrue: [self errorMultipleDeclarationsForNotation: aNotation]! !

!XMLUnparsedEntityValidator methodsFor: 'validating' stamp: 'monty 7/13/2016 03:02'!
validateNotationAttributes
	"this validatin has to be delayed until the entire DTD
	is processed so NOTATION attributes can be declared before
	the notations in their enumeration are"
	self notationAttributeValidators valuesDo: [:validator |
		validator allowedValues do: [:each |
			(self notations includes: each)
				ifFalse: [
					self
						errorUndeclaredNotation: each
						inAttribute: validator attribute]]]! !

!XMLUnparsedEntityValidator methodsFor: 'validating' stamp: 'monty 7/13/2016 03:04'!
validateUnparsedEntities
	"this validatin has to be delayed until the entire DTD
	is processed so unparsed entities can be declared before
	their notations"
	self unparsedEntities valuesDo: [:each |
		(self notations includes: each ndata)
			ifFalse: [
				self
					errorUndeclaredNotation: each ndata
					inEntity: each name]]! !

!Stream methodsFor: '*XML-Parser' stamp: 'monty 12/11/2014 06:32'!
parseXML
	^ XMLDOMParser parse: self! !

!UndefinedObject methodsFor: '*XML-Parser' stamp: 'monty 3/24/2016 20:03'!
isXMLChar
	"hack optimization to avoid testing for nil before sending
	this to objects from streams in XMLParser"
	^ false! !

!UndefinedObject methodsFor: '*XML-Parser' stamp: 'monty 3/25/2016 17:35'!
isXMLNCNameChar
	"hack optimization to avoid testing for nil before sending
	this to objects from streams in XMLParser"
	^ false! !

!UndefinedObject methodsFor: '*XML-Parser' stamp: 'monty 3/25/2016 17:35'!
isXMLNCNameStartChar
	"hack optimization to avoid testing for nil before sending
	this to objects from streams in XMLParser"
	^ false! !

!UndefinedObject methodsFor: '*XML-Parser' stamp: 'monty 3/24/2016 20:03'!
isXMLNameChar
	"hack optimization to avoid testing for nil before sending
	this to objects from streams in XMLParser"
	^ false! !

!UndefinedObject methodsFor: '*XML-Parser' stamp: 'monty 3/24/2016 20:03'!
isXMLNameStartChar
	"hack optimization to avoid testing for nil before sending
	this to objects from streams in XMLParser"
	^ false! !

!UndefinedObject methodsFor: '*XML-Parser' stamp: 'monty 3/12/2016 19:09'!
isXMLSeparator
	"hack optimization to avoid testing for nil before sending
	this to objects from streams in XMLParser"
	^ false! !

!UndefinedObject methodsFor: '*XML-Parser' stamp: 'monty 3/12/2016 19:09'!
isXMLWhitespace
	"hack optimization to avoid testing for nil before sending
	this to objects from streams in XMLParser"
	^ false! !

!Character class methodsFor: '*XML-Parser' stamp: 'monty 3/12/2016 19:10'!
xmlSeparators
	^ String
		with: 32 asCharacter "space"
		with: 10 asCharacter "lf"
		with: 9 asCharacter "tab"
		with: 13 asCharacter "cr"! !

!Character class methodsFor: '*XML-Parser' stamp: 'monty 3/12/2016 19:10'!
xmlWhitespace
	^ self xmlSeparators copyWith: 12 asCharacter "form feed"! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 3/25/2016 05:35'!
isXMLChar
	"Range tests are faster and use less memory than a BitmapCharacterSet, and
	a temp is used because Character in newer SpurVM images has no 'value'
	inst var.
	
	Char ::=
		#x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | [#x10000-#x10FFFF]"

	| tempValue |

	(tempValue := self asciiValue) >= 16r20
		ifTrue: [
			tempValue <= 16rD7FF
				ifTrue: [^ true].

			tempValue >= 16rE000
				ifFalse: [^ false].
			tempValue <= 16rFFFD
				ifTrue: [^ true].

			tempValue >= 16r10000
				ifFalse: [^ false].
			^ tempValue <= 16r10FFFF].

	tempValue = 16r9
		ifTrue: [^ true].
	tempValue = 16rA
		ifTrue: [^ true].
	^ tempValue = 16rD.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 3/26/2016 01:50'!
isXMLNCNameChar
	"should be the same as isXMLNameChar except without a test for $:"

	| tempValue |

	(tempValue := self asciiValue) <= 16r7A "$z asciiValue hex"
		ifTrue: [
			tempValue >= 16r61 "$a asciiValue hex"
				ifTrue: [^ true].

			tempValue >= 16r41 "$A asciiValue hex"
				ifTrue: [
					tempValue <= 16r5A "$Z asciiValue hex"
						ifTrue: [^ true].
					^ tempValue = 16r5F]. "$_ asciiValue hex"

			tempValue >= 16r30 "$0 asciiValue hex"
				ifTrue: [^ tempValue <= 16r39]. "$9 asciiValue hex; no $: test"

			tempValue = 16r2E "$. asciiValue hex"
				ifTrue: [^ true].
			^ tempValue = 16r2D]. "$- asciiValue hex"

	tempValue >= 16rC0
		ifFalse: [^ tempValue = 16rB7].
	tempValue <= 16rD6
		ifTrue: [^ true].

	tempValue >= 16rD8
		ifFalse: [^ false].
	tempValue <= 16rF6
		ifTrue: [^ true].

	tempValue >= 16rF8
		ifFalse: [^ false].
	tempValue <= 16r2FF
		ifTrue: [^ true].

	tempValue >= 16r300
		ifFalse: [^ false].
	tempValue <= 16r36F
		ifTrue: [^ true].

	tempValue >= 16r370
		ifFalse: [^ false].
	tempValue <= 16r37D
		ifTrue: [^ true].

	tempValue >= 16r37F
		ifFalse: [^ false].
	tempValue <= 16r1FFF
		ifTrue: [^ true].

	tempValue >= 16r200C
		ifFalse: [^ false].
	tempValue <= 16r200D
		ifTrue: [^ true].

	tempValue >= 16r203F
		ifFalse: [^ false].
	tempValue <= 16r2040
		ifTrue: [^ true].

	tempValue >= 16r2070
		ifFalse: [^ false].
	tempValue <= 16r218F
		ifTrue: [^ true].

	tempValue >= 16r2C00
		ifFalse: [^ false].
	tempValue <= 16r2FEF
		ifTrue: [^ true].

	tempValue >= 16r3001
		ifFalse: [^ false].
	tempValue <= 16rD7FF
		ifTrue: [^ true].

	tempValue >= 16rF900
		ifFalse: [^ false].
	tempValue <= 16rFDCF
		ifTrue: [^ true].

	tempValue >= 16rFDF0
		ifFalse: [^ false].
	tempValue <= 16rFFFD
		ifTrue: [^ true].

	tempValue >= 16r10000
		ifFalse: [^ false].
	^ tempValue <= 16rEFFFF.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 7/11/2016 22:35'!
isXMLNCNameStartChar
	"should be the same as isXMLNameStartChar except without a test for $:"

	| tempValue |

	(tempValue := self asciiValue) <= 16r7A "$z asciiValue hex"
		ifTrue: [
			tempValue >= 16r61 "$a asciiValue hex"
				ifTrue: [^ true].

			tempValue >= 16r41 "$A asciiValue hex"
				ifFalse: [^ false].
			tempValue <= 16r5A "$Z asciiValue hex"
				ifTrue: [^ true].

			^ tempValue = 16r5F]. "$_ asciiValue hex; no $: test"

	tempValue >= 16rC0
		ifFalse: [^ false].
	tempValue <= 16rD6
		ifTrue: [^ true].

	tempValue >= 16rD8
		ifFalse: [^ false].
	tempValue <= 16rF6
		ifTrue: [^ true].

	tempValue >= 16rF8
		ifFalse: [^ false].
	tempValue <= 16r2FF
		ifTrue: [^ true].

	tempValue >= 16r370
		ifFalse: [^ false].
	tempValue <= 16r37D
		ifTrue: [^ true].

	tempValue >= 16r37F
		ifFalse: [^ false].
	tempValue <= 16r1FFF
		ifTrue: [^ true].

	tempValue >= 16r200C
		ifFalse: [^ false].
	tempValue <= 16r200D
		ifTrue: [^ true].

	tempValue >= 16r2070
		ifFalse: [^ false].
	tempValue <= 16r218F
		ifTrue: [^ true].

	tempValue >= 16r2C00
		ifFalse: [^ false].
	tempValue <= 16r2FEF
		ifTrue: [^ true].

	tempValue >= 16r3001
		ifFalse: [^ false].
	tempValue <= 16rD7FF
		ifTrue: [^ true].

	tempValue >= 16rF900
		ifFalse: [^ false].
	tempValue <= 16rFDCF
		ifTrue: [^ true].

	tempValue >= 16rFDF0
		ifFalse: [^ false].
	tempValue <= 16rFFFD
		ifTrue: [^ true].

	tempValue >= 16r10000
		ifFalse: [^ false].
	^ tempValue <= 16rEFFFF.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 3/26/2016 01:49'!
isXMLNameChar
	"Range tests are faster and use less memory than a BitmapCharacterSet, and
	a temp is used because Character in newer SpurVM images has no 'value'
	inst var.
	
	NameStartChar ::=
		':' | [A-Z] | '_' | [a-z] | [#xC0-#xD6] | [#xD8-#xF6] | [#xF8-#x2FF] |
		[#x370-#x37D] | [#x37F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] |
		[#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] |
		[#x10000-#xEFFFF]
	NameChar ::=
		NameStartChar | '-' | '.' | [0-9] | #xB7 | [#x0300-#x036F] |
		[#x203F-#x2040]"

	| tempValue |

	(tempValue := self asciiValue) <= 16r7A "$z asciiValue hex"
		ifTrue: [
			tempValue >= 16r61 "$a asciiValue hex"
				ifTrue: [^ true].

			tempValue >= 16r41 "$A asciiValue hex"
				ifTrue: [
					tempValue <= 16r5A "$Z asciiValue hex"
						ifTrue: [^ true].
					^ tempValue = 16r5F]. "$_ asciiValue hex"

			tempValue >= 16r30 "$0 asciiValue hex"
				ifTrue: [
					tempValue <= 16r39 "$9 asciiValue hex"
						ifTrue: [^ true].
					^ tempValue = 16r3A]. "$: asciiValue hex"

			tempValue = 16r2E "$. asciiValue hex"
				ifTrue: [^ true].
			^ tempValue = 16r2D]. "$- asciiValue hex"

	tempValue >= 16rC0
		ifFalse: [^ tempValue = 16rB7].
	tempValue <= 16rD6
		ifTrue: [^ true].

	tempValue >= 16rD8
		ifFalse: [^ false].
	tempValue <= 16rF6
		ifTrue: [^ true].

	tempValue >= 16rF8
		ifFalse: [^ false].
	tempValue <= 16r2FF
		ifTrue: [^ true].

	tempValue >= 16r300
		ifFalse: [^ false].
	tempValue <= 16r36F
		ifTrue: [^ true].

	tempValue >= 16r370
		ifFalse: [^ false].
	tempValue <= 16r37D
		ifTrue: [^ true].

	tempValue >= 16r37F
		ifFalse: [^ false].
	tempValue <= 16r1FFF
		ifTrue: [^ true].

	tempValue >= 16r200C
		ifFalse: [^ false].
	tempValue <= 16r200D
		ifTrue: [^ true].

	tempValue >= 16r203F
		ifFalse: [^ false].
	tempValue <= 16r2040
		ifTrue: [^ true].

	tempValue >= 16r2070
		ifFalse: [^ false].
	tempValue <= 16r218F
		ifTrue: [^ true].

	tempValue >= 16r2C00
		ifFalse: [^ false].
	tempValue <= 16r2FEF
		ifTrue: [^ true].

	tempValue >= 16r3001
		ifFalse: [^ false].
	tempValue <= 16rD7FF
		ifTrue: [^ true].

	tempValue >= 16rF900
		ifFalse: [^ false].
	tempValue <= 16rFDCF
		ifTrue: [^ true].

	tempValue >= 16rFDF0
		ifFalse: [^ false].
	tempValue <= 16rFFFD
		ifTrue: [^ true].

	tempValue >= 16r10000
		ifFalse: [^ false].
	^ tempValue <= 16rEFFFF.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 3/25/2016 05:43'!
isXMLNameStartChar
	"Range tests are faster and use less memory than a BitmapCharacterSet, and
	a temp is used because Character in newer SpurVM images has no 'value'
	inst var.
	
	NameStartChar ::=
		':' | [A-Z] | '_' | [a-z] | [#xC0-#xD6] | [#xD8-#xF6] | [#xF8-#x2FF] |
		[#x370-#x37D] | [#x37F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] |
		[#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] |
		[#x10000-#xEFFFF]"

	| tempValue |

	(tempValue := self asciiValue) <= 16r7A "$z asciiValue hex"
		ifTrue: [
			tempValue >= 16r61 "$a asciiValue hex"
				ifTrue: [^ true].

			tempValue >= 16r41 "$A asciiValue hex"
				ifTrue: [
					tempValue <= 16r5A "$Z asciiValue hex"
						ifTrue: [^ true].
					^ tempValue = 16r5F]. "$_ asciiValue hex"
	
			^ tempValue = 16r3A]. "$: asciiValue hex"

	tempValue >= 16rC0
		ifFalse: [^ false].
	tempValue <= 16rD6
		ifTrue: [^ true].

	tempValue >= 16rD8
		ifFalse: [^ false].
	tempValue <= 16rF6
		ifTrue: [^ true].

	tempValue >= 16rF8
		ifFalse: [^ false].
	tempValue <= 16r2FF
		ifTrue: [^ true].

	tempValue >= 16r370
		ifFalse: [^ false].
	tempValue <= 16r37D
		ifTrue: [^ true].

	tempValue >= 16r37F
		ifFalse: [^ false].
	tempValue <= 16r1FFF
		ifTrue: [^ true].

	tempValue >= 16r200C
		ifFalse: [^ false].
	tempValue <= 16r200D
		ifTrue: [^ true].

	tempValue >= 16r2070
		ifFalse: [^ false].
	tempValue <= 16r218F
		ifTrue: [^ true].

	tempValue >= 16r2C00
		ifFalse: [^ false].
	tempValue <= 16r2FEF
		ifTrue: [^ true].

	tempValue >= 16r3001
		ifFalse: [^ false].
	tempValue <= 16rD7FF
		ifTrue: [^ true].

	tempValue >= 16rF900
		ifFalse: [^ false].
	tempValue <= 16rFDCF
		ifTrue: [^ true].

	tempValue >= 16rFDF0
		ifFalse: [^ false].
	tempValue <= 16rFFFD
		ifTrue: [^ true].

	tempValue >= 16r10000
		ifFalse: [^ false].
	^ tempValue <= 16rEFFFF.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 3/25/2016 05:39'!
isXMLSeparator
	"a temp is used because Character in newer SpurVM images
	has no 'value' inst var"
	| tempValue |

	(tempValue := self asciiValue) = 32 "space"
		ifTrue: [^ true].
	tempValue = 10 "line feed"
		ifTrue: [^ true].
	tempValue = 9 "tab"
		ifTrue: [^ true].
	^ tempValue = 13. "carriage return; no form feeds"! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 3/25/2016 05:38'!
isXMLWhitespace
	"a temp is used because Character in newer SpurVM images
	has no 'value' inst var"
	| tempValue |

	(tempValue := self asciiValue) = 32 "space"
		ifTrue: [^ true].
	tempValue = 10 "line feed"
		ifTrue: [^ true].
	tempValue = 9 "tab"
		ifTrue: [^ true].
	tempValue = 13 "carriage return"
		ifTrue: [^ true].
	^ tempValue = 12. "form feed"! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 3/25/2016 02:03'!
xmlSplit: aString
	"faster, portable version"
	| parts |

	parts := OrderedCollection new.
	self
		xmlSplit: aString
		do: [:each | parts addLast: each].
	^ parts.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 4/5/2016 03:12'!
xmlSplit: aString do: aBlock
	"faster, portable version"
	| start |

	start := 1.
	1 to: aString size do: [:end |
		(aString at: end) = self
			ifTrue: [
				aBlock value: (aString copyFrom: start to: end - 1).
				start := end + 1]].
	start > aString size
		ifTrue: [
			"either aString is empty or it ends with self"
			aBlock value: '']
		ifFalse: [aBlock value: (aString copyFrom: start to: aString size)].! !

XMLIntegerReader initialize!

XMLAbstractFactory initialize!

XMLURIParser initialize!

XMLAttributeValidator initialize!

XMLEncodingDetector initialize!

DTDAttributeDefinitionParser initialize!

XMLNestedStreamReader initialize!

XMLParserTokenizer initialize!

DTDElementContentModelParser initialize!

XMLFileHandle initialize!

XMLStreamConverter initialize!

XMLURI initialize!

XMLWriteStreamAdapter initialize!

XMLWellFormedParserTokenizer initialize!
